/*
  Copyright (C) 2002-2020 CERN for the benefit of the ATLAS collaboration
*/

#ifndef JIVEXML_MDTPREPDATARETRIEVER_H
#define JIVEXML_MDTPREPDATARETRIEVER_H

#include "JiveXML/IDataRetriever.h"
#include "AthenaBaseComps/AthAlgTool.h"
#include "GaudiKernel/ServiceHandle.h"
#include "GaudiKernel/ToolHandle.h"

#include "MuonIdHelpers/IMuonIdHelperSvc.h"

#include <string>

namespace JiveXML {
  
  class MdtPrepDataRetriever : virtual public IDataRetriever, public AthAlgTool {
    
  public:
    
    /// Standard Constructor
    MdtPrepDataRetriever(const std::string& type, const std::string& name, const IInterface* parent);
  
    /// Retrieve all the data
    virtual StatusCode retrieve(ToolHandle<IFormatTool> &FormatTool);

    /// Return the name of the data type
    virtual std::string dataTypeName() const { return m_typeName; };

    ///Default AthAlgTool methods
    StatusCode initialize();

  private:
   
   ///The data type that is generated by this retriever
    const std::string m_typeName;

    ///The storegate key for the MDT collection
    std::string m_sgKey;

    ServiceHandle<Muon::IMuonIdHelperSvc> m_idHelperSvc {this, "MuonIdHelperSvc", "Muon::MuonIdHelperSvc/MuonIdHelperSvc"};

    int m_adcCut;
    bool m_obeyMasked;
  };
  
}
#endif
