/*
  Copyright (C) 2002-2021 CERN for the benefit of the ATLAS collaboration
*/

#ifndef JIVEXML_TRIGRPCDATARETRIEVER_H
#define JIVEXML_TRIGRPCDATARETRIEVER_H


#include "AthenaBaseComps/AthAlgTool.h"
#include "JiveXML/IDataRetriever.h"
#include "GaudiKernel/ServiceHandle.h"
#include "GaudiKernel/ToolHandle.h"
#include "StoreGate/ReadCondHandleKey.h"

#include "MuonReadoutGeometry/MuonDetectorManager.h"
#include "MuonRPC_CnvTools/IRPC_RDO_Decoder.h"
#include "RPC_CondCabling/RpcCablingCondData.h"
#include "MuonIdHelpers/IMuonIdHelperSvc.h"

#include <string>

namespace JiveXML {

  class TrigRpcDataRetriever : virtual public IDataRetriever, public AthAlgTool {
    
  public:

    /// Standard Constructor
    TrigRpcDataRetriever(const std::string& type,const std::string& name,const IInterface* parent);

      /// Retrieve all the data
    virtual StatusCode retrieve(ToolHandle<IFormatTool> &FormatTool);

    /// Return the name of the data type
    virtual std::string dataTypeName() const { return m_typeName; };

    ///Default AthAlgTool methods
    StatusCode initialize();
   
  private:
    
   ///The data type that is generated by this retriever
    const std::string m_typeName;

    Gaudi::Property<std::string> m_sgKey{this,"StoreGateKey","RPCPAD","StoreGate key for the RPC RDO container"};

    ServiceHandle<Muon::IMuonIdHelperSvc> m_idHelperSvc {this, "MuonIdHelperSvc", "Muon::MuonIdHelperSvc/MuonIdHelperSvc"};

    ToolHandle<Muon::IRPC_RDO_Decoder> m_rpcDecoder{this,"RpcRDO_Decoder","Muon::RpcRDO_Decoder"}; 
    
    SG::ReadCondHandleKey<RpcCablingCondData> m_rpcCab{this, "ReadKey", "RpcCablingCondData", "Key of RpcCablingCondData"};
    SG::ReadCondHandleKey<MuonGM::MuonDetectorManager> m_DetectorManagerKey {this, "DetectorManagerKey", "MuonDetectorManager", "Key of input MuonDetectorManager condition data"};
  };
  
}
#endif
