# Copyright (C) 2002-2020 CERN for the benefit of the ATLAS collaboration

#
# File: CaloClusterCorrection/python/CaloSwEtaoff_v3.py
# Created: Nov 2006, sss
# Purpose: Eta offset (s-shape) corrections, v3.
#
# These corrections were derived using 11.0.41 simulation and 12.0
# reconstruction, for both electrons and photons, 5x5, 3x5, 3x7 clusters.
#
# This correction was in CaloClusterCorrection-00-02-58, in 12.0.3.
#
# See CaloSwEtaoff_v3.h for more details.
#

from CaloClusterCorrection.constants import \
     CALOCORR_EMB1, CALOCORR_EMB2, CALOCORR_EME1, CALOCORR_EME2, \
     EMB1, EME1, EMB2, EME2
from AthenaCommon.SystemOfUnits import GeV

###############################################################################
# Tables for v3 sshape correction, layer 1, 5x5 electron clusters
#


ele5_layer1_etaoffv3_55_b = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
    [0, 0, 0, 0, 0, 0], # Removed by hand, see below
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele5_layer1_etaoffv3_55_e = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele10_layer1_etaoffv3_55_b = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0015706614,   0.00062781902 ],
  [               0,               0,               0,               0,      -6.3444008,     -0.44514953 ],
  [               0,               0,               0,               0,       -2.386005,        1.036892 ],
  [               0,               0,               0,               0,       10.361853,       3.1626612 ],
  [               0,               0,               0,               0,       3.3264339,      0.16645127 ],
  [               0,               0,               0,               0,  -0.00091942655,  -0.00018379891 ],
  [               0,               0,               0,               0,      -2.1914544,       1.6585196 ],
  [               0,               0,               0,               0,       2.5477847,      0.35548449 ],
 ],
 [
  [               0,               0,               0,    0.0022725477,   -0.0033716158,    0.0013920363 ],
  [               0,               0,               0,       -411.0477,       474.52622,      -119.66925 ],
  [               0,               0,               0,        82.09395,      -74.375286,       16.987238 ],
  [               0,               0,               0,       46.823476,      -37.186028,       6.8032089 ],
  [               0,               0,               0,      -6.1746957,        3.857455,      -0.7950942 ],
  [               0,               0,               0,   0.00041506748,  -0.00068625035,    0.0003291814 ],
  [               0,               0,               0,       16.693188,      -16.354496,       6.8442456 ],
  [               0,               0,               0,       9.3874015,      -10.183035,       4.3040714 ],
 ],
 [
  [               0,               0,               0,               0,               0,   0.00019554784 ],
  [               0,               0,               0,               0,               0,       45.870785 ],
  [               0,               0,               0,               0,               0,      -7.5286336 ],
  [               0,               0,               0,               0,               0,       3.9566071 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0075985908,      0.54972286,      -0.4316456 ],
  [               0,               0,               0,      -1.5352101,       1.6075213,       0.1916814 ],
  [               0,               0,               0,       1.2395091,      -1.2755063,     -0.21897513 ],
  [               0,               0,               0,       1.1587431,      -1.0905256,      -0.3226703 ],
  [               0,               0,               0,    -0.035145291,     0.085432153,     -0.05153608 ],
  [               0,               0,               0,     -0.17593984,      0.32624917,     -0.15068605 ],
  [               0,               0,               0,      -1.0309338,       2.7933241,      -1.5139337 ],
  [               0,               0,               0,     -0.26593067,      0.75597341,       1.0353853 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     -0.13107283,      0.14211434,     0.069508038 ],
  [               0,               0,               0,       663.54822,      -1633.2742,       985.11169 ],
  [               0,               0,               0,       7297.6118,      -20810.523,       14835.383 ],
  [               0,               0,               0,       5112.2031,      -14542.381,       10340.002 ],
  [               0,               0,               0,       476.42899,      -1345.0564,         948.815 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele10_layer1_etaoffv3_55_e = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.020981628,     0.040734868 ],
  [               0,               0,               0,               0,      -2.7010188,       5.5153956 ],
  [               0,               0,               0,               0,       1.9176511,      -3.9062123 ],
  [               0,               0,               0,               0,       3.2416568,      -6.0058017 ],
  [               0,               0,               0,               0,    -0.078389339,      0.17368643 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele25_layer1_etaoffv3_55_b = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,  -0.00044184751,    0.0018388959 ],
  [               0,               0,               0,               0,      -2.4869014,        1.821924 ],
  [               0,               0,               0,               0,       1.6661017,     -0.97780021 ],
  [               0,               0,               0,               0,       1.7232846,     -0.93155174 ],
  [               0,               0,               0,               0,     -0.07006519,      -0.0029612 ],
  [               0,               0,               0,               0,  -0.00043859683,   2.0825465e-05 ],
  [               0,               0,               0,               0,      -3.6951445,       3.1186172 ],
  [               0,               0,               0,               0,      0.86812033,       1.0604254 ],
 ],
 [
  [               0,               0,               0,    0.0014950805,   -0.0028498712,    0.0016034598 ],
  [               0,               0,               0,       -41.35124,       33.572831,      -3.1045143 ],
  [               0,               0,               0,       11.153688,      -10.472834,       1.9834852 ],
  [               0,               0,               0,       7.2444823,      -0.3797206,      -1.9990723 ],
  [               0,               0,               0,      -1.9941959,       2.3915074,     -0.82972197 ],
  [               0,               0,               0,    -0.003159871,    0.0020769778,  -0.00029213418 ],
  [               0,               0,               0,       8.4390544,      -14.060555,       6.8137052 ],
  [               0,               0,               0,      -13.806472,       19.616392,       -5.562683 ],
 ],
 [
  [               0,               0,               0,               0,     0.025892759,  -0.00012947131 ],
  [               0,               0,               0,               0,      -11.134826,       9.4502392 ],
  [               0,               0,               0,               0,      -20.382269,       15.679564 ],
  [               0,               0,               0,               0,       20.660542,      -16.429544 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0026498005,   -0.0070837882,    0.0049237295 ],
  [               0,               0,               0,      -19.045169,       34.893713,       -13.18859 ],
  [               0,               0,               0,        17.03737,       -30.76539,        12.92295 ],
  [               0,               0,               0,      -1.2774867,       9.6544941,      -7.6481886 ],
  [               0,               0,               0,      -4.9748464,        10.24534,      -5.2726434 ],
  [               0,               0,               0,  -3.4214434e-05,  -0.00056306694,   0.00042221303 ],
  [               0,               0,               0,       4.9299195,      -11.563646,       7.7862999 ],
  [               0,               0,               0,      -4.8706384,       14.125837,      -8.8046659 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.016978431,     0.017512703,   -0.0030990581,    -0.072446913 ],
  [               0,               0,      -156.03229,       226.23109,       323.12637,      -463.90112 ],
  [               0,               0,       1273.4395,      -1836.7859,      -2639.7385,       3805.6262 ],
  [               0,               0,      -457.01675,        651.8902,       878.66888,      -1255.1594 ],
  [               0,               0,       192.83768,      -268.20703,      -388.37036,       539.53577 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele25_layer1_etaoffv3_55_e = [
 [
  [               0,               0,               0,               0,      0.44954631,     -0.63378936 ],
  [               0,               0,               0,               0,       96.088539,      -140.25194 ],
  [               0,               0,               0,               0,      -82.864655,       120.00001 ],
  [               0,               0,               0,               0,       9.9662724,      -13.894251 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,       1.0573786,    -0.042377956,      -2.1588733 ],
  [               0,               0,               0,      -144.65092,       54.314568,       233.65034 ],
  [               0,               0,               0,       91.283401,       42.273254,       -260.1897 ],
  [               0,               0,               0,      -88.765846,      -1.1214867,       187.49055 ],
  [               0,               0,               0,     0.037112545,     0.020102786,     -0.11981502 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,       2126.8567 ],
  [               0,               0,               0,               0,               0,     0.024206102 ],
  [               0,               0,               0,               0,               0,    -0.024201646 ],
  [               0,               0,               0,               0,               0,   7.2605178e-07 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0035585887,    0.0024062472,     0.002231353,    0.0079824124 ],
  [               0,               0,      -1.2341887,      -3.0967455,       18.946318,      -16.760548 ],
  [               0,               0,       6.6378312,      -23.911489,        25.84226,      -7.8449321 ],
  [               0,               0,     -0.61755121,       12.577948,      -35.192223,       26.118864 ],
  [               0,               0,      -1.0471343,       5.2306342,      -8.7021627,       4.8253818 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0019307922,   -0.0028480527 ],
  [               0,               0,               0,               0,      -9.6849003,       22.361542 ],
  [               0,               0,               0,               0,      -0.6357137,      0.39466339 ],
  [               0,               0,               0,               0,        1.604297,      -3.3460672 ],
  [               0,               0,               0,               0,     -0.65675688,       1.3717926 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.041173596,    -0.088445932,     -0.19239816,      0.41666228 ],
  [               0,               0,      -35.740318,       184.44098,      -297.27213,       143.98109 ],
  [               0,               0,       8.2044106,      -39.917191,       57.734322,      -22.359846 ],
  [               0,               0,      -34.403839,        235.5327,      -534.73792,       401.74631 ],
  [               0,               0,       17.319733,      -112.69702,       244.25945,      -176.30406 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      -1.4802551,       3.4373825 ],
  [               0,               0,               0,               0,      -16.743881,         39.1782 ],
  [               0,               0,               0,               0,       5.3986483,      -13.050788 ],
  [               0,               0,               0,               0,        3.144839,      -7.1905832 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0096382368,     -0.02116129 ],
  [               0,               0,               0,               0,       -56.92577,       137.53165 ],
  [               0,               0,               0,               0,       12.672087,      -30.739578 ],
  [               0,               0,               0,               0,       7.2578621,      -17.484474 ],
  [               0,               0,               0,               0,      -1.1810434,       2.9555435 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0082226871,    0.0093056606,    -0.064076796 ],
  [               0,               0,               0,      -55.386269,       224.50885,      -211.76149 ],
  [               0,               0,               0,       681.43481,       -3324.949,       4053.9829 ],
  [               0,               0,               0,      -11.140888,       66.950546,      -96.893883 ],
  [               0,               0,               0,    0.0095915161,   -0.0096312435,    -0.029354062 ],
  [               0,               0,               0,       23.413624,      -101.67561,       109.12014 ],
  [               0,               0,               0,       21.456503,      -212.96181,       383.86746 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0064971559,   -0.0055787498 ],
  [               0,               0,               0,               0,      -1602.3992,        3985.532 ],
  [               0,               0,               0,               0,               0,     -0.40000001 ],
  [               0,               0,               0,               0,      -107.22143,       263.87305 ],
  [               0,               0,               0,               0,     0.041471589,     -0.21492033 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele50_layer1_etaoffv3_55_b = [
 [
  [               0,               0,      -810.54034,       59.110523,      -0.7605409,    0.0014721213 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00067471702,   0.00058228141 ],
  [               0,               0,               0,               0,      -1.4778765,       3.9769259 ],
  [               0,               0,               0,               0,     -0.85124941,     -0.39009087 ],
  [               0,               0,               0,               0,     0.010725257,     -0.87529986 ],
  [               0,               0,               0,               0,      0.37312234,     -0.25230508 ],
  [               0,               0,               0,               0,   -0.0001247544,   0.00011391728 ],
  [               0,               0,               0,               0,     0.099909324,      0.67096458 ],
  [               0,               0,               0,               0,       4.1843834,     -0.13630718 ],
 ],
 [
  [               0,               0,               0,    0.0019214215,   -0.0023310352,    0.0015985409 ],
  [               0,               0,               0,      -14.093939,        16.75639,      -1.6236217 ],
  [               0,               0,               0,        2.907173,      -3.2847784,     0.044738212 ],
  [               0,               0,               0,      0.54601658,     -0.15466303,      -1.1211276 ],
  [               0,               0,               0,       -1.308857,       1.3753372,     -0.47957891 ],
  [               0,               0,               0,    0.0013659088,    -0.001766656,   0.00053899409 ],
  [               0,               0,               0,      -12.692372,       19.735783,      -5.1889124 ],
  [               0,               0,               0,       18.232702,      -20.415094,       4.8812525 ],
 ],
 [
  [               0,               0,               0,               0,     0.023271454,    0.0088902591 ],
  [               0,               0,               0,               0,      -11.911762,       10.033204 ],
  [               0,               0,               0,               0,      -8.5924807,       6.3258781 ],
  [               0,               0,               0,               0,       14.524152,      -11.550827 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0027413515,   0.00037746405,     0.008729784 ],
  [               0,               0,               0,      0.70862722,     -0.95613715,       1.3717366 ],
  [               0,               0,               0,     -0.34363708,      0.55552443,      -1.0315936 ],
  [               0,               0,               0,   -0.0048784847,     -0.22051965,     -0.66563473 ],
  [               0,               0,               0,    -0.026670551,     0.026184216,    -0.035637228 ],
  [               0,               0,               0,  -1.2738806e-05,  -5.2232868e-05,   6.9367227e-05 ],
  [               0,               0,               0,      -1.6887284,       1.4333075,       3.1171733 ],
  [               0,               0,               0,       22.060372,      -45.500493,       27.203559 ],
 ],
 [
  [               0,               0,               0,               0,               0,       132.65952 ],
  [               0,               0,               0,               0,               0,     0.036768053 ],
  [               0,               0,               0,               0,               0,    -0.036751382 ],
  [               0,               0,               0,               0,               0,    -0.036755461 ],
  [               0,               0,               0,               0,               0,  -1.6823894e-06 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.019404886,    0.0090145785,    -0.012676204,    -0.052498702 ],
  [               0,               0,       -39.14032,      -17.499323,       24.256966,       106.20184 ],
  [               0,               0,      -29.623285,      -14.894537,        15.97854,       73.004768 ],
  [               0,               0,               0,               0,               0,              -1 ],
  [               0,               0,       1.0788304,       1.5908196,        2.157037,       2.3678267 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     -0.26620767,      0.30457628,      0.49314412,     -0.54036999 ],
  [               0,               0,       -472.2774,       680.99585,        993.6286,       -1427.376 ],
  [               0,               0,      -619.59186,       993.20032,       852.70081,       -1439.283 ],
  [               0,               0,       180.67104,      -210.70621,      -633.77478,       805.70074 ],
  [               0,               0,       1920.9401,      -2729.6006,      -3922.6433,       5572.8628 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele50_layer1_etaoffv3_55_e = [
 [
  [               0,               0,               0,               0,  -7.2486245e-08,   1.0475105e-07 ],
  [               0,               0,               0,               0,   3.1373175e+16,    3.137677e+16 ],
  [               0,               0,               0,               0,  -1.3704538e+08,   1.9644334e+08 ],
  [               0,               0,               0,               0,    5.218105e+08,  -7.4254989e+08 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,    0.0074664513 ],
  [               0,               0,               0,               0,               0,       2.8379662 ],
  [               0,               0,               0,               0,               0,     -0.94771838 ],
  [               0,               0,               0,               0,               0,      -1.0040702 ],
  [               0,               0,               0,               0,               0,     -0.12331583 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.20620742,      0.31222755 ],
  [               0,               0,               0,               0,      -259.48917,       391.18036 ],
  [               0,               0,               0,               0,        4.216845,      -7.4673195 ],
  [               0,               0,               0,               0,       49.776829,      -73.691872 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.023981767,    -0.041922353,    -0.068588257,      0.12136133 ],
  [               0,               0,      -227.09409,       1113.3864,       -1814.328,       984.41028 ],
  [               0,               0,       -1.718468,       7.6401849,      -10.248902,       2.9206145 ],
  [               0,               0,      -55.563663,       269.80737,      -435.19772,       232.31953 ],
  [               0,               0,      -14.335973,       68.329529,      -108.24363,       57.046799 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0071496489,    -0.011316138 ],
  [               0,               0,               0,               0,      -3.8108895,       9.4696236 ],
  [               0,               0,               0,               0,    -0.041766047,      -1.0047622 ],
  [               0,               0,               0,               0,    -0.068516485,     -0.90794033 ],
  [               0,               0,               0,               0,     -0.18737386,      0.38730463 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0046153986,    -0.011410213,    -0.022649197,      0.05757878 ],
  [               0,               0,       2.4761786,      -2.7157643,      -15.604957,       24.618496 ],
  [               0,               0,       4.5338049,      -20.668133,       27.424736,      -9.5793114 ],
  [               0,               0,      -1.5833465,       12.669076,      -30.797817,       22.404881 ],
  [               0,               0,      0.74331546,      -7.1013594,       20.454021,      -18.465231 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0004624748,    0.0021443858 ],
  [               0,               0,               0,               0,      -137.78442,       319.20065 ],
  [               0,               0,               0,               0,       46.045773,      -107.47071 ],
  [               0,               0,               0,               0,      -1.3581876,       3.5544765 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.019170683,    -0.041885834 ],
  [               0,               0,               0,               0,      -11.452419,       29.373341 ],
  [               0,               0,               0,               0,      -0.7189883,      0.54809988 ],
  [               0,               0,               0,               0,       2.0811918,      -6.0060315 ],
  [               0,               0,               0,               0,      0.14917986,     -0.30093074 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,    0.0015537221 ],
  [               0,               0,               0,               0,               0,       1.9156358 ],
  [               0,               0,               0,               0,               0,     -0.94005489 ],
  [               0,               0,               0,               0,               0,      -2.0138381 ],
  [               0,               0,               0,               0,               0,     -0.94542211 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0053983643,   -0.0060201357,     0.054186586 ],
  [               0,               0,               0,       3.9992926,      -20.499985,       32.941387 ],
  [               0,               0,               0,      -103.79864,       490.03943,      -579.55225 ],
  [               0,               0,               0,      -31.348848,       163.09621,       -211.0486 ],
  [               0,               0,               0,     0.010546393,   -0.0097815394,    -0.037788507 ],
  [               0,               0,               0,      0.30021322,      0.24000639,      -1.4313816 ],
  [               0,               0,               0,      -3.3678761,     -0.10071801,       17.905855 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.042884566,     -0.09323182 ],
  [               0,               0,               0,               0,      -3001.8792,       7469.6494 ],
  [               0,               0,               0,               0,               0,     -0.40000001 ],
  [               0,               0,               0,               0,      -9.9822845,       23.901722 ],
  [               0,               0,               0,               0,       1.0682977,      -1.9516869 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele75_layer1_etaoffv3_55_b = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0012114314,   0.00052920269 ],
  [               0,               0,               0,               0,      -3.0755401,       4.4848029 ],
  [               0,               0,               0,               0,      0.52070112,     -0.89925603 ],
  [               0,               0,               0,               0,      -1.8347284,     -0.58855803 ],
  [               0,               0,               0,               0,     -0.48238642,    0.0057502958 ],
  [               0,               0,               0,               0,  -0.00035572146,   2.9319637e-05 ],
  [               0,               0,               0,               0,      0.96242765,      0.88438537 ],
  [               0,               0,               0,               0,      -3.1194079,      -1.2393436 ],
 ],
 [
  [               0,               0,               0,    0.0039709084,  -0.00010058496,  -0.00021736998 ],
  [               0,               0,               0,      -2.4794654,      -7.1541585,       8.1484303 ],
  [               0,               0,               0,        7.905507,       -8.643309,        1.334247 ],
  [               0,               0,               0,       30.184079,      -37.923466,       10.750763 ],
  [               0,               0,               0,       2.1735631,      -2.7949853,      0.79995551 ],
  [               0,               0,               0,    0.0017415994,    -0.001401817,   0.00018463411 ],
  [               0,               0,               0,       1.8032266,      -1.9533315,       2.1037341 ],
  [               0,               0,               0,       6.6776852,      -15.592889,       6.0447877 ],
 ],
 [
  [               0,               0,               0,               0,      0.02010949,    0.0062756864 ],
  [               0,               0,               0,               0,      -3.1099041,       3.0718672 ],
  [               0,               0,               0,               0,      -22.609722,       17.440681 ],
  [               0,               0,               0,               0,       20.403421,      -16.227043 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0009502164,    0.0010074978,    0.0015670827 ],
  [               0,               0,               0,     0.070797516,     -0.85192706,       2.3237342 ],
  [               0,               0,               0,    0.0054383293,      0.19758448,      -1.1345184 ],
  [               0,               0,               0,     -0.21490661,      0.80151191,      -1.6615892 ],
  [               0,               0,               0,    -0.063278477,      0.20000794,      -0.2047194 ],
  [               0,               0,               0,   4.5310684e-05,  -9.6955035e-06,  -5.6628254e-05 ],
  [               0,               0,               0,       0.5058623,     -0.23211853,      0.41652464 ],
  [               0,               0,               0,       1.8921567,       26.992635,      -16.510791 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.029607091,     0.034340594,   -0.0037819885,       -0.140716 ],
  [               0,               0,       21.759768,      -59.759033,      -62.756866,       153.51743 ],
  [               0,               0,       942.77496,      -1357.3046,      -1959.5039,       2819.4348 ],
  [               0,               0,      -720.64575,       1020.4964,       1407.0114,      -1993.7295 ],
  [               0,               0,       140.38144,      -194.70084,      -282.59283,       391.34317 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele75_layer1_etaoffv3_55_e = [
 [
  [               0,               0,               0,               0,      0.64288831,       -0.886141 ],
  [               0,               0,               0,               0,      -92.593445,       133.75571 ],
  [               0,               0,               0,               0,       55.503551,      -80.249146 ],
  [               0,               0,               0,               0,      -5.2212858,        7.629025 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.019276291,     0.044009633 ],
  [               0,               0,               0,               0,      -94.033699,       140.35669 ],
  [               0,               0,               0,               0,       41.402252,      -61.960648 ],
  [               0,               0,               0,               0,      -105.87991,       152.54935 ],
  [               0,               0,               0,               0,      0.13977931,     -0.23501356 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.47784209,      0.72131538 ],
  [               0,               0,               0,               0,      -268.15393,         402.517 ],
  [               0,               0,               0,               0,       81.245064,      -121.69636 ],
  [               0,               0,               0,               0,      -16.619169,       24.889744 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.01241299,    -0.023218596,    -0.036029782,     0.068510838 ],
  [               0,               0,      -55.079514,       311.48608,      -569.14539,       340.53934 ],
  [               0,               0,       28.414856,      -143.98045,       242.63547,      -136.98296 ],
  [               0,               0,      -33.813877,       160.85127,       -253.1902,       130.91792 ],
  [               0,               0,       7.9557052,      -38.791725,        63.26582,      -34.442554 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0074158283,    -0.012312487 ],
  [               0,               0,               0,               0,      -5.1453314,       12.476102 ],
  [               0,               0,               0,               0,     -0.43363041,     -0.22356078 ],
  [               0,               0,               0,               0,      -2.2524817,       3.2950499 ],
  [               0,               0,               0,               0,     -0.53654528,       1.0638697 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0015976786,    -0.006278364,    -0.010326751,     0.039438836 ],
  [               0,               0,       4.3930359,      -20.517244,       31.872501,      -15.192503 ],
  [               0,               0,       26.260386,      -162.32101,       333.54816,      -228.94623 ],
  [               0,               0,      -17.903048,       118.47925,      -261.01224,       190.33833 ],
  [               0,               0,       11.724734,      -76.016525,       164.23045,      -118.19415 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0014942049,    0.0031642404 ],
  [               0,               0,               0,               0,      -56.508217,       131.39648 ],
  [               0,               0,               0,               0,       3.7131042,      -9.7139664 ],
  [               0,               0,               0,               0,       11.377805,      -25.952042 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00061379519,    0.0017372831 ],
  [               0,               0,               0,               0,       1.5004278,     -0.85101509 ],
  [               0,               0,               0,               0,      0.12901545,      -1.4966727 ],
  [               0,               0,               0,               0,      -1.3314807,       1.9222633 ],
  [               0,               0,               0,               0,    0.0017824705,     0.046940539 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,    0.0017110794 ],
  [               0,               0,               0,               0,               0,       2.4819045 ],
  [               0,               0,               0,               0,               0,      -1.2165251 ],
  [               0,               0,               0,               0,               0,      -1.7095548 ],
  [               0,               0,               0,               0,               0,     -0.69955027 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.089106828,      0.45993495,     -0.58324748 ],
  [               0,               0,               0,      -356.59937,       1709.9064,      -2042.1978 ],
  [               0,               0,               0,      -68.841576,       333.74286,      -405.95871 ],
  [               0,               0,               0,      -88.243584,       432.98386,      -531.02673 ],
  [               0,               0,               0,       1.0928816,      -5.3201823,       6.4743123 ],
  [               0,               0,               0,        -579.698,       2822.7024,      -3433.1365 ],
  [               0,               0,               0,       61.758965,       -385.4913,       569.58978 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0072481688,    0.0098869307 ],
  [               0,               0,               0,               0,     -0.93215138,       8.7392273 ],
  [               0,               0,               0,               0,               0,     -0.40000001 ],
  [               0,               0,               0,               0,      -40.547676,        99.50988 ],
  [               0,               0,               0,               0,      0.39986828,     -0.43932018 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele100_layer1_etaoffv3_55_b = [
 [
  [               0,               0,       7943.6973,      -22.056589,       -0.467381,   0.00091530057 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00019713932,   0.00060636781 ],
  [               0,               0,               0,               0,     -0.55630455,       4.3435392 ],
  [               0,               0,               0,               0,     -0.17369028,     -0.76331606 ],
  [               0,               0,               0,               0,    -0.075814278,     -0.81629093 ],
  [               0,               0,               0,               0,     -0.14315277,    -0.046450725 ],
  [               0,               0,               0,               0,   -0.0001106568,   1.9458399e-05 ],
  [               0,               0,               0,               0,       3.5892644,      0.61304421 ],
  [               0,               0,               0,               0,       1.0469833,     -0.31432491 ],
 ],
 [
  [               0,               0,               0,    0.0042776561,   -0.0051466599,    0.0023422381 ],
  [               0,               0,               0,      -19.399081,       22.215956,       -2.634307 ],
  [               0,               0,               0,      -2.4788266,         3.38868,      -1.9390723 ],
  [               0,               0,               0,       2.5469546,      -2.3068627,     -0.54606379 ],
  [               0,               0,               0,      0.81870142,       -1.196722,      0.26590982 ],
  [               0,               0,               0,  -0.00039031168,   0.00038124597,  -0.00010839834 ],
  [               0,               0,               0,      -2.6613995,       1.0036841,       2.5765793 ],
  [               0,               0,               0,       2.0260772,      -2.1997964,      0.67183505 ],
 ],
 [
  [               0,               0,               0,               0,     0.023439476,    0.0096109565 ],
  [               0,               0,               0,               0,      -5.1328783,       4.6203222 ],
  [               0,               0,               0,               0,      -15.428502,       11.773486 ],
  [               0,               0,               0,               0,       15.557879,      -12.372809 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0011041246,   -0.0011953223,      0.00150178 ],
  [               0,               0,               0,      -3.3374958,       6.2157309,     -0.03714725 ],
  [               0,               0,               0,      0.29811896,     -0.65651085,     -0.65975813 ],
  [               0,               0,               0,       2.4648417,       -5.578169,       1.7893914 ],
  [               0,               0,               0,      0.15889136,      -0.2597194,    -0.058295325 ],
  [               0,               0,               0,  -0.00022270595,   0.00050516328,  -0.00023144163 ],
  [               0,               0,               0,     -0.44190604,       1.9293299,      0.76032819 ],
  [               0,               0,               0,       7.4985267,      -18.726895,       13.108216 ],
 ],
 [
  [               0,               0,               0,               0,               0,     0.068541579 ],
  [               0,               0,               0,               0,               0,      0.44600102 ],
  [               0,               0,               0,               0,               0,     -0.42588347 ],
  [               0,               0,               0,               0,               0,     -0.42376623 ],
  [               0,               0,               0,               0,               0,   -0.0025564202 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0066420389,    0.0046696258,   4.4628599e-05,   -0.0085441843 ],
  [               0,               0,      -52.928528,      -23.857561,       33.517651,       144.39526 ],
  [               0,               0,        22.93606,       10.412103,      -14.855282,      -61.092197 ],
  [               0,               0,               0,               0,               0,              -1 ],
  [               0,               0,    -0.082910053,     -0.16964053,      -0.1629196,      -0.2204686 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     -0.10799616,      0.11432731,      0.19193298,     -0.18462949 ],
  [               0,               0,       52.189709,      -63.024658,      -91.406029,        112.9214 ],
  [               0,               0,      -370.02151,       633.95575,       308.35834,      -656.04321 ],
  [               0,               0,       280.87955,      -336.91873,       -896.8186,       1146.5931 ],
  [               0,               0,       2281.9983,      -3242.6511,      -4657.9219,       6617.3999 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele100_layer1_etaoffv3_55_e = [
 [
  [               0,               0,               0,               0,   -0.0055499054,     0.015357244 ],
  [               0,               0,               0,               0,       121.29137,      -172.81703 ],
  [               0,               0,               0,               0,        -99.7883,       141.87549 ],
  [               0,               0,               0,               0,       14.714136,      -20.375965 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0015688377,    0.0051045716 ],
  [               0,               0,               0,               0,       363.29254,      -527.50781 ],
  [               0,               0,               0,               0,       28.202797,      -42.858788 ],
  [               0,               0,               0,               0,      -111.33593,       161.94656 ],
  [               0,               0,               0,               0,      0.13165331,     -0.22351781 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.14332119,      0.21983179 ],
  [               0,               0,               0,               0,       -304.4982,       458.00012 ],
  [               0,               0,               0,               0,       7.2286196,      -12.086808 ],
  [               0,               0,               0,               0,       33.529194,      -49.560787 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,   0.00026360073 ],
  [               0,               0,               0,               0,               0,      -39.294945 ],
  [               0,               0,               0,               0,               0,       4.4307947 ],
  [               0,               0,               0,               0,               0,      -0.6371901 ],
  [               0,               0,               0,               0,               0,        4.686213 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0018690402,   -0.0032475027,   -0.0074724769,     0.014737793 ],
  [               0,               0,      -33.272385,       173.31223,      -297.14444,       169.94513 ],
  [               0,               0,      -26.836735,        130.1712,      -210.52263,       112.46611 ],
  [               0,               0,        7.509479,      -36.200153,       57.315598,      -30.797928 ],
  [               0,               0,      -9.0465841,       43.808781,      -70.745026,       38.170681 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0055813654,   -0.0084978072 ],
  [               0,               0,               0,               0,      -3.9157767,       9.9659767 ],
  [               0,               0,               0,               0,     -0.51966941,     -0.14710687 ],
  [               0,               0,               0,               0,    -0.089824319,     -0.90971774 ],
  [               0,               0,               0,               0,     -0.11291238,      0.26277101 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.010310157,     -0.02558028,    -0.050200175,      0.12703042 ],
  [               0,               0,       4.6980457,       -11.45173,      -8.3129873,       26.578136 ],
  [               0,               0,       5.7908335,      -29.659842,       47.339268,      -23.410379 ],
  [               0,               0,      -4.7987223,       32.554527,      -73.119942,        53.32658 ],
  [               0,               0,       5.9311876,      -39.741692,       88.724297,      -65.942146 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00025341049,    0.0030143363 ],
  [               0,               0,               0,               0,      -174.14209,       402.76596 ],
  [               0,               0,               0,               0,       45.774055,      -106.76493 ],
  [               0,               0,               0,               0,       1.8806877,      -3.9221008 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00040068926,    0.0019181304 ],
  [               0,               0,               0,               0,      -5.0564418,       14.666621 ],
  [               0,               0,               0,               0,      0.90847921,      -3.3267112 ],
  [               0,               0,               0,               0,       2.3447216,       -6.638514 ],
  [               0,               0,               0,               0,       0.1392608,     -0.26263636 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.068229288,    -0.012630222 ],
  [               0,               0,               0,               0,      -4.6265254,        11.59407 ],
  [               0,               0,               0,               0,         18.0518,      -43.780022 ],
  [               0,               0,               0,               0,      0.93247896,      -2.7380328 ],
  [               0,               0,               0,               0,     0.058764566,     -0.14968126 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0060177422,    0.0022838656,     0.037564985 ],
  [               0,               0,               0,      -12.196206,       56.512695,      -57.996574 ],
  [               0,               0,               0,      -99.452385,       469.80203,      -556.06848 ],
  [               0,               0,               0,      -21.513437,       114.84082,      -151.86601 ],
  [               0,               0,               0,    0.0088183712,   -0.0069333818,    -0.034421183 ],
  [               0,               0,               0,     -0.39018974,       1.9253165,      -1.3798422 ],
  [               0,               0,               0,      -4.2613478,      0.84034801,        21.05356 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.45159081,      -1.0956219 ],
  [               0,               0,               0,               0,     0.025300529,        9.491106 ],
  [               0,               0,               0,               0,               0,     -0.40000001 ],
  [               0,               0,               0,               0,      -21.631268,          52.896 ],
  [               0,               0,               0,               0,       0.1303283,      0.46828949 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele200_layer1_etaoffv3_55_b = [
 [
  [               0,               0,       39457.484,      -276.35422,      0.12928578,   0.00030840401 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,  -0.00024631534,   0.00078709003 ],
  [               0,               0,               0,               0,       2.9604265,       3.1431188 ],
  [               0,               0,               0,               0,      0.27098953,     -0.87524154 ],
  [               0,               0,               0,               0,     -0.76054382,     -0.74380306 ],
  [               0,               0,               0,               0,     -0.31860005,    -0.013244144 ],
  [               0,               0,               0,               0,  -7.2075088e-05,   4.7195778e-05 ],
  [               0,               0,               0,               0,     -0.57484319,       2.5282195 ],
  [               0,               0,               0,               0,      -1.4474607,      -1.4571138 ],
 ],
 [
  [               0,               0,               0,    0.0051089647,   -0.0021108824,    0.0007133248 ],
  [               0,               0,               0,       0.1732595,      -10.489963,       8.5830451 ],
  [               0,               0,               0,       10.306481,      -10.693137,       1.8782197 ],
  [               0,               0,               0,       14.972617,      -15.264464,       2.7526962 ],
  [               0,               0,               0,      0.33509144,     -0.16674944,     -0.12428781 ],
  [               0,               0,               0,   -0.0019866431,    0.0017664682,  -0.00040156508 ],
  [               0,               0,               0,      -2.9198306,      0.23022381,       3.2533405 ],
  [               0,               0,               0,      -16.322737,       21.419634,      -11.937827 ],
 ],
 [
  [               0,               0,               0,               0,     0.022569278,    0.0054508913 ],
  [               0,               0,               0,               0,       9.1294098,       -6.684999 ],
  [               0,               0,               0,               0,      -26.208273,       20.314625 ],
  [               0,               0,               0,               0,       17.440903,      -13.870676 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0010932888,  -3.2636672e-05,    0.0031448101 ],
  [               0,               0,               0,       -2.795892,       6.8236616,      -2.1188089 ],
  [               0,               0,               0,       1.6891938,      -3.2406473,      0.68613084 ],
  [               0,               0,               0,       3.4969109,      -7.3465731,       2.7585914 ],
  [               0,               0,               0,    -0.073704686,     0.014550243,    -0.052703132 ],
  [               0,               0,               0,    0.0010737816,   -0.0022428693,    0.0012137218 ],
  [               0,               0,               0,       0.5772491,      -2.9166338,       4.3508862 ],
  [               0,               0,               0,     -0.42586878,       1.6275659,      -3.3801035 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.60531855,     -0.83168411,      -1.2222939,       1.6861122 ],
  [               0,               0,      -3658.2859,       5209.2021,       7389.4712,      -10513.374 ],
  [               0,               0,       1142.4681,      -1601.9917,      -2577.0603,       3619.1831 ],
  [               0,               0,      -210.09431,       318.28726,       250.30019,      -397.72018 ],
  [               0,               0,       1524.3723,      -2161.0974,      -3108.7478,       4406.0615 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele200_layer1_etaoffv3_55_e = [
 [
  [               0,               0,               0,               0,     0.063873865,    -0.063253731 ],
  [               0,               0,               0,               0,       -7.630147,       12.419241 ],
  [               0,               0,               0,               0,       23.306496,      -34.582355 ],
  [               0,               0,               0,               0,      -26.393133,       38.154125 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00031374383,     0.026099194 ],
  [               0,               0,               0,               0,      -204.22023,       301.82166 ],
  [               0,               0,               0,               0,       106.82129,      -158.10228 ],
  [               0,               0,               0,               0,      -152.90417,       220.68594 ],
  [               0,               0,               0,               0,     0.087414205,     -0.12439174 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.14516376,      0.22102937 ],
  [               0,               0,               0,               0,      -401.36325,       604.15161 ],
  [               0,               0,               0,               0,      -57.153126,       83.508591 ],
  [               0,               0,               0,               0,       89.269524,      -132.35161 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,  -0.00026558538,   -0.0004630436,   1.1337162e-05,    0.0036306826 ],
  [               0,               0,       5.6485977,      -7.8354077,      -13.898318,       21.960716 ],
  [               0,               0,       3.0528789,      -4.2612858,      -9.4253111,        12.33695 ],
  [               0,               0,       3.3576133,      -5.3140349,      -8.4860983,       12.342109 ],
  [               0,               0,      0.42976406,     -0.50282896,      -1.6074566,       2.2121882 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.007600762,    -0.011626304 ],
  [               0,               0,               0,               0,      -3.0372596,       8.0219374 ],
  [               0,               0,               0,               0,    -0.098723844,     -0.95300955 ],
  [               0,               0,               0,               0,      -0.4127112,     -0.34114459 ],
  [               0,               0,               0,               0,     -0.35526538,      0.71151298 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   -0.0002546605,  -0.00028265672,   0.00058543665,    0.0056437585 ],
  [               0,               0,       2.7938206,      -5.5980792,      -13.132394,       29.005621 ],
  [               0,               0,    -0.077076271,     0.019289043,      0.99792743,      -2.6570952 ],
  [               0,               0,      -2.2973855,       5.2442279,       10.060228,      -24.092754 ],
  [               0,               0,     -0.28405449,      0.51012582,       1.9118201,      -3.5834577 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0065610069,     0.014635429 ],
  [               0,               0,               0,               0,      -27.014797,       62.873398 ],
  [               0,               0,               0,               0,       17.643892,      -41.257797 ],
  [               0,               0,               0,               0,       0.8640117,       -1.935614 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00070609088,    0.0022513643 ],
  [               0,               0,               0,               0,      -2.8486648,       9.1291542 ],
  [               0,               0,               0,               0,       3.0118766,      -8.2982168 ],
  [               0,               0,               0,               0,    -0.035387918,        -1.08334 ],
  [               0,               0,               0,               0,     -0.24998026,      0.63869375 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,    0.0012649589 ],
  [               0,               0,               0,               0,               0,       2.5806346 ],
  [               0,               0,               0,               0,               0,     -0.85195959 ],
  [               0,               0,               0,               0,               0,      -2.0268774 ],
  [               0,               0,               0,               0,               0,       -1.122219 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.046177663,      0.12079469 ],
  [               0,               0,               0,               0,       5.9714403,      -7.3195119 ],
  [               0,               0,               0,               0,      -17.727022,       41.412674 ],
  [               0,               0,               0,               0,       13.624075,      -32.955322 ],
  [               0,               0,               0,               0,     0.069967486,     -0.16948652 ],
  [               0,               0,               0,               0,      -2.6423967,       7.2821603 ],
  [               0,               0,               0,               0,      -8.8312922,       18.895977 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.012697118,    0.0022184877 ],
  [               0,               0,               0,               0,      0.55397409,       6.7909932 ],
  [               0,               0,               0,               0,               0,     -0.40000001 ],
  [               0,               0,               0,               0,      -61.048275,       150.38797 ],
  [               0,               0,               0,               0,     0.041154616,      0.54461634 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele500_layer1_etaoffv3_55_b = [
 [
  [               0,               0,       18467.344,      -24.996426,        -0.67596,   0.00084440724 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,  -0.00020211055,   0.00093346828 ],
  [               0,               0,               0,               0,     0.013338142,       3.3933823 ],
  [               0,               0,               0,               0,      0.65705903,      -1.0596921 ],
  [               0,               0,               0,               0,     0.085011696,      -1.0387125 ],
  [               0,               0,               0,               0,     -0.26817771,    -0.014347828 ],
  [               0,               0,               0,               0,   2.3693636e-05,   3.9870199e-05 ],
  [               0,               0,               0,               0,      -1.0569009,       2.7272573 ],
  [               0,               0,               0,               0,     -0.36089338,      -1.6733981 ],
 ],
 [
  [               0,               0,               0,   -0.0043899312,    0.0020836766,  -0.00070345883 ],
  [               0,               0,               0,      -4.5597154,       11.566445,      -8.2555142 ],
  [               0,               0,               0,      -3.3824961,       4.8682165,     -0.87837793 ],
  [               0,               0,               0,      -6.3387742,       8.4542936,      -1.7304735 ],
  [               0,               0,               0,     0.072096862,     -0.17438868,      0.20852204 ],
  [               0,               0,               0,   9.9416012e-05,   -0.0001740911,   0.00013065794 ],
  [               0,               0,               0,      0.70428708,      0.46627798,       1.7242215 ],
  [               0,               0,               0,      0.44286792,     -0.95584501,      -1.5716139 ],
 ],
 [
  [               0,               0,               0,               0,    0.0054068482,   -0.0014960802 ],
  [               0,               0,               0,               0,       12.320723,      -8.2099152 ],
  [               0,               0,               0,               0,      -195.72704,       154.61855 ],
  [               0,               0,               0,               0,       149.86905,       -119.1945 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00022538024,    0.0015480234 ],
  [               0,               0,               0,               0,     -0.64636547,       2.8892142 ],
  [               0,               0,               0,               0,      0.14790587,      -1.0674936 ],
  [               0,               0,               0,               0,      0.31929508,      -1.4950855 ],
  [               0,               0,               0,               0,     0.031778314,     -0.16636273 ],
  [               0,               0,               0,               0,   4.1898673e-05,   6.3203575e-06 ],
  [               0,               0,               0,               0,      -1.1012376,       3.2358108 ],
  [               0,               0,               0,               0,     -0.13729376,       3.9681082 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.27312961,     -0.43364999,        -0.59638,      0.94905394 ],
  [               0,               0,      -819.70917,       1190.7024,       1660.5543,      -2405.3748 ],
  [               0,               0,      -681.59473,       1109.4988,       816.06036,      -1443.1254 ],
  [               0,               0,       161.38496,      -153.38361,       -764.1507,       930.18018 ],
  [               0,               0,       3132.4138,      -4452.3594,      -6395.9873,       9089.3223 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele500_layer1_etaoffv3_55_e = [
 [
  [               0,               0,               0,               0,       975.13684,      -1338.6165 ],
  [               0,               0,               0,               0,      0.64078259,     -0.86082429 ],
  [               0,               0,               0,               0,     -0.63987488,      0.85952085 ],
  [               0,               0,               0,               0,   -0.0010923888,     0.001669741 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.097634219,       0.1455294 ],
  [               0,               0,               0,               0,       -558.6684,       824.85278 ],
  [               0,               0,               0,               0,      -484.23325,       712.92383 ],
  [               0,               0,               0,               0,       303.09286,      -441.16412 ],
  [               0,               0,               0,               0,      0.22947548,    -0.051107358 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.33416227,      0.50416273 ],
  [               0,               0,               0,               0,      -305.74182,       461.24371 ],
  [               0,               0,               0,               0,      -2.3670866,       2.1090395 ],
  [               0,               0,               0,               0,       34.807152,      -51.414711 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.002230275,   -0.0015748303,   -0.0067084087,    0.0079006767 ],
  [               0,               0,      -25.651943,       138.60046,      -250.71732,       153.58659 ],
  [               0,               0,      -23.231165,       110.41486,      -173.40359,       88.698235 ],
  [               0,               0,       3.9636848,      -21.072693,       37.354881,      -23.164248 ],
  [               0,               0,      -5.3072152,       26.795063,      -45.321465,        25.73513 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0068936585,    -0.011000214 ],
  [               0,               0,               0,               0,      -5.4692502,       13.289225 ],
  [               0,               0,               0,               0,    -0.021053845,      -1.1402717 ],
  [               0,               0,               0,               0,     -0.83666557,      0.44966453 ],
  [               0,               0,               0,               0,     -0.53096008,       1.0735759 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0013022922,   -0.0029063867,   -0.0063305376,     0.017342795 ],
  [               0,               0,      -8.2778206,       49.579983,      -98.666931,       68.017998 ],
  [               0,               0,       19.875858,      -126.41358,       268.01813,      -190.65944 ],
  [               0,               0,      -26.755375,       171.61105,       -366.3605,       259.15158 ],
  [               0,               0,       12.032299,      -78.699478,       171.37708,      -124.19704 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,  -0.00016169692,    0.0026361698 ],
  [               0,               0,               0,               0,       -406.2562,       937.81964 ],
  [               0,               0,               0,               0,       89.038704,       -206.9225 ],
  [               0,               0,               0,               0,       10.768179,      -23.932135 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.01722401,     -0.03665261 ],
  [               0,               0,               0,               0,      -8.0925074,       21.646381 ],
  [               0,               0,               0,               0,      0.01581577,      -1.2790937 ],
  [               0,               0,               0,               0,       1.7502825,      -5.3047886 ],
  [               0,               0,               0,               0,     0.038633391,     -0.03215982 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0014979596,    0.0036002754 ],
  [               0,               0,               0,               0,      -9.9301443,      -16.140903 ],
  [               0,               0,               0,               0,      -19082.914,       45847.195 ],
  [               0,               0,               0,               0,       6118.7754,      -14692.006 ],
  [               0,               0,               0,               0,      -1.1306506,      -120.71579 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.030712685,     0.083361134 ],
  [               0,               0,               0,               0,       10.520472,      -17.848017 ],
  [               0,               0,               0,               0,      -5.8542476,       12.637672 ],
  [               0,               0,               0,               0,       4.0930886,      -9.8724222 ],
  [               0,               0,               0,               0,   -0.0018406323,    0.0045054532 ],
  [               0,               0,               0,               0,       16.585424,      -103.73822 ],
  [               0,               0,               0,               0,       2.7012405,        2.605267 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,       1.1922566,      -2.8794312 ],
  [               0,               0,               0,               0,       5.6184545,       26.025528 ],
  [               0,               0,               0,               0,  -0.00038987695,       159.15186 ],
  [               0,               0,               0,               0,      -4.6237426,       11.241631 ],
  [               0,               0,               0,               0,       1.1420208,      -1.4920483 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele1000_layer1_etaoffv3_55_b = [
 [
  [               0,               0,       -3222.738,       150.86826,      -1.0524111,   0.00099216774 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -3.730359e-05,   0.00086544774 ],
  [               0,               0,               0,               0,      0.21078671,       3.8222405 ],
  [               0,               0,               0,               0,      0.42713022,      -1.0684548 ],
  [               0,               0,               0,               0,     -0.19495071,      -1.0287218 ],
  [               0,               0,               0,               0,     -0.28333998,   -0.0092520819 ],
  [               0,               0,               0,               0,    3.280688e-06,   3.6647938e-05 ],
  [               0,               0,               0,               0,     -0.73014652,       2.6353862 ],
  [               0,               0,               0,               0,      -1.5661243,      -1.5359055 ],
 ],
 [
  [               0,               0,               0,               0,  -0.00093500579,  -0.00061075627 ],
  [               0,               0,               0,               0,       1.3798477,      -3.8521469 ],
  [               0,               0,               0,               0,      0.16071648,      0.83467838 ],
  [               0,               0,               0,               0,      0.26566913,       1.0182274 ],
  [               0,               0,               0,               0,      0.14549937,     0.044165568 ],
  [               0,               0,               0,               0,  -6.2729655e-05,   8.7633259e-05 ],
  [               0,               0,               0,               0,      0.24281375,       2.2323183 ],
  [               0,               0,               0,               0,      -2.2717229,     -0.81558908 ],
 ],
 [
  [               0,               0,               0,               0,       0.5441153,     -0.43158194 ],
  [               0,               0,               0,               0,       -752.2793,       602.11621 ],
  [               0,               0,               0,               0,       67.743362,      -55.573318 ],
  [               0,               0,               0,               0,       150.18768,      -119.44189 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0004865326,   0.00049503341,   -0.0030102541 ],
  [               0,               0,               0,        1.691027,      -4.0822769,      0.30023766 ],
  [               0,               0,               0,    -0.015573414,     -0.10430527,       1.0177762 ],
  [               0,               0,               0,      -2.2322187,       4.5299305,      -1.1125538 ],
  [               0,               0,               0,     -0.14099303,      0.36514518,    -0.097714595 ],
  [               0,               0,               0,   0.00076139053,   -0.0015880071,   0.00088353225 ],
  [               0,               0,               0,     -0.58991491,     -0.11177395,       2.8825038 ],
  [               0,               0,               0,     -0.22830616,     -0.13514416,      -1.7983974 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.51863915,     -0.80002582,      -1.1123065,       1.7186226 ],
  [               0,               0,       632.95667,      -879.49493,       -1265.056,       1763.2408 ],
  [               0,               0,      -945.12872,       1460.9943,        1413.354,      -2245.2805 ],
  [               0,               0,       22.484987,       95.247574,      -658.76703,       676.92188 ],
  [               0,               0,        3459.394,      -4917.0762,      -7068.3521,       10044.789 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele1000_layer1_etaoffv3_55_e = [
 [
  [               0,               0,               0,               0,      0.53269619,     -0.74324083 ],
  [               0,               0,               0,               0,      -57.760056,       84.382744 ],
  [               0,               0,               0,               0,       30.962984,      -45.510502 ],
  [               0,               0,               0,               0,      -17.662186,       25.653631 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00076478301,    0.0089363866 ],
  [               0,               0,               0,               0,       5.5942173,      -3.4334507 ],
  [               0,               0,               0,               0,       26.793867,      -40.938255 ],
  [               0,               0,               0,               0,      -63.474575,       91.127235 ],
  [               0,               0,               0,               0,       0.1330024,    -0.090107508 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.32343629,      0.48894948 ],
  [               0,               0,               0,               0,      -362.81281,       546.30334 ],
  [               0,               0,               0,               0,        4.885498,       -8.704628 ],
  [               0,               0,               0,               0,        22.55175,      -33.209896 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,  -0.00041571527,  -0.00064621947,   0.00032067043,    0.0063007548 ],
  [               0,               0,      0.23113365,       5.9019723,      -18.941782,        16.04594 ],
  [               0,               0,     -0.91733873,       3.6808889,      -5.2218328,       1.5157394 ],
  [               0,               0,     -0.76367408,       1.5775146,      0.35116363,      -2.5555117 ],
  [               0,               0,     -0.40267175,       1.5211055,      -1.7537922,      0.61991513 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.008178642,    -0.012726598 ],
  [               0,               0,               0,               0,      -5.3892789,       12.770225 ],
  [               0,               0,               0,               0,       0.3311587,      -1.8863642 ],
  [               0,               0,               0,               0,      0.67609251,      -2.4509649 ],
  [               0,               0,               0,               0,     -0.25803778,      0.54886031 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,  -0.00018392694,  -4.2508549e-05,   0.00067111652,    0.0035911547 ],
  [               0,               0,      -0.0882309,      0.36371753,      0.19907403,       1.8094956 ],
  [               0,               0,      0.92417806,      -1.9549718,      -4.0610485,       7.3072243 ],
  [               0,               0,    -0.012069495,      0.18992449,     -0.34108225,      -1.1581291 ],
  [               0,               0,      -0.3239387,      0.64354694,       1.7704866,      -3.4822197 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.23265059,      0.53791708 ],
  [               0,               0,               0,               0,      -150.06541,       347.74942 ],
  [               0,               0,               0,               0,      -21.000591,       46.437408 ],
  [               0,               0,               0,               0,       67.724236,      -155.04523 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.010837702,    -0.022255275 ],
  [               0,               0,               0,               0,       -12.48433,       32.499966 ],
  [               0,               0,               0,               0,       1.9554857,      -5.9101849 ],
  [               0,               0,               0,               0,       1.7988555,      -5.4578576 ],
  [               0,               0,               0,               0,     -0.18743996,      0.51342493 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.044325657,      0.10646402 ],
  [               0,               0,               0,               0,       -1.486508,       4.5680842 ],
  [               0,               0,               0,               0,      -5165.1821,       12392.958 ],
  [               0,               0,               0,               0,       3062.8962,      -7341.0264 ],
  [               0,               0,               0,               0,     -0.73070168,       -4.360065 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     -0.51148349,        2.464211,       -2.959609 ],
  [               0,               0,               0,      -956.58704,       4661.4678,      -5669.7251 ],
  [               0,               0,               0,       56.790508,      -290.36356,       368.18109 ],
  [               0,               0,               0,      -112.35757,       556.05908,       -687.4007 ],
  [               0,               0,               0,    -0.028623616,      0.16237876,     -0.22389677 ],
  [               0,               0,               0,      -78.072968,       378.86633,      -460.54099 ],
  [               0,               0,               0,       101.52191,      -501.34052,       615.38031 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.031547472,    -0.042895351 ],
  [               0,               0,               0,               0,       1.2822292,       10.844081 ],
  [               0,               0,               0,               0,               0,     -0.40000001 ],
  [               0,               0,               0,               0,      -25.704468,       63.091343 ],
  [               0,               0,               0,               0,      0.10658435,      0.67865926 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]


ele_layer1_etaoffv3_55_b = [
    ele5_layer1_etaoffv3_55_b,
    ele10_layer1_etaoffv3_55_b,
    ele25_layer1_etaoffv3_55_b,
    ele50_layer1_etaoffv3_55_b,
    ele75_layer1_etaoffv3_55_b,
    ele100_layer1_etaoffv3_55_b,
    ele200_layer1_etaoffv3_55_b,
    ele500_layer1_etaoffv3_55_b,
    ele1000_layer1_etaoffv3_55_b,
    ]

ele_layer1_etaoffv3_55_e = [
    ele5_layer1_etaoffv3_55_e,
    ele10_layer1_etaoffv3_55_e,
    ele25_layer1_etaoffv3_55_e,
    ele50_layer1_etaoffv3_55_e,
    ele75_layer1_etaoffv3_55_e,
    ele100_layer1_etaoffv3_55_e,
    ele200_layer1_etaoffv3_55_e,
    ele500_layer1_etaoffv3_55_e,
    ele1000_layer1_etaoffv3_55_e,
    ]



###############################################################################
# Tables for v3 sshape correction, layer 1, 5x5 photon clusters
#


gam5_layer1_etaoffv3_55_b = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0011783374,    0.0011967889 ],
  [               0,               0,               0,               0,      -4.8040476,       3.9178371 ],
  [               0,               0,               0,               0,       1.9053017,      -1.3821375 ],
  [               0,               0,               0,               0,       1.7486053,      -1.2487376 ],
  [               0,               0,               0,               0,    -0.007682234,    -0.039602771 ],
  [               0,               0,               0,               0,   -0.0016199796,   0.00044775763 ],
  [               0,               0,               0,               0,      -1.0360069,        2.302706 ],
  [               0,               0,               0,               0,      -10.535007,       1.9511348 ],
 ],
 [
  [               0,               0,               0,    -0.002335516,    0.0032222793,  -0.00074671482 ],
  [               0,               0,               0,      -231.67409,       290.53479,      -82.348267 ],
  [               0,               0,               0,        51.06633,      -55.136017,       13.624092 ],
  [               0,               0,               0,       144.47852,      -204.00272,       72.488518 ],
  [               0,               0,               0,       17.851112,      -25.973106,       8.9175301 ],
  [               0,               0,               0,    0.0075503094,    -0.009789777,    0.0032386759 ],
  [               0,               0,               0,       51.785755,      -44.895641,       12.148882 ],
  [               0,               0,               0,      -46.757492,       23.279966,      -4.2156854 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
    [0, 0, 0, 0, 0, 0], # Removed by hand, see below
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam5_layer1_etaoffv3_55_e = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam10_layer1_etaoffv3_55_b = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0099341208,    0.0046440633 ],
  [               0,               0,               0,               0,       1.5993174,       1.4532664 ],
  [               0,               0,               0,               0,       2.7442207,      -1.7085524 ],
  [               0,               0,               0,               0,       -1.529268,     -0.37177095 ],
  [               0,               0,               0,               0,   0.00080676662,      0.19658448 ],
  [               0,               0,               0,               0,   0.00092154619,  -0.00078528753 ],
  [               0,               0,               0,               0,      -4.7761006,       1.6081935 ],
  [               0,               0,               0,               0,      -1.7298484,     -0.15141805 ],
 ],
 [
  [               0,               0,               0,  -0.00050817383,     0.020505553,    0.0007980821 ],
  [               0,               0,               0,     -0.85428643,      0.79333729,      0.62578094 ],
  [               0,               0,               0,      0.73189098,     -0.78691798,     -0.45676136 ],
  [               0,               0,               0,       1.5227063,      -1.6890569,     -0.23032393 ],
  [               0,               0,               0,      0.14171585,     -0.16775456,     0.033631276 ],
  [               0,               0,               0,    0.0026899038,   -0.0030482239,   0.00087101321 ],
  [               0,               0,               0,       88.419998,      -116.23006,       36.036507 ],
  [               0,               0,               0,      -79.572136,        90.40873,      -24.676268 ],
 ],
 [
  [               0,               0,               0,               0,               0,     0.020849766 ],
  [               0,               0,               0,               0,               0,      0.61081254 ],
  [               0,               0,               0,               0,               0,     -0.55221134 ],
  [               0,               0,               0,               0,               0,       0.0169261 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.011164073,    -0.014361354,    0.0052002971 ],
  [               0,               0,               0,        2.181107,      -6.9365878,       6.4995108 ],
  [               0,               0,               0,       1.6255578,       -3.696758,       1.2415196 ],
  [               0,               0,               0,       1.3601394,      -2.9588921,      0.64461225 ],
  [               0,               0,               0,      -0.8602789,       2.4354155,      -1.7612019 ],
  [               0,               0,               0,  -0.00050596846,    0.0013060553,  -0.00078203558 ],
  [               0,               0,               0,      -11.543426,       25.752415,      -10.774867 ],
  [               0,               0,               0,      -16.052189,       37.119839,      -18.881962 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,      0.18232779,     -0.15262015,     -0.14147063 ],
  [               0,               0,               0,      -743.05603,        2053.407,       -1413.152 ],
  [               0,               0,               0,       999.24939,      -2881.4851,        2075.969 ],
  [               0,               0,               0,      -1539.4957,       4307.7114,      -3013.9285 ],
  [               0,               0,               0,       225.82169,      -631.29285,       440.68625 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam10_layer1_etaoffv3_55_e = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      -2.0442274,       5.2060618,       9.2079916,      -18.297171 ],
  [               0,               0,       2.3526785,      -10.213893,       14.242667,      -6.1244078 ],
  [               0,               0,      -2.2282865,       9.6063786,      -13.258021,       5.5957136 ],
  [               0,               0,      -2.7229848,       12.121185,      -17.518148,       8.0005283 ],
  [               0,               0,    -0.089575931,      0.45581865,     -0.77294457,      0.43679717 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      -348.17642,       713.95709 ],
  [               0,               0,               0,               0,     -0.35963207,      0.70453465 ],
  [               0,               0,               0,               0,      0.35965019,     -0.70453954 ],
  [               0,               0,               0,               0,      0.35957667,     -0.70440608 ],
  [               0,               0,               0,               0,   4.8238646e-05,  -8.6311848e-05 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam25_layer1_etaoffv3_55_b = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,  -1.8310473e-05,   6.5525091e-05 ],
  [               0,               0,               0,               0,       30.693556,       8.9190168 ],
  [               0,               0,               0,               0,      -7.2400761,       5.6414542 ],
  [               0,               0,               0,               0,      -14.784348,        12.34689 ],
  [               0,               0,               0,               0,      -2.8611383,      0.81268609 ],
  [               0,               0,               0,               0,  -0.00035461373,  -0.00014066129 ],
  [               0,               0,               0,               0,      -1.0779366,       1.5924522 ],
  [               0,               0,               0,               0,      0.43692467,       1.1123832 ],
 ],
 [
  [               0,               0,               0,   -0.0020051524,    0.0038145734,  -0.00057396817 ],
  [               0,               0,               0,       13.299584,      -15.806419,       8.2863932 ],
  [               0,               0,               0,      -1.0958405,      0.33366767,     -0.85206532 ],
  [               0,               0,               0,       2.1943634,      -4.6152496,      0.66331291 ],
  [               0,               0,               0,     -0.46501949,      0.35753122,     -0.18992639 ],
  [               0,               0,               0,   -0.0015080664,    0.0015212896,  -0.00042130367 ],
  [               0,               0,               0,      -0.6729545,      0.81157166,       1.9385171 ],
  [               0,               0,               0,        15.11422,      -22.426657,       6.6144319 ],
 ],
 [
  [               0,               0,               0,               0,     0.026131505,    0.0024548981 ],
  [               0,               0,               0,               0,      -10.483573,       8.8824501 ],
  [               0,               0,               0,               0,      -10.850929,       8.1347237 ],
  [               0,               0,               0,               0,       16.861692,      -13.413177 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0025460909,     0.001332792,  -0.00061042677 ],
  [               0,               0,               0,       3.2697558,      -8.1892166,       6.3704262 ],
  [               0,               0,               0,      -1.8001276,        4.057023,       -3.121789 ],
  [               0,               0,               0,       -1.005877,       2.4770477,      -2.4777877 ],
  [               0,               0,               0,    -0.077908494,      0.28243065,     -0.28463078 ],
  [               0,               0,               0,  -0.00025262835,   0.00045817441,   -0.0001833116 ],
  [               0,               0,               0,     -0.99852711,      0.95278549,       4.3969173 ],
  [               0,               0,               0,     -0.64678013,       1.1341269,       2.7714019 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.03791935,     0.029601717,    -0.015918411,     -0.13844872 ],
  [               0,               0,      -6.5220046,      -49.014088,       -26.77141,       163.09569 ],
  [               0,               0,       866.64764,      -1245.9347,      -1790.6627,       2572.9514 ],
  [               0,               0,      -247.21725,       335.73331,       474.92819,      -644.96069 ],
  [               0,               0,       56.822018,      -75.412369,      -111.14262,       146.59172 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam25_layer1_etaoffv3_55_e = [
 [
  [               0,               0,               0,               0,      -1495.9517,       2176.9421 ],
  [               0,               0,               0,               0,       10.717705,      -15.393338 ],
  [               0,               0,               0,               0,      -10.654681,       15.302591 ],
  [               0,               0,               0,               0,    -0.033619042,     0.048494708 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00041778249,   5.2129762e-05 ],
  [               0,               0,               0,               0,       12.269341,       21.972227 ],
  [               0,               0,               0,               0,      0.16367792,    -0.015688516 ],
  [               0,               0,               0,               0,       3.2348835,       3.8334708 ],
  [               0,               0,               0,               0,     0.024321381,     -0.28994095 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.72174478,       1.0892977 ],
  [               0,               0,               0,               0,      -186.58978,       280.74988 ],
  [               0,               0,               0,               0,       115.57764,       -172.9538 ],
  [               0,               0,               0,               0,       -66.50164,        99.05249 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    -0.014035672,    -0.010385011,     0.017295767,      0.10735353 ],
  [               0,               0,     0.070208825,       5.4447937,      -17.915043,       14.938394 ],
  [               0,               0,      -8.9715538,       39.796787,      -58.436565,       27.889832 ],
  [               0,               0,       4.3617449,      -26.732033,       52.018585,      -33.126118 ],
  [               0,               0,       2.0528731,      -10.189385,       16.822906,      -9.2380991 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0049391831,   -0.0080106249 ],
  [               0,               0,               0,               0,      -8.5248394,       19.292034 ],
  [               0,               0,               0,               0,       1.5782127,      -4.1228414 ],
  [               0,               0,               0,               0,      -3.5433848,       5.9173775 ],
  [               0,               0,               0,               0,     -0.66925639,       1.3117073 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,  -0.00076136267,   -0.0004703819,    0.0016249351,    0.0099368803 ],
  [               0,               0,       0.5403747,      -0.2055676,       -3.168071,       4.6699052 ],
  [               0,               0,       1.6107274,      -3.2927237,      -7.7647486,       14.685385 ],
  [               0,               0,       2.1872475,      -4.3623266,      -11.431172,       21.860312 ],
  [               0,               0,     -0.22908372,      0.43315673,         1.30487,      -2.5031676 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,       359.07816,      -823.72473 ],
  [               0,               0,               0,               0,       -15.17205,       35.057045 ],
  [               0,               0,               0,               0,       15.219572,      -35.165855 ],
  [               0,               0,               0,               0,     -0.13896377,      0.32043901 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0035604483,     0.011870033 ],
  [               0,               0,               0,               0,      -5.4489379,       15.144194 ],
  [               0,               0,               0,               0,       4.8992877,      -12.730033 ],
  [               0,               0,               0,               0,       3.6690004,      -9.7044096 ],
  [               0,               0,               0,               0,      0.33133516,     -0.74622661 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.028212231,    0.0052127801,      0.16687976 ],
  [               0,               0,               0,       10.625097,       1.7891358,      -63.616486 ],
  [               0,               0,               0,       31.598627,      -162.29919,       206.47186 ],
  [               0,               0,               0,      -31.557556,       156.81187,      -194.66135 ],
  [               0,               0,               0,   -0.0051875636,   -0.0042475066,     0.041997839 ],
  [               0,               0,               0,       1.8212934,     -0.15918756,      -7.8887935 ],
  [               0,               0,               0,       5.5760145,      0.26207533,       -34.78043 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0043692295,   -0.0038402285 ],
  [               0,               0,               0,               0,      -3608.1365,       8972.4541 ],
  [               0,               0,               0,               0,               0,     -0.40000001 ],
  [               0,               0,               0,               0,       10.110676,      -26.422779 ],
  [               0,               0,               0,               0,     0.080225818,    -0.029207064 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam50_layer1_etaoffv3_55_b = [
 [
  [               0,               0,       16444.957,      -117.24649,     -0.14527971,   0.00074026507 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,  -0.00056921557,   0.00083849969 ],
  [               0,               0,               0,               0,       1.2633437,       3.7711315 ],
  [               0,               0,               0,               0,       1.5669674,      -1.2246104 ],
  [               0,               0,               0,               0,     -0.98316091,     -0.79196072 ],
  [               0,               0,               0,               0,     -0.65432549,     0.050143514 ],
  [               0,               0,               0,               0,   0.00030755586,  -5.8369242e-05 ],
  [               0,               0,               0,               0,      -0.8219794,        1.584446 ],
  [               0,               0,               0,               0,      -1.9045931,     -0.47303954 ],
 ],
 [
  [               0,               0,               0,   6.0386781e-05,  -0.00015579157,   0.00066743762 ],
  [               0,               0,               0,      -7.3466339,       5.9512453,       3.1036105 ],
  [               0,               0,               0,      -7.4376483,      -2.6265917,      -1.5292208 ],
  [               0,               0,               0,      -5.0952816,       -5.916029,      -1.3894496 ],
  [               0,               0,               0,       4.8176508,      -4.6587238,    -0.033303756 ],
  [               0,               0,               0,    0.0072136535,   -0.0018502699,    0.0021282278 ],
  [               0,               0,               0,     -0.30753362,      0.44578892,      0.15862907 ],
  [               0,               0,               0,     -0.21283785,     -0.26967758,      -1.1956372 ],
 ],
 [
  [               0,               0,               0,               0,     0.020553965,     0.011249019 ],
  [               0,               0,               0,               0,        5.913229,      -4.1645646 ],
  [               0,               0,               0,               0,      -19.284252,       14.841058 ],
  [               0,               0,               0,               0,       13.986178,      -11.123606 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,  -0.00038524231,    0.0013589221,   0.00082867732 ],
  [               0,               0,               0,      -2.2788951,       3.7709143,      0.65077561 ],
  [               0,               0,               0,       2.4556973,      -5.1233301,        1.709681 ],
  [               0,               0,               0,       2.6083016,      -5.2760806,       1.2841609 ],
  [               0,               0,               0,     -0.78258824,       2.1881902,      -1.6724832 ],
  [               0,               0,               0,   0.00043904857,   -0.0015033414,     0.001349765 ],
  [               0,               0,               0,        4.406939,      -8.0006418,       3.9933608 ],
  [               0,               0,               0,       3.8702457,      -10.920832,        6.373178 ],
 ],
 [
  [               0,               0,               0,               0,               0,   0.00026743513 ],
  [               0,               0,               0,               0,               0,       18.542793 ],
  [               0,               0,               0,               0,               0,      -8.5443029 ],
  [               0,               0,               0,               0,               0,    -0.025566809 ],
  [               0,               0,               0,               0,               0,     -0.79589438 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.09453924,     0.046163652,    -0.056199152,     -0.24756324 ],
  [               0,               0,       1.9957367,       5.1564679,       7.2260523,      -22.493402 ],
  [               0,               0,      -17.911469,      -8.1945801,       11.904509,       48.635189 ],
  [               0,               0,               0,               0,               0,              -1 ],
  [               0,               0,    -0.092881322,    -0.090364672,     -0.18568826,      -0.8020485 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,      0.57533264,      -1.3469944,      0.75995421 ],
  [               0,               0,               0,        -1373.92,       3688.3794,      -2458.5374 ],
  [               0,               0,               0,       3836.4365,      -11086.632,       8007.4722 ],
  [               0,               0,               0,      -1436.8373,       3971.7544,      -2744.3484 ],
  [               0,               0,               0,       2558.0667,      -7264.8643,       5156.9707 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam50_layer1_etaoffv3_55_e = [
 [
  [               0,               0,               0,               0,  -4.0844363e-08,   5.9019712e-08 ],
  [               0,               0,               0,               0,        -7213476,       -11795847 ],
  [               0,               0,               0,               0,         1344501,       -13038157 ],
  [               0,               0,               0,               0,         7316760,       3471799.5 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,  -0.00048565149,    0.0062049883 ],
  [               0,               0,               0,               0,     -0.37578329,       5.0976067 ],
  [               0,               0,               0,               0,      -4.8251271,       6.1218996 ],
  [               0,               0,               0,               0,      -64.740738,        93.36245 ],
  [               0,               0,               0,               0,      0.10585812,     -0.25540876 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.095034033,      0.14548095 ],
  [               0,               0,               0,               0,      -336.19073,       506.88745 ],
  [               0,               0,               0,               0,      -86.699318,       127.53297 ],
  [               0,               0,               0,               0,       125.52957,      -186.27657 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    -0.050720599,      0.31584606,     -0.60712296,      0.37312582 ],
  [               0,               0,      -5.4973893,       25.930895,      -43.950504,       28.243248 ],
  [               0,               0,      -5.4683127,       31.480673,      -58.623795,       34.539795 ],
  [               0,               0,       8.4018774,      -33.443893,       41.723644,      -16.566702 ],
  [               0,               0,      -1.8067771,       9.9491711,      -18.341028,       11.322811 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0064928737,    -0.010189401 ],
  [               0,               0,               0,               0,       -4.830986,       11.695884 ],
  [               0,               0,               0,               0,    -0.085506029,     -0.96431899 ],
  [               0,               0,               0,               0,     -0.58683825,     0.039717805 ],
  [               0,               0,               0,               0,     -0.30553302,      0.62825513 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0093086474,    -0.020715749,    -0.041899774,      0.09586101 ],
  [               0,               0,      -5.2435603,       3.9408419,       56.197029,       -83.84259 ],
  [               0,               0,       9.0608339,      -55.337978,       112.26707,      -76.862251 ],
  [               0,               0,      -2.1132741,       13.018627,      -26.372942,       16.411156 ],
  [               0,               0,       3.7390871,      -26.052549,       60.156986,       -46.00753 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      3.1599e-06,    0.0034576459 ],
  [               0,               0,               0,               0,      -175.67839,       406.28766 ],
  [               0,               0,               0,               0,       79.801178,      -184.95921 ],
  [               0,               0,               0,               0,      -5.5085497,       13.083632 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.01111974,    -0.023384849 ],
  [               0,               0,               0,               0,      -9.9956694,       26.471928 ],
  [               0,               0,               0,               0,      0.17045557,      -1.6784714 ],
  [               0,               0,               0,               0,     -0.31512988,     -0.42367041 ],
  [               0,               0,               0,               0,     -0.28838712,      0.74604148 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,     0.021835081 ],
  [               0,               0,               0,               0,               0,      0.54564029 ],
  [               0,               0,               0,               0,               0,     -0.49102488 ],
  [               0,               0,               0,               0,               0,     -0.53838712 ],
  [               0,               0,               0,               0,               0,    -0.055533674 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     -0.45004591,       2.1657503,      -2.5976715 ],
  [               0,               0,               0,      -818.92944,       3986.4358,       -4843.561 ],
  [               0,               0,               0,        6.623373,      -44.261162,       66.513741 ],
  [               0,               0,               0,      -112.48145,       556.20264,      -687.06171 ],
  [               0,               0,               0,    -0.016700493,      0.10484725,     -0.15473765 ],
  [               0,               0,               0,       190.67519,        -928.354,       1130.8373 ],
  [               0,               0,               0,      -146.75076,       723.92621,      -895.46698 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.011557623,    -0.016545895 ],
  [               0,               0,               0,               0,      -3289.5295,       8182.0928 ],
  [               0,               0,               0,               0,       76699.078,      -190886.17 ],
  [               0,               0,               0,               0,      -48.114841,       118.00428 ],
  [               0,               0,               0,               0,      0.10393414,       0.1431403 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam75_layer1_etaoffv3_55_b = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0014347619,    0.0013219268 ],
  [               0,               0,               0,               0,       1.8024346,       3.3392859 ],
  [               0,               0,               0,               0,     0.030355753,     -0.89702165 ],
  [               0,               0,               0,               0,     -0.25807133,      -1.0840129 ],
  [               0,               0,               0,               0,     -0.13257506,    -0.078687139 ],
  [               0,               0,               0,               0,   0.00088966009,  -0.00030902235 ],
  [               0,               0,               0,               0,       2.0894458,      0.57198894 ],
  [               0,               0,               0,               0,      -1.0033476,      -1.6685237 ],
 ],
 [
  [               0,               0,               0,    0.0023113145,    -0.001643758,    0.0011034504 ],
  [               0,               0,               0,      0.95795083,      -5.2021699,       6.3035746 ],
  [               0,               0,               0,      -3.9496057,       5.0472946,      -2.5056701 ],
  [               0,               0,               0,       2.1863463,       -2.789819,     -0.35221058 ],
  [               0,               0,               0,       1.0722526,      -1.4884626,       0.3474687 ],
  [               0,               0,               0,   0.00011541466,  -0.00022028203,   8.3705425e-05 ],
  [               0,               0,               0,      -18.733704,      -8.4800243,      -10.103798 ],
  [               0,               0,               0,      -10.271093,      -1.2475628,       1.5630715 ],
 ],
 [
  [               0,               0,               0,               0,     0.019411175,     0.013145425 ],
  [               0,               0,               0,               0,       14.302109,      -10.843347 ],
  [               0,               0,               0,               0,      -25.222315,       19.572813 ],
  [               0,               0,               0,               0,       12.998444,      -10.340406 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,  -0.00027671529,     0.001217909,   0.00031076104 ],
  [               0,               0,               0,       10.360596,      -24.666843,       17.440636 ],
  [               0,               0,               0,      -3.8396661,       8.6002197,      -5.7078009 ],
  [               0,               0,               0,      -1.8710563,        4.436348,      -3.9613755 ],
  [               0,               0,               0,     0.032029636,   -0.0043695122,     -0.22347715 ],
  [               0,               0,               0,   -0.0011590821,    0.0025466389,   -0.0014197221 ],
  [               0,               0,               0,      -7.8669028,        17.81452,      -7.2214413 ],
  [               0,               0,               0,       -2.904211,       6.5662241,      -11.233186 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.67104477,     -0.98560745,       -1.395373,       2.0562072 ],
  [               0,               0,      -188.68962,        274.9837,        383.7854,      -553.25122 ],
  [               0,               0,      -533.89948,       862.71661,       609.28412,      -1074.8217 ],
  [               0,               0,       122.33428,      -105.27903,         -656.94,       792.82581 ],
  [               0,               0,       3077.8923,      -4379.4668,      -6269.7407,       8919.2188 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam75_layer1_etaoffv3_55_e = [
 [
  [               0,               0,               0,               0,        1.357533,      -1.9269474 ],
  [               0,               0,               0,               0,      -128.51178,       185.44371 ],
  [               0,               0,               0,               0,       56.814236,      -81.967781 ],
  [               0,               0,               0,               0,      -5.0060935,       7.3784647 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,       0.1611328,     -0.23314971 ],
  [               0,               0,               0,               0,       200.37962,      -289.72058 ],
  [               0,               0,               0,               0,      -221.74594,       325.02719 ],
  [               0,               0,               0,               0,       555.10675,      -803.01538 ],
  [               0,               0,               0,               0,      0.18006206,     0.072332196 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.18252115,        0.278882 ],
  [               0,               0,               0,               0,      -311.06317,       468.29892 ],
  [               0,               0,               0,               0,       42.951874,      -65.311066 ],
  [               0,               0,               0,               0,      -3.3653238,       5.3114071 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.017698975,    -0.026019875,    -0.046773393,       0.0716822 ],
  [               0,               0,       0.5918569,        -1.21213,      -3.3985884,       7.8481336 ],
  [               0,               0,     -0.75601977,       1.6066077,       1.6037461,       -4.646986 ],
  [               0,               0,      -1.1960576,       2.1154501,       3.4098649,      -6.9724579 ],
  [               0,               0,     -0.13093869,      0.17705935,      0.28186017,      -0.3091681 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.013022968,    -0.020350233 ],
  [               0,               0,               0,               0,      -3.7608933,       8.9534512 ],
  [               0,               0,               0,               0,      0.93607086,      -2.8896072 ],
  [               0,               0,               0,               0,       1.3623445,      -3.7025969 ],
  [               0,               0,               0,               0,    -0.041014645,      0.09761937 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0096302601,     -0.02447268,     -0.04720284,        0.123136 ],
  [               0,               0,       7.2643571,      -33.803207,       50.644634,      -22.548441 ],
  [               0,               0,        20.84774,       -128.9734,       264.50476,      -180.94193 ],
  [               0,               0,      -5.4031086,       33.774792,      -70.607498,       48.269279 ],
  [               0,               0,       8.5661116,      -55.789982,       121.18872,      -87.753242 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0012894197,    0.0029402489 ],
  [               0,               0,               0,               0,      -109.73257,       253.85851 ],
  [               0,               0,               0,               0,       54.257687,      -125.93571 ],
  [               0,               0,               0,               0,       1.4333352,      -3.0175169 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00060558849,    0.0020282129 ],
  [               0,               0,               0,               0,      -1.8825443,       7.0016718 ],
  [               0,               0,               0,               0,       1.6366923,      -5.0662637 ],
  [               0,               0,               0,               0,      0.51079631,      -2.3573673 ],
  [               0,               0,               0,               0,     0.058479782,    -0.086030811 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,   0.00075620192 ],
  [               0,               0,               0,               0,               0,      0.36048082 ],
  [               0,               0,               0,               0,               0,      0.66248477 ],
  [               0,               0,               0,               0,               0,      0.28741631 ],
  [               0,               0,               0,               0,               0,      -1.4823608 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   0.00041976586,   0.00015010008,    0.0060179033 ],
  [               0,               0,               0,      -9.2630558,       2.2596507,       56.209137 ],
  [               0,               0,               0,      -100.77993,       492.56546,      -603.44324 ],
  [               0,               0,               0,      -18.362123,       89.723389,      -109.45832 ],
  [               0,               0,               0,    0.0072795372,   0.00070069812,    -0.045483604 ],
  [               0,               0,               0,       25.887468,      -189.07562,       304.83176 ],
  [               0,               0,               0,      -25.270851,       1.6059542,       137.60194 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.045667671,     0.035077881 ],
  [               0,               0,               0,               0,       154.97379,      -383.03494 ],
  [               0,               0,               0,               0,       20337.107,      -50610.625 ],
  [               0,               0,               0,               0,      -103.49996,       255.72923 ],
  [               0,               0,               0,               0,       2.1838443,      -5.3504896 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam100_layer1_etaoffv3_55_b = [
 [
  [               0,               0,       29298.014,       -217.5034,    -0.077593409,    0.0012074746 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,  -0.00071301567,    0.0027376644 ],
  [               0,               0,               0,               0,      0.15029706,        2.869844 ],
  [               0,               0,               0,               0,      0.13564453,      -1.4003315 ],
  [               0,               0,               0,               0,     -0.31991231,      -1.2785722 ],
  [               0,               0,               0,               0,     -0.13266972,     0.015641479 ],
  [               0,               0,               0,               0,    0.0003035104,  -0.00041323906 ],
  [               0,               0,               0,               0,      0.26585701,       1.3648449 ],
  [               0,               0,               0,               0,      -0.2101474,      -1.4547414 ],
 ],
 [
  [               0,               0,               0,    -0.031327732,     0.021887083,   -0.0070651863 ],
  [               0,               0,               0,      -3.2202716,       5.7182221,      -3.5042169 ],
  [               0,               0,               0,       1.6783167,       -2.727782,       1.8945549 ],
  [               0,               0,               0,       1.3540937,      -2.4055271,       1.8768524 ],
  [               0,               0,               0,     0.035639364,    -0.068010151,      0.05529907 ],
  [               0,               0,               0,   -0.0006086569,   0.00071359123,   -0.0001441655 ],
  [               0,               0,               0,       9.1511755,      -11.478523,        6.273818 ],
  [               0,               0,               0,      -0.8266961,       1.0116317,      -2.1468809 ],
 ],
 [
  [               0,               0,               0,               0,     0.021605261,    0.0088662617 ],
  [               0,               0,               0,               0,       15.213936,      -11.554061 ],
  [               0,               0,               0,               0,      -29.738659,        23.14979 ],
  [               0,               0,               0,               0,       15.745097,       -12.52341 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.084386289,      0.19385514,    -0.075524926 ],
  [               0,               0,               0,   -0.0096104108,    -0.086891696,      0.70957434 ],
  [               0,               0,               0,     0.076896533,    -0.081856109,     -0.54623848 ],
  [               0,               0,               0,      0.10717753,     -0.14352663,     -0.53301215 ],
  [               0,               0,               0,    -0.013291858,     0.030962637,    -0.025293667 ],
  [               0,               0,               0,   0.00035280842,  -0.00081819779,   0.00051518605 ],
  [               0,               0,               0,      -4.9749017,        10.06177,      -2.3883946 ],
  [               0,               0,               0,      -2.4648118,        4.870872,       2.0433078 ],
 ],
 [
  [               0,               0,               0,               0,               0,    0.0051770103 ],
  [               0,               0,               0,               0,               0,       1.6863858 ],
  [               0,               0,               0,               0,               0,      -1.5611352 ],
  [               0,               0,               0,               0,               0,      -1.2046876 ],
  [               0,               0,               0,               0,               0,    -0.033251788 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0023490351,    0.0009516413,   -0.0012319065,   -0.0065086791 ],
  [               0,               0,       -5.191153,      0.90217084,      -6.9964151,      -4.0275264 ],
  [               0,               0,      -7.0851102,      -41.307529,      -54.023804,      -9.2945127 ],
  [               0,               0,       12.859993,       142147.03,       96761.078,       35182.859 ],
  [               0,               0,       3.4235196,       66.705032,      -8.0693712,       16.768381 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.66352296,     -0.90572381,      -1.3310084,       1.8229156 ],
  [               0,               0,       191.06297,      -315.76767,      -424.61499,       700.78381 ],
  [               0,               0,        931.7334,      -1274.1165,      -2314.6304,       3189.7107 ],
  [               0,               0,      -101.28865,       209.06093,      -167.68539,       104.43527 ],
  [               0,               0,       2709.3232,       -3848.177,      -5517.9697,       7835.6152 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam100_layer1_etaoffv3_55_e = [
 [
  [               0,               0,               0,               0,      -1.8081698,       2.6547165 ],
  [               0,               0,               0,               0,        50.65414,      -72.467545 ],
  [               0,               0,               0,               0,      -54.279804,       77.658348 ],
  [               0,               0,               0,               0,       7.8093572,      -11.127714 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.021444269,    -0.030285001 ],
  [               0,               0,               0,               0,      -1013.9719,       1501.1021 ],
  [               0,               0,               0,               0,      -916.96509,       1349.9886 ],
  [               0,               0,               0,               0,       934.22192,       -1353.899 ],
  [               0,               0,               0,               0,      0.34689948,    -0.087509632 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.24699277,      0.37372595 ],
  [               0,               0,               0,               0,      -328.38553,       494.64777 ],
  [               0,               0,               0,               0,       6.4932394,      -11.038664 ],
  [               0,               0,               0,               0,       34.262264,      -50.594288 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,  -0.00038784204,  -0.00016905025,   0.00060248357,     0.003185963 ],
  [               0,               0,      0.59556133,    -0.076648831,      -1.5762973,       2.3984118 ],
  [               0,               0,       1.5588473,      -2.2379105,      -5.6597328,       7.2550921 ],
  [               0,               0,      0.40686113,      -1.0314399,     -0.69830686,       1.0330507 ],
  [               0,               0,     -0.34476745,      0.62176162,      0.63055235,      -1.1117142 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0053498554,   -0.0076997411 ],
  [               0,               0,               0,               0,      -3.0188711,        8.205163 ],
  [               0,               0,               0,               0,    -0.044461053,      -1.1159879 ],
  [               0,               0,               0,               0,      -0.2279702,     -0.68863726 ],
  [               0,               0,               0,               0,     -0.27951145,        0.585796 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   0.00032552256,   -0.0012998369,  -0.00094189733,    0.0076314253 ],
  [               0,               0,      -2.7874413,       19.562548,      -45.115067,       37.275169 ],
  [               0,               0,       24.275923,      -155.11761,       330.38977,      -235.82715 ],
  [               0,               0,      -18.876961,       119.22991,      -250.18663,       173.25449 ],
  [               0,               0,        12.20324,       -80.00322,       174.67079,      -126.93929 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.079823941,      0.18793812 ],
  [               0,               0,               0,               0,      -182.73587,         422.064 ],
  [               0,               0,               0,               0,       47.206802,       -109.9112 ],
  [               0,               0,               0,               0,       13.706871,      -31.141085 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0043522199,   -0.0067422423 ],
  [               0,               0,               0,               0,      -1.3616576,       5.8966575 ],
  [               0,               0,               0,               0,     -0.72018594,       0.4254643 ],
  [               0,               0,               0,               0,     -0.93771577,       1.0479037 ],
  [               0,               0,               0,               0,      0.16560198,     -0.31756222 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,     0.001287644 ],
  [               0,               0,               0,               0,               0,       2.7732518 ],
  [               0,               0,               0,               0,               0,      -1.0720414 ],
  [               0,               0,               0,               0,               0,      -2.0639563 ],
  [               0,               0,               0,               0,               0,     -0.87803823 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.014993253,     0.041292842,   -0.0043637226 ],
  [               0,               0,               0,      -20.758821,       111.95871,      -141.17662 ],
  [               0,               0,               0,      -74.617378,       344.50089,      -398.36157 ],
  [               0,               0,               0,      -23.264292,       126.30935,       -169.3288 ],
  [               0,               0,               0,     0.010068818,   0.00015087947,    -0.058923993 ],
  [               0,               0,               0,       5.5648112,      -25.106972,       29.015434 ],
  [               0,               0,               0,      -8.0419741,       23.978559,      -13.123154 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.089523286,     0.073092483,     -0.18394452,      -1.3511142 ],
  [               0,               0,       -158.8768,      -130.63341,       328.64893,       2435.8142 ],
  [               0,               0,     -0.12287489,     -0.38230103,     -0.99026006,      -2.5736816 ],
  [               0,               0,      -19.108362,      -9.9173288,       46.459599,        235.8764 ],
  [               0,               0,    0.0086019225,     0.024846543,     0.045406174,     -0.26819918 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam200_layer1_etaoffv3_55_b = [
 [
  [               0,               0,       15334.867,       -47.28046,     -0.55462193,     0.001197804 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0019258711,    0.0087115457 ],
  [               0,               0,               0,               0,      0.29570055,       2.1391389 ],
  [               0,               0,               0,               0,     -0.12804651,      -1.2377374 ],
  [               0,               0,               0,               0,     -0.32944319,      -1.2256041 ],
  [               0,               0,               0,               0,     -0.05374486,  -6.1545041e-05 ],
  [               0,               0,               0,               0,   0.00066104234,   -0.0013932076 ],
  [               0,               0,               0,               0,      0.30848211,       1.2088065 ],
  [               0,               0,               0,               0,     -0.10293474,      -1.5695961 ],
 ],
 [
  [               0,               0,               0,   -0.0066092946,    0.0093151955,   -0.0018305759 ],
  [               0,               0,               0,       11.761585,      -17.434546,       9.4810257 ],
  [               0,               0,               0,      -2.1810744,       7.8138013,      -3.5672667 ],
  [               0,               0,               0,        4.916492,     -0.70702839,      -1.7533056 ],
  [               0,               0,               0,       5.4650459,      -7.6709371,       1.9853222 ],
  [               0,               0,               0,    0.0085438015,  -0.00021620795,   -0.0012071002 ],
  [               0,               0,               0,   -0.0033025483,     -0.55315357,      0.70209104 ],
  [               0,               0,               0,      0.64460331,      0.38792714,      0.69340909 ],
 ],
 [
  [               0,               0,               0,               0,     0.020459721,     0.008997621 ],
  [               0,               0,               0,               0,       11.606323,      -8.6534758 ],
  [               0,               0,               0,               0,      -29.426563,       22.877197 ],
  [               0,               0,               0,               0,       17.226519,      -13.702612 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,  -5.0257557e-07,   0.00033746767,    0.0011681693 ],
  [               0,               0,               0,     -0.62859499,      0.89157325,       2.6090288 ],
  [               0,               0,               0,     -0.43923238,       1.1866341,      -1.7585135 ],
  [               0,               0,               0,      0.79503477,      -1.8849978,     -0.29257682 ],
  [               0,               0,               0,      0.10242564,      -0.2676717,   -0.0020962174 ],
  [               0,               0,               0,   0.00012536989,  -0.00029408099,   0.00020436192 ],
  [               0,               0,               0,      0.46976256,     -0.27474409,       2.0336099 ],
  [               0,               0,               0,     -0.91239977,      -0.2692678,      -2.3217525 ],
 ],
 [
  [               0,               0,               0,               0,               0,       1.9211318 ],
  [               0,               0,               0,               0,               0,       0.1376356 ],
  [               0,               0,               0,               0,               0,     -0.13615675 ],
  [               0,               0,               0,               0,               0,     -0.13705698 ],
  [               0,               0,               0,               0,               0,  -0.00017828165 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0018793161,    0.0008329589,   -0.0013380449,   -0.0048607606 ],
  [               0,               0,      -5.3410416,      -5.1441088,      -6.9840999,      -5.4730601 ],
  [               0,               0,      -32.225475,      -43.460442,      -58.543156,      -79.160522 ],
  [               0,               0,               0,               0,               0,              -1 ],
  [               0,               0,       25.064789,       35.493229,       50.243938,       70.802216 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,       0.6000846,       -0.902152,      -1.2663372,         1.90931 ],
  [               0,               0,       503.36896,      -705.05035,      -1008.8378,       1418.2689 ],
  [               0,               0,      -571.32361,       914.42987,       678.54877,      -1170.1139 ],
  [               0,               0,       209.37402,       -206.7934,       -910.0304,       1107.5945 ],
  [               0,               0,       3312.1213,      -4709.2104,      -6759.3311,       9608.5635 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam200_layer1_etaoffv3_55_e = [
 [
  [               0,               0,               0,               0,      0.89576435,      -1.2700237 ],
  [               0,               0,               0,               0,      -51.805584,       75.658714 ],
  [               0,               0,               0,               0,       25.386513,      -37.366875 ],
  [               0,               0,               0,               0,      -20.157146,       29.295332 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,       0.2874389,     -0.41588345 ],
  [               0,               0,               0,               0,       687.27417,      -998.79028 ],
  [               0,               0,               0,               0,        0.312345,      -2.1270428 ],
  [               0,               0,               0,               0,      -158.02249,       231.91196 ],
  [               0,               0,               0,               0,      0.21428557,      -0.1427146 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.35043544,       0.5290916 ],
  [               0,               0,               0,               0,      -343.82718,       517.46124 ],
  [               0,               0,               0,               0,       23.177023,       -35.80447 ],
  [               0,               0,               0,               0,       12.901422,      -18.868082 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,   6.2838029e-05 ],
  [               0,               0,               0,               0,               0,      -39.997829 ],
  [               0,               0,               0,               0,               0,      -3.2392185 ],
  [               0,               0,               0,               0,               0,      -4.0296645 ],
  [               0,               0,               0,               0,               0,       22.130262 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,  -0.00051127438,  -0.00023772659,   0.00079048477,    0.0040589273 ],
  [               0,               0,       1.8307964,      -2.2419569,      -4.9885921,       8.2099075 ],
  [               0,               0,      0.20400427,    -0.044135805,      -1.7080868,      0.84182799 ],
  [               0,               0,     -0.12402148,     0.032399323,      0.46610686,      -1.3984562 ],
  [               0,               0,     -0.10994729,      0.22336492,      0.11354504,     -0.23316769 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0066780508,    -0.010584448 ],
  [               0,               0,               0,               0,      -6.6055355,       15.553938 ],
  [               0,               0,               0,               0,      0.18100552,      -1.6011064 ],
  [               0,               0,               0,               0,     -0.35939348,     -0.47570959 ],
  [               0,               0,               0,               0,     -0.40855464,      0.84859043 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   0.00084780733,    -0.002321088,   -0.0030244342,     0.011789037 ],
  [               0,               0,      -2.7348993,       17.874266,      -38.876438,       31.204313 ],
  [               0,               0,       23.667723,      -151.51872,       323.11649,      -230.80014 ],
  [               0,               0,      -19.365072,       119.39382,      -244.45389,       164.98186 ],
  [               0,               0,       11.118433,      -73.212273,       160.50826,      -117.09859 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.024732305,    -0.053094555 ],
  [               0,               0,               0,               0,      -232.53226,       536.92969 ],
  [               0,               0,               0,               0,        77.27256,      -179.19533 ],
  [               0,               0,               0,               0,       2.5942886,      -5.4990339 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00028039972,    0.0022470518 ],
  [               0,               0,               0,               0,      -2.0676367,       8.1208763 ],
  [               0,               0,               0,               0,     -0.53575969,    -0.075942785 ],
  [               0,               0,               0,               0,      0.18611869,      -1.5941405 ],
  [               0,               0,               0,               0,      0.05625239,    -0.043904953 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,   0.00076961645 ],
  [               0,               0,               0,               0,               0,       3.2243047 ],
  [               0,               0,               0,               0,               0,      -0.6100468 ],
  [               0,               0,               0,               0,               0,      -2.3680253 ],
  [               0,               0,               0,               0,               0,      -1.4814018 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.060113989,      0.18009852,    -0.073078342 ],
  [               0,               0,               0,      -77.015511,       435.44553,       -595.9082 ],
  [               0,               0,               0,       215.59019,      -1065.4958,       1314.0378 ],
  [               0,               0,               0,      -132.45232,       654.04767,      -806.87469 ],
  [               0,               0,               0,     0.012136344,    -0.059682805,     0.075363353 ],
  [               0,               0,               0,       109.17151,      -533.16827,        651.9541 ],
  [               0,               0,               0,      -12.343112,       93.242729,        -157.185 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.02523653,    -0.045671824 ],
  [               0,               0,               0,               0,      -3075.5317,       7652.3037 ],
  [               0,               0,               0,               0,       384.41803,      -963.12817 ],
  [               0,               0,               0,               0,      -90.426414,       222.89401 ],
  [               0,               0,               0,               0,     0.091356456,      0.24230132 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam500_layer1_etaoffv3_55_b = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00019871628,   0.00058204803 ],
  [               0,               0,               0,               0,       2.4878662,       4.3854394 ],
  [               0,               0,               0,               0,     -0.95951527,     -0.69267398 ],
  [               0,               0,               0,               0,      -0.8308571,     -0.54966086 ],
  [               0,               0,               0,               0,     -0.21490584,   -0.0096896719 ],
  [               0,               0,               0,               0,  -3.5689609e-05,   8.6257249e-05 ],
  [               0,               0,               0,               0,      -0.9831326,       2.0840545 ],
  [               0,               0,               0,               0,      -3.0975828,      -1.4098248 ],
 ],
 [
  [               0,               0,               0,    0.0035676288,   -0.0043547791,    0.0022692394 ],
  [               0,               0,               0,      -24.468372,       28.462093,      -3.7994146 ],
  [               0,               0,               0,      -7.1228447,        9.449605,       -1.987574 ],
  [               0,               0,               0,      -5.3621993,       5.7611637,      -1.3994027 ],
  [               0,               0,               0,      -4.1295657,       8.4436369,       -5.597105 ],
  [               0,               0,               0,   -0.0050669275,   0.00055170764,    0.0048439945 ],
  [               0,               0,               0,       1.3982439,      -1.3776748,      0.51727784 ],
  [               0,               0,               0,      -4.8460131,       6.1601582,     -0.90779972 ],
 ],
 [
  [               0,               0,               0,               0,      0.11254045,    -0.059997883 ],
  [               0,               0,               0,               0,       4.6655869,      -3.1297495 ],
  [               0,               0,               0,               0,      -18.188923,       13.938903 ],
  [               0,               0,               0,               0,       12.922526,      -10.278912 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00048726573,    0.0008596222 ],
  [               0,               0,               0,               0,      -1.2710594,       4.6283784 ],
  [               0,               0,               0,               0,      0.20479438,      -1.1915667 ],
  [               0,               0,               0,               0,    -0.064407915,      -1.4201797 ],
  [               0,               0,               0,               0,     -0.02648088,     -0.17555539 ],
  [               0,               0,               0,               0,  -2.6975845e-06,    4.324667e-05 ],
  [               0,               0,               0,               0,      0.51305437,       1.9545453 ],
  [               0,               0,               0,               0,      -2.7731273,     -0.63029695 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.34463114,     -0.39302686,     -0.63481623,      0.71515501 ],
  [               0,               0,      -1106.0393,       1518.1755,       2187.3801,      -2991.2393 ],
  [               0,               0,        1775.782,      -2532.9873,      -3779.4873,       5390.3262 ],
  [               0,               0,      -791.87042,       1135.5491,       1448.2941,      -2080.7715 ],
  [               0,               0,        930.2854,       -1314.556,      -1891.3788,       2671.5425 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam500_layer1_etaoffv3_55_e = [
 [
  [               0,               0,               0,               0,      -3.2046492,       4.6736708 ],
  [               0,               0,               0,               0,      -12.811275,       20.298634 ],
  [               0,               0,               0,               0,       68.366386,      -99.372643 ],
  [               0,               0,               0,               0,      -59.980762,       86.312035 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.029124958,     0.048860874 ],
  [               0,               0,               0,               0,       489.37018,      -710.33588 ],
  [               0,               0,               0,               0,      -54.259602,       78.315254 ],
  [               0,               0,               0,               0,       47.603668,      -69.325859 ],
  [               0,               0,               0,               0,      0.16293713,    -0.058497448 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      -0.3844429,      0.57903683 ],
  [               0,               0,               0,               0,      -407.84775,       614.26776 ],
  [               0,               0,               0,               0,       6.1130919,      -10.502569 ],
  [               0,               0,               0,               0,       29.882156,       -44.07513 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   -0.0010343373,   -0.0011579994,   0.00030994296,      0.01087321 ],
  [               0,               0,      -4.3370104,       26.829876,      -49.450932,       29.878372 ],
  [               0,               0,      -1.0108491,       7.9934525,      -18.639847,       12.360463 ],
  [               0,               0,      0.31183422,       3.6268303,      -14.929951,       12.263034 ],
  [               0,               0,    -0.021212017,      0.59262705,      -1.7031265,       1.3415271 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.006391244,    -0.009726814 ],
  [               0,               0,               0,               0,      -6.3890052,       15.103521 ],
  [               0,               0,               0,               0,      0.36798871,      -2.0145037 ],
  [               0,               0,               0,               0,       0.4545446,      -2.1270294 ],
  [               0,               0,               0,               0,     -0.34684107,      0.72291702 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,  -0.00021643515,  -0.00011535079,   0.00062384456,    0.0040570283 ],
  [               0,               0,        1.212321,      -2.1769388,      -5.6953545,        13.77842 ],
  [               0,               0,         1.50302,      -3.1668401,      -7.2501283,       13.908038 ],
  [               0,               0,     -0.60385048,       1.6360143,        2.220978,      -7.4760036 ],
  [               0,               0,     -0.48320392,       1.0141906,       2.4917078,       -5.147378 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.26869041,      0.62069935 ],
  [               0,               0,               0,               0,      -215.32648,       497.86484 ],
  [               0,               0,               0,               0,      -72.020721,       163.59451 ],
  [               0,               0,               0,               0,       107.94471,      -247.35101 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.026496258,     0.065907955 ],
  [               0,               0,               0,               0,       16.285343,      -35.217152 ],
  [               0,               0,               0,               0,      -1.1764117,       1.3603166 ],
  [               0,               0,               0,               0,    -0.016530626,      -1.1290051 ],
  [               0,               0,               0,               0,      0.34221616,      -0.7152015 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.016193703,     0.038926847 ],
  [               0,               0,               0,               0,       19.861702,      -7.6677966 ],
  [               0,               0,               0,               0,       -3739.563,       8975.7236 ],
  [               0,               0,               0,               0,       4895.4033,      -11734.997 ],
  [               0,               0,               0,               0,       2.3881714,      -17.072485 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.024277288,     0.025707047,     0.088491306 ],
  [               0,               0,               0,       28.621719,      -50.646847,      -35.122639 ],
  [               0,               0,               0,       3.6761396,      -34.275978,       59.301064 ],
  [               0,               0,               0,      -160.99297,       795.33453,      -981.40894 ],
  [               0,               0,               0,     0.015325101,    -0.085914999,      0.12146008 ],
  [               0,               0,               0,       194.66547,      -948.59412,       1156.4039 ],
  [               0,               0,               0,       28.810608,      -113.70294,       102.92166 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,       2.5092173,      -6.2142124 ],
  [               0,               0,               0,               0,      -3914.0103,       9733.4346 ],
  [               0,               0,               0,               0,               0,     -0.40000001 ],
  [               0,               0,               0,               0,      -504.84052,       1250.2173 ],
  [               0,               0,               0,               0,      0.24638806,     -0.75827509 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam1000_layer1_etaoffv3_55_b = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,  -0.00011543587,    0.0011657159 ],
  [               0,               0,               0,               0,     -0.97041833,       4.6882191 ],
  [               0,               0,               0,               0,      0.57568008,     -0.98638093 ],
  [               0,               0,               0,               0,      0.28173712,      -1.6550145 ],
  [               0,               0,               0,               0,      -0.3455247,     -0.12920794 ],
  [               0,               0,               0,               0,   8.1600692e-06,   0.00017673663 ],
  [               0,               0,               0,               0,      -1.2709703,        1.397451 ],
  [               0,               0,               0,               0,     -0.25706217,      0.82943344 ],
 ],
 [
  [               0,               0,               0,               0,  -0.00083874102,  -0.00037543772 ],
  [               0,               0,               0,               0,       3.8112748,      -6.8766842 ],
  [               0,               0,               0,               0,      0.21112967,      0.77919048 ],
  [               0,               0,               0,               0,      0.73779196,      0.86112446 ],
  [               0,               0,               0,               0,      0.21129511,     0.066207148 ],
  [               0,               0,               0,               0,   1.8010791e-05,   3.3032175e-05 ],
  [               0,               0,               0,               0,       2.1743443,       1.1507652 ],
  [               0,               0,               0,               0,      -4.6428661,     0.069627054 ],
 ],
 [
  [               0,               0,               0,               0,      0.47908032,     -0.37960464 ],
  [               0,               0,               0,               0,      -576.06836,       461.49518 ],
  [               0,               0,               0,               0,      -31.122963,       23.210266 ],
  [               0,               0,               0,               0,       144.56134,      -114.97266 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0077008032,    -0.017482666,     0.011596261 ],
  [               0,               0,               0,      -3.9651482,       8.8070955,      -2.1901789 ],
  [               0,               0,               0,      -2.4941752,       5.5501032,       -4.033145 ],
  [               0,               0,               0,       0.8230049,      -1.5544364,      -0.6925059 ],
  [               0,               0,               0,      0.79322505,      -1.7706085,      0.81118619 ],
  [               0,               0,               0,   0.00047487082,  -0.00099172455,   0.00055761723 ],
  [               0,               0,               0,      -2.0707948,       7.2332549,      -7.8769765 ],
  [               0,               0,               0,       4.5727282,      -8.6398134,      0.47282422 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,       3.5411594,      -4.8674097,      -6.9911375,       9.6124744 ],
  [               0,               0,       3626.5442,      -5197.7573,      -7641.4243,       10956.855 ],
  [               0,               0,       1308.7487,      -1884.6888,      -2714.3491,       3906.8999 ],
  [               0,               0,      -1242.1597,       1779.3302,         2386.24,      -3421.1641 ],
  [               0,               0,        127.6636,       -173.1337,      -255.81071,       346.02444 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam1000_layer1_etaoffv3_55_e = [
 [
  [               0,               0,               0,               0,      0.01198518,   -0.0010546284 ],
  [               0,               0,               0,               0,      -43.501667,       64.831924 ],
  [               0,               0,               0,               0,       59.945835,      -87.549889 ],
  [               0,               0,               0,               0,      -54.099266,       78.121025 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.051559061,      0.07647147 ],
  [               0,               0,               0,               0,       14.847151,       17.066999 ],
  [               0,               0,               0,               0,      -1948.1503,       2870.1184 ],
  [               0,               0,               0,               0,       1778.9226,      -2584.5098 ],
  [               0,               0,               0,               0,      0.61488074,      0.31735432 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.41040552,      0.61832237 ],
  [               0,               0,               0,               0,      -447.45996,       673.05707 ],
  [               0,               0,               0,               0,       15.541031,        -24.4972 ],
  [               0,               0,               0,               0,       19.360456,      -28.456402 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,  -0.00021296096,  -0.00027409385,   0.00021259829,     0.002904604 ],
  [               0,               0,       6.4317622,      -9.2454615,      -16.871399,       26.951406 ],
  [               0,               0,     -0.77996182,       1.3479321,       1.8276316,      -4.3669925 ],
  [               0,               0,      -1.6122802,       2.4347887,       4.3399611,      -7.6063638 ],
  [               0,               0,      0.13984825,     -0.23447537,     -0.38119197,      0.73961264 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0010462975,    0.0040225689 ],
  [               0,               0,               0,               0,     -0.27748924,       3.8076375 ],
  [               0,               0,               0,               0,       1.4796274,      -4.1366324 ],
  [               0,               0,               0,               0,     -0.10103721,      -1.0195045 ],
  [               0,               0,               0,               0,     -0.28338358,      0.61672455 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,  -0.00040927445,  -0.00035045287,   0.00090113888,    0.0071390672 ],
  [               0,               0,        2.539659,      -4.8094883,      -12.267637,       26.381128 ],
  [               0,               0,      0.08425083,     -0.02855511,     -0.48190466,     -0.99520618 ],
  [               0,               0,     -0.34491426,       1.2508978,      0.35070416,      -4.2762356 ],
  [               0,               0,     -0.17582949,      0.22248884,       1.6112316,      -2.6687315 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00064363895,    0.0028589405 ],
  [               0,               0,               0,               0,      -260.87137,       602.02563 ],
  [               0,               0,               0,               0,       67.547668,      -156.81834 ],
  [               0,               0,               0,               0,       9.6101313,      -21.628941 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.012678817,    -0.026925026 ],
  [               0,               0,               0,               0,       -6.073585,       17.799822 ],
  [               0,               0,               0,               0,      -4.5599003,       9.3407269 ],
  [               0,               0,               0,               0,      0.20429642,      -1.6524928 ],
  [               0,               0,               0,               0,      0.45426089,     -0.96609128 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0001253701,    0.0025907606 ],
  [               0,               0,               0,               0,      -401.94516,       965.37585 ],
  [               0,               0,               0,               0,       3.2786777,      -8.4230471 ],
  [               0,               0,               0,               0,        13.71995,      -34.515331 ],
  [               0,               0,               0,               0,    -0.024325719,     -0.35457459 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     -0.20889968,      0.96937013,      -1.1154206 ],
  [               0,               0,               0,       -284.9823,       1385.7567,      -1673.2623 ],
  [               0,               0,               0,       85.226929,      -443.63687,       572.38635 ],
  [               0,               0,               0,       -13.09869,       80.369522,      -117.41942 ],
  [               0,               0,               0,    0.0066426136,     0.012198045,    -0.066541925 ],
  [               0,               0,               0,       523.96344,      -2560.0044,       3125.8347 ],
  [               0,               0,               0,       125.82971,      -621.29523,       763.03955 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.065647669,     -0.13196914 ],
  [               0,               0,               0,               0,     0.052444201,       9.0274172 ],
  [               0,               0,               0,               0,               0,     -0.40000001 ],
  [               0,               0,               0,               0,      -55.099873,       135.65038 ],
  [               0,               0,               0,               0,      0.11515028,      0.39801738 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]


gam_layer1_etaoffv3_55_b = [
    gam5_layer1_etaoffv3_55_b,
    gam10_layer1_etaoffv3_55_b,
    gam25_layer1_etaoffv3_55_b,
    gam50_layer1_etaoffv3_55_b,
    gam75_layer1_etaoffv3_55_b,
    gam100_layer1_etaoffv3_55_b,
    gam200_layer1_etaoffv3_55_b,
    gam500_layer1_etaoffv3_55_b,
    gam1000_layer1_etaoffv3_55_b,
    ]

gam_layer1_etaoffv3_55_e = [
    gam5_layer1_etaoffv3_55_e,
    gam10_layer1_etaoffv3_55_e,
    gam25_layer1_etaoffv3_55_e,
    gam50_layer1_etaoffv3_55_e,
    gam75_layer1_etaoffv3_55_e,
    gam100_layer1_etaoffv3_55_e,
    gam200_layer1_etaoffv3_55_e,
    gam500_layer1_etaoffv3_55_e,
    gam1000_layer1_etaoffv3_55_e,
    ]



###############################################################################
# Tables for v3 sshape correction, layer 1, 3x5 electron clusters
#

ele5_layer1_etaoffv3_35_b = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
    [0, 0, 0, 0, 0, 0], # Removed by hand, see below
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele5_layer1_etaoffv3_35_e = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele10_layer1_etaoffv3_35_b = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0054645368,     0.010835519 ],
  [               0,               0,               0,               0,      -2.2018785,       1.4325579 ],
  [               0,               0,               0,               0,      -13.859774,       3.1056084 ],
  [               0,               0,               0,               0,      -13.787603,       3.0850858 ],
  [               0,               0,               0,               0,      0.20537826,    -0.061122814 ],
  [               0,               0,               0,               0,      -0.2105215,     0.057455304 ],
  [               0,               0,               0,               0,      -0.1282433,      0.26315808 ],
  [               0,               0,               0,               0,  -0.00057660049,       1.5607133 ],
 ],
 [
  [               0,               0,               0,      0.00334336,   -0.0050194573,    0.0020356621 ],
  [               0,               0,               0,      -328.25852,       378.32943,      -95.377506 ],
  [               0,               0,               0,       91.027735,      -84.252019,       18.910651 ],
  [               0,               0,               0,       64.277502,      -57.297857,       11.782491 ],
  [               0,               0,               0,      -7.2840657,       5.1432395,      -1.0610306 ],
  [               0,               0,               0,   -0.0019550964,    0.0017979955,  -0.00029235901 ],
  [               0,               0,               0,       19.829431,      -18.947883,       7.1929797 ],
  [               0,               0,               0,       47.789828,      -49.370852,       14.140231 ],
 ],
 [
  [               0,               0,               0,               0,               0,   0.00019586603 ],
  [               0,               0,               0,               0,               0,       45.673374 ],
  [               0,               0,               0,               0,               0,      -7.4925227 ],
  [               0,               0,               0,               0,               0,       3.9421911 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0068571423,    -0.010747271,    0.0043250915 ],
  [               0,               0,               0,       15.108793,      -45.585953,       29.584968 ],
  [               0,               0,               0,       26.672793,      -53.194359,       26.583624 ],
  [               0,               0,               0,      -13.104099,        17.86022,       1.1559382 ],
  [               0,               0,               0,      -7.0763084,       13.506303,      -5.6630005 ],
  [               0,               0,               0,   -0.0018353038,    0.0021898851,  -0.00077928746 ],
  [               0,               0,               0,      -1.6101141,       4.7075895,      -1.8627983 ],
  [               0,               0,               0,       12.392702,      -24.126784,       12.086764 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.070285261,      0.13972805,    -0.051412717 ],
  [               0,               0,               0,      -631.25415,       1923.7402,      -1454.9023 ],
  [               0,               0,               0,       4695.4009,      -13496.873,       9697.5684 ],
  [               0,               0,               0,       2739.5354,      -7754.9868,       5485.9214 ],
  [               0,               0,               0,       272.60098,      -747.16431,       510.61957 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele10_layer1_etaoffv3_35_e = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.046229027,     0.087072447 ],
  [               0,               0,               0,               0,     -0.63957226,       1.8692856 ],
  [               0,               0,               0,               0,      0.47921556,      -1.4468138 ],
  [               0,               0,               0,               0,       1.7225711,      -3.4291186 ],
  [               0,               0,               0,               0,     0.093077108,     -0.12709765 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele25_layer1_etaoffv3_35_b = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,  -1.2005652e-05,    0.0005529591 ],
  [               0,               0,               0,               0,       -3.451765,       3.4199709 ],
  [               0,               0,               0,               0,        1.066909,     -0.47359697 ],
  [               0,               0,               0,               0,       0.8196006,     -0.23339621 ],
  [               0,               0,               0,               0,     -0.26709091,   0.00085329336 ],
  [               0,               0,               0,               0,  -0.00034567089,   2.0499352e-05 ],
  [               0,               0,               0,               0,      -3.5765854,       3.0729446 ],
  [               0,               0,               0,               0,       1.4035124,      0.83510787 ],
 ],
 [
  [               0,               0,               0,      0.02401338,      -0.0182524,    0.0040549252 ],
  [               0,               0,               0,      -6.3372739,      -3.0015096,       5.8089418 ],
  [               0,               0,               0,       32.307155,      -36.883347,       9.5403114 ],
  [               0,               0,               0,       23.713757,      -24.688904,       5.4919709 ],
  [               0,               0,               0,      -1.9918614,       2.9122664,      -1.0608353 ],
  [               0,               0,               0,   -0.0015120554,   0.00094803535,  -0.00010502579 ],
  [               0,               0,               0,       1.4915998,      -3.9109362,       3.5684251 ],
  [               0,               0,               0,      -16.358212,       24.019949,      -7.8587456 ],
 ],
 [
  [               0,               0,               0,               0,     0.025892759,  -0.00012947131 ],
  [               0,               0,               0,               0,      -11.134826,       9.4502392 ],
  [               0,               0,               0,               0,      -20.382269,       15.679564 ],
  [               0,               0,               0,               0,       20.660542,      -16.429544 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.002035729,   -0.0056569181,    0.0040469087 ],
  [               0,               0,               0,       -5.548927,       12.261087,      -3.7867592 ],
  [               0,               0,               0,        18.28038,      -30.919365,       12.086454 ],
  [               0,               0,               0,       11.923207,      -17.324481,       6.0934532 ],
  [               0,               0,               0,      -5.3525422,       10.127223,      -4.8507088 ],
  [               0,               0,               0,  -8.3528971e-06,  -0.00024885047,   9.6328689e-05 ],
  [               0,               0,               0,       6.1850857,      -13.717843,       8.5764959 ],
  [               0,               0,               0,      -3.1456925,       9.4468011,      -5.6843209 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     -0.14505997,       0.2404443,      0.31981027,     -0.51865858 ],
  [               0,               0,       9408.8418,      -13205.399,      -19778.965,       27776.064 ],
  [               0,               0,        2125.968,      -3069.1758,      -4417.1128,       6374.5864 ],
  [               0,               0,       758.07617,      -1059.9508,      -1547.7303,       2162.5876 ],
  [               0,               0,       200.57275,      -274.26016,      -400.08212,       545.88116 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele25_layer1_etaoffv3_35_e = [
 [
  [               0,               0,               0,               0,               0,   -0.0050752843 ],
  [               0,               0,               0,               0,               0,       5.3609123 ],
  [               0,               0,               0,               0,               0,     -0.79632646 ],
  [               0,               0,               0,               0,               0,     -0.59245169 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,      -1.6672838,    0.0055373521,       3.6081324 ],
  [               0,               0,               0,      -127.61105,       7.6448541,       265.66867 ],
  [               0,               0,               0,       113.99134,       7.4780569,      -258.03735 ],
  [               0,               0,               0,      -79.334724,      -84.903374,       289.33401 ],
  [               0,               0,               0,     0.014205037,     0.040130645,     -0.10193985 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,     0.013157598 ],
  [               0,               0,               0,               0,               0,       2.0825377 ],
  [               0,               0,               0,               0,               0,      -1.1668438 ],
  [               0,               0,               0,               0,               0,      0.11849387 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   -0.0068984856,   -0.0034407438,     0.012101951,     0.056996956 ],
  [               0,               0,       -1.084901,    -0.077105053,       8.5487804,      -8.5166693 ],
  [               0,               0,       5.0343189,      -19.664083,       24.262381,      -9.5813332 ],
  [               0,               0,      -2.1494119,       16.407898,      -35.949188,       23.839586 ],
  [               0,               0,      -1.1339613,       5.6378045,      -9.3258324,       5.1358209 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0019257423,   -0.0028579885 ],
  [               0,               0,               0,               0,      -9.7617512,       22.574915 ],
  [               0,               0,               0,               0,     -0.63100141,       0.3977305 ],
  [               0,               0,               0,               0,       1.2676725,      -2.6660333 ],
  [               0,               0,               0,               0,     -0.74494243,       1.5453506 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.042847455,    -0.091959998,     -0.20062041,      0.43405852 ],
  [               0,               0,      -35.462486,       183.82817,       -298.4725,       146.59093 ],
  [               0,               0,       6.8707809,      -31.180223,       38.676025,      -8.5135927 ],
  [               0,               0,      -33.134125,       227.34521,      -517.17334,       389.21338 ],
  [               0,               0,       16.492867,      -107.33644,       232.68469,      -167.98087 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      -1.4802008,       3.4372575 ],
  [               0,               0,               0,               0,      -16.743101,       39.176407 ],
  [               0,               0,               0,               0,       5.3976917,      -13.048594 ],
  [               0,               0,               0,               0,       3.1447072,      -7.1902781 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0096621364,    -0.021225633 ],
  [               0,               0,               0,               0,      -57.668755,       139.30634 ],
  [               0,               0,               0,               0,       12.674744,      -30.744438 ],
  [               0,               0,               0,               0,       7.3218298,      -17.629063 ],
  [               0,               0,               0,               0,      -1.1756026,       2.9434536 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0052236966,    0.0010594606,     0.035559576 ],
  [               0,               0,               0,      -1.1705779,      0.87047696,       11.259214 ],
  [               0,               0,               0,      -2.7380674,      0.69105911,       12.906754 ],
  [               0,               0,               0,       1.9244112,       3.0071166,       -18.63335 ],
  [               0,               0,               0,    0.0028607196,  -8.3882609e-05,    -0.015400004 ],
  [               0,               0,               0,       4.5541606,     -0.44051811,      -25.595512 ],
  [               0,               0,               0,      -12.945931,     -0.70638257,       70.872925 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.007729379,    -0.007598829 ],
  [               0,               0,               0,               0,      -1340.6198,       3335.1809 ],
  [               0,               0,               0,               0,               0,     -0.40000001 ],
  [               0,               0,               0,               0,      -101.58306,       249.96935 ],
  [               0,               0,               0,               0,     0.057020042,     -0.19481789 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele50_layer1_etaoffv3_35_b = [
 [
  [               0,               0,       3869.9277,       123.48212,      -1.2619745,    0.0016548653 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00084421891,   0.00056418939 ],
  [               0,               0,               0,               0,      -1.7056677,       3.9896097 ],
  [               0,               0,               0,               0,      -1.1923214,      -0.3869964 ],
  [               0,               0,               0,               0,    -0.038755576,     -0.94804659 ],
  [               0,               0,               0,               0,      0.26859711,     -0.19612616 ],
  [               0,               0,               0,               0,  -0.00012557948,    8.058125e-05 ],
  [               0,               0,               0,               0,      0.48872253,      0.86945405 ],
  [               0,               0,               0,               0,       5.3415962,     -0.45380873 ],
 ],
 [
  [               0,               0,               0,    0.0019508352,   -0.0023719713,    0.0016127316 ],
  [               0,               0,               0,      -14.193298,       16.880126,      -1.6624338 ],
  [               0,               0,               0,       2.8808386,      -3.2472476,     0.031596526 ],
  [               0,               0,               0,      0.59422167,     -0.20776069,      -1.1068477 ],
  [               0,               0,               0,      -1.2919143,       1.3541252,     -0.47297287 ],
  [               0,               0,               0,    0.0013738748,   -0.0017747074,   0.00054078539 ],
  [               0,               0,               0,      -12.849774,       19.904957,       -5.233258 ],
  [               0,               0,               0,       18.747261,      -20.949806,       5.0120554 ],
 ],
 [
  [               0,               0,               0,               0,     0.022654353,    0.0083813788 ],
  [               0,               0,               0,               0,      -12.068476,       10.165203 ],
  [               0,               0,               0,               0,      -9.1774626,       6.7855806 ],
  [               0,               0,               0,               0,       15.082092,      -11.994566 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0027762922,   0.00047171695,     0.008671441 ],
  [               0,               0,               0,      0.73733368,      -1.0205764,       1.4064744 ],
  [               0,               0,               0,     -0.36436835,      0.60130554,      -1.0559894 ],
  [               0,               0,               0,    -0.027434141,     -0.17050411,       -0.692292 ],
  [               0,               0,               0,    -0.026053722,     0.025047827,    -0.035114298 ],
  [               0,               0,               0,  -1.3259463e-05,  -5.1068666e-05,   6.8872301e-05 ],
  [               0,               0,               0,      -1.6244376,       1.2656121,       3.2199176 ],
  [               0,               0,               0,       21.586054,      -44.468166,       26.661231 ],
 ],
 [
  [               0,               0,               0,               0,               0,       49.321396 ],
  [               0,               0,               0,               0,               0,     0.048936583 ],
  [               0,               0,               0,               0,               0,    -0.048890509 ],
  [               0,               0,               0,               0,               0,    -0.048903733 ],
  [               0,               0,               0,               0,               0,  -4.6160458e-06 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,       -3.296737,      -1.5346832,       2.1620667,       9.0588531 ],
  [               0,               0,       2.5998435,       5.5169401,       6.8776007,       12.413194 ],
  [               0,               0,      -11.121909,      -5.0607457,       7.4867129,       30.283232 ],
  [               0,               0,               0,               0,               0,              -1 ],
  [               0,               0,     -0.14005485,     -0.17171147,     -0.27863732,     -0.84419119 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     -0.13208513,      0.13388902,      0.23199417,     -0.21105722 ],
  [               0,               0,      -255.60362,       372.96152,       547.66083,      -793.13611 ],
  [               0,               0,      -366.28671,       624.34326,       318.52002,       -661.8949 ],
  [               0,               0,       339.18304,      -441.06992,      -945.19312,       1258.3899 ],
  [               0,               0,       2190.4517,      -3111.6272,      -4471.2852,       6350.3706 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele50_layer1_etaoffv3_35_e = [
 [
  [               0,               0,               0,               0,               0,     0.033666216 ],
  [               0,               0,               0,               0,               0,      -0.8906045 ],
  [               0,               0,               0,               0,               0,      0.70327741 ],
  [               0,               0,               0,               0,               0,   -0.0056259171 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,    0.0098674241 ],
  [               0,               0,               0,               0,               0,        2.466495 ],
  [               0,               0,               0,               0,               0,      -1.0810128 ],
  [               0,               0,               0,               0,               0,      -1.1922649 ],
  [               0,               0,               0,               0,               0,    -0.069446974 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.084758393,      0.13111988 ],
  [               0,               0,               0,               0,      -271.33585,       408.97052 ],
  [               0,               0,               0,               0,       -28.96464,       41.648956 ],
  [               0,               0,               0,               0,       64.629646,       -95.74469 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.025679369,    -0.044922478,     -0.07354366,       0.1301319 ],
  [               0,               0,      -231.86729,        1137.578,      -1855.0172,       1007.0993 ],
  [               0,               0,       27.783651,      -137.41801,       227.02032,      -126.18809 ],
  [               0,               0,      -67.734245,       329.30911,      -531.97711,       284.69275 ],
  [               0,               0,      -27.963364,       135.29185,      -217.71063,       116.58251 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0071511506,    -0.011306583 ],
  [               0,               0,               0,               0,      -3.8075168,       9.4570007 ],
  [               0,               0,               0,               0,    -0.017532865,      -1.0518025 ],
  [               0,               0,               0,               0,    -0.045299083,     -0.95270616 ],
  [               0,               0,               0,               0,     -0.18541336,      0.38340691 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0064882212,    -0.013146392,    -0.034069087,     0.071298815 ],
  [               0,               0,      -18.068304,       90.707352,      -139.82661,       64.324089 ],
  [               0,               0,       24.890905,      -160.48569,       345.10284,      -248.56985 ],
  [               0,               0,      -30.673491,       190.89055,        -394.384,       269.43143 ],
  [               0,               0,       14.338054,      -95.548943,       211.95396,      -156.44534 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00042408219,    0.0022312268 ],
  [               0,               0,               0,               0,      -137.82732,       319.30008 ],
  [               0,               0,               0,               0,        46.11311,      -107.62569 ],
  [               0,               0,               0,               0,      -1.3813643,       3.6078913 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.019009061,    -0.041546628 ],
  [               0,               0,               0,               0,      -11.770888,       30.144438 ],
  [               0,               0,               0,               0,     -0.72361976,       0.5582214 ],
  [               0,               0,               0,               0,       2.1499372,      -6.1673079 ],
  [               0,               0,               0,               0,      0.15050516,     -0.30324325 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,    0.0016447601 ],
  [               0,               0,               0,               0,               0,       1.8533443 ],
  [               0,               0,               0,               0,               0,     -0.94026864 ],
  [               0,               0,               0,               0,               0,      -1.9649354 ],
  [               0,               0,               0,               0,               0,     -0.89505225 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0052165198,   -0.0063340273,     0.053911068 ],
  [               0,               0,               0,       4.4231458,      -22.888542,       36.228096 ],
  [               0,               0,               0,      -102.20041,        482.5116,      -570.69104 ],
  [               0,               0,               0,      -31.902176,        165.7047,      -214.12224 ],
  [               0,               0,               0,     0.010634225,    -0.010109441,    -0.037534203 ],
  [               0,               0,               0,      0.28199115,      0.25314194,      -1.3585107 ],
  [               0,               0,               0,       -3.356791,     -0.17670488,        18.01351 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.028151263,    -0.056453537 ],
  [               0,               0,               0,               0,      -2983.6223,       7424.4634 ],
  [               0,               0,               0,               0,      -0.1111811,       1391.2585 ],
  [               0,               0,               0,               0,      -10.739266,        25.78928 ],
  [               0,               0,               0,               0,      0.55131418,     -0.65768683 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele75_layer1_etaoffv3_35_b = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0012066729,   0.00052521101 ],
  [               0,               0,               0,               0,      -3.1206885,       4.5061121 ],
  [               0,               0,               0,               0,      0.52158823,     -0.89648844 ],
  [               0,               0,               0,               0,      -1.8459794,     -0.58415922 ],
  [               0,               0,               0,               0,     -0.48406809,    0.0055336158 ],
  [               0,               0,               0,               0,  -0.00035522769,   3.0486245e-05 ],
  [               0,               0,               0,               0,       0.9767844,      0.88047354 ],
  [               0,               0,               0,               0,      -3.1524901,      -1.2399492 ],
 ],
 [
  [               0,               0,               0,   0.00042111152,    0.0017510855,  -0.00022312135 ],
  [               0,               0,               0,       4.2207886,      -11.349667,       8.3467983 ],
  [               0,               0,               0,      -6.6840536,       7.9355254,      -3.1459369 ],
  [               0,               0,               0,       13.043291,       -17.40168,       4.5671709 ],
  [               0,               0,               0,       2.7828536,      -3.5612066,      0.97693022 ],
  [               0,               0,               0,    0.0010405996,  -0.00059829566,   8.1757373e-07 ],
  [               0,               0,               0,     -0.65040573,      -1.6675797,       2.5013049 ],
  [               0,               0,               0,      -6.9839506,       2.1717379,       0.6727581 ],
 ],
 [
  [               0,               0,               0,               0,      0.02010949,    0.0062756864 ],
  [               0,               0,               0,               0,      -3.1099041,       3.0718672 ],
  [               0,               0,               0,               0,      -22.609722,       17.440681 ],
  [               0,               0,               0,               0,       20.403421,      -16.227043 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0010732791,   0.00095934135,     0.001381391 ],
  [               0,               0,               0,     0.093521821,     -0.96677326,       2.4398631 ],
  [               0,               0,               0,     0.034001333,      0.13879608,      -1.1073457 ],
  [               0,               0,               0,     -0.20964708,      0.80736904,       -1.679802 ],
  [               0,               0,               0,    -0.072785018,      0.22758594,     -0.22493641 ],
  [               0,               0,               0,  -4.2152795e-05,   2.3309531e-06,   6.0740019e-05 ],
  [               0,               0,               0,      0.46664161,     -0.13877956,      0.37490398 ],
  [               0,               0,               0,       1.7711478,       27.285676,      -7.2495888 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.030315962,     0.036569562,   -0.0028745863,     -0.14847745 ],
  [               0,               0,       25.352798,      -69.042488,       -73.09407,       176.81812 ],
  [               0,               0,       975.78888,      -1405.5981,      -2026.8169,       2917.9138 ],
  [               0,               0,      -708.48419,        1003.078,       1383.1627,      -1959.6135 ],
  [               0,               0,       141.29729,      -195.24796,      -283.97916,       391.74997 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele75_layer1_etaoffv3_35_e = [
 [
  [               0,               0,               0,               0,      0.16365589,     -0.23181126 ],
  [               0,               0,               0,               0,       418.35162,      -605.64832 ],
  [               0,               0,               0,               0,      -298.15509,       430.24564 ],
  [               0,               0,               0,               0,       67.571655,      -97.151642 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,       0.1299983,      -0.1870411 ],
  [               0,               0,               0,               0,       26.925541,      -33.672974 ],
  [               0,               0,               0,               0,      -189.29008,       277.72021 ],
  [               0,               0,               0,               0,      -134.50897,       195.81889 ],
  [               0,               0,               0,               0,      0.14572729,      -0.3248156 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.27055901,      0.40861821 ],
  [               0,               0,               0,               0,        -309.431,       464.67184 ],
  [               0,               0,               0,               0,      -3.9982858,       5.0456033 ],
  [               0,               0,               0,               0,       64.978554,      -96.356209 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   -0.0005248253,  -0.00034968054,    0.0006906669,    0.0045379247 ],
  [               0,               0,       5.9623423,      -7.3409443,      -22.449326,       31.954035 ],
  [               0,               0,     -0.94170344,       1.0570046,        3.711628,      -5.7995205 ],
  [               0,               0,      -3.0431764,       5.7337909,       5.9379096,      -12.652284 ],
  [               0,               0,    -0.077983066,    -0.033586219,      0.77148068,     -0.77624583 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0074145948,    -0.012310799 ],
  [               0,               0,               0,               0,      -5.1536541,       12.492185 ],
  [               0,               0,               0,               0,     -0.43864009,     -0.21398251 ],
  [               0,               0,               0,               0,      -2.2481835,        3.287184 ],
  [               0,               0,               0,               0,     -0.53513783,       1.0612441 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0015037809,   -0.0060561621,   -0.0099291224,     0.038466226 ],
  [               0,               0,       4.6728811,      -22.065838,       34.689777,      -16.865343 ],
  [               0,               0,       26.410503,      -163.26714,       335.54221,      -230.35252 ],
  [               0,               0,      -17.796043,       117.85734,      -259.80936,       189.56319 ],
  [               0,               0,       11.793708,        -76.4552,       165.16055,      -118.85135 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0014942049,    0.0031642404 ],
  [               0,               0,               0,               0,      -56.508217,       131.39648 ],
  [               0,               0,               0,               0,       3.7131042,      -9.7139664 ],
  [               0,               0,               0,               0,       11.377805,      -25.952042 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00061301386,    0.0017345443 ],
  [               0,               0,               0,               0,       1.5119832,     -0.87531489 ],
  [               0,               0,               0,               0,      0.12755658,      -1.4933041 ],
  [               0,               0,               0,               0,       -1.337254,       1.9356369 ],
  [               0,               0,               0,               0,    0.0012771043,     0.048170704 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,    0.0017166996 ],
  [               0,               0,               0,               0,               0,       2.4752293 ],
  [               0,               0,               0,               0,               0,      -1.2157953 ],
  [               0,               0,               0,               0,               0,       -1.706978 ],
  [               0,               0,               0,               0,               0,     -0.69715512 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     -0.17317608,      0.83538181,     -0.99902523 ],
  [               0,               0,               0,      -316.16586,       1529.1906,      -1842.0431 ],
  [               0,               0,               0,      -41.733124,       203.81728,      -250.33972 ],
  [               0,               0,               0,      -15.773548,       78.204651,       -96.84996 ],
  [               0,               0,               0,     0.055367365,     -0.28254905,      0.36039585 ],
  [               0,               0,               0,       316.94589,       -1499.746,       1773.2042 ],
  [               0,               0,               0,      -188.25336,       959.30554,      -1220.6487 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0069242553,     0.010242987 ],
  [               0,               0,               0,               0,      -0.7410118,        8.310463 ],
  [               0,               0,               0,               0,               0,     -0.40000001 ],
  [               0,               0,               0,               0,       -39.28096,       96.377235 ],
  [               0,               0,               0,               0,      0.39694312,     -0.42769584 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele100_layer1_etaoffv3_35_b = [
 [
  [               0,               0,       25077.189,      -94.653938,     -0.55447972,   0.00096256472 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00014012139,    0.0006243907 ],
  [               0,               0,               0,               0,     -0.29647906,       4.2552174 ],
  [               0,               0,               0,               0,    -0.061524126,     -0.79133151 ],
  [               0,               0,               0,               0,    -0.046190718,     -0.83127942 ],
  [               0,               0,               0,               0,     -0.15962566,    -0.043388036 ],
  [               0,               0,               0,               0,  -0.00010435563,   1.8613779e-05 ],
  [               0,               0,               0,               0,       3.9750827,      0.51109202 ],
  [               0,               0,               0,               0,      0.77780814,     -0.19662311 ],
 ],
 [
  [               0,               0,               0,    0.0042752576,   -0.0051430812,    0.0023409184 ],
  [               0,               0,               0,      -19.413103,       22.225874,      -2.6350413 ],
  [               0,               0,               0,      -2.4712406,       3.3803513,      -1.9370846 ],
  [               0,               0,               0,        2.550109,      -2.3093255,     -0.54544964 ],
  [               0,               0,               0,       0.8172434,      -1.1947988,      0.26536898 ],
  [               0,               0,               0,   -0.0003917299,   0.00038267744,  -0.00010876255 ],
  [               0,               0,               0,      -2.6490825,      0.99670427,       2.5745815 ],
  [               0,               0,               0,       2.0475462,      -2.2125785,      0.67143071 ],
 ],
 [
  [               0,               0,               0,               0,     0.023439476,    0.0096109565 ],
  [               0,               0,               0,               0,      -5.1328783,       4.6203222 ],
  [               0,               0,               0,               0,      -15.428502,       11.773486 ],
  [               0,               0,               0,               0,       15.557879,      -12.372809 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0033408755,   -0.0073087022,    0.0053903455 ],
  [               0,               0,               0,      -6.3935986,       13.770349,      -4.5019339 ],
  [               0,               0,               0,     -0.60754659,       1.6923895,      -2.1274152 ],
  [               0,               0,               0,       2.9958871,      -6.7871708,       2.4685189 ],
  [               0,               0,               0,      0.65607697,      -1.5029636,      0.69391254 ],
  [               0,               0,               0,    3.736673e-05,  -0.00014120834,   0.00016396544 ],
  [               0,               0,               0,      -5.5259086,       14.245419,      -6.5056834 ],
  [               0,               0,               0,       6.9409384,      -17.458475,       12.394709 ],
 ],
 [
  [               0,               0,               0,               0,               0,       0.1329781 ],
  [               0,               0,               0,               0,               0,       0.3555674 ],
  [               0,               0,               0,               0,               0,     -0.34519657 ],
  [               0,               0,               0,               0,               0,     -0.34412867 ],
  [               0,               0,               0,               0,               0,   -0.0013180081 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0066420389,    0.0046696258,   4.4628599e-05,   -0.0085441843 ],
  [               0,               0,      -52.928528,      -23.857561,       33.517651,       144.39526 ],
  [               0,               0,        22.93606,       10.412103,      -14.855282,      -61.092197 ],
  [               0,               0,               0,               0,               0,              -1 ],
  [               0,               0,    -0.082910053,     -0.16964053,      -0.1629196,      -0.2204686 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    -0.019905444,   -0.0073836376,     0.013523549,     0.061549671 ],
  [               0,               0,      -180.50282,        269.6481,       378.77271,      -559.10083 ],
  [               0,               0,      -238.51727,       443.01721,       26.162354,      -246.61832 ],
  [               0,               0,       363.68161,      -458.61792,      -1061.3982,       1388.5135 ],
  [               0,               0,       2483.7466,      -3528.4211,      -5069.7593,       7200.6411 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele100_layer1_etaoffv3_35_e = [
 [
  [               0,               0,               0,               0,               0,      -127187.87 ],
  [               0,               0,               0,               0,               0,    0.0051434361 ],
  [               0,               0,               0,               0,               0,   -0.0051433775 ],
  [               0,               0,               0,               0,               0,  -1.1038586e-08 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.010963574,    -0.008584884 ],
  [               0,               0,               0,               0,       358.87469,      -520.97351 ],
  [               0,               0,               0,               0,       28.567886,      -43.404808 ],
  [               0,               0,               0,               0,      -95.283936,         138.308 ],
  [               0,               0,               0,               0,      0.11263025,     -0.19947313 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.11224361,      0.17426357 ],
  [               0,               0,               0,               0,      -244.48135,       368.48392 ],
  [               0,               0,               0,               0,       12.174127,      -19.506727 ],
  [               0,               0,               0,               0,       18.121632,      -26.668716 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,     0.000470864 ],
  [               0,               0,               0,               0,               0,      -9.4703779 ],
  [               0,               0,               0,               0,               0,       4.3317933 ],
  [               0,               0,               0,               0,               0,      0.10232953 ],
  [               0,               0,               0,               0,               0,       2.4837377 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0018273592,      -0.0031822,   -0.0073292004,     0.014516071 ],
  [               0,               0,      -33.005669,       172.00008,      -295.00732,        168.7907 ],
  [               0,               0,      -26.711046,       129.57091,      -209.57228,       111.96745 ],
  [               0,               0,       7.5681577,      -36.478836,       57.751411,      -31.022215 ],
  [               0,               0,      -9.0260792,       43.714466,      -70.602295,       38.099632 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0055821547,   -0.0084995655 ],
  [               0,               0,               0,               0,      -3.9220541,       9.9783659 ],
  [               0,               0,               0,               0,     -0.51781714,     -0.15064265 ],
  [               0,               0,               0,               0,    -0.088714473,     -0.91196477 ],
  [               0,               0,               0,               0,     -0.11318803,      0.26327658 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.010352684,    -0.025682233,    -0.050398167,      0.12751023 ],
  [               0,               0,       4.7442179,      -11.804281,      -7.4446592,       25.879534 ],
  [               0,               0,       5.8432584,      -30.005217,       48.094307,      -23.957924 ],
  [               0,               0,      -4.8330836,       32.790634,      -73.662849,       53.743622 ],
  [               0,               0,       5.9828863,      -40.066322,       89.402229,      -66.413109 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00026601533,    0.0029914442 ],
  [               0,               0,               0,               0,      -173.63504,       401.59647 ],
  [               0,               0,               0,               0,       45.612991,      -106.39415 ],
  [               0,               0,               0,               0,       1.9001297,      -3.9673405 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00040212396,    0.0019032269 ],
  [               0,               0,               0,               0,      -5.0234022,       14.598677 ],
  [               0,               0,               0,               0,      0.89268625,      -3.2903166 ],
  [               0,               0,               0,               0,       2.3279624,      -6.5992665 ],
  [               0,               0,               0,               0,      0.14127202,     -0.26694787 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,       32.488464,       14.821446 ],
  [               0,               0,               0,               0,     -0.55403507,       1.3843366 ],
  [               0,               0,               0,               0,      0.57507998,      -1.4351088 ],
  [               0,               0,               0,               0,      0.54951072,      -1.3738061 ],
  [               0,               0,               0,               0,      0.10171502,     -0.24381247 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0055657122,   0.00072581408,     0.038734447 ],
  [               0,               0,               0,      -10.630958,       48.124386,      -46.885113 ],
  [               0,               0,               0,      -95.292015,       449.91641,      -532.30566 ],
  [               0,               0,               0,      -22.157648,       117.85023,       -155.3774 ],
  [               0,               0,               0,    0.0088905338,   -0.0069922032,    -0.034736827 ],
  [               0,               0,               0,     -0.50035906,       2.0565295,      -1.0449632 ],
  [               0,               0,               0,      -4.3015118,      0.98277974,       20.930611 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.22133926,     -0.52279228 ],
  [               0,               0,               0,               0,      -0.2935434,       10.388731 ],
  [               0,               0,               0,               0,               0,     -0.40000001 ],
  [               0,               0,               0,               0,      -24.843533,       60.841785 ],
  [               0,               0,               0,               0,      0.11614889,       0.5037806 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele200_layer1_etaoffv3_35_b = [
 [
  [               0,               0,      -2525.7751,       184.85469,      -1.2848883,    0.0010368607 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0005412333,   0.00051443393 ],
  [               0,               0,               0,               0,       -0.737792,       4.4903314 ],
  [               0,               0,               0,               0,       -0.622759,     -0.59169026 ],
  [               0,               0,               0,               0,      -1.2177388,       -0.580836 ],
  [               0,               0,               0,               0,     -0.19207579,    -0.053474423 ],
  [               0,               0,               0,               0,  -7.5565631e-05,   4.5590207e-05 ],
  [               0,               0,               0,               0,       1.7127017,       1.8283159 ],
  [               0,               0,               0,               0,      -2.9944215,      -1.1365259 ],
 ],
 [
  [               0,               0,               0,    0.0052900993,   -0.0023073983,   0.00075077756 ],
  [               0,               0,               0,      0.71748804,      -11.324428,       8.9077723 ],
  [               0,               0,               0,       10.526464,      -10.985796,       1.9756254 ],
  [               0,               0,               0,       15.035316,      -15.371008,       2.7944381 ],
  [               0,               0,               0,      0.29544025,     -0.11476701,     -0.14211836 ],
  [               0,               0,               0,   -0.0019349144,    0.0017087276,  -0.00038551588 ],
  [               0,               0,               0,      -2.7812914,      0.16413653,       3.2329989 ],
  [               0,               0,               0,       -16.84043,       22.195791,      -12.224216 ],
 ],
 [
  [               0,               0,               0,               0,     0.022569278,    0.0054508913 ],
  [               0,               0,               0,               0,       9.1294098,       -6.684999 ],
  [               0,               0,               0,               0,      -26.208273,       20.314625 ],
  [               0,               0,               0,               0,       17.440903,      -13.870676 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0010920306,  -4.3679231e-05,    0.0031568499 ],
  [               0,               0,               0,      -2.8079153,        6.846943,      -2.1312061 ],
  [               0,               0,               0,       1.6980834,      -3.2569663,      0.69354152 ],
  [               0,               0,               0,       3.5030036,      -7.3582682,       2.7642653 ],
  [               0,               0,               0,    -0.074617932,     0.015946754,    -0.053083777 ],
  [               0,               0,               0,    0.0010768259,   -0.0022487268,    0.0012165109 ],
  [               0,               0,               0,      0.56940696,      -2.8995917,       4.3421994 ],
  [               0,               0,               0,     -0.43097495,       1.6417433,      -3.3882571 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.96621281,      -1.3609825,      -1.9775631,       2.7921071 ],
  [               0,               0,      -4404.0654,       6281.7354,       8906.3154,      -12693.997 ],
  [               0,               0,       912.66357,      -1258.2335,      -2203.1025,       3053.6438 ],
  [               0,               0,       387.28033,      -515.32568,      -1032.8333,       1394.4657 ],
  [               0,               0,       2341.3796,      -3321.9426,      -4778.0957,       6777.5684 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele200_layer1_etaoffv3_35_e = [
 [
  [               0,               0,               0,               0,               0,       32194.451 ],
  [               0,               0,               0,               0,               0,   -0.0075363093 ],
  [               0,               0,               0,               0,               0,    0.0075361151 ],
  [               0,               0,               0,               0,               0,   6.6853119e-08 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00016672353,     0.027176278 ],
  [               0,               0,               0,               0,      -223.54077,        330.1676 ],
  [               0,               0,               0,               0,       121.29354,      -179.35059 ],
  [               0,               0,               0,               0,      -163.69627,        236.3324 ],
  [               0,               0,               0,               0,     0.060612887,    -0.086868703 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      -0.1397121,      0.21272214 ],
  [               0,               0,               0,               0,      -398.74707,       600.43549 ],
  [               0,               0,               0,               0,      -71.862236,       105.36404 ],
  [               0,               0,               0,               0,       101.01111,      -149.79977 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,  -0.00036980354,  -0.00024898854,   0.00025983748,    0.0030704965 ],
  [               0,               0,       4.6055689,      -6.7278013,      -8.8597937,         15.4153 ],
  [               0,               0,       3.9838636,      -5.4055386,      -13.113562,       17.345718 ],
  [               0,               0,        4.260684,       -7.141933,       -9.709342,       15.277357 ],
  [               0,               0,      0.35711789,     -0.15664022,      -2.1455884,       2.4884679 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0075833052,    -0.011602903 ],
  [               0,               0,               0,               0,      -3.0380044,       8.0266075 ],
  [               0,               0,               0,               0,     -0.12090474,     -0.90981686 ],
  [               0,               0,               0,               0,     -0.40787324,     -0.35050461 ],
  [               0,               0,               0,               0,     -0.35108241,      0.70345145 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   -0.0002552876,  -0.00028190165,   0.00058831263,    0.0056442837 ],
  [               0,               0,       2.7702153,       -5.548562,      -13.015206,       28.757538 ],
  [               0,               0,     -0.07483539,     0.013769727,      0.98829085,      -2.6332123 ],
  [               0,               0,      -2.2862506,        5.218492,       10.012226,       -23.98159 ],
  [               0,               0,     -0.28160357,      0.50497437,       1.8993338,       -3.557271 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0065610069,     0.014635429 ],
  [               0,               0,               0,               0,      -27.014797,       62.873398 ],
  [               0,               0,               0,               0,       17.643892,      -41.257797 ],
  [               0,               0,               0,               0,       0.8640117,       -1.935614 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00070577988,    0.0022513573 ],
  [               0,               0,               0,               0,      -2.8486266,       9.1293936 ],
  [               0,               0,               0,               0,       3.0118809,      -8.2982883 ],
  [               0,               0,               0,               0,    -0.035421256,      -1.0832595 ],
  [               0,               0,               0,               0,     -0.24989749,      0.63851953 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,    0.0012649589 ],
  [               0,               0,               0,               0,               0,       2.5806346 ],
  [               0,               0,               0,               0,               0,     -0.85195959 ],
  [               0,               0,               0,               0,               0,      -2.0268774 ],
  [               0,               0,               0,               0,               0,       -1.122219 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.046031766,      0.12044401 ],
  [               0,               0,               0,               0,       5.8101931,      -6.9288311 ],
  [               0,               0,               0,               0,        -17.7327,       41.428562 ],
  [               0,               0,               0,               0,        13.67523,       -33.08012 ],
  [               0,               0,               0,               0,    -0.070446573,      0.17065436 ],
  [               0,               0,               0,               0,       -2.645421,       7.2864761 ],
  [               0,               0,               0,               0,      -8.7788944,       21.906107 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.012697118,    0.0022184877 ],
  [               0,               0,               0,               0,      0.55397409,       6.7909932 ],
  [               0,               0,               0,               0,               0,     -0.40000001 ],
  [               0,               0,               0,               0,      -61.048275,       150.38797 ],
  [               0,               0,               0,               0,     0.041154616,      0.54461634 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele500_layer1_etaoffv3_35_b = [
 [
  [               0,               0,       31409.502,       -172.8237,     -0.23557456,   0.00064664253 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,  -0.00019716696,   0.00097032191 ],
  [               0,               0,               0,               0,     0.086592975,       3.3014309 ],
  [               0,               0,               0,               0,      0.60155613,       -1.071505 ],
  [               0,               0,               0,               0,     0.044892326,      -1.0520887 ],
  [               0,               0,               0,               0,     -0.25733452,     -0.01401143 ],
  [               0,               0,               0,               0,   1.4503349e-05,    4.068739e-05 ],
  [               0,               0,               0,               0,      -1.0160645,       2.7611582 ],
  [               0,               0,               0,               0,      -0.4188708,      -1.6644519 ],
 ],
 [
  [               0,               0,               0,   -0.0058961152,    0.0024528979,  -0.00059548747 ],
  [               0,               0,               0,     -0.61972742,       8.5444255,      -7.6673797 ],
  [               0,               0,               0,      -8.8682975,       10.910109,      -2.4271333 ],
  [               0,               0,               0,      -12.507469,       15.049829,      -3.3753196 ],
  [               0,               0,               0,      0.10986382,     -0.30866142,      0.25788985 ],
  [               0,               0,               0,    0.0014238428,   -0.0016116158,   0.00050326515 ],
  [               0,               0,               0,      -13.014828,        16.30918,      -2.6350288 ],
  [               0,               0,               0,       4.7138361,      -5.5997191,     -0.35186031 ],
 ],
 [
  [               0,               0,               0,               0,    0.0054068482,   -0.0014960802 ],
  [               0,               0,               0,               0,       12.320723,      -8.2099152 ],
  [               0,               0,               0,               0,      -195.72704,       154.61855 ],
  [               0,               0,               0,               0,       149.86905,       -119.1945 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0002549302,    0.0015407411 ],
  [               0,               0,               0,               0,     -0.67792918,       2.8993763 ],
  [               0,               0,               0,               0,      0.16205288,      -1.0801349 ],
  [               0,               0,               0,               0,      0.32359362,      -1.4965253 ],
  [               0,               0,               0,               0,     0.031287994,     -0.16448696 ],
  [               0,               0,               0,               0,   4.1875124e-05,    6.643235e-06 ],
  [               0,               0,               0,               0,      -1.0825685,       3.2220859 ],
  [               0,               0,               0,               0,    -0.072337777,       3.9220065 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    -0.075574927,     0.053496942,     0.097422332,    -0.019301614 ],
  [               0,               0,      -2985.4773,       4542.4966,       5300.8262,      -8131.2642 ],
  [               0,               0,      -894.29364,       1405.1082,       1262.7609,      -2064.2954 ],
  [               0,               0,       549.12305,      -704.88702,      -1577.4875,       2086.8452 ],
  [               0,               0,       3550.1692,      -5045.9526,      -7249.3506,       10301.699 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele500_layer1_etaoffv3_35_e = [
 [
  [               0,               0,               0,               0,     0.061076801,     -0.08664722 ],
  [               0,               0,               0,               0,      -105.15008,      -253.24252 ],
  [               0,               0,               0,               0,      -66.703735,       98.329041 ],
  [               0,               0,               0,               0,      -135.46085,       196.53188 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.057100426,     0.084788077 ],
  [               0,               0,               0,               0,      -64.013374,        105.9312 ],
  [               0,               0,               0,               0,      -1394.8142,       2055.1284 ],
  [               0,               0,               0,               0,       1246.8893,       -1812.329 ],
  [               0,               0,               0,               0,      0.53096336,     -0.15872456 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.34021392,       0.5132699 ],
  [               0,               0,               0,               0,      -298.48605,       450.40384 ],
  [               0,               0,               0,               0,      0.14841761,      -1.6315395 ],
  [               0,               0,               0,               0,       31.617731,      -46.675621 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0020020925,   -0.0012451569,   -0.0061175879,    0.0070564863 ],
  [               0,               0,       -25.49264,       137.85332,      -249.47685,       152.85677 ],
  [               0,               0,      -23.324862,       110.78231,      -173.85889,       88.870529 ],
  [               0,               0,       4.0837412,      -21.750002,       38.605316,      -23.922613 ],
  [               0,               0,      -5.3593473,       27.044928,      -45.715088,       25.938442 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0068907049,    -0.010995269 ],
  [               0,               0,               0,               0,      -5.4719262,       13.295052 ],
  [               0,               0,               0,               0,    -0.021199325,      -1.1399826 ],
  [               0,               0,               0,               0,     -0.83649993,      0.44930071 ],
  [               0,               0,               0,               0,     -0.53102076,       1.0736936 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0013000357,   -0.0029011895,   -0.0063230395,     0.017326042 ],
  [               0,               0,      -8.2870998,       49.634693,      -98.771904,       68.082603 ],
  [               0,               0,       19.871197,      -126.37826,       267.93045,      -190.58797 ],
  [               0,               0,       -26.75832,        171.6319,      -366.40924,       259.18924 ],
  [               0,               0,       12.030585,      -78.688713,       171.35469,      -124.18164 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,  -0.00016169692,    0.0026361698 ],
  [               0,               0,               0,               0,       -406.2562,       937.81964 ],
  [               0,               0,               0,               0,       89.038704,       -206.9225 ],
  [               0,               0,               0,               0,       10.768179,      -23.932135 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.01740047,    -0.037083443 ],
  [               0,               0,               0,               0,      -8.3317308,        22.21958 ],
  [               0,               0,               0,               0,     0.039551269,      -1.3360542 ],
  [               0,               0,               0,               0,       1.7688575,      -5.3491573 ],
  [               0,               0,               0,               0,     0.028058959,     -0.00680946 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0035265745,    0.0084719416 ],
  [               0,               0,               0,               0,      -10.095737,      -15.489921 ],
  [               0,               0,               0,               0,      -50509.375,       121211.67 ],
  [               0,               0,               0,               0,       16052.481,      -38490.984 ],
  [               0,               0,               0,               0,      -13.628344,      -39.451389 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0093200896,    0.0081876619,     0.042363677 ],
  [               0,               0,               0,      -3.3146117,       26.833105,      -36.544292 ],
  [               0,               0,               0,       -119.4208,       557.23969,      -650.82593 ],
  [               0,               0,               0,      -17.102222,       97.395836,      -135.43674 ],
  [               0,               0,               0,      0.01046516,   -0.0011270175,    -0.057911936 ],
  [               0,               0,               0,      -2.5005891,       13.966924,       -18.14924 ],
  [               0,               0,               0,      -4.6068778,       7.9989867,       5.5663357 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,       1.1922566,      -2.8794312 ],
  [               0,               0,               0,               0,       5.6184545,       26.025528 ],
  [               0,               0,               0,               0,  -0.00038987695,       159.15186 ],
  [               0,               0,               0,               0,      -4.6237426,       11.241631 ],
  [               0,               0,               0,               0,       1.1420208,      -1.4920483 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele1000_layer1_etaoffv3_35_b = [
 [
  [               0,               0,       16036.048,      -4.7617946,     -0.67598003,   0.00039580744 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,  -7.9219964e-05,   0.00090915453 ],
  [               0,               0,               0,               0,      0.28702292,       3.7229528 ],
  [               0,               0,               0,               0,      0.45718592,      -1.0989323 ],
  [               0,               0,               0,               0,     -0.16764042,      -1.0606879 ],
  [               0,               0,               0,               0,      -0.2784844,   -0.0084635827 ],
  [               0,               0,               0,               0,   2.6870662e-06,    3.502514e-05 ],
  [               0,               0,               0,               0,     -0.85104515,       2.7261632 ],
  [               0,               0,               0,               0,        -1.59014,      -1.5346509 ],
 ],
 [
  [               0,               0,               0,               0,  -0.00093059674,  -0.00061985486 ],
  [               0,               0,               0,               0,       1.3839959,      -3.8418958 ],
  [               0,               0,               0,               0,      0.14932111,      0.84289676 ],
  [               0,               0,               0,               0,      0.25006421,       1.0277602 ],
  [               0,               0,               0,               0,      0.14497832,      0.04378854 ],
  [               0,               0,               0,               0,  -6.0622176e-05,   8.6479294e-05 ],
  [               0,               0,               0,               0,       0.1906867,       2.2655332 ],
  [               0,               0,               0,               0,       -2.230168,     -0.83602082 ],
 ],
 [
  [               0,               0,               0,               0,      0.54409528,     -0.43156597 ],
  [               0,               0,               0,               0,      -752.27307,       602.11133 ],
  [               0,               0,               0,               0,          67.736,      -55.567463 ],
  [               0,               0,               0,               0,       150.18961,      -119.44343 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0015199845,    0.0026466018,   0.00076156556 ],
  [               0,               0,               0,     -0.20283077,      0.29635719,       2.1336055 ],
  [               0,               0,               0,     -0.49757008,       1.1651909,      -1.5780363 ],
  [               0,               0,               0,       1.4492573,      -2.7554203,     0.088956375 ],
  [               0,               0,               0,      0.10142892,     -0.21607201,    -0.021294445 ],
  [               0,               0,               0,   0.00061977806,   -0.0012640448,   0.00069329608 ],
  [               0,               0,               0,     -0.52568563,     -0.37303812,       3.1075238 ],
  [               0,               0,               0,      -0.1410978,      -0.3491398,      -1.7734466 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.51330054,     -0.79008365,      -1.1005833,       1.6970896 ],
  [               0,               0,       814.93579,       -1139.155,      -1634.4115,       2290.1687 ],
  [               0,               0,       -879.2674,       1365.1809,        1279.837,      -2051.0327 ],
  [               0,               0,       74.956329,       29.687122,      -787.12158,       841.09595 ],
  [               0,               0,       3457.3777,      -4913.6304,      -7064.9321,       10038.744 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele1000_layer1_etaoffv3_35_e = [
 [
  [               0,               0,               0,               0,      -0.2048219,      0.29695022 ],
  [               0,               0,               0,               0,      -646.03973,        571.8526 ],
  [               0,               0,               0,               0,      -268.90378,       388.90671 ],
  [               0,               0,               0,               0,       311.64142,      -447.45312 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0005879167,    0.0093551679 ],
  [               0,               0,               0,               0,       5.0826731,      -2.7074637 ],
  [               0,               0,               0,               0,       29.067663,      -44.286274 ],
  [               0,               0,               0,               0,      -67.211967,       96.556885 ],
  [               0,               0,               0,               0,      0.12920579,     -0.08659514 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.32603931,      0.49284622 ],
  [               0,               0,               0,               0,      -360.34802,       542.62421 ],
  [               0,               0,               0,               0,       5.1683745,      -9.1237402 ],
  [               0,               0,               0,               0,       22.055426,      -32.472645 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,  -0.00042396341,  -0.00063925295,   0.00033956673,    0.0062866104 ],
  [               0,               0,      0.27391249,        5.745892,      -18.765709,       15.988444 ],
  [               0,               0,     -0.90817177,       3.6260669,      -5.1173539,        1.451409 ],
  [               0,               0,     -0.76545495,       1.5779569,      0.36296266,       -2.568202 ],
  [               0,               0,     -0.39754203,       1.4975177,       -1.717472,      0.60117102 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0081756776,    -0.012721663 ],
  [               0,               0,               0,               0,      -5.3894658,       12.770978 ],
  [               0,               0,               0,               0,      0.33054113,      -1.8852332 ],
  [               0,               0,               0,               0,      0.67568606,      -2.4501953 ],
  [               0,               0,               0,               0,     -0.25804895,      0.54889905 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,  -0.00018335973,  -4.2079311e-05,    0.0006703285,    0.0035857158 ],
  [               0,               0,     -0.08877974,       0.3636069,       0.2005465,       1.8119624 ],
  [               0,               0,      0.92329532,      -1.9531519,      -4.0570598,        7.298995 ],
  [               0,               0,    -0.013311519,      0.19238521,     -0.33507833,      -1.1700692 ],
  [               0,               0,     -0.32395637,      0.64356339,       1.7705083,      -3.4821739 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.095702633,      0.22360469 ],
  [               0,               0,               0,               0,      -225.27509,       520.40594 ],
  [               0,               0,               0,               0,       47.575615,      -111.04151 ],
  [               0,               0,               0,               0,       20.297285,      -46.143822 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.010855946,    -0.022299649 ],
  [               0,               0,               0,               0,      -12.487933,       32.510227 ],
  [               0,               0,               0,               0,       1.9486805,      -5.8944373 ],
  [               0,               0,               0,               0,       1.7924913,       -5.442965 ],
  [               0,               0,               0,               0,       -0.186849,      0.51211208 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.044325657,      0.10646402 ],
  [               0,               0,               0,               0,       -1.486508,       4.5680842 ],
  [               0,               0,               0,               0,      -5165.1821,       12392.958 ],
  [               0,               0,               0,               0,       3062.8962,      -7341.0264 ],
  [               0,               0,               0,               0,     -0.73070168,       -4.360065 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.076982982,     -0.37233341,      0.45868579 ],
  [               0,               0,               0,      -1221.3219,       5933.6016,      -7197.8501 ],
  [               0,               0,               0,       119.16742,      -585.42621,       717.28754 ],
  [               0,               0,               0,      -132.25739,         648.336,      -794.36725 ],
  [               0,               0,               0,     -0.68992996,       3.3547194,      -4.0776701 ],
  [               0,               0,               0,       1109.0172,      -5383.8369,       6529.1392 ],
  [               0,               0,               0,       1126.4727,      -5431.9624,       6541.0518 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.031547472,    -0.042895351 ],
  [               0,               0,               0,               0,       1.2822292,       10.844081 ],
  [               0,               0,               0,               0,               0,     -0.40000001 ],
  [               0,               0,               0,               0,      -25.704468,       63.091343 ],
  [               0,               0,               0,               0,      0.10658435,      0.67865926 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]


ele_layer1_etaoffv3_35_b = [
    ele5_layer1_etaoffv3_35_b,
    ele10_layer1_etaoffv3_35_b,
    ele25_layer1_etaoffv3_35_b,
    ele50_layer1_etaoffv3_35_b,
    ele75_layer1_etaoffv3_35_b,
    ele100_layer1_etaoffv3_35_b,
    ele200_layer1_etaoffv3_35_b,
    ele500_layer1_etaoffv3_35_b,
    ele1000_layer1_etaoffv3_35_b,
    ]

ele_layer1_etaoffv3_35_e = [
    ele5_layer1_etaoffv3_35_e,
    ele10_layer1_etaoffv3_35_e,
    ele25_layer1_etaoffv3_35_e,
    ele50_layer1_etaoffv3_35_e,
    ele75_layer1_etaoffv3_35_e,
    ele100_layer1_etaoffv3_35_e,
    ele200_layer1_etaoffv3_35_e,
    ele500_layer1_etaoffv3_35_e,
    ele1000_layer1_etaoffv3_35_e,
    ]



###############################################################################
# Tables for v3 sshape correction, layer 1, 3x5 photon clusters
#

gam5_layer1_etaoffv3_35_b = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0030896794,    0.0048066955 ],
  [               0,               0,               0,               0,       2.7044201,       2.9409697 ],
  [               0,               0,               0,               0,      0.87795943,      -1.8132097 ],
  [               0,               0,               0,               0,      0.00416695,      -1.7705373 ],
  [               0,               0,               0,               0,     -0.12602545,    -0.015210825 ],
  [               0,               0,               0,               0,   0.00063823897,   0.00068382209 ],
  [               0,               0,               0,               0,      -1.0130422,       2.1650372 ],
  [               0,               0,               0,               0,     -0.47973776,       1.4614716 ],
 ],
 [
  [               0,               0,               0,      0.01775288,    -0.024279218,    0.0087084267 ],
  [               0,               0,               0,       161.06863,      -131.89241,       26.891159 ],
  [               0,               0,               0,       1.4361209,      -9.2061386,         3.84603 ],
  [               0,               0,               0,      -19.295015,        20.10257,      -4.2963638 ],
  [               0,               0,               0,       8.0695734,      -10.042666,       2.9202685 ],
  [               0,               0,               0,     0.011514179,    -0.013354405,    0.0039466699 ],
  [               0,               0,               0,       40.162033,      -54.609737,       15.717414 ],
  [               0,               0,               0,       19.443254,      -54.642723,       15.647727 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
    [0, 0, 0, 0, 0, 0], # Removed by hand, see below
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam5_layer1_etaoffv3_35_e = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam10_layer1_etaoffv3_35_b = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,  -8.4308733e-05,   0.00039284254 ],
  [               0,               0,               0,               0,       1.9512004,        2.982213 ],
  [               0,               0,               0,               0,     -0.31819794,     -0.51388836 ],
  [               0,               0,               0,               0,      -5.5325751,      0.95202142 ],
  [               0,               0,               0,               0,      -1.7502965,      0.28278089 ],
  [               0,               0,               0,               0,  -0.00038508611,  -8.5867796e-05 ],
  [               0,               0,               0,               0,      -1.3911054,       1.4106479 ],
  [               0,               0,               0,               0,       3.2259352,      0.57431561 ],
 ],
 [
  [               0,               0,               0,     0.015017855,     0.020229587,   0.00064828654 ],
  [               0,               0,               0,     -0.88005912,      0.81432074,      0.53398502 ],
  [               0,               0,               0,      0.78950918,     -0.85037911,     -0.38851124 ],
  [               0,               0,               0,       1.3607517,      -1.4933707,     -0.22765733 ],
  [               0,               0,               0,     0.096464254,     -0.10832106,     0.019299788 ],
  [               0,               0,               0,    0.0030023688,   -0.0034181229,   0.00097200822 ],
  [               0,               0,               0,       92.121582,      -120.95792,       37.373623 ],
  [               0,               0,               0,      -78.407433,       88.815964,      -24.248043 ],
 ],
 [
  [               0,               0,               0,               0,               0,     0.019201091 ],
  [               0,               0,               0,               0,               0,      0.63092536 ],
  [               0,               0,               0,               0,               0,     -0.56708866 ],
  [               0,               0,               0,               0,               0,     0.018374341 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.023259727,    -0.064080715,     0.051635988 ],
  [               0,               0,               0,      -2.3849649,       5.9038582,      -2.6598184 ],
  [               0,               0,               0,       2.0201669,      -4.7872419,       2.0709946 ],
  [               0,               0,               0,       1.7420906,      -4.2912297,       1.8285844 ],
  [               0,               0,               0,      0.10219464,     -0.24629034,      0.10981528 ],
  [               0,               0,               0,   -0.0020534738,    0.0044757216,   -0.0023852556 ],
  [               0,               0,               0,       5.5264583,      -23.500439,       13.193666 ],
  [               0,               0,               0,      -57.215816,       126.34231,      -66.583511 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,      0.26003179,     -0.40453365,     0.058955804 ],
  [               0,               0,               0,      -1212.9658,       3375.4683,      -2342.6064 ],
  [               0,               0,               0,       1795.9749,      -5206.8706,       3772.0388 ],
  [               0,               0,               0,      -1186.7299,       3287.4426,      -2276.7434 ],
  [               0,               0,               0,       1606.5468,      -4557.9961,       3232.1719 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam10_layer1_etaoffv3_35_e = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   -0.0018878384,   0.00026712331,    0.0039660106,    0.0069287759 ],
  [               0,               0,       33.664692,      -155.32129,       239.15918,      -122.00838 ],
  [               0,               0,       2.3636453,      -19.284147,       43.655418,      -30.877256 ],
  [               0,               0,       1.4278455,      -14.218515,       33.322338,      -23.397715 ],
  [               0,               0,      -1.7599759,       9.0559015,      -15.643936,       9.0772648 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      -546.32233,       1120.6642 ],
  [               0,               0,               0,               0,     -0.31040981,      0.60781169 ],
  [               0,               0,               0,               0,      0.31042233,     -0.60781676 ],
  [               0,               0,               0,               0,      0.31037325,     -0.60772753 ],
  [               0,               0,               0,               0,   2.9795849e-05,  -5.3255288e-05 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam25_layer1_etaoffv3_35_b = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,  -4.9479902e-05,   6.6028966e-05 ],
  [               0,               0,               0,               0,       51.390713,       9.3523817 ],
  [               0,               0,               0,               0,      -13.578383,       6.1263037 ],
  [               0,               0,               0,               0,      -7.0075173,       11.707676 ],
  [               0,               0,               0,               0,     -0.39012247,      0.61525398 ],
  [               0,               0,               0,               0,  -0.00050893519,  -0.00012984373 ],
  [               0,               0,               0,               0,      -1.0518252,       1.5365748 ],
  [               0,               0,               0,               0,      0.10336533,       1.1762496 ],
 ],
 [
  [               0,               0,               0,   0.00052428374,    0.0020349391,  -0.00026432914 ],
  [               0,               0,               0,       9.6835442,      -12.314179,       7.3271809 ],
  [               0,               0,               0,      -0.5182988,     -0.63778067,     -0.55345339 ],
  [               0,               0,               0,       3.7108235,      -6.7655039,       1.3370651 ],
  [               0,               0,               0,     -0.12234017,     0.042479318,     -0.10936002 ],
  [               0,               0,               0,   -0.0020112023,    0.0019424573,  -0.00050911942 ],
  [               0,               0,               0,     -0.49747354,      0.40816191,       2.0182202 ],
  [               0,               0,               0,       18.601067,      -26.886801,       7.9568405 ],
 ],
 [
  [               0,               0,               0,               0,      0.02618443,     0.002657681 ],
  [               0,               0,               0,               0,      -10.491592,       8.8867121 ],
  [               0,               0,               0,               0,      -10.659539,       7.9841094 ],
  [               0,               0,               0,               0,       16.694633,      -13.280283 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0024369597,    0.0013306736,  -0.00055167492 ],
  [               0,               0,               0,       3.2496166,      -8.1523905,       6.3598313 ],
  [               0,               0,               0,      -1.7428263,       3.9286151,       -3.048033 ],
  [               0,               0,               0,      -1.0084615,       2.4941289,      -2.4925656 ],
  [               0,               0,               0,    -0.082897149,      0.29515934,     -0.29359174 ],
  [               0,               0,               0,  -0.00029606803,   0.00054618751,  -0.00022646248 ],
  [               0,               0,               0,      -1.0387297,      0.99009579,       4.3791313 ],
  [               0,               0,               0,     -0.78397137,       1.5407119,       2.5352423 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.18881994,     -0.18697636,     -0.32599181,      0.30547544 ],
  [               0,               0,      -811.82947,       1096.4445,       1586.0446,      -2129.8823 ],
  [               0,               0,       2271.4468,      -3259.7356,      -4722.6802,       6775.6104 ],
  [               0,               0,       268.31296,      -401.45242,       -586.8927,       872.90582 ],
  [               0,               0,       426.81464,      -597.44226,      -859.24841,       1201.6631 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam25_layer1_etaoffv3_35_e = [
 [
  [               0,               0,               0,               0,               0,   -0.0016569169 ],
  [               0,               0,               0,               0,               0,        1530.156 ],
  [               0,               0,               0,               0,               0,     0.042800225 ],
  [               0,               0,               0,               0,               0,      -1.1861337 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.098424837,      0.14508191 ],
  [               0,               0,               0,               0,       -10.02386,       53.977631 ],
  [               0,               0,               0,               0,        1602.099,      -2353.5496 ],
  [               0,               0,               0,               0,      -240.98061,       351.87326 ],
  [               0,               0,               0,               0,       1.8508099,       -2.144006 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.01690965,     0.047833312 ],
  [               0,               0,               0,               0,       -173.2218,        260.4588 ],
  [               0,               0,               0,               0,       115.83805,      -173.36549 ],
  [               0,               0,               0,               0,      -74.209015,       110.47808 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    -0.012540843,   -0.0087115476,     0.016409624,     0.096037515 ],
  [               0,               0,      0.54624838,        1.985739,      -10.343878,       9.7340269 ],
  [               0,               0,      -7.6568208,       34.314228,      -51.155914,       24.888716 ],
  [               0,               0,       3.5600667,      -21.776632,       42.172001,      -26.776674 ],
  [               0,               0,       1.6534665,      -8.2005796,       13.527421,      -7.4208059 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0059920293,   -0.0097260056 ],
  [               0,               0,               0,               0,      -7.4512234,       16.886028 ],
  [               0,               0,               0,               0,       1.7909085,      -4.5425706 ],
  [               0,               0,               0,               0,      -2.4202116,       3.7067215 ],
  [               0,               0,               0,               0,     -0.56330264,       1.1037006 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,  -0.00073125475,  -0.00046573352,    0.0015540192,    0.0097115971 ],
  [               0,               0,      0.57508546,     -0.32640669,      -3.1670079,       4.8957863 ],
  [               0,               0,       1.5973473,      -3.3292758,      -7.4487119,       14.311156 ],
  [               0,               0,       2.1232257,      -4.2275596,      -11.108858,       21.182127 ],
  [               0,               0,     -0.25334355,       0.4770087,       1.4594033,       -2.796407 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,       359.07816,      -823.72473 ],
  [               0,               0,               0,               0,       -15.17205,       35.057045 ],
  [               0,               0,               0,               0,       15.219572,      -35.165855 ],
  [               0,               0,               0,               0,     -0.13896377,      0.32043901 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0035457779,     0.011342278 ],
  [               0,               0,               0,               0,      -5.4077749,       15.293084 ],
  [               0,               0,               0,               0,        5.498457,      -14.164721 ],
  [               0,               0,               0,               0,       3.2617815,      -8.7541552 ],
  [               0,               0,               0,               0,      0.36279988,     -0.81404442 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.026392262,     0.007539209,      0.15001701 ],
  [               0,               0,               0,       6.8502293,       18.753607,      -82.416924 ],
  [               0,               0,               0,       34.166195,      -174.30232,       220.46532 ],
  [               0,               0,               0,      -35.916389,       177.97595,      -220.34869 ],
  [               0,               0,               0,   -0.0054223388,  -0.00034075885,     0.033838376 ],
  [               0,               0,               0,       1.0018122,      0.32582369,      -4.2016449 ],
  [               0,               0,               0,       5.8406239,      0.39910361,       -36.67812 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.010236971,    -0.016876616 ],
  [               0,               0,               0,               0,      -3494.4546,       8691.0176 ],
  [               0,               0,               0,               0,               0,     -0.40000001 ],
  [               0,               0,               0,               0,      -26.700972,       64.806473 ],
  [               0,               0,               0,               0,      0.24577919,     -0.36410072 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam50_layer1_etaoffv3_35_b = [
 [
  [               0,               0,       10132.821,       4.2081466,     -0.72377694,    0.0013550519 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,  -0.00099877233,   0.00083434104 ],
  [               0,               0,               0,               0,       3.7876635,       3.5542173 ],
  [               0,               0,               0,               0,       2.5277152,      -1.2072322 ],
  [               0,               0,               0,               0,      -1.3184597,     -0.79396605 ],
  [               0,               0,               0,               0,      -1.0676636,     0.065358467 ],
  [               0,               0,               0,               0,   0.00046794492,  -5.5064764e-05 ],
  [               0,               0,               0,               0,     -0.16288169,       1.2528615 ],
  [               0,               0,               0,               0,      -3.4110489,     0.093952149 ],
 ],
 [
  [               0,               0,               0,    0.0034007374,    -0.002118394,    0.0011277228 ],
  [               0,               0,               0,        0.826415,      -4.3004537,       5.6473994 ],
  [               0,               0,               0,      0.38411474,      -0.8918913,     -0.69642425 ],
  [               0,               0,               0,       2.5788045,      -3.3493185,     -0.16220732 ],
  [               0,               0,               0,     0.097735889,     -0.13096762,    -0.072978556 ],
  [               0,               0,               0,   0.00096726191,  -0.00098156719,   0.00027306131 ],
  [               0,               0,               0,      -2.4970114,        1.179567,        2.352381 ],
  [               0,               0,               0,       1.5832241,      -2.9801147,      -2.8070736 ],
 ],
 [
  [               0,               0,               0,               0,     0.018761851,     0.010253859 ],
  [               0,               0,               0,               0,       6.1288786,      -4.3170061 ],
  [               0,               0,               0,               0,      -20.796743,       16.029064 ],
  [               0,               0,               0,               0,       15.332538,      -12.194397 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    1.847571e-06,   -0.0014582404,    0.0029380005 ],
  [               0,               0,               0,       -3.531641,       9.0949678,      -2.8344936 ],
  [               0,               0,               0,       1.8178115,      -3.2428217,       0.4030602 ],
  [               0,               0,               0,       2.4754865,      -5.3751054,       1.5761598 ],
  [               0,               0,               0,     -0.18315361,      0.15816969,     -0.13323316 ],
  [               0,               0,               0,   -0.0002691634,   0.00057871832,  -0.00026720995 ],
  [               0,               0,               0,      -5.1659393,       9.0647945,      -6.0561442 ],
  [               0,               0,               0,      -5.4814453,       13.338348,       -3.547255 ],
 ],
 [
  [               0,               0,               0,               0,               0,   0.00026717689 ],
  [               0,               0,               0,               0,               0,       18.549328 ],
  [               0,               0,               0,               0,               0,      -8.5416193 ],
  [               0,               0,               0,               0,               0,    -0.022528015 ],
  [               0,               0,               0,               0,               0,     -0.79681915 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0031549449,     0.001426013,   -0.0021664924,   -0.0083271088 ],
  [               0,               0,      -5.2464089,      -4.9404545,      -6.7806287,      -5.3049483 ],
  [               0,               0,      -17.904217,      -23.751377,      -31.382967,      -41.710846 ],
  [               0,               0,   0.00010301659,     0.030978817,    -0.010527588,      -1.0115291 ],
  [               0,               0,       13.340928,       18.818447,       26.717705,       37.578838 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.41739646,     -0.59773153,     -0.85838693,       1.2360024 ],
  [               0,               0,       125.07809,      -181.74809,      -253.30212,       374.28265 ],
  [               0,               0,       354.35095,      -419.54614,      -1221.7209,        1567.973 ],
  [               0,               0,       427.88129,      -545.32336,      -1235.1003,       1625.9517 ],
  [               0,               0,        3092.333,      -4396.2817,      -6295.6792,       8948.4062 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam50_layer1_etaoffv3_35_e = [
 [
  [               0,               0,               0,               0,               0,        214478.2 ],
  [               0,               0,               0,               0,               0,   -0.0043743481 ],
  [               0,               0,               0,               0,               0,    0.0043743104 ],
  [               0,               0,               0,               0,               0,   6.4179417e-09 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.31281587,     -0.45090973 ],
  [               0,               0,               0,               0,      -18.482735,        31.20281 ],
  [               0,               0,               0,               0,       36.955605,      -55.675056 ],
  [               0,               0,               0,               0,      -52.418015,       75.468307 ],
  [               0,               0,               0,               0,      0.10723363,     -0.19981812 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.064135745,      0.10027498 ],
  [               0,               0,               0,               0,      -288.49255,       435.39035 ],
  [               0,               0,               0,               0,      -43.897423,       63.780895 ],
  [               0,               0,               0,               0,         73.4916,      -108.93246 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,   0.00055569876 ],
  [               0,               0,               0,               0,               0,    -0.003478437 ],
  [               0,               0,               0,               0,               0,     -0.29811373 ],
  [               0,               0,               0,               0,               0,     -0.37771094 ],
  [               0,               0,               0,               0,               0,       3.5410421 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.089595169,     -0.15618399,     -0.24990195,      0.44119227 ],
  [               0,               0,      -50.570717,        237.4996,      -368.25812,       189.43384 ],
  [               0,               0,      -9.7268162,       54.225586,      -100.24983,       60.558743 ],
  [               0,               0,      -3.1153679,       22.780043,      -50.592113,       34.536854 ],
  [               0,               0,      -1.6277322,       7.6968379,      -12.084893,       6.3136258 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0063709165,   -0.0099868029 ],
  [               0,               0,               0,               0,      -4.9342952,       11.912959 ],
  [               0,               0,               0,               0,     -0.06051546,      -1.0113775 ],
  [               0,               0,               0,               0,     -0.58941901,     0.047353256 ],
  [               0,               0,               0,               0,     -0.30791605,      0.63362008 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0039827209,    -0.010146546,    -0.018518014,     0.049761463 ],
  [               0,               0,      -11.688093,       67.488831,      -123.87252,       73.551697 ],
  [               0,               0,       13.197472,      -78.973587,       156.69397,      -104.24204 ],
  [               0,               0,      -14.628161,       96.368874,      -210.76138,       151.93657 ],
  [               0,               0,       13.035727,      -85.280045,       185.86488,      -134.89717 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00010844284,    0.0033257324 ],
  [               0,               0,               0,               0,      -167.51183,       387.46375 ],
  [               0,               0,               0,               0,       75.272141,      -174.53535 ],
  [               0,               0,               0,               0,      -4.9634619,       11.818313 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00038950195,    0.0016819898 ],
  [               0,               0,               0,               0,       -2.189317,       8.2035761 ],
  [               0,               0,               0,               0,       0.3053371,      -1.9900105 ],
  [               0,               0,               0,               0,    -0.094362415,     -0.93302822 ],
  [               0,               0,               0,               0,    -0.057392634,      0.20649362 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,     0.022613877 ],
  [               0,               0,               0,               0,               0,      0.53909212 ],
  [               0,               0,               0,               0,               0,      -0.4864271 ],
  [               0,               0,               0,               0,               0,     -0.53207004 ],
  [               0,               0,               0,               0,               0,    -0.053616691 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.016408606,   -0.0076688882,      -0.0703049 ],
  [               0,               0,               0,      -8.9119043,      -25.063503,       120.99465 ],
  [               0,               0,               0,      -147.82454,        720.7226,      -880.08429 ],
  [               0,               0,               0,      -4.5845809,       23.803799,      -30.642361 ],
  [               0,               0,               0,   -0.0077808942,   -0.0012319586,     0.049279612 ],
  [               0,               0,               0,       67.631851,       -322.1109,        384.2384 ],
  [               0,               0,               0,       8.3109426,      -7.0557699,      -40.106899 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0056036185,   -0.0010771659 ],
  [               0,               0,               0,               0,      -3285.0837,       8171.0879 ],
  [               0,               0,               0,               0,       111704.05,      -278002.88 ],
  [               0,               0,               0,               0,      -74.654488,       183.67143 ],
  [               0,               0,               0,               0,      0.10373662,     0.082601309 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam75_layer1_etaoffv3_35_b = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,  -2.9727653e-06,   0.00076969276 ],
  [               0,               0,               0,               0,     -0.35366359,       4.3393555 ],
  [               0,               0,               0,               0,      0.64795399,     -0.87035388 ],
  [               0,               0,               0,               0,      -0.6468823,     -0.99100214 ],
  [               0,               0,               0,               0,      0.53427517,       -0.502648 ],
  [               0,               0,               0,               0,    0.0007328447,  -0.00039304781 ],
  [               0,               0,               0,               0,       0.8974964,      0.18959866 ],
  [               0,               0,               0,               0,     -0.45536584,      -2.6616819 ],
 ],
 [
  [               0,               0,               0,    0.0017892024,  -0.00094719481,   0.00089573045 ],
  [               0,               0,               0,       2.0570605,      -6.6100664,       6.7052894 ],
  [               0,               0,               0,      -3.2785792,       4.2287259,      -2.2729096 ],
  [               0,               0,               0,       2.2517493,      -2.8822529,     -0.32379699 ],
  [               0,               0,               0,      0.99963576,      -1.4028254,      0.32639581 ],
  [               0,               0,               0,   0.00077821524,  -0.00095252576,   0.00028022172 ],
  [               0,               0,               0,      -11.943915,      -11.238369,      -10.299148 ],
  [               0,               0,               0,       -8.720006,      -2.5040071,       2.1794994 ],
 ],
 [
  [               0,               0,               0,               0,     0.019411175,     0.013145425 ],
  [               0,               0,               0,               0,       14.302109,      -10.843347 ],
  [               0,               0,               0,               0,      -25.222315,       19.572813 ],
  [               0,               0,               0,               0,       12.998444,      -10.340406 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,  -0.00032494601,    0.0009290335,    0.0013721987 ],
  [               0,               0,               0,      -2.5366147,        5.128078,     -0.28967601 ],
  [               0,               0,               0,       1.3729904,       -2.906143,      0.50776172 ],
  [               0,               0,               0,       1.4461474,      -2.9242828,      0.16754061 ],
  [               0,               0,               0,     -0.16379414,       0.3825736,     -0.34356195 ],
  [               0,               0,               0,   0.00060007913,   -0.0014320621,   0.00081834878 ],
  [               0,               0,               0,       1.4469632,      0.13363388,      0.21468668 ],
  [               0,               0,               0,       17.543074,      -41.416752,       16.922169 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.52270466,      -0.7851373,      -1.1010553,       1.6595556 ],
  [               0,               0,       571.36365,      -806.84314,      -1136.0582,       1609.1459 ],
  [               0,               0,      -465.62262,       770.15039,       439.96744,      -843.05737 ],
  [               0,               0,       172.31818,       -169.2702,      -782.98547,       957.67621 ],
  [               0,               0,       3215.0264,      -4573.3691,       -6551.228,       9317.1875 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam75_layer1_etaoffv3_35_e = [
 [
  [               0,               0,               0,               0,    -0.025464978,     0.035331648 ],
  [               0,               0,               0,               0,       145.58571,      -191.66927 ],
  [               0,               0,               0,               0,        103.5946,      -152.11885 ],
  [               0,               0,               0,               0,        122.0877,      -175.80353 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      -2.4416995,       3.6121612 ],
  [               0,               0,               0,               0,       99.718475,      -148.04967 ],
  [               0,               0,               0,               0,      -145.18184,       214.85393 ],
  [               0,               0,               0,               0,       172.29231,      -248.80113 ],
  [               0,               0,               0,               0,      0.12611476,      -0.1611724 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.19357012,       0.2943705 ],
  [               0,               0,               0,               0,      -350.63455,        527.5481 ],
  [               0,               0,               0,               0,       15.128934,      -23.901163 ],
  [               0,               0,               0,               0,       24.257908,       -35.73645 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    -0.012032251,     0.019790772,      0.02618894,    -0.038427785 ],
  [               0,               0,     -0.33092064,       18.295044,      -56.492756,       46.231369 ],
  [               0,               0,      -4.1734867,       13.259763,      -9.8559103,      -2.0454359 ],
  [               0,               0,      -3.1549215,        7.369628,       1.3168263,      -8.9720116 ],
  [               0,               0,      -0.2061125,      0.98865765,      -1.5885879,      0.89001626 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0095695816,    -0.015133274 ],
  [               0,               0,               0,               0,      -4.6471753,       11.012434 ],
  [               0,               0,               0,               0,      0.51198167,      -2.1277158 ],
  [               0,               0,               0,               0,      0.90470463,      -2.8632739 ],
  [               0,               0,               0,               0,    -0.088957198,      0.19992343 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,  -0.00058402267,  -0.00038643822,    0.0013542576,    0.0087190876 ],
  [               0,               0,       1.4337864,      -2.4033012,      -6.2369885,       12.629129 ],
  [               0,               0,         1.03101,      -2.4951746,       -4.160778,       8.9784727 ],
  [               0,               0,     -0.47960809,       1.0905937,       1.7305598,      -5.1198077 ],
  [               0,               0,     -0.27339482,      0.48379743,       1.8402851,      -3.4243906 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00079226017,    0.0027532426 ],
  [               0,               0,               0,               0,      -152.21661,       351.87042 ],
  [               0,               0,               0,               0,       65.932045,      -152.92459 ],
  [               0,               0,               0,               0,    -0.071410537,       0.5322203 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00061312795,    0.0020396602 ],
  [               0,               0,               0,               0,      -1.9278303,       7.0905104 ],
  [               0,               0,               0,               0,       1.6593105,      -5.1171203 ],
  [               0,               0,               0,               0,      0.52110952,      -2.3809807 ],
  [               0,               0,               0,               0,     0.050437074,      -0.0679814 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,   0.00075620192 ],
  [               0,               0,               0,               0,               0,      0.36048082 ],
  [               0,               0,               0,               0,               0,      0.66248477 ],
  [               0,               0,               0,               0,               0,      0.28741631 ],
  [               0,               0,               0,               0,               0,      -1.4823608 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.015403526,     0.051473983,    -0.025240934 ],
  [               0,               0,               0,      -27.567989,       166.06882,      -233.99428 ],
  [               0,               0,               0,      0.80357498,      -9.1152077,        15.93317 ],
  [               0,               0,               0,      -107.83315,        528.6228,      -647.71387 ],
  [               0,               0,               0,   0.00058881124,    0.0048418161,    -0.014728559 ],
  [               0,               0,               0,      -52.066128,       280.47061,      -377.16953 ],
  [               0,               0,               0,      -10.357135,       9.5024643,        32.07996 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.048829958,     0.028878734 ],
  [               0,               0,               0,               0,       153.72484,       -379.9469 ],
  [               0,               0,               0,               0,       20045.662,      -49885.359 ],
  [               0,               0,               0,               0,      -103.11269,       254.76833 ],
  [               0,               0,               0,               0,       2.4657905,      -6.0532284 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam100_layer1_etaoffv3_35_b = [
 [
  [               0,               0,       6768.3867,       64.236382,     -0.96123409,    0.0014519172 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00028722009,   0.00073110336 ],
  [               0,               0,               0,               0,      -1.0547084,       4.3957491 ],
  [               0,               0,               0,               0,       -5.505177,       3.1939089 ],
  [               0,               0,               0,               0,      -5.7666159,       2.9470372 ],
  [               0,               0,               0,               0,      -12.080764,      -4.9918871 ],
  [               0,               0,               0,               0,   -0.0084220544,    -0.011747456 ],
  [               0,               0,               0,               0,      -0.1128652,     0.084802665 ],
  [               0,               0,               0,               0,     -0.64796644,      -1.8854136 ],
 ],
 [
  [               0,               0,               0,     -0.25239441,      0.18317387,    -0.051390562 ],
  [               0,               0,               0,      -1.4340862,       2.5365849,      -1.5791903 ],
  [               0,               0,               0,       1.1298528,      -1.9783328,        1.299407 ],
  [               0,               0,               0,       1.0934885,      -1.9488796,       1.3036242 ],
  [               0,               0,               0,      0.01130803,    -0.020244731,     0.012382376 ],
  [               0,               0,               0,  -0.00061982422,   0.00068731193,  -0.00010161579 ],
  [               0,               0,               0,       6.2747283,      -7.9141488,       5.0327339 ],
  [               0,               0,               0,     0.058474071,     -0.16102593,      -1.7050011 ],
 ],
 [
  [               0,               0,               0,               0,     0.022176227,    0.0084309205 ],
  [               0,               0,               0,               0,       14.428528,      -10.927464 ],
  [               0,               0,               0,               0,      -28.963757,       22.531525 ],
  [               0,               0,               0,               0,       15.648305,      -12.446163 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.085080996,      0.19555925,    -0.077607982 ],
  [               0,               0,               0,     0.078963161,     -0.28172654,      0.81773776 ],
  [               0,               0,               0,    0.0050849002,     0.075491771,     -0.63218063 ],
  [               0,               0,               0,     0.032119326,     0.020002581,     -0.62213111 ],
  [               0,               0,               0,    -0.014937009,     0.034623098,    -0.027525408 ],
  [               0,               0,               0,    0.0003218218,  -0.00075219001,   0.00048263877 ],
  [               0,               0,               0,      -4.2782974,       8.5923748,      -1.6542296 ],
  [               0,               0,               0,      -2.5022664,       5.0184517,       1.9268495 ],
 ],
 [
  [               0,               0,               0,               0,               0,    0.0051849065 ],
  [               0,               0,               0,               0,               0,       1.6844983 ],
  [               0,               0,               0,               0,               0,      -1.5579526 ],
  [               0,               0,               0,               0,               0,      -1.2040988 ],
  [               0,               0,               0,               0,               0,    -0.033307996 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0016487919,   0.00075032993,   -0.0011183624,   -0.0040848106 ],
  [               0,               0,      -5.0675006,      -5.0878987,      -7.0750971,      -5.8391199 ],
  [               0,               0,      -3.1113842,      -3.7794864,      -4.4926696,       -5.583425 ],
  [               0,               0,               0,               0,               0,              -1 ],
  [               0,               0,       1.9179223,        2.713284,       3.8344331,       5.1133294 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,       0.7350508,      -1.0212826,      -1.4885132,       2.0746248 ],
  [               0,               0,        400.6846,      -602.70801,      -838.00671,       1265.6403 ],
  [               0,               0,       879.50696,      -1183.8835,      -2293.2913,       3127.4795 ],
  [               0,               0,        269.6886,      -302.78467,      -980.16486,       1227.7799 ],
  [               0,               0,        3266.634,      -4640.3066,      -6654.9316,       9451.3389 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam100_layer1_etaoffv3_35_e = [
 [
  [               0,               0,               0,               0,               0,     -0.04529706 ],
  [               0,               0,               0,               0,               0,      0.78808212 ],
  [               0,               0,               0,               0,               0,     -0.62586093 ],
  [               0,               0,               0,               0,               0,    -0.041551538 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.022235956,    -0.031247148 ],
  [               0,               0,               0,               0,      -629.55365,       934.03473 ],
  [               0,               0,               0,               0,      -751.30817,       1105.8929 ],
  [               0,               0,               0,               0,       1035.1813,      -1503.3593 ],
  [               0,               0,               0,               0,      0.27695116,    -0.043890126 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.24131133,      0.36530113 ],
  [               0,               0,               0,               0,      -326.69299,       492.09854 ],
  [               0,               0,               0,               0,       2.7648461,      -5.5080557 ],
  [               0,               0,               0,               0,       34.384365,      -50.776009 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,  -0.00036957522,  -2.7631218e-05,  -0.00085570285,    0.0051909476 ],
  [               0,               0,        6.552269,      -29.700748,       47.860523,      -25.305887 ],
  [               0,               0,       -11.51657,       63.790504,      -116.57317,       69.230896 ],
  [               0,               0,       12.967463,      -61.671921,       96.686577,       -50.99448 ],
  [               0,               0,      -5.2342162,       24.751534,      -38.976967,       20.509453 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0050070742,   -0.0071522635 ],
  [               0,               0,               0,               0,      -3.0038605,        8.238246 ],
  [               0,               0,               0,               0,      -0.1589364,     -0.89876896 ],
  [               0,               0,               0,               0,     -0.30210295,     -0.54388016 ],
  [               0,               0,               0,               0,     -0.28627777,      0.60123217 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   0.00059437338,   -0.0018769228,   -0.0023597777,     0.010680544 ],
  [               0,               0,      -3.1654048,       21.545397,        -48.2831,        38.67075 ],
  [               0,               0,       23.637371,      -151.14365,        322.1673,      -230.16934 ],
  [               0,               0,      -18.395185,       115.75173,      -241.93178,       166.79851 ],
  [               0,               0,       11.866367,      -77.891762,       170.26718,      -123.88369 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.023369435,    -0.049572784 ],
  [               0,               0,               0,               0,      -214.67493,       495.60321 ],
  [               0,               0,               0,               0,       68.465126,       -158.8389 ],
  [               0,               0,               0,               0,       4.3449168,      -9.5949039 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0015792932,  -0.00034898057 ],
  [               0,               0,               0,               0,     -0.74515641,       4.5054679 ],
  [               0,               0,               0,               0,     -0.48859024,      -0.1222804 ],
  [               0,               0,               0,               0,     -0.46731257,    -0.051918294 ],
  [               0,               0,               0,               0,      0.23899512,     -0.48681614 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,    0.0012864822 ],
  [               0,               0,               0,               0,               0,       2.7671497 ],
  [               0,               0,               0,               0,               0,       -1.070349 ],
  [               0,               0,               0,               0,               0,      -2.0671773 ],
  [               0,               0,               0,               0,               0,     -0.87880576 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     -0.10894006,       0.4492557,     -0.43970805 ],
  [               0,               0,               0,      -114.08656,       600.03925,      -776.86865 ],
  [               0,               0,               0,       257.48715,      -1262.4646,       1545.3903 ],
  [               0,               0,               0,      -172.88974,       848.47473,      -1040.5957 ],
  [               0,               0,               0,    0.0097368918,    -0.049016617,     0.063018218 ],
  [               0,               0,               0,       113.89611,      -558.43549,       685.39288 ],
  [               0,               0,               0,      -102.25237,       531.43225,      -690.83777 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0011664334,    0.0013406151,  -0.00054220902,    -0.010493279 ],
  [               0,               0,      -156.27415,      -128.44312,       323.47015,       2395.7527 ],
  [               0,               0,     -0.36327484,     -0.89369458,      -2.2343092,       -5.968636 ],
  [               0,               0,      -6.4296236,      0.48958406,       20.363731,       43.071766 ],
  [               0,               0,     0.011788584,     0.029124495,     0.064971678,     -0.15413439 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam200_layer1_etaoffv3_35_b = [
 [
  [               0,               0,       310.80252,       118.24477,     -0.99869442,   0.00099128706 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,  -0.00037366664,    0.0011264172 ],
  [               0,               0,               0,               0,      0.27687821,       3.9913344 ],
  [               0,               0,               0,               0,       1.0762202,      -1.1315992 ],
  [               0,               0,               0,               0,     -0.57859153,      -1.0658265 ],
  [               0,               0,               0,               0,     -0.57774693,    0.0018065083 ],
  [               0,               0,               0,               0,   3.3767701e-05,  -0.00016157854 ],
  [               0,               0,               0,               0,      -0.6825037,       1.0464954 ],
  [               0,               0,               0,               0,      -1.7857215,      -1.5451415 ],
 ],
 [
  [               0,               0,               0,    0.0004086268,    0.0012158108,   0.00035035313 ],
  [               0,               0,               0,       6.7700696,      -12.461679,       8.1215219 ],
  [               0,               0,               0,     -0.33139905,      0.54218942,      -1.1962587 ],
  [               0,               0,               0,      0.48545253,     -0.34584323,      -1.2410783 ],
  [               0,               0,               0,     -0.39715517,      0.42668277,     -0.23674659 ],
  [               0,               0,               0,  -0.00056962465,   0.00073531328,  -0.00020030746 ],
  [               0,               0,               0,     -0.23740473,      0.27792165,       2.5608108 ],
  [               0,               0,               0,       0.2138388,    -0.062486961,      -2.4367776 ],
 ],
 [
  [               0,               0,               0,               0,    0.0047119139,    0.0011069298 ],
  [               0,               0,               0,               0,       28.173269,      -21.241228 ],
  [               0,               0,               0,               0,      -115.26564,       90.779785 ],
  [               0,               0,               0,               0,       88.299927,      -70.232323 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,  -5.8285579e-05,   0.00028204251,     0.001259268 ],
  [               0,               0,               0,     -0.54645318,       1.0319268,       2.3540649 ],
  [               0,               0,               0,     -0.50439852,        1.321565,      -1.8027453 ],
  [               0,               0,               0,      0.69659197,      -1.7346117,     -0.32070208 ],
  [               0,               0,               0,     0.098031841,     -0.27954206,     0.011918317 ],
  [               0,               0,               0,   0.00015830171,  -0.00037210883,   0.00024883778 ],
  [               0,               0,               0,      0.53730005,     -0.30001852,       2.0032778 ],
  [               0,               0,               0,      -1.2693881,      0.13158298,      -2.1789441 ],
 ],
 [
  [               0,               0,               0,               0,               0,       3.3531506 ],
  [               0,               0,               0,               0,               0,      0.11426009 ],
  [               0,               0,               0,               0,               0,     -0.11342052 ],
  [               0,               0,               0,               0,               0,     -0.11392861 ],
  [               0,               0,               0,               0,               0,  -0.00010214139 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0026692874,     0.001201508,   -0.0018489144,   -0.0070058042 ],
  [               0,               0,      -5.2790051,      -5.0972719,      -6.9920783,      -5.5881472 ],
  [               0,               0,      -15.674266,      -20.901674,      -27.815134,      -37.331867 ],
  [               0,               0,               0,               0,               0,              -1 ],
  [               0,               0,       11.378847,        16.12961,       22.854191,       32.069103 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.73009986,      -1.0274043,      -1.4910598,       2.1051526 ],
  [               0,               0,       740.98633,      -1070.8018,      -1506.5221,       2183.7756 ],
  [               0,               0,       737.59271,      -967.23206,      -2062.2393,       2769.1438 ],
  [               0,               0,       274.67657,      -298.93491,      -1047.1877,       1300.9119 ],
  [               0,               0,       3659.3887,      -5198.0601,      -7463.2051,       10599.026 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam200_layer1_etaoffv3_35_e = [
 [
  [               0,               0,               0,               0,               0,    0.0062497496 ],
  [               0,               0,               0,               0,               0,      -2.2249932 ],
  [               0,               0,               0,               0,               0,      0.83944404 ],
  [               0,               0,               0,               0,               0,      0.41669258 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.061024655,       0.0906009 ],
  [               0,               0,               0,               0,      -909.01752,       1341.9399 ],
  [               0,               0,               0,               0,      -1157.1649,       1705.6829 ],
  [               0,               0,               0,               0,       952.03241,      -1384.0037 ],
  [               0,               0,               0,               0,      0.33718094,    -0.073046871 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.35517848,      0.53609288 ],
  [               0,               0,               0,               0,      -335.89005,       505.67331 ],
  [               0,               0,               0,               0,       20.152664,       -31.30617 ],
  [               0,               0,               0,               0,       14.765266,      -21.638435 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,   6.2838029e-05 ],
  [               0,               0,               0,               0,               0,      -39.997829 ],
  [               0,               0,               0,               0,               0,      -3.2392185 ],
  [               0,               0,               0,               0,               0,      -4.0296645 ],
  [               0,               0,               0,               0,               0,       22.130262 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,  -0.00049480423,  -0.00024020569,   0.00074232148,    0.0039136284 ],
  [               0,               0,       2.1030862,      -2.6704488,      -5.5581951,       9.1705933 ],
  [               0,               0,      0.25255418,     -0.16065679,      -1.6711396,      0.87134647 ],
  [               0,               0,     -0.09611614,     0.016255662,       0.3467648,      -1.2824413 ],
  [               0,               0,      -0.1149479,      0.22076765,      0.16376786,     -0.28142676 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0070527615,    -0.011161734 ],
  [               0,               0,               0,               0,      -6.2212143,        14.71144 ],
  [               0,               0,               0,               0,      0.29488426,      -1.8244995 ],
  [               0,               0,               0,               0,     -0.22230701,     -0.74494702 ],
  [               0,               0,               0,               0,     -0.38023242,      0.79039067 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   -0.0018545595,    0.0037285991,    0.0094916737,    -0.016230661 ],
  [               0,               0,      -1.8829647,       19.963114,      -60.286373,       59.112862 ],
  [               0,               0,       21.230623,      -135.20734,       286.74402,      -203.77682 ],
  [               0,               0,      -20.007275,       124.25496,      -256.53812,       174.87679 ],
  [               0,               0,       10.967936,       -72.04969,       157.58138,        -114.684 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.18809594,      0.43576172 ],
  [               0,               0,               0,               0,      -164.78429,       381.29553 ],
  [               0,               0,               0,               0,        12.89815,      -31.288294 ],
  [               0,               0,               0,               0,       43.660992,      -99.826645 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00035966371,    0.0020815798 ],
  [               0,               0,               0,               0,      -2.2439885,       8.5135365 ],
  [               0,               0,               0,               0,     -0.42147097,     -0.33780652 ],
  [               0,               0,               0,               0,      0.22856028,      -1.6943023 ],
  [               0,               0,               0,               0,     0.055596422,    -0.043280751 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,   0.00090302282 ],
  [               0,               0,               0,               0,               0,       2.9041364 ],
  [               0,               0,               0,               0,               0,     -0.72311658 ],
  [               0,               0,               0,               0,               0,      -2.2112727 ],
  [               0,               0,               0,               0,               0,      -1.2612936 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.056892086,      0.16704814,    -0.060534868 ],
  [               0,               0,               0,      -75.732773,       429.96512,       -590.1062 ],
  [               0,               0,               0,       165.41481,      -821.56995,       1017.5823 ],
  [               0,               0,               0,      -119.49205,       591.16913,      -730.59747 ],
  [               0,               0,               0,     0.010895577,    -0.050792407,     0.061114013 ],
  [               0,               0,               0,        164.4861,      -802.57794,       979.94318 ],
  [               0,               0,               0,      -13.086167,       94.401917,      -155.56105 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,       1.5812387,      -3.9135556 ],
  [               0,               0,               0,               0,      -3130.0374,        7787.209 ],
  [               0,               0,               0,               0,       693.85101,      -1730.9268 ],
  [               0,               0,               0,               0,      -194.67302,       481.80716 ],
  [               0,               0,               0,               0,     0.097573757,      0.15024088 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam500_layer1_etaoffv3_35_b = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00015595322,   0.00054659357 ],
  [               0,               0,               0,               0,       2.7634697,       4.4874291 ],
  [               0,               0,               0,               0,      -1.0535986,     -0.60283339 ],
  [               0,               0,               0,               0,     -0.72329068,     -0.48806414 ],
  [               0,               0,               0,               0,     -0.20419891,    -0.016982572 ],
  [               0,               0,               0,               0,  -3.0976345e-05,   9.3992559e-05 ],
  [               0,               0,               0,               0,     -0.89786363,       2.0352092 ],
  [               0,               0,               0,               0,      -2.8053217,      -1.3985664 ],
 ],
 [
  [               0,               0,               0,    0.0030339633,   -0.0040254681,    0.0022319013 ],
  [               0,               0,               0,       -25.86562,       30.721764,      -4.5380321 ],
  [               0,               0,               0,      -5.8321748,       9.2255802,      -2.3033783 ],
  [               0,               0,               0,      -3.5346837,       4.3623252,      -1.2575999 ],
  [               0,               0,               0,      -14.556558,       12.221576,      -4.8347259 ],
  [               0,               0,               0,    0.0052787517,   -0.0033258914,    0.0039873645 ],
  [               0,               0,               0,      0.52155942,     -0.68258059,      0.39000696 ],
  [               0,               0,               0,      -5.5506759,       6.1685381,     -0.75683284 ],
 ],
 [
  [               0,               0,               0,               0,      0.11254045,    -0.059997883 ],
  [               0,               0,               0,               0,       4.6655869,      -3.1297495 ],
  [               0,               0,               0,               0,      -18.188923,       13.938903 ],
  [               0,               0,               0,               0,       12.922526,      -10.278912 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00021279816,    0.0011745488 ],
  [               0,               0,               0,               0,      -1.8501583,       4.7800493 ],
  [               0,               0,               0,               0,       0.3771022,      -1.3152544 ],
  [               0,               0,               0,               0,      0.63421291,      -1.9392102 ],
  [               0,               0,               0,               0,     0.027145645,     -0.20706604 ],
  [               0,               0,               0,               0,  -7.1330949e-05,   8.0900973e-06 ],
  [               0,               0,               0,               0,      -1.1131728,       3.3009858 ],
  [               0,               0,               0,               0,       0.3314147,      0.20400076 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.45719126,     -0.68287718,     -0.96360373,       1.4454515 ],
  [               0,               0,       279.49753,      -387.20743,      -560.19537,       781.84644 ],
  [               0,               0,       -451.8313,       788.16827,       274.26855,      -683.10132 ],
  [               0,               0,       224.51566,      -238.33978,      -933.28064,       1160.6927 ],
  [               0,               0,        3649.771,      -5188.0039,      -7448.8618,        10586.08 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam500_layer1_etaoffv3_35_e = [
 [
  [               0,               0,               0,               0,      0.11067088,     -0.15758084 ],
  [               0,               0,               0,               0,  -1.8618769e+10,   6.1840957e+09 ],
  [               0,               0,               0,               0,      -316.67697,        459.8215 ],
  [               0,               0,               0,               0,       14.471878,      -20.248713 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.49885544,     -0.72352755 ],
  [               0,               0,               0,               0,       562.08606,      -816.20862 ],
  [               0,               0,               0,               0,       50.684635,      -76.483047 ],
  [               0,               0,               0,               0,      -156.07227,       228.91698 ],
  [               0,               0,               0,               0,      0.17381851,    -0.042436469 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.38583839,      0.58111823 ],
  [               0,               0,               0,               0,      -405.03339,       610.07544 ],
  [               0,               0,               0,               0,       5.7488928,      -9.9611044 ],
  [               0,               0,               0,               0,       29.811562,      -43.970196 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   -0.0029487014,   0.00028999388,    0.0054906784,    0.0079302099 ],
  [               0,               0,       31.271967,      -142.34917,       217.92268,      -110.86487 ],
  [               0,               0,       10.058844,      -48.317215,       76.368904,       -40.77618 ],
  [               0,               0,       6.4180965,      -28.587032,         41.1633,      -19.985453 ],
  [               0,               0,       1.0988097,      -4.7649345,       6.8387041,      -3.2082772 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0060839914,   -0.0092925029 ],
  [               0,               0,               0,               0,       -6.392756,       15.211607 ],
  [               0,               0,               0,               0,      0.18472913,      -1.6614267 ],
  [               0,               0,               0,               0,    -0.024839884,      -1.1904495 ],
  [               0,               0,               0,               0,     -0.39855093,      0.82717294 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,  -9.5928444e-05,  -1.6214804e-05,   0.00046179627,    0.0028363469 ],
  [               0,               0,      0.62958151,      -1.2658794,      -3.3302519,       10.205504 ],
  [               0,               0,       1.3363498,       -2.825192,      -6.5238514,       12.424055 ],
  [               0,               0,     -0.89766753,       2.1850276,        3.601907,      -10.075804 ],
  [               0,               0,     -0.49956265,       1.0431892,       2.5326166,      -5.2104039 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0020024008,  -0.00096289435 ],
  [               0,               0,               0,               0,      -332.52063,       767.06122 ],
  [               0,               0,               0,               0,       74.643761,      -173.33458 ],
  [               0,               0,               0,               0,       11.791193,      -26.513557 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00067521672,    0.0020016239 ],
  [               0,               0,               0,               0,       1.4647738,     -0.43873167 ],
  [               0,               0,               0,               0,     -0.28583893,     -0.73772371 ],
  [               0,               0,               0,               0,     -0.82410711,      0.76728767 ],
  [               0,               0,               0,               0,     -0.34234178,      0.89302069 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.016193703,     0.038926847 ],
  [               0,               0,               0,               0,       19.861702,      -7.6677966 ],
  [               0,               0,               0,               0,       -3739.563,       8975.7236 ],
  [               0,               0,               0,               0,       4895.4033,      -11734.997 ],
  [               0,               0,               0,               0,       2.3881714,      -17.072485 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.15802705,      0.39450014 ],
  [               0,               0,               0,               0,       84.384918,      -196.47913 ],
  [               0,               0,               0,               0,       4.4215655,      -12.629594 ],
  [               0,               0,               0,               0,     -0.50818968,       1.4913756 ],
  [               0,               0,               0,               0,      0.12824079,     -0.31431499 ],
  [               0,               0,               0,               0,      0.33458525,      -1.9100994 ],
  [               0,               0,               0,               0,       9.7288733,      -22.623686 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.054501515,     -0.11887339 ],
  [               0,               0,               0,               0,        -3956.24,       9838.0225 ],
  [               0,               0,               0,               0,   1.3992701e-05,     -0.47281918 ],
  [               0,               0,               0,               0,      -217.88164,         538.185 ],
  [               0,               0,               0,               0,      0.20807774,     -0.52532887 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam1000_layer1_etaoffv3_35_b = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   4.4698769e-05,    0.0007849324 ],
  [               0,               0,               0,               0,     -0.03783476,       5.5026259 ],
  [               0,               0,               0,               0,      0.63233387,      -1.2390471 ],
  [               0,               0,               0,               0,     -0.57515222,      -1.1589423 ],
  [               0,               0,               0,               0,      -0.4520852,     0.001503008 ],
  [               0,               0,               0,               0,   8.8435874e-05,  -1.8074323e-05 ],
  [               0,               0,               0,               0,      -18.353703,       2.5213799 ],
  [               0,               0,               0,               0,      -5.6391444,     0.011914624 ],
 ],
 [
  [               0,               0,               0,               0,  -0.00081413944,  -0.00040111772 ],
  [               0,               0,               0,               0,       3.6022353,       -6.686101 ],
  [               0,               0,               0,               0,      0.18818246,      0.79720598 ],
  [               0,               0,               0,               0,      0.70277202,      0.88498729 ],
  [               0,               0,               0,               0,      0.21393952,     0.062540554 ],
  [               0,               0,               0,               0,   1.8272436e-05,   3.2458109e-05 ],
  [               0,               0,               0,               0,       2.0094757,        1.269923 ],
  [               0,               0,               0,               0,      -4.5672388,     0.035926297 ],
 ],
 [
  [               0,               0,               0,               0,      0.47908032,     -0.37960464 ],
  [               0,               0,               0,               0,      -576.06836,       461.49518 ],
  [               0,               0,               0,               0,      -31.122963,       23.210266 ],
  [               0,               0,               0,               0,       144.56134,      -114.97266 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0082104588,    -0.018599706,     0.012206017 ],
  [               0,               0,               0,      -4.4811177,        9.980423,       -2.841269 ],
  [               0,               0,               0,      -2.3681335,        5.234045,      -3.8438499 ],
  [               0,               0,               0,      0.96370071,      -1.8765883,      -0.5151934 ],
  [               0,               0,               0,      0.82618326,      -1.8386624,      0.84564584 ],
  [               0,               0,               0,    0.0005180139,   -0.0010828556,    0.0006038759 ],
  [               0,               0,               0,      -2.6799264,       8.6880112,      -8.7220564 ],
  [               0,               0,               0,       3.6060612,      -6.3345428,      -0.8495127 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.52912778,     -0.83903044,      -1.1448056,       1.8163455 ],
  [               0,               0,       825.72601,      -1169.6696,      -1629.2839,       2312.5674 ],
  [               0,               0,      -892.48865,       1449.5687,       1093.3278,        -1918.37 ],
  [               0,               0,      -169.30476,       371.63318,      -326.91202,       197.58167 ],
  [               0,               0,        3961.519,      -5638.0967,      -8074.4409,       11489.402 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam1000_layer1_etaoffv3_35_e = [
 [
  [               0,               0,               0,               0,     0.058852483,    -0.083835281 ],
  [               0,               0,               0,               0,      -2057.1482,      -3007.4607 ],
  [               0,               0,               0,               0,      -417.19965,       603.64966 ],
  [               0,               0,               0,               0,      -8.0274849,       13.998417 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.003681618,   -0.0016447623 ],
  [               0,               0,               0,               0,      -3.6984732,       15.069304 ],
  [               0,               0,               0,               0,      -211.42143,       309.78238 ],
  [               0,               0,               0,               0,       216.84441,      -315.36792 ],
  [               0,               0,               0,               0,      0.24562845,     0.029362526 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.41201857,      0.62073547 ],
  [               0,               0,               0,               0,      -446.44391,       671.53497 ],
  [               0,               0,               0,               0,       15.685191,      -24.709421 ],
  [               0,               0,               0,               0,        19.16194,      -28.161802 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,  -0.00047969847,  -0.00024889479,   0.00069031084,    0.0037786847 ],
  [               0,               0,         4.37463,      -6.1149588,      -11.152306,         17.8272 ],
  [               0,               0,      0.18445365,     -0.23510192,     -0.98285198,       0.1978959 ],
  [               0,               0,     -0.97420543,       1.3464458,        2.720387,      -4.8952808 ],
  [               0,               0,     0.078395516,     -0.12443133,     -0.21056399,      0.41116688 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0010637566,    0.0041360124 ],
  [               0,               0,               0,               0,      -0.4454703,       4.0304127 ],
  [               0,               0,               0,               0,       1.5608351,      -4.2826266 ],
  [               0,               0,               0,               0,    -0.016090292,       -1.181469 ],
  [               0,               0,               0,               0,     -0.26240027,      0.57459205 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,  -0.00038266616,  -0.00032318264,   0.00086769566,    0.0068060565 ],
  [               0,               0,       2.3836927,      -4.5436692,      -11.572103,       25.220797 ],
  [               0,               0,    0.0090057189,      0.13791692,     -0.14126368,      -1.7498722 ],
  [               0,               0,     -0.38718256,       1.3324021,      0.57509786,      -4.7164426 ],
  [               0,               0,     -0.15956676,      0.18731214,       1.5276413,      -2.4885063 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.33969229,      0.78460717 ],
  [               0,               0,               0,               0,      -149.66676,       346.50989 ],
  [               0,               0,               0,               0,      -48.788872,       110.45457 ],
  [               0,               0,               0,               0,       85.915131,      -196.89453 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.017105024,    -0.036815245 ],
  [               0,               0,               0,               0,      -7.5853386,       21.031248 ],
  [               0,               0,               0,               0,      -3.5353353,       6.9339914 ],
  [               0,               0,               0,               0,       1.2467769,       -4.163568 ],
  [               0,               0,               0,               0,      0.43218622,     -0.93216211 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00011861396,    0.0026025388 ],
  [               0,               0,               0,               0,      -401.79379,       965.01501 ],
  [               0,               0,               0,               0,        2.954607,      -7.6468763 ],
  [               0,               0,               0,               0,       13.624998,      -34.288292 ],
  [               0,               0,               0,               0,    -0.024406971,     -0.35494593 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.029649427,     0.035312142,     0.096639119 ],
  [               0,               0,               0,       12.884025,       57.400738,      -204.71477 ],
  [               0,               0,               0,      -173.98045,       809.93878,      -943.37415 ],
  [               0,               0,               0,      -14.000388,       86.143028,      -125.93508 ],
  [               0,               0,               0,    0.0014925045,    0.0012061945,   -0.0081563462 ],
  [               0,               0,               0,      -128.21375,       630.72858,      -774.71906 ],
  [               0,               0,               0,      0.10263837,       22.571896,      -58.482372 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.065647669,     -0.13196914 ],
  [               0,               0,               0,               0,     0.052444201,       9.0274172 ],
  [               0,               0,               0,               0,               0,     -0.40000001 ],
  [               0,               0,               0,               0,      -55.099873,       135.65038 ],
  [               0,               0,               0,               0,      0.11515028,      0.39801738 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]


gam_layer1_etaoffv3_35_b = [
    gam5_layer1_etaoffv3_35_b,
    gam10_layer1_etaoffv3_35_b,
    gam25_layer1_etaoffv3_35_b,
    gam50_layer1_etaoffv3_35_b,
    gam75_layer1_etaoffv3_35_b,
    gam100_layer1_etaoffv3_35_b,
    gam200_layer1_etaoffv3_35_b,
    gam500_layer1_etaoffv3_35_b,
    gam1000_layer1_etaoffv3_35_b,
    ]

gam_layer1_etaoffv3_35_e = [
    gam5_layer1_etaoffv3_35_e,
    gam10_layer1_etaoffv3_35_e,
    gam25_layer1_etaoffv3_35_e,
    gam50_layer1_etaoffv3_35_e,
    gam75_layer1_etaoffv3_35_e,
    gam100_layer1_etaoffv3_35_e,
    gam200_layer1_etaoffv3_35_e,
    gam500_layer1_etaoffv3_35_e,
    gam1000_layer1_etaoffv3_35_e,
    ]



###############################################################################
# Tables for v3 sshape correction, layer 1, 3x7 electron clusters
#

ele5_layer1_etaoffv3_37_b = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
    [0, 0, 0, 0, 0, 0], # Removed by hand, see below
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele5_layer1_etaoffv3_37_e = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele10_layer1_etaoffv3_37_b = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0082553151,     0.012007935 ],
  [               0,               0,               0,               0,      -2.2476542,       1.4021163 ],
  [               0,               0,               0,               0,      -16.323183,       3.6871582 ],
  [               0,               0,               0,               0,      -16.196393,       3.6541606 ],
  [               0,               0,               0,               0,      0.34926038,    -0.098038254 ],
  [               0,               0,               0,               0,     -0.28872738,     0.076863252 ],
  [               0,               0,               0,               0,     -0.16676506,      0.24583819 ],
  [               0,               0,               0,               0,    0.0062207933,       1.5567139 ],
 ],
 [
  [               0,               0,               0,    0.0011688703,   -0.0017219564,   0.00068227459 ],
  [               0,               0,               0,      -692.85153,       815.55392,      -213.86021 ],
  [               0,               0,               0,       269.38371,      -111.40475,       -34.04256 ],
  [               0,               0,               0,       167.14282,      -40.706993,      -47.288446 ],
  [               0,               0,               0,      -167.60452,       52.192742,       5.9553903 ],
  [               0,               0,               0,     0.077149093,     -0.10509345,     0.037627682 ],
  [               0,               0,               0,      0.59965032,      -0.9990485,       0.6044403 ],
  [               0,               0,               0,       9.1900627,      -5.8524359,     -0.70801476 ],
 ],
 [
  [               0,               0,               0,               0,               0,   0.00077003555 ],
  [               0,               0,               0,               0,               0,       3.0492783 ],
  [               0,               0,               0,               0,               0,      -2.4558482 ],
  [               0,               0,               0,               0,               0,      0.92366743 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.012395777,    0.0016134129,   -0.0020893515 ],
  [               0,               0,               0,      -4.6501817,       10.475799,      -5.1753897 ],
  [               0,               0,               0,       4.4376468,      -10.108534,       5.1438087 ],
  [               0,               0,               0,       4.2668684,      -9.7179587,       4.9450788 ],
  [               0,               0,               0,    -0.029766722,      0.11045031,    -0.092722299 ],
  [               0,               0,               0,   -0.0026804866,    0.0054457849,    -0.002735008 ],
  [               0,               0,               0,      -20.132733,       40.207691,      -15.514626 ],
  [               0,               0,               0,      -41.686907,       71.257998,      -20.667286 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.076936983,      0.12809616,    -0.021494271 ],
  [               0,               0,               0,      -65.428947,       361.79318,       -377.4527 ],
  [               0,               0,               0,        4944.542,       -14198.22,       10191.049 ],
  [               0,               0,               0,       3297.3479,       -9336.876,       6607.3638 ],
  [               0,               0,               0,       324.15912,       -895.0603,       616.67468 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele10_layer1_etaoffv3_37_e = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.015297238,     0.030519824 ],
  [               0,               0,               0,               0,      -1.2557422,       3.1535304 ],
  [               0,               0,               0,               0,       1.0497549,      -2.4493721 ],
  [               0,               0,               0,               0,       2.3926232,       -4.574512 ],
  [               0,               0,               0,               0,    -0.017287549,     0.078265853 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele25_layer1_etaoffv3_37_b = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   5.5882871e-05,   0.00037851426 ],
  [               0,               0,               0,               0,     0.027283272,    -0.011337498 ],
  [               0,               0,               0,               0,       0.7669484,       1.6505199 ],
  [               0,               0,               0,               0,     -0.85174813,       2.8050918 ],
  [               0,               0,               0,               0,     -0.58562802,      0.12620511 ],
  [               0,               0,               0,               0,  -5.6694285e-05,  -0.00013314643 ],
  [               0,               0,               0,               0,      0.40076307,       1.7574674 ],
  [               0,               0,               0,               0,     0.064529855,       1.2319876 ],
 ],
 [
  [               0,               0,               0,       0.0047446,   -0.0069889578,    0.0027442121 ],
  [               0,               0,               0,      -67.216932,       63.410835,      -10.953344 ],
  [               0,               0,               0,       52.746768,      -51.651306,        12.26264 ],
  [               0,               0,               0,        42.07765,      -24.236817,       1.6045165 ],
  [               0,               0,               0,      -7.6692341,       8.8237701,      -2.6236403 ],
  [               0,               0,               0,    0.0024258615,   -0.0037367378,      0.00120581 ],
  [               0,               0,               0,       3.6306762,      -5.3567299,       3.4673339 ],
  [               0,               0,               0,      -16.219683,       22.191286,      -6.4294914 ],
 ],
 [
  [               0,               0,               0,               0,     0.026664153,  -0.00075031188 ],
  [               0,               0,               0,               0,      -12.350151,       10.418055 ],
  [               0,               0,               0,               0,      -18.853256,        14.46418 ],
  [               0,               0,               0,               0,       20.360682,      -16.191481 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0081123805,    -0.012409691,    0.0057995685 ],
  [               0,               0,               0,       4.0268825,      -10.524559,       8.1844281 ],
  [               0,               0,               0,      -1.4278067,       2.9447174,      -2.1829654 ],
  [               0,               0,               0,      -1.5581489,       3.2574093,      -2.4503647 ],
  [               0,               0,               0,     -0.46888096,       1.3458817,      -1.0140545 ],
  [               0,               0,               0,  -8.2116059e-05,   0.00036249068,  -0.00027657276 ],
  [               0,               0,               0,       -48.63823,        88.81651,      -34.275573 ],
  [               0,               0,               0,       69.804667,      -132.63979,       59.920973 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0099384626,     0.045596592,     0.024257457,     -0.14834736 ],
  [               0,               0,      -82.229759,       100.34919,        158.8838,      -186.99765 ],
  [               0,               0,       1522.5142,      -2197.3076,      -3155.3303,        4551.791 ],
  [               0,               0,       -469.3692,       666.27728,        903.5603,      -1284.2509 ],
  [               0,               0,       208.66528,      -287.81772,      -418.25616,       576.01129 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele25_layer1_etaoffv3_37_e = [
 [
  [               0,               0,               0,               0,               0,      -116939.97 ],
  [               0,               0,               0,               0,               0,    0.0054477337 ],
  [               0,               0,               0,               0,               0,   -0.0054476522 ],
  [               0,               0,               0,               0,               0,  -2.4621755e-08 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   4.2749183e-05,   9.7003707e-05,   0.00069223082 ],
  [               0,               0,               0,       5.7458439,        8.374754,       16.205114 ],
  [               0,               0,               0,     -0.13528226,     -0.34701136,      -1.1279981 ],
  [               0,               0,               0,      0.42036608,      0.71401846,      0.18878672 ],
  [               0,               0,               0,     0.063773826,     0.093515359,     -0.16273516 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,     0.026174447 ],
  [               0,               0,               0,               0,               0,       1.3506526 ],
  [               0,               0,               0,               0,               0,     -0.95751333 ],
  [               0,               0,               0,               0,               0,     0.055819154 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.29737017,     -0.34181112,      -1.0391645,       1.3278582 ],
  [               0,               0,      -29.032743,       138.77121,      -225.67191,       125.65726 ],
  [               0,               0,      -9.9358797,        67.22789,      -137.40147,       87.711021 ],
  [               0,               0,      -16.663673,       101.92185,      -197.11797,       122.03161 ],
  [               0,               0,      -5.1303539,       26.636204,      -46.111095,       26.618551 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0019115055,   -0.0030437557 ],
  [               0,               0,               0,               0,      -4.8197784,       13.753852 ],
  [               0,               0,               0,               0,      -5.1286263,       9.4318733 ],
  [               0,               0,               0,               0,      -4.1941385,       8.4361105 ],
  [               0,               0,               0,               0,      -1.2388253,       2.4960167 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,       0.0193928,    -0.044990778,    -0.081691034,      0.19234601 ],
  [               0,               0,      -268.06473,       1628.1852,      -3277.3479,       2187.9724 ],
  [               0,               0,      -274.82837,       1773.1069,      -3809.1868,       2723.7964 ],
  [               0,               0,       -29.54472,       204.73892,       -468.5177,       353.58585 ],
  [               0,               0,       120.54794,      -775.33356,       1660.8297,      -1184.8101 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      -1.8521698,        4.301486 ],
  [               0,               0,               0,               0,      -13.487175,        31.62545 ],
  [               0,               0,               0,               0,       5.5988827,      -13.457766 ],
  [               0,               0,               0,               0,        2.265614,      -5.1778626 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.016264288,    -0.036166593 ],
  [               0,               0,               0,               0,      -35.680866,       86.773819 ],
  [               0,               0,               0,               0,       9.9346523,      -24.444366 ],
  [               0,               0,               0,               0,       6.2464228,      -15.369427 ],
  [               0,               0,               0,               0,     -0.76294386,       1.9137734 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0034408218,     0.014406499,   -0.0073695746 ],
  [               0,               0,               0,      -1.0964979,      -13.659677,        45.98735 ],
  [               0,               0,               0,      -14.503793,       60.420937,      -62.855022 ],
  [               0,               0,               0,       6.0107098,       249.77023,      -614.96936 ],
  [               0,               0,               0,      0.39469466,    0.0097913146,      -2.1540425 ],
  [               0,               0,               0,       12.501919,      -60.339554,       72.834229 ],
  [               0,               0,               0,      -4.5295143,       21.332355,      -34.476765 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0070373639,   -0.0036480429 ],
  [               0,               0,               0,               0,       -1009.565,       2512.5398 ],
  [               0,               0,               0,               0,               0,     -0.40000001 ],
  [               0,               0,               0,               0,      -97.751945,       240.54333 ],
  [               0,               0,               0,               0,     0.049559265,     -0.11497449 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele50_layer1_etaoffv3_37_b = [
 [
  [               0,               0,       18979.906,      -44.672588,     -0.75722378,    0.0014192684 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00081508435,   0.00057071577 ],
  [               0,               0,               0,               0,      -1.6763384,       3.9566655 ],
  [               0,               0,               0,               0,       -1.020954,     -0.43398207 ],
  [               0,               0,               0,               0,    -0.059878063,     -0.94906418 ],
  [               0,               0,               0,               0,      0.24265394,     -0.19415023 ],
  [               0,               0,               0,               0,   -0.0001423053,   8.3134659e-05 ],
  [               0,               0,               0,               0,      0.60120198,      0.81823207 ],
  [               0,               0,               0,               0,       5.1827874,     -0.50820459 ],
 ],
 [
  [               0,               0,               0,     0.002069764,   -0.0025599956,    0.0016967153 ],
  [               0,               0,               0,      -14.518322,       17.363269,      -1.8705837 ],
  [               0,               0,               0,       3.1465693,      -3.5505989,      0.10831277 ],
  [               0,               0,               0,      0.36053035,      0.11965902,      -1.2216444 ],
  [               0,               0,               0,      -1.3711555,       1.4483834,     -0.50101242 ],
  [               0,               0,               0,    0.0014713432,   -0.0018961938,   0.00057714605 ],
  [               0,               0,               0,      -13.452928,       20.507738,      -5.3865991 ],
  [               0,               0,               0,        17.26031,      -18.859229,       4.3222535 ],
 ],
 [
  [               0,               0,               0,               0,     0.022564214,    0.0085350657 ],
  [               0,               0,               0,               0,      -11.860503,       9.9998484 ],
  [               0,               0,               0,               0,      -9.3691397,       6.9382691 ],
  [               0,               0,               0,               0,       15.042174,       -11.96296 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0074740777,   -0.0013487236,     0.027640562 ],
  [               0,               0,               0,      0.60600621,     -0.94066865,       1.0428937 ],
  [               0,               0,               0,     -0.45598989,      0.75319468,     -0.90783265 ],
  [               0,               0,               0,     -0.35719098,      0.51649966,      -0.7941084 ],
  [               0,               0,               0,   -0.0095373294,    0.0079166132,    -0.010989758 ],
  [               0,               0,               0,  -0.00020954251,   0.00036883358,  -0.00013814491 ],
  [               0,               0,               0,     -0.50869734,     -0.92815758,       3.9244951 ],
  [               0,               0,               0,        12.57297,      -22.958547,        14.88213 ],
 ],
 [
  [               0,               0,               0,               0,               0,        39.57077 ],
  [               0,               0,               0,               0,               0,     0.060773537 ],
  [               0,               0,               0,               0,               0,    -0.060731575 ],
  [               0,               0,               0,               0,               0,    -0.060725231 ],
  [               0,               0,               0,               0,               0,  -4.7461317e-06 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      -2.7149739,      -1.2643332,       1.7811857,       7.4658656 ],
  [               0,               0,       2.2481542,       5.3143106,       7.1022491,       13.457803 ],
  [               0,               0,      -11.959024,      -5.4400339,       8.0476084,       32.572323 ],
  [               0,               0,               0,               0,               0,              -1 ],
  [               0,               0,     -0.14262712,     -0.17322721,     -0.28389677,     -0.86431527 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     -0.13800994,      0.14397144,       0.2454506,      -0.2336254 ],
  [               0,               0,      -235.03223,       343.32553,       505.78235,      -732.80835 ],
  [               0,               0,      -343.33618,       591.35101,       272.09518,      -595.16327 ],
  [               0,               0,       340.76468,      -444.17184,      -944.27704,        1258.813 ],
  [               0,               0,       2172.8472,      -3086.5347,      -4435.3525,       6299.1611 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele50_layer1_etaoffv3_37_e = [
 [
  [               0,               0,               0,               0,               0,     0.011010897 ],
  [               0,               0,               0,               0,               0,      -1.6098773 ],
  [               0,               0,               0,               0,               0,      0.89027846 ],
  [               0,               0,               0,               0,               0,    -0.025074229 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,     0.015648859 ],
  [               0,               0,               0,               0,               0,       1.8783462 ],
  [               0,               0,               0,               0,               0,      -1.0392964 ],
  [               0,               0,               0,               0,               0,      -1.1670015 ],
  [               0,               0,               0,               0,               0,    -0.048338473 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.10975598,      0.16779803 ],
  [               0,               0,               0,               0,        -319.672,       481.25717 ],
  [               0,               0,               0,               0,      -43.491837,       63.320007 ],
  [               0,               0,               0,               0,         87.8685,      -130.26147 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.024243452,    -0.042333953,    -0.069535092,      0.12276942 ],
  [               0,               0,      -256.09088,       1256.6353,      -2049.6858,       1113.0577 ],
  [               0,               0,       12.472229,      -62.554646,       105.41825,      -60.563145 ],
  [               0,               0,      -64.705193,       313.26517,      -503.71536,       268.14255 ],
  [               0,               0,      -21.896948,       105.18339,      -168.03391,       89.332436 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0071272594,    -0.011264571 ],
  [               0,               0,               0,               0,       -3.950753,       9.7156076 ],
  [               0,               0,               0,               0,     0.048046835,      -1.1719561 ],
  [               0,               0,               0,               0,     0.078469276,      -1.1914868 ],
  [               0,               0,               0,               0,     -0.15308236,      0.31275293 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.007251462,    -0.017681038,    -0.035245195,     0.087668762 ],
  [               0,               0,       3.6499696,      -5.8011336,      -17.757999,       31.614882 ],
  [               0,               0,       6.6342735,      -33.600536,       53.874004,      -27.549458 ],
  [               0,               0,      -2.6789949,       20.587687,      -49.803207,        37.51223 ],
  [               0,               0,       1.5335615,      -11.897328,       30.057255,      -24.816486 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00018855515,    0.0028392603 ],
  [               0,               0,               0,               0,      -125.05164,       289.85196 ],
  [               0,               0,               0,               0,       45.295559,      -105.70384 ],
  [               0,               0,               0,               0,      -1.1014107,       2.9386775 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.02179384,     -0.04778346 ],
  [               0,               0,               0,               0,      -11.671741,       29.754354 ],
  [               0,               0,               0,               0,      0.18801773,       -1.577408 ],
  [               0,               0,               0,               0,       2.5400434,      -7.0865226 ],
  [               0,               0,               0,               0,      0.16801868,     -0.35553369 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,   0.00072401168 ],
  [               0,               0,               0,               0,               0,   -0.0041264114 ],
  [               0,               0,               0,               0,               0,      0.55023462 ],
  [               0,               0,               0,               0,               0,      -0.7205317 ],
  [               0,               0,               0,               0,               0,      -1.9301178 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0041210754,    -0.012034433,     0.061209876 ],
  [               0,               0,               0,       12.582551,      -63.895618,       87.626015 ],
  [               0,               0,               0,      -101.95731,        480.0643,      -566.19696 ],
  [               0,               0,               0,      -38.114471,       196.46674,       -252.1833 ],
  [               0,               0,               0,     0.011620183,    -0.013133312,    -0.035914533 ],
  [               0,               0,               0,      0.40556884,     0.082532652,      -1.6723872 ],
  [               0,               0,               0,      -2.6430483,     -0.31050587,       14.026166 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.031931356,    -0.065633662 ],
  [               0,               0,               0,               0,      -2980.4658,       7416.6528 ],
  [               0,               0,               0,               0,               0,     -0.40000001 ],
  [               0,               0,               0,               0,      -11.552912,        27.81431 ],
  [               0,               0,               0,               0,       0.9468652,      -1.6275445 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele75_layer1_etaoffv3_37_b = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00058011839,   0.00047127006 ],
  [               0,               0,               0,               0,        -2.83051,        4.892377 ],
  [               0,               0,               0,               0,      -1.3823491,     -0.23983297 ],
  [               0,               0,               0,               0,      0.29523122,      -1.0272524 ],
  [               0,               0,               0,               0,      0.29640268,     -0.22944458 ],
  [               0,               0,               0,               0,  -0.00027236022,   9.8353609e-05 ],
  [               0,               0,               0,               0,     -0.18811439,       1.1988737 ],
  [               0,               0,               0,               0,     -0.61816933,     -0.37097585 ],
 ],
 [
  [               0,               0,               0,    0.0034231828,   0.00033867037,  -0.00033953465 ],
  [               0,               0,               0,     -0.60011584,      -10.379681,       9.3269656 ],
  [               0,               0,               0,       5.6768012,      -5.7166582,      0.48793878 ],
  [               0,               0,               0,       36.502874,      -45.511492,       13.015947 ],
  [               0,               0,               0,       3.5086584,      -4.4193395,       1.2689204 ],
  [               0,               0,               0,    0.0015225663,  -0.00094195264,   1.3438752e-05 ],
  [               0,               0,               0,      0.32300762,     -0.42852003,       1.6950162 ],
  [               0,               0,               0,        1.357775,      -9.4335109,        4.440021 ],
 ],
 [
  [               0,               0,               0,               0,     0.022149587,    0.0072697066 ],
  [               0,               0,               0,               0,      -3.3145149,       3.2112007 ],
  [               0,               0,               0,               0,      -19.732744,       15.170383 ],
  [               0,               0,               0,               0,       18.218197,      -14.489149 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0017205562,   0.00066731688,    0.0041740843 ],
  [               0,               0,               0,      0.26770717,       1.9553002,     -0.59477681 ],
  [               0,               0,               0,       0.9768926,      -2.8006219,      0.88677648 ],
  [               0,               0,               0,       1.3493546,      -4.0226029,       1.5737892 ],
  [               0,               0,               0,    -0.093306577,     0.086278047,    -0.070133099 ],
  [               0,               0,               0,   0.00056648441,   -0.0015720178,    0.0010034618 ],
  [               0,               0,               0,       3.4945932,       -6.864562,       5.2938152 ],
  [               0,               0,               0,     -0.41146103,       1.8465292,       2.9569103 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.030546207,     0.036317006,   -0.0032770615,     -0.14804526 ],
  [               0,               0,       24.630775,      -68.101608,      -71.699654,       175.01068 ],
  [               0,               0,       967.52441,       -1393.717,      -2009.4418,       2892.9402 ],
  [               0,               0,      -720.25464,       1019.8612,       1407.1021,      -1993.7441 ],
  [               0,               0,       138.90205,      -191.87427,      -279.14215,       384.94028 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele75_layer1_etaoffv3_37_e = [
 [
  [               0,               0,               0,               0,     0.067403078,    -0.094308227 ],
  [               0,               0,               0,               0,         440.841,      -638.26697 ],
  [               0,               0,               0,               0,      -245.98811,       355.16742 ],
  [               0,               0,               0,               0,      -1.1440737,       1.7391926 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.13908572,     -0.19924927 ],
  [               0,               0,               0,               0,        798.9563,      -1163.3082 ],
  [               0,               0,               0,               0,      -58.090595,       84.291733 ],
  [               0,               0,               0,               0,      -454.26758,       663.16815 ],
  [               0,               0,               0,               0,      0.15429156,     -0.33369064 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.14102235,        0.217383 ],
  [               0,               0,               0,               0,      -218.37634,       328.68872 ],
  [               0,               0,               0,               0,     -0.26654524,     -0.79940838 ],
  [               0,               0,               0,               0,       33.132286,      -49.022491 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.013049521,    -0.024257513,    -0.038051683,     0.071784616 ],
  [               0,               0,      -60.247498,       340.34598,      -621.52014,       371.62057 ],
  [               0,               0,       29.272589,      -148.12183,       249.36235,      -140.65494 ],
  [               0,               0,      -37.568867,       177.68118,      -278.08273,       143.05463 ],
  [               0,               0,       7.8733573,      -38.686798,       63.548717,      -34.826809 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0064981147,    -0.010761874 ],
  [               0,               0,               0,               0,      -5.6059933,       13.525821 ],
  [               0,               0,               0,               0,     -0.73364377,      0.39052784 ],
  [               0,               0,               0,               0,      -2.6009789,       3.9775891 ],
  [               0,               0,               0,               0,     -0.53853995,       1.0590872 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   0.00049449055,   -0.0041514821,   -0.0054412284,     0.030704869 ],
  [               0,               0,       4.5738645,      -20.314926,       27.962526,      -9.7011852 ],
  [               0,               0,       20.572466,      -128.67471,        267.5108,      -185.93536 ],
  [               0,               0,      -16.745333,       108.82656,      -235.88161,       169.43359 ],
  [               0,               0,       9.6582375,      -62.654331,       135.42055,      -97.492546 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.33456287,      0.77660841 ],
  [               0,               0,               0,               0,      -15.583259,       37.211433 ],
  [               0,               0,               0,               0,      -18.322027,       40.992771 ],
  [               0,               0,               0,               0,       19.704588,      -45.113731 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00069223362,    0.0018424658 ],
  [               0,               0,               0,               0,       2.0339794,      -2.2800896 ],
  [               0,               0,               0,               0,     -0.13333195,     -0.86954141 ],
  [               0,               0,               0,               0,      -1.2584383,       1.7572374 ],
  [               0,               0,               0,               0,     0.076921418,     -0.13963813 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,    0.0028618148 ],
  [               0,               0,               0,               0,               0,        1.862132 ],
  [               0,               0,               0,               0,               0,      -1.1270396 ],
  [               0,               0,               0,               0,               0,      -1.4760505 ],
  [               0,               0,               0,               0,               0,     -0.43677524 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.016413612,     0.082145996,    -0.094130985 ],
  [               0,               0,               0,      -606.89734,       2942.0859,       -3558.832 ],
  [               0,               0,               0,      -253.83212,        1232.368,      -1497.3463 ],
  [               0,               0,               0,      -62.872375,        765.1189,      -1467.4163 ],
  [               0,               0,               0,     -0.81488544,     0.021709958,       4.5835209 ],
  [               0,               0,               0,      -11.264228,       57.129295,      -72.328575 ],
  [               0,               0,               0,      -3.9019055,       18.434555,      -21.734314 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.008460681,    0.0066163656 ],
  [               0,               0,               0,               0,     -0.95406991,       8.7971592 ],
  [               0,               0,               0,               0,               0,     -0.40000001 ],
  [               0,               0,               0,               0,       -37.78981,       92.687515 ],
  [               0,               0,               0,               0,      0.69200999,      -1.1556425 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele100_layer1_etaoffv3_37_b = [
 [
  [               0,               0,       21259.445,      -64.625786,      -0.6259445,     0.001021922 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00015670431,   0.00062219986 ],
  [               0,               0,               0,               0,     -0.35875539,       4.2567374 ],
  [               0,               0,               0,               0,     -0.13302187,     -0.78411233 ],
  [               0,               0,               0,               0,     -0.00897742,     -0.83088627 ],
  [               0,               0,               0,               0,     -0.15188185,    -0.042274623 ],
  [               0,               0,               0,               0,  -0.00011259744,   1.8289583e-05 ],
  [               0,               0,               0,               0,       3.5717391,      0.61162411 ],
  [               0,               0,               0,               0,       1.0331639,     -0.29778313 ],
 ],
 [
  [               0,               0,               0,    0.0042620638,   -0.0051334729,    0.0023405648 ],
  [               0,               0,               0,      -19.096807,       21.786703,      -2.4949111 ],
  [               0,               0,               0,      -2.6373425,       3.5674969,      -1.9878892 ],
  [               0,               0,               0,       2.5730084,      -2.2687691,     -0.57420115 ],
  [               0,               0,               0,      0.85918211,      -1.2345483,      0.27247116 ],
  [               0,               0,               0,  -0.00040871266,   0.00039221202,  -0.00010904253 ],
  [               0,               0,               0,      -2.6900639,       0.9956202,       2.5746864 ],
  [               0,               0,               0,       1.1961922,     -0.91769151,      0.18769785 ],
 ],
 [
  [               0,               0,               0,               0,     0.024388285,    0.0099486699 ],
  [               0,               0,               0,               0,      -5.2931151,       4.7414999 ],
  [               0,               0,               0,               0,      -14.744508,       11.234737 ],
  [               0,               0,               0,               0,       15.019686,      -11.944862 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0018275476,    0.0001976572,    0.0067092845 ],
  [               0,               0,               0,       1.0812121,      -1.7902116,       1.9927978 ],
  [               0,               0,               0,     -0.50835628,       0.9913465,      -1.3544201 ],
  [               0,               0,               0,     -0.34194244,      0.52637973,      -1.1590252 ],
  [               0,               0,               0,    -0.052532613,      0.07818668,    -0.072712219 ],
  [               0,               0,               0,  -4.8826009e-05,   1.3130176e-05,   5.1839658e-05 ],
  [               0,               0,               0,     -0.71941123,      0.42018143,       3.4341463 ],
  [               0,               0,               0,      -7.1354346,       16.133845,      -4.3657982 ],
 ],
 [
  [               0,               0,               0,               0,               0,    0.0016557554 ],
  [               0,               0,               0,               0,               0,       1.8440925 ],
  [               0,               0,               0,               0,               0,     -0.98058939 ],
  [               0,               0,               0,               0,               0,     -0.85907978 ],
  [               0,               0,               0,               0,               0,     -0.10460033 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,       -9.614397,       -4.472558,       6.3195562,       26.459299 ],
  [               0,               0,       64.779388,       34.577698,       -33.68996,      -158.36098 ],
  [               0,               0,    -0.011398018,      -3.7649834,      -2.5208671,       10.967765 ],
  [               0,               0,               0,               0,               0,              -1 ],
  [               0,               0,     -0.12525155,     -0.15672164,      -0.2074185,     -0.65014899 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    -0.022732779,   -0.0057172342,     0.017549152,     0.060684685 ],
  [               0,               0,      -191.20055,       285.43225,       400.87442,      -591.70593 ],
  [               0,               0,      -287.77786,       515.63568,       120.85187,      -386.45474 ],
  [               0,               0,       375.61108,      -474.37698,      -1091.1094,       1428.3208 ],
  [               0,               0,       2516.2844,      -3574.7336,      -5136.4824,       7295.6006 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele100_layer1_etaoffv3_37_e = [
 [
  [               0,               0,               0,               0,               0,      -162030.48 ],
  [               0,               0,               0,               0,               0,    0.0049069449 ],
  [               0,               0,               0,               0,               0,   -0.0049068979 ],
  [               0,               0,               0,               0,               0,  -6.8984414e-09 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.086829297,     -0.12237787 ],
  [               0,               0,               0,               0,       590.32184,      -858.66711 ],
  [               0,               0,               0,               0,      -40.768478,       58.837498 ],
  [               0,               0,               0,               0,      -136.67561,       199.88979 ],
  [               0,               0,               0,               0,      0.18385664,     -0.34963584 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.10252132,      0.16057125 ],
  [               0,               0,               0,               0,      -213.56931,       322.09088 ],
  [               0,               0,               0,               0,       18.947506,       -29.57379 ],
  [               0,               0,               0,               0,       9.4063978,      -13.721263 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,   0.00065994973 ],
  [               0,               0,               0,               0,               0,      -39.977097 ],
  [               0,               0,               0,               0,               0,       11.189283 ],
  [               0,               0,               0,               0,               0,     -0.15665863 ],
  [               0,               0,               0,               0,               0,       1.1041028 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0026777585,     -0.00477533,   -0.0097023239,     0.018941568 ],
  [               0,               0,      -39.394455,        202.1069,      -341.81793,       192.77956 ],
  [               0,               0,      -28.348125,        138.8576,       -226.7533,       122.37915 ],
  [               0,               0,       6.0676479,      -28.840775,       44.802303,      -23.716232 ],
  [               0,               0,       -9.141202,         43.9646,      -70.499336,       37.764709 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0056359293,   -0.0086786477 ],
  [               0,               0,               0,               0,      -4.2108307,       10.581224 ],
  [               0,               0,               0,               0,     -0.47308928,     -0.23305216 ],
  [               0,               0,               0,               0,     -0.19561519,     -0.70879072 ],
  [               0,               0,               0,               0,     -0.12448812,      0.28006825 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0094721811,    -0.023552816,    -0.046128359,      0.11712329 ],
  [               0,               0,        5.612606,      -17.365623,       4.3138518,       17.713615 ],
  [               0,               0,       6.6088548,      -35.418179,       60.810818,      -33.878574 ],
  [               0,               0,      -5.2370396,       35.060566,      -77.771202,       56.097397 ],
  [               0,               0,       5.8449173,      -39.271004,       87.877632,      -65.443977 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00029558197,    0.0029912381 ],
  [               0,               0,               0,               0,      -168.03995,       388.68936 ],
  [               0,               0,               0,               0,       42.080868,      -98.247368 ],
  [               0,               0,               0,               0,       3.0978684,      -6.7368417 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00038262247,    0.0019034242 ],
  [               0,               0,               0,               0,      -5.2668433,       15.197485 ],
  [               0,               0,               0,               0,      0.79360253,      -3.0540318 ],
  [               0,               0,               0,               0,       2.5816028,      -7.2000895 ],
  [               0,               0,               0,               0,       0.2102931,      -0.4337956 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.068007283,    -0.051276419 ],
  [               0,               0,               0,               0,      -13.101539,       31.988523 ],
  [               0,               0,               0,               0,       30.832327,      -74.495247 ],
  [               0,               0,               0,               0,       3.6629286,      -9.3672018 ],
  [               0,               0,               0,               0,     0.056450494,     -0.14755514 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.007485555,     0.009740063,     0.028124012 ],
  [               0,               0,               0,      -2.0120513,       7.4598498,      0.89727437 ],
  [               0,               0,               0,      -91.086349,        427.8161,      -503.46634 ],
  [               0,               0,               0,       -32.64642,       169.74728,      -219.53482 ],
  [               0,               0,               0,    0.0091928206,   -0.0056284396,    -0.039714314 ],
  [               0,               0,               0,      -2.0018559,       9.9572926,      -11.395513 ],
  [               0,               0,               0,       -6.979033,       17.208992,      -2.8095491 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.050207365,    -0.096523069 ],
  [               0,               0,               0,               0,    -0.082472079,       9.7201719 ],
  [               0,               0,               0,               0,               0,     -0.40000001 ],
  [               0,               0,               0,               0,       -27.54056,       67.512161 ],
  [               0,               0,               0,               0,      0.14436831,      0.42974067 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele200_layer1_etaoffv3_37_b = [
 [
  [               0,               0,       37506.484,      -206.73219,     -0.22184464,   0.00046217392 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00054094347,   0.00050742987 ],
  [               0,               0,               0,               0,       -1.030571,       4.6392867 ],
  [               0,               0,               0,               0,     -0.64158399,     -0.58378429 ],
  [               0,               0,               0,               0,      -1.0800122,     -0.62145259 ],
  [               0,               0,               0,               0,     -0.17050231,    -0.061644195 ],
  [               0,               0,               0,               0,  -7.3808681e-05,   4.4008958e-05 ],
  [               0,               0,               0,               0,       1.8647972,       1.7261543 ],
  [               0,               0,               0,               0,      -3.4469335,      -1.0291552 ],
 ],
 [
  [               0,               0,               0,    0.0080470317,    -0.011867249,    0.0047159638 ],
  [               0,               0,               0,      -50.377681,       49.770517,      -8.3365069 ],
  [               0,               0,               0,       33.472203,      -30.386596,       6.0133775 ],
  [               0,               0,               0,       30.934254,        -25.3266,       3.7726488 ],
  [               0,               0,               0,      -2.9050852,       2.6462655,     -0.75856388 ],
  [               0,               0,               0,    0.0029664194,   -0.0045110998,    0.0015115214 ],
  [               0,               0,               0,      -4.4708173,       6.8810999,     -0.80076461 ],
  [               0,               0,               0,      -5.7338128,       7.6947541,      -7.5274255 ],
 ],
 [
  [               0,               0,               0,               0,     0.022256367,    0.0054606339 ],
  [               0,               0,               0,               0,       8.3250885,      -6.0424137 ],
  [               0,               0,               0,               0,      -26.017214,       20.160507 ],
  [               0,               0,               0,               0,       17.644098,      -14.032248 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.001127556,   3.3827325e-06,    0.0031491143 ],
  [               0,               0,               0,      -3.0859208,       7.2362542,      -2.2212474 ],
  [               0,               0,               0,       1.7058579,       -3.201493,      0.62481375 ],
  [               0,               0,               0,       3.6018246,      -7.4245196,       2.7178193 ],
  [               0,               0,               0,     -0.06003799,   -0.0064753372,    -0.047500828 ],
  [               0,               0,               0,    0.0011500634,   -0.0023688787,    0.0012588224 ],
  [               0,               0,               0,      0.59868595,      -3.0338838,       4.4702057 ],
  [               0,               0,               0,     -0.36248645,       1.5842598,      -3.4295541 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.96918535,      -1.3652381,      -1.9837464,       2.8009536 ],
  [               0,               0,      -4403.4844,       6280.8511,         8905.25,      -12692.363 ],
  [               0,               0,       912.35364,       -1257.613,      -2203.5674,       3053.9463 ],
  [               0,               0,       391.22641,      -520.86786,      -1041.2961,       1406.3571 ],
  [               0,               0,       2349.4141,      -3333.3586,      -4794.5039,       6800.8794 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele200_layer1_etaoffv3_37_e = [
 [
  [               0,               0,               0,               0,               0,     0.004786938 ],
  [               0,               0,               0,               0,               0,   7.2837458e-05 ],
  [               0,               0,               0,               0,               0,     -0.51832551 ],
  [               0,               0,               0,               0,               0,      0.62621719 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00080572063,     0.026685104 ],
  [               0,               0,               0,               0,      -206.79376,       305.55765 ],
  [               0,               0,               0,               0,       111.84186,      -165.46239 ],
  [               0,               0,               0,               0,      -151.90057,       219.19554 ],
  [               0,               0,               0,               0,     0.070968352,     -0.10294254 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.15059955,      0.22904401 ],
  [               0,               0,               0,               0,      -391.01227,       588.66779 ],
  [               0,               0,               0,               0,      -66.491051,       97.383308 ],
  [               0,               0,               0,               0,       97.065292,      -143.92365 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,  -0.00035136039,  -0.00040076702,   0.00020763651,    0.0036027553 ],
  [               0,               0,        5.178256,      -7.1996164,      -12.612721,       20.083366 ],
  [               0,               0,       2.9351761,      -4.0938945,      -9.1155672,       11.890747 ],
  [               0,               0,       3.2350886,       -5.181179,      -8.0795364,       11.847123 ],
  [               0,               0,      0.37067595,     -0.40814662,      -1.4768718,       1.9950103 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0080131823,     -0.01217355 ],
  [               0,               0,               0,               0,      -2.8109767,       7.4874282 ],
  [               0,               0,               0,               0,    -0.019122275,      -1.0952446 ],
  [               0,               0,               0,               0,     -0.27051333,     -0.61125964 ],
  [               0,               0,               0,               0,     -0.32038474,      0.63916785 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,  -0.00028100552,  -0.00031441153,   0.00061258097,    0.0059943837 ],
  [               0,               0,         3.13185,      -6.2385154,      -14.608103,       31.772827 ],
  [               0,               0,     -0.17321631,      0.20069714,       1.5132663,      -3.6386845 ],
  [               0,               0,       -2.328836,       5.3172832,       10.150298,       -24.31233 ],
  [               0,               0,     -0.27775013,        0.496571,       1.8854913,      -3.5310254 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0065118233,     0.014439584 ],
  [               0,               0,               0,               0,      -26.973259,       62.780182 ],
  [               0,               0,               0,               0,       17.503408,      -40.937359 ],
  [               0,               0,               0,               0,      0.86914575,      -1.9466347 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.031074336,     0.076900266 ],
  [               0,               0,               0,               0,       10.386408,      -21.915089 ],
  [               0,               0,               0,               0,       2.3429656,      -6.6938267 ],
  [               0,               0,               0,               0,     -0.60479885,      0.25148356 ],
  [               0,               0,               0,               0,      0.14327493,     -0.28928858 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,     0.003944383 ],
  [               0,               0,               0,               0,               0,       1.3037038 ],
  [               0,               0,               0,               0,               0,     -0.81805187 ],
  [               0,               0,               0,               0,               0,      -1.2248108 ],
  [               0,               0,               0,               0,               0,     -0.37191066 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,     0.009056001 ],
  [               0,               0,               0,               0,               0,       6.5443668 ],
  [               0,               0,               0,               0,               0,      -1.5068249 ],
  [               0,               0,               0,               0,               0,     0.061640773 ],
  [               0,               0,               0,               0,               0,  -0.00010120284 ],
  [               0,               0,               0,               0,               0,        5.228097 ],
  [               0,               0,               0,               0,               0,      -3.3886659 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0073348237,     0.052006889 ],
  [               0,               0,               0,               0,      0.14069965,        8.126421 ],
  [               0,               0,               0,               0,               0,     -0.40000001 ],
  [               0,               0,               0,               0,      -57.406208,       141.39787 ],
  [               0,               0,               0,               0,     0.066504031,       0.5137201 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele500_layer1_etaoffv3_37_b = [
 [
  [               0,               0,       42827.184,      -295.62622,      0.12172319,   0.00045149901 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,  -0.00022567676,    0.0010021494 ],
  [               0,               0,               0,               0,      0.03996017,       3.2622764 ],
  [               0,               0,               0,               0,      0.62862689,      -1.0908956 ],
  [               0,               0,               0,               0,     0.083513559,      -1.0734913 ],
  [               0,               0,               0,               0,     -0.25297648,    -0.013598441 ],
  [               0,               0,               0,               0,   1.9604872e-05,   3.8903155e-05 ],
  [               0,               0,               0,               0,      -1.1039882,       2.8128039 ],
  [               0,               0,               0,               0,     -0.40324917,      -1.6730276 ],
 ],
 [
  [               0,               0,               0,   -0.0058785897,    0.0024299136,  -0.00058722043 ],
  [               0,               0,               0,     -0.51269226,       8.4575685,      -7.6571639 ],
  [               0,               0,               0,      -8.9676308,        11.02503,      -2.4609519 ],
  [               0,               0,               0,      -12.701373,       15.272093,      -3.4374568 ],
  [               0,               0,               0,      0.10474279,      -0.3052905,      0.25843764 ],
  [               0,               0,               0,    0.0014557174,   -0.0016478863,   0.00051329227 ],
  [               0,               0,               0,      -13.146639,       16.463788,       -2.679665 ],
  [               0,               0,               0,       4.6407391,      -5.5197003,      -0.3735774 ],
 ],
 [
  [               0,               0,               0,               0,    0.0058622709,   -0.0018617706 ],
  [               0,               0,               0,               0,       13.304334,      -8.9936714 ],
  [               0,               0,               0,               0,      -195.73642,        154.6255 ],
  [               0,               0,               0,               0,       150.04523,      -119.33477 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00017427093,    0.0015961463 ],
  [               0,               0,               0,               0,     -0.60962561,       2.8510375 ],
  [               0,               0,               0,               0,      0.14835058,      -1.0659709 ],
  [               0,               0,               0,               0,      0.32153619,      -1.4955467 ],
  [               0,               0,               0,               0,     0.030561681,      -0.1660203 ],
  [               0,               0,               0,               0,   4.3988971e-05,   4.9827781e-06 ],
  [               0,               0,               0,               0,         -1.1039,       3.2285677 ],
  [               0,               0,               0,               0,     -0.12827156,       3.9571924 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.41465649,     -0.64265287,     -0.90515286,       1.4041142 ],
  [               0,               0,      -2946.0122,       4327.6187,       5674.3374,      -8341.2393 ],
  [               0,               0,       -998.8009,       1556.7629,       1468.6958,      -2363.2639 ],
  [               0,               0,         521.401,      -664.09344,      -1526.1245,       2011.0453 ],
  [               0,               0,       3570.9873,      -5075.5884,      -7292.1377,       10362.596 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele500_layer1_etaoffv3_37_e = [
 [
  [               0,               0,               0,               0,     0.056599788,    -0.080200724 ],
  [               0,               0,               0,               0,      -39.260284,      -108.50809 ],
  [               0,               0,               0,               0,      -22.824799,       35.079659 ],
  [               0,               0,               0,               0,      -172.10764,       249.33026 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.060474899,     0.089953862 ],
  [               0,               0,               0,               0,      -75.229454,       120.82205 ],
  [               0,               0,               0,               0,      -1122.3031,       1653.1696 ],
  [               0,               0,               0,               0,       983.04938,      -1428.8604 ],
  [               0,               0,               0,               0,       0.4600623,     -0.14426585 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.34948555,       0.5271669 ],
  [               0,               0,               0,               0,      -291.47662,       439.80838 ],
  [               0,               0,               0,               0,       1.1324997,      -3.0911324 ],
  [               0,               0,               0,               0,        31.25194,       -46.12701 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0036118471,   -0.0035744696,     -0.01029645,     0.013036414 ],
  [               0,               0,      -26.873417,       144.95251,      -262.18509,       160.71191 ],
  [               0,               0,      -22.731586,       108.35546,      -170.62718,       87.477753 ],
  [               0,               0,       3.1206095,      -16.556065,       29.353296,      -18.470873 ],
  [               0,               0,      -5.0589323,       25.615664,      -43.493282,       24.814524 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0070566009,    -0.011257233 ],
  [               0,               0,               0,               0,      -5.1647172,       12.653595 ],
  [               0,               0,               0,               0,    -0.045281768,      -1.0886424 ],
  [               0,               0,               0,               0,     -0.80857641,      0.39217266 ],
  [               0,               0,               0,               0,     -0.49995053,       1.0094776 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0016241058,   -0.0034588238,   -0.0079735424,     0.020158829 ],
  [               0,               0,       -8.961051,       53.083324,      -104.47955,       71.153175 ],
  [               0,               0,       19.119431,      -121.52382,       257.49716,      -183.12325 ],
  [               0,               0,       -27.31389,         175.097,      -373.60406,       264.16235 ],
  [               0,               0,       12.238213,      -80.094437,       174.51402,      -126.54019 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0039627911,     0.011346877 ],
  [               0,               0,               0,               0,       -407.2937,        940.2641 ],
  [               0,               0,               0,               0,       88.026711,      -204.60851 ],
  [               0,               0,               0,               0,       11.510143,      -25.628761 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.017690904,    -0.037736632 ],
  [               0,               0,               0,               0,      -8.4826431,       22.565834 ],
  [               0,               0,               0,               0,      0.18602115,      -1.6782897 ],
  [               0,               0,               0,               0,       1.7329855,      -5.2676344 ],
  [               0,               0,               0,               0,   -0.0032617883,     0.064412184 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.004929848,     0.011849944 ],
  [               0,               0,               0,               0,      -9.2344589,      -17.491404 ],
  [               0,               0,               0,               0,      -5200.6641,       12498.379 ],
  [               0,               0,               0,               0,       1434.6285,      -3446.6494 ],
  [               0,               0,               0,               0,      -7.6413975,      -19.096498 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0095765637,     0.008645298,     0.042905204 ],
  [               0,               0,               0,      -3.5327764,       27.763226,      -37.716312 ],
  [               0,               0,               0,      -124.21214,       579.96985,      -677.76617 ],
  [               0,               0,               0,      -18.507683,       104.60011,      -144.64214 ],
  [               0,               0,               0,     0.011178054,   -0.0013338653,      -0.0616442 ],
  [               0,               0,               0,      -2.6928864,       15.296572,      -20.263002 ],
  [               0,               0,               0,      -4.5407453,       8.5631323,       3.7395437 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.036442816,   -0.0009948574 ],
  [               0,               0,               0,               0,       5.1018138,       27.076166 ],
  [               0,               0,               0,               0,               0,     -0.40000001 ],
  [               0,               0,               0,               0,      -7.5497913,       18.482059 ],
  [               0,               0,               0,               0,       1.2801821,      -1.8372844 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele1000_layer1_etaoffv3_37_b = [
 [
  [               0,               0,       12312.123,       14.392639,     -0.71127129,   0.00050897902 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,  -8.5663396e-05,   0.00092212835 ],
  [               0,               0,               0,               0,      0.31415937,       3.6953231 ],
  [               0,               0,               0,               0,      0.45925112,       -1.107013 ],
  [               0,               0,               0,               0,     -0.18061667,      -1.0662699 ],
  [               0,               0,               0,               0,     -0.27874802,    -0.007966734 ],
  [               0,               0,               0,               0,   2.3302424e-06,    3.428433e-05 ],
  [               0,               0,               0,               0,     -0.87273147,       2.7568909 ],
  [               0,               0,               0,               0,      -1.6105761,      -1.5338026 ],
 ],
 [
  [               0,               0,               0,               0,  -0.00092962077,  -0.00061099253 ],
  [               0,               0,               0,               0,       1.3889318,      -3.8696683 ],
  [               0,               0,               0,               0,      0.15961728,      0.83536805 ],
  [               0,               0,               0,               0,      0.26333858,       1.0219338 ],
  [               0,               0,               0,               0,      0.14581522,     0.044838515 ],
  [               0,               0,               0,               0,  -6.1234148e-05,   8.6241489e-05 ],
  [               0,               0,               0,               0,      0.21149743,        2.256207 ],
  [               0,               0,               0,               0,      -2.3314897,     -0.78474644 ],
 ],
 [
  [               0,               0,               0,               0,      0.54112631,     -0.42920542 ],
  [               0,               0,               0,               0,      -751.36316,       601.38806 ],
  [               0,               0,               0,               0,       65.522575,      -53.805519 ],
  [               0,               0,               0,               0,       150.93961,      -120.04003 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0022523974,    0.0046938601,  -0.00061521107 ],
  [               0,               0,               0,      0.29248957,     -0.98877834,       2.9786564 ],
  [               0,               0,               0,     -0.35155694,       0.7820525,      -1.3422604 ],
  [               0,               0,               0,       1.4812661,      -2.8613579,      0.15452582 ],
  [               0,               0,               0,     0.046558284,     -0.06726756,     -0.11988201 ],
  [               0,               0,               0,   0.00060124147,   -0.0012399194,   0.00068722799 ],
  [               0,               0,               0,     -0.42519284,     -0.46613705,       3.0736251 ],
  [               0,               0,               0,     -0.15428769,     -0.40994089,      -1.7313569 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.51433074,     -0.79049444,      -1.1019925,        1.696907 ],
  [               0,               0,       835.68683,      -1168.9293,      -1676.4749,       2350.5259 ],
  [               0,               0,      -863.75464,       1342.0352,       1250.6963,      -2007.3987 ],
  [               0,               0,       75.019386,       28.616268,      -783.62311,       838.10559 ],
  [               0,               0,       3448.6553,      -4901.2661,      -7046.9087,       10013.196 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele1000_layer1_etaoffv3_37_e = [
 [
  [               0,               0,               0,               0,     0.071746498,      -0.1020021 ],
  [               0,               0,               0,               0,  -1.8420529e+09,  -1.4584941e+09 ],
  [               0,               0,               0,               0,      -185.06882,       268.35275 ],
  [               0,               0,               0,               0,      -87.247139,       127.74439 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00060890953,    0.0095308879 ],
  [               0,               0,               0,               0,       4.9740224,      -2.6342604 ],
  [               0,               0,               0,               0,       30.787886,      -46.814285 ],
  [               0,               0,               0,               0,       -65.75367,       94.433846 ],
  [               0,               0,               0,               0,      0.12855487,    -0.087105006 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.33490461,      0.50607777 ],
  [               0,               0,               0,               0,      -349.00934,       525.65076 ],
  [               0,               0,               0,               0,       5.0128031,      -8.8933878 ],
  [               0,               0,               0,               0,       22.455097,      -33.061367 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    -0.053174779,      0.21613006,      -0.2788291,       0.1138602 ],
  [               0,               0,       10.371162,      -30.183598,       15.854593,       11.275277 ],
  [               0,               0,     -0.71970552,      0.33296418,       4.4685011,      -6.2405663 ],
  [               0,               0,       5.9818969,      -33.749016,         61.8974,      -38.225288 ],
  [               0,               0,      -4.9273028,         24.7076,      -41.348213,       23.148565 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0081912288,    -0.012750562 ],
  [               0,               0,               0,               0,      -5.3840475,       12.763103 ],
  [               0,               0,               0,               0,      0.32753301,      -1.8790296 ],
  [               0,               0,               0,               0,      0.67045897,      -2.4431727 ],
  [               0,               0,               0,               0,     -0.25506437,      0.54187578 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,  -0.00017176165,  -3.4695524e-05,   0.00065083272,    0.0034725873 ],
  [               0,               0,    -0.083157748,      0.33114362,      0.16536801,       1.9854776 ],
  [               0,               0,      0.91131657,      -1.9293697,      -4.0064578,       7.2027173 ],
  [               0,               0,    -0.048364367,       0.2640053,     -0.16432087,      -1.5204476 ],
  [               0,               0,     -0.32999194,      0.65597367,       1.7998697,      -3.5440221 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.16746274,      0.38850108 ],
  [               0,               0,               0,               0,      -193.66312,       447.73245 ],
  [               0,               0,               0,               0,       22.215021,      -52.762283 ],
  [               0,               0,               0,               0,       35.310684,      -80.637375 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.011249931,    -0.023172425 ],
  [               0,               0,               0,               0,      -12.314932,       32.067375 ],
  [               0,               0,               0,               0,       1.9458745,      -5.8860655 ],
  [               0,               0,               0,               0,       1.7573894,      -5.3640895 ],
  [               0,               0,               0,               0,     -0.18655004,      0.50896335 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00025743345,   0.00096616521 ],
  [               0,               0,               0,               0,      -1.0313067,       5.3739433 ],
  [               0,               0,               0,               0,      -0.1020502,     -0.62105596 ],
  [               0,               0,               0,               0,     -0.23703749,      -1.5127069 ],
  [               0,               0,               0,               0,     -0.11702042,     -0.57846344 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.010928608,     0.051765207,    -0.052574057 ],
  [               0,               0,               0,      -17.035294,       77.832855,      -80.265068 ],
  [               0,               0,               0,       125.06193,      -614.00751,       751.93811 ],
  [               0,               0,               0,      -134.70491,       660.08643,      -808.46936 ],
  [               0,               0,               0,   -0.0022153105,  -0.00033471442,     0.013679534 ],
  [               0,               0,               0,       36.051689,      -95.203964,       13.214182 ],
  [               0,               0,               0,      -11.614936,      0.31666628,       65.294617 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.14767291,      0.40474802 ],
  [               0,               0,               0,               0,      -3.1102777,       22.267883 ],
  [               0,               0,               0,               0,               0,     -0.40000001 ],
  [               0,               0,               0,               0,      -26.116096,       64.132462 ],
  [               0,               0,               0,               0,     -0.13203025,       1.2966125 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]


ele_layer1_etaoffv3_37_b = [
    ele5_layer1_etaoffv3_37_b,
    ele10_layer1_etaoffv3_37_b,
    ele25_layer1_etaoffv3_37_b,
    ele50_layer1_etaoffv3_37_b,
    ele75_layer1_etaoffv3_37_b,
    ele100_layer1_etaoffv3_37_b,
    ele200_layer1_etaoffv3_37_b,
    ele500_layer1_etaoffv3_37_b,
    ele1000_layer1_etaoffv3_37_b,
    ]

ele_layer1_etaoffv3_37_e = [
    ele5_layer1_etaoffv3_37_e,
    ele10_layer1_etaoffv3_37_e,
    ele25_layer1_etaoffv3_37_e,
    ele50_layer1_etaoffv3_37_e,
    ele75_layer1_etaoffv3_37_e,
    ele100_layer1_etaoffv3_37_e,
    ele200_layer1_etaoffv3_37_e,
    ele500_layer1_etaoffv3_37_e,
    ele1000_layer1_etaoffv3_37_e,
    ]



###############################################################################
# Tables for v3 sshape correction, layer 1, 3x7 photon clusters
#

gam5_layer1_etaoffv3_37_b = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.019566501,    0.0056323111 ],
  [               0,               0,               0,               0,      -4.4975562,       2.9028616 ],
  [               0,               0,               0,               0,       2.7188234,      -1.6853194 ],
  [               0,               0,               0,               0,       1.9949682,       -1.839355 ],
  [               0,               0,               0,               0,     -0.17463715,    -0.028333571 ],
  [               0,               0,               0,               0,    0.0006240342,   0.00095373741 ],
  [               0,               0,               0,               0,      -2.3228507,       2.0519505 ],
  [               0,               0,               0,               0,      -2.4069431,       1.5174382 ],
 ],
 [
  [               0,               0,               0,   -0.0062272679,    0.0065215202,   -0.0010784442 ],
  [               0,               0,               0,       24.951969,       5.7864323,      -6.7244515 ],
  [               0,               0,               0,        2.559495,      -41.563965,       26.672422 ],
  [               0,               0,               0,       301.44019,      -348.87494,       96.440651 ],
  [               0,               0,               0,      -1.5628783,       18.772331,      -12.683232 ],
  [               0,               0,               0,      0.01560323,    -0.028983725,     0.012642517 ],
  [               0,               0,               0,       5.2939873,      -1.5049545,      0.25250387 ],
  [               0,               0,               0,       10.127971,      -14.208724,       4.8130894 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
    [0, 0, 0, 0, 0, 0], # Removed by hand, see below
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam5_layer1_etaoffv3_37_e = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam10_layer1_etaoffv3_37_b = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0058897212,     0.002959328 ],
  [               0,               0,               0,               0,        3.755049,       1.0882094 ],
  [               0,               0,               0,               0,      -13.520554,     -0.96715933 ],
  [               0,               0,               0,               0,      -13.450398,     -0.80141699 ],
  [               0,               0,               0,               0,       7.9584355,      0.19215992 ],
  [               0,               0,               0,               0,    -0.047019925,   -0.0011136037 ],
  [               0,               0,               0,               0,     -0.46169072,      0.24231575 ],
  [               0,               0,               0,               0,       1.0434574,       1.0485588 ],
 ],
 [
  [               0,               0,               0,      0.04238344,    -0.034623805,    0.0077795545 ],
  [               0,               0,               0,       30.675367,      -42.415924,       15.464958 ],
  [               0,               0,               0,      -1.0529828,      0.84889686,     -0.81785411 ],
  [               0,               0,               0,       -2.087307,       2.3652139,      -1.3752562 ],
  [               0,               0,               0,    -0.088206016,      0.52530771,     -0.35379332 ],
  [               0,               0,               0,   -0.0063027851,    0.0049626958,  -0.00092337938 ],
  [               0,               0,               0,      -2.9678276,       -3.668659,       4.6647716 ],
  [               0,               0,               0,      -20.041588,       26.923223,      -7.5539303 ],
 ],
 [
  [               0,               0,               0,               0,               0,        2.635736 ],
  [               0,               0,               0,               0,               0,      0.11252269 ],
  [               0,               0,               0,               0,               0,     -0.11207007 ],
  [               0,               0,               0,               0,               0,   0.00013111871 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0019679498,    0.0011019355,    0.0066143237 ],
  [               0,               0,               0,       0.8719421,      0.99142843,      -1.1844245 ],
  [               0,               0,               0,    -0.015519082,      -2.0424924,       1.6104473 ],
  [               0,               0,               0,      0.98063856,      -4.2428198,       2.7149076 ],
  [               0,               0,               0,      0.38475898,     -0.81846917,      0.35801658 ],
  [               0,               0,               0,    0.0015714072,   -0.0027976248,    0.0010922642 ],
  [               0,               0,               0,       15.743665,      -33.671429,       19.161043 ],
  [               0,               0,               0,      -13.257767,       29.416231,      -13.987031 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,      0.11863397,    -0.011964181,     -0.21389046 ],
  [               0,               0,               0,        495.9967,      -1500.8831,       1136.0526 ],
  [               0,               0,               0,       1984.4369,      -5719.6421,       4119.8428 ],
  [               0,               0,               0,      -1612.2043,         4519.04,      -3167.5735 ],
  [               0,               0,               0,       229.82574,       -634.4118,       436.88644 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam10_layer1_etaoffv3_37_e = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0056755487,    0.0062053907,    0.0055765188,    0.0028574502 ],
  [               0,               0,       2.1373386,      -7.0135026,       5.9045749,      0.17157502 ],
  [               0,               0,       1.4032658,      -10.535279,       23.084305,      -16.106766 ],
  [               0,               0,      -2.4594817,        8.996541,      -9.9073515,       2.5081122 ],
  [               0,               0,     -0.59147263,       3.0072682,      -5.1080213,       2.8993943 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.16412666,      0.33936667 ],
  [               0,               0,               0,               0,       -3.652369,       7.2933707 ],
  [               0,               0,               0,               0,       3.7348552,      -7.3885722 ],
  [               0,               0,               0,               0,       3.5506938,      -7.0541744 ],
  [               0,               0,               0,               0,     0.094370991,     -0.16903435 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam25_layer1_etaoffv3_37_b = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,  -0.00012323668,   0.00021457471 ],
  [               0,               0,               0,               0,       3.5581937,       6.5090995 ],
  [               0,               0,               0,               0,      -115.37304,      -10.077041 ],
  [               0,               0,               0,               0,      -119.98327,      -10.131657 ],
  [               0,               0,               0,               0,      -2.7399476,     -0.24883433 ],
  [               0,               0,               0,               0,    -0.025545565,   -0.0026057474 ],
  [               0,               0,               0,               0,     -0.18290798,      0.32801211 ],
  [               0,               0,               0,               0,    -0.011566305,       1.5829817 ],
 ],
 [
  [               0,               0,               0,   0.00052740722,    0.0019761357,  -0.00022989871 ],
  [               0,               0,               0,       5.1495705,      -7.5490379,       6.0709057 ],
  [               0,               0,               0,      0.47915557,      -1.6284747,     -0.32211143 ],
  [               0,               0,               0,       4.4924059,      -7.5120463,       1.5098372 ],
  [               0,               0,               0,     -0.20307353,      0.12443565,     -0.13093813 ],
  [               0,               0,               0,    -0.001306998,    0.0012293248,  -0.00033464123 ],
  [               0,               0,               0,     -0.57178926,      0.67028409,       1.9195009 ],
  [               0,               0,               0,       18.111593,        -26.4596,       7.9077368 ],
 ],
 [
  [               0,               0,               0,               0,     0.023750424,  -0.00043652189 ],
  [               0,               0,               0,               0,       -4.799839,       4.3830438 ],
  [               0,               0,               0,               0,      -18.736006,       14.381065 ],
  [               0,               0,               0,               0,       20.738419,      -16.494518 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0027738183,   -0.0013940054,   0.00078817894 ],
  [               0,               0,               0,       2.4739711,      -6.6882172,       5.9908123 ],
  [               0,               0,               0,      -1.4091407,       3.0574427,      -2.5358644 ],
  [               0,               0,               0,      0.41516042,     -0.80324936,     -0.68143183 ],
  [               0,               0,               0,    -0.030712022,      0.21797945,     -0.30211657 ],
  [               0,               0,               0,   5.7812598e-05,  -0.00011780432,   9.0237685e-05 ],
  [               0,               0,               0,       6.5862594,       1.3563924,      -8.4690914 ],
  [               0,               0,               0,       19.751419,      -42.788036,       25.569326 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.15287194,     -0.13527386,      -0.2521579,      0.19939259 ],
  [               0,               0,      -691.19122,       922.94617,       1343.0807,       -1780.549 ],
  [               0,               0,       2099.4001,      -3013.7212,      -4356.8408,       6252.5229 ],
  [               0,               0,       335.85913,      -498.31631,         -713.62,       1054.7946 ],
  [               0,               0,       333.30673,      -464.91837,      -669.33209,       932.57697 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam25_layer1_etaoffv3_37_e = [
 [
  [               0,               0,               0,               0,               0,     0.011935097 ],
  [               0,               0,               0,               0,               0,      -1.8676134 ],
  [               0,               0,               0,               0,               0,      0.69514525 ],
  [               0,               0,               0,               0,               0,      0.21974672 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   4.7404101e-05,   0.00053869729 ],
  [               0,               0,               0,               0,        12.45722,       21.929377 ],
  [               0,               0,               0,               0,      -1.5339345,      -2.4778535 ],
  [               0,               0,               0,               0,       1.9748818,       2.1273997 ],
  [               0,               0,               0,               0,      0.36995408,      0.24291287 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.34984794,      0.52855921 ],
  [               0,               0,               0,               0,        -157.729,       238.40031 ],
  [               0,               0,               0,               0,       39.297073,      -59.664639 ],
  [               0,               0,               0,               0,      -4.7452288,       7.3315601 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   -0.0074948869,   -0.0060209222,    0.0087264888,     0.059193444 ],
  [               0,               0,      -3.5996978,       26.112799,      -55.861794,       37.885551 ],
  [               0,               0,      -13.357316,       60.076309,      -89.573112,       43.665771 ],
  [               0,               0,        7.351028,      -43.416996,       82.503838,      -51.514412 ],
  [               0,               0,       2.6832044,      -13.335958,       22.039398,      -12.109596 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0028432815,    0.0061459392 ],
  [               0,               0,               0,               0,       14.990563,       -23.69384 ],
  [               0,               0,               0,               0,        4.712214,       -9.809886 ],
  [               0,               0,               0,               0,       3.1245942,      -6.1746902 ],
  [               0,               0,               0,               0,      -0.1039477,      0.25861785 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,  -0.00051195931,  -7.8215991e-05,    0.0012998553,    0.0052187419 ],
  [               0,               0,     -0.85920733,       2.9585314,       2.9005611,      -8.5594969 ],
  [               0,               0,       2.6935225,       -5.542438,      -12.558986,       24.594364 ],
  [               0,               0,       2.7068646,      -5.2268038,      -14.863544,       28.088106 ],
  [               0,               0,     -0.51521671,      0.99177909,       2.8001089,      -5.4469614 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,       45.230961,      -103.75344 ],
  [               0,               0,               0,               0,      -30.989962,       71.599678 ],
  [               0,               0,               0,               0,       31.236603,      -72.162727 ],
  [               0,               0,               0,               0,      -1.0331506,       2.3823478 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0036295198,     0.011649947 ],
  [               0,               0,               0,               0,      -4.1948528,       12.380634 ],
  [               0,               0,               0,               0,       4.4449072,      -11.690481 ],
  [               0,               0,               0,               0,       3.3180425,      -8.9115143 ],
  [               0,               0,               0,               0,      0.62430555,      -1.4384826 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0095339259,    0.0089391423,     0.043686364 ],
  [               0,               0,               0,      -90.650375,        451.7865,      -556.89734 ],
  [               0,               0,               0,       150.91313,      -735.23413,       893.85583 ],
  [               0,               0,               0,        -156.813,       763.99316,      -930.33105 ],
  [               0,               0,               0,    0.0020758528,     0.028107885,    -0.081592001 ],
  [               0,               0,               0,      -8.9648533,      -5.1666641,       65.801025 ],
  [               0,               0,               0,       3.1424453,      -5.6095176,      -5.0639095 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0063534267,   -0.0074612894 ],
  [               0,               0,               0,               0,      -3491.2837,       8683.1719 ],
  [               0,               0,               0,               0,               0,     -0.40000001 ],
  [               0,               0,               0,               0,      -13.115494,       31.214542 ],
  [               0,               0,               0,               0,      0.11756368,   -0.0043200455 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam50_layer1_etaoffv3_37_b = [
 [
  [               0,               0,       19186.695,      -88.133743,     -0.48497629,    0.0012692058 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,  -0.00043122459,    0.0014427929 ],
  [               0,               0,               0,               0,      0.29402301,       3.2886982 ],
  [               0,               0,               0,               0,      0.47313419,      -1.3700746 ],
  [               0,               0,               0,               0,     -0.50125223,       -1.161345 ],
  [               0,               0,               0,               0,     -0.26594248,     0.020779928 ],
  [               0,               0,               0,               0,   0.00019782152,   -0.0001597691 ],
  [               0,               0,               0,               0,      0.39358291,       1.4939713 ],
  [               0,               0,               0,               0,     -0.91330534,      -1.2923979 ],
 ],
 [
  [               0,               0,               0,    0.0030718402,   -0.0020407313,    0.0011767271 ],
  [               0,               0,               0,     -0.96968347,      -2.0298474,       4.9502406 ],
  [               0,               0,               0,      0.17859709,     -0.49326563,     -0.84233087 ],
  [               0,               0,               0,       2.5845175,      -3.1574054,      -0.2744467 ],
  [               0,               0,               0,      0.16212924,     -0.22832578,    -0.043999497 ],
  [               0,               0,               0,   0.00082685053,  -0.00084738591,   0.00024029889 ],
  [               0,               0,               0,      -2.2439847,      0.79323608,       2.4943197 ],
  [               0,               0,               0,       1.1266046,      -1.6060709,      -3.4672878 ],
 ],
 [
  [               0,               0,               0,               0,     0.020481506,     0.011006957 ],
  [               0,               0,               0,               0,       5.2831407,      -3.6607628 ],
  [               0,               0,               0,               0,      -18.591179,       14.287498 ],
  [               0,               0,               0,               0,       13.986037,       -11.12344 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   0.00012345836,   0.00019144593,    0.0014477434 ],
  [               0,               0,               0,     -0.21212274,      -1.0483794,       3.8462894 ],
  [               0,               0,               0,     -0.44610494,       1.5640752,       -2.214483 ],
  [               0,               0,               0,      0.93034357,      -1.7071656,     -0.56702274 ],
  [               0,               0,               0,      0.13268019,     -0.29805347,     0.030061979 ],
  [               0,               0,               0,   0.00015424634,  -0.00049765984,   0.00040809609 ],
  [               0,               0,               0,       2.2281609,      -3.3115535,       3.0851412 ],
  [               0,               0,               0,     -0.25700641,     0.032538671,       2.0446429 ],
 ],
 [
  [               0,               0,               0,               0,               0,   0.00026741863 ],
  [               0,               0,               0,               0,               0,       12.907967 ],
  [               0,               0,               0,               0,               0,      -5.1563754 ],
  [               0,               0,               0,               0,               0,    -0.094984777 ],
  [               0,               0,               0,               0,               0,      -0.9712019 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0029525016,    0.0013316114,   -0.0020350278,   -0.0077753379 ],
  [               0,               0,      -5.2248783,      -5.0313272,       -6.890172,      -5.4327221 ],
  [               0,               0,      -17.879757,      -24.309704,       -32.25227,      -43.483852 ],
  [               0,               0,   0.00016140207,       195.62215,    -0.035585217,      -1.0155841 ],
  [               0,               0,       13.565405,       19.172691,       27.082426,       38.138329 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.39380813,     -0.56791276,     -0.81306463,       1.1791723 ],
  [               0,               0,       256.79837,      -367.30432,      -517.32056,       746.05853 ],
  [               0,               0,        283.6701,       -316.2308,      -1079.0175,       1359.3491 ],
  [               0,               0,       441.50385,      -568.29572,      -1252.0717,       1657.3776 ],
  [               0,               0,       3083.0137,      -4383.6323,      -6276.3818,       8922.2021 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam50_layer1_etaoffv3_37_e = [
 [
  [               0,               0,               0,               0,               0,       128373.85 ],
  [               0,               0,               0,               0,               0,   -0.0052904291 ],
  [               0,               0,               0,               0,               0,    0.0052903672 ],
  [               0,               0,               0,               0,               0,   5.0356204e-09 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.023988666,     0.036432974 ],
  [               0,               0,               0,               0,       799.00909,      -1158.8315 ],
  [               0,               0,               0,               0,      -611.01794,        900.5282 ],
  [               0,               0,               0,               0,       812.70795,      -1182.0814 ],
  [               0,               0,               0,               0,     0.072606839,     -0.45693961 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.062348321,     0.098196827 ],
  [               0,               0,               0,               0,      -264.68448,       399.49149 ],
  [               0,               0,               0,               0,      -27.268398,       39.089676 ],
  [               0,               0,               0,               0,       53.674423,      -79.489876 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0012885783,   -0.0027042825,   -0.0043317005,     0.011984306 ],
  [               0,               0,      -37.323483,        185.1609,      -304.83301,        168.1801 ],
  [               0,               0,      -13.948867,       67.428146,      -108.57369,       57.195469 ],
  [               0,               0,       2.3811102,      -9.6693325,       12.130027,      -5.4231162 ],
  [               0,               0,      -2.3072047,       11.526284,      -19.255453,       10.783957 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0068465434,    -0.010711855 ],
  [               0,               0,               0,               0,      -4.6366663,       11.178283 ],
  [               0,               0,               0,               0,      0.12007135,       -1.339987 ],
  [               0,               0,               0,               0,     -0.38406816,     -0.34445304 ],
  [               0,               0,               0,               0,      -0.2808755,      0.57134527 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0091457041,    -0.020310082,    -0.041158207,     0.093914405 ],
  [               0,               0,      -5.7194605,       6.4562345,       51.850346,      -81.347939 ],
  [               0,               0,        10.03591,      -61.542934,       125.47899,      -86.269386 ],
  [               0,               0,      -3.8872147,       23.795216,      -48.097759,       30.943027 ],
  [               0,               0,        4.034224,        -28.0193,       64.505577,      -49.203232 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00026046211,    0.0030118178 ],
  [               0,               0,               0,               0,      -168.91006,       390.64505 ],
  [               0,               0,               0,               0,       69.125153,      -160.38722 ],
  [               0,               0,               0,               0,      -3.0148892,       7.3285065 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00035684122,    0.0016790541 ],
  [               0,               0,               0,               0,      -2.8863294,       9.8636627 ],
  [               0,               0,               0,               0,      0.48305917,      -2.3974199 ],
  [               0,               0,               0,               0,      0.21288341,      -1.6484632 ],
  [               0,               0,               0,               0,    -0.080964975,      0.25670284 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,     0.010218455 ],
  [               0,               0,               0,               0,               0,      0.65913546 ],
  [               0,               0,               0,               0,               0,     -0.56033212 ],
  [               0,               0,               0,               0,               0,     -0.66789144 ],
  [               0,               0,               0,               0,               0,     -0.11914371 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.011272831,     0.076581359,     -0.11165876 ],
  [               0,               0,               0,      -666.49835,       3207.6113,      -3851.5022 ],
  [               0,               0,               0,       116.34254,      -569.50208,       695.31067 ],
  [               0,               0,               0,       34.601807,      -7.9648957,      -215.71683 ],
  [               0,               0,               0,    -0.059727281,     0.026568679,      0.53504485 ],
  [               0,               0,               0,      -4.8784618,       23.896912,      -29.265125 ],
  [               0,               0,               0,       12.300871,      -53.501431,       51.272133 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0075906022,    0.0013913187,     0.055706125 ],
  [               0,               0,               0,      -667.20477,       3.4743116,       4118.8804 ],
  [               0,               0,               0,       14498.655,        4.530695,      -89814.781 ],
  [               0,               0,               0,      -15.845221,       13.670547,       62.108288 ],
  [               0,               0,               0,     0.037887774,     0.087600209,     -0.10655543 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam75_layer1_etaoffv3_37_b = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,  -0.00013035953,   0.00078244729 ],
  [               0,               0,               0,               0,      0.14820278,       4.2644973 ],
  [               0,               0,               0,               0,      0.70665938,      -1.1027609 ],
  [               0,               0,               0,               0,     0.037338592,      -1.1121839 ],
  [               0,               0,               0,               0,     -0.19403806,    -0.058764651 ],
  [               0,               0,               0,               0,   0.00016758544,  -3.9844988e-05 ],
  [               0,               0,               0,               0,       8.2376347,     -0.47036192 ],
  [               0,               0,               0,               0,      -3.3626132,      -1.8628469 ],
 ],
 [
  [               0,               0,               0,    0.0048249434,   -0.0046211439,    0.0019141634 ],
  [               0,               0,               0,      -9.3600874,       6.9985251,       2.9112062 ],
  [               0,               0,               0,       3.6488113,      -14.028506,       9.3566351 ],
  [               0,               0,               0,       8.2667532,      -20.048153,       10.594807 ],
  [               0,               0,               0,       22.641731,      -46.764381,       -3.957021 ],
  [               0,               0,               0,      0.13947079,      -0.1511205,      0.06760323 ],
  [               0,               0,               0,       0.1698586,     -0.21730621,      0.11625274 ],
  [               0,               0,               0,      -1.7068582,      0.19474988,       1.1959749 ],
 ],
 [
  [               0,               0,               0,               0,     0.019591585,     0.012319445 ],
  [               0,               0,               0,               0,       17.119638,      -13.081682 ],
  [               0,               0,               0,               0,      -28.234161,       21.965702 ],
  [               0,               0,               0,               0,       13.714909,      -10.910196 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.032029394,     0.074201226,    -0.031564936 ],
  [               0,               0,               0,       1.3868558,      -3.2315531,       2.8025141 ],
  [               0,               0,               0,     -0.83389699,        1.935071,      -1.8601695 ],
  [               0,               0,               0,     -0.67554283,       1.6087797,      -1.7455901 ],
  [               0,               0,               0,    -0.058996614,      0.13707018,     -0.10156087 ],
  [               0,               0,               0,  -0.00015399783,   0.00039337476,  -0.00027330546 ],
  [               0,               0,               0,       6.7394032,      -16.582209,       12.519445 ],
  [               0,               0,               0,      -12.629567,       25.136032,      -17.367277 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.52408445,       -0.790075,      -1.1059057,       1.6725998 ],
  [               0,               0,       825.98773,      -1171.7933,      -1645.0872,       2338.6475 ],
  [               0,               0,      -496.67215,       816.20947,       498.08524,      -929.54266 ],
  [               0,               0,       196.29732,      -201.44431,      -837.76093,       1031.6704 ],
  [               0,               0,       3243.4253,      -4613.9575,      -6609.1519,       9399.9639 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam75_layer1_etaoffv3_37_e = [
 [
  [               0,               0,               0,               0,    -0.024603963,     0.033967339 ],
  [               0,               0,               0,               0,      -21710.473,       31634.963 ],
  [               0,               0,               0,               0,       105.30622,      -154.36023 ],
  [               0,               0,               0,               0,       115.83475,      -166.85448 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0025930495,    0.0084320121 ],
  [               0,               0,               0,               0,       97.314865,      -140.47705 ],
  [               0,               0,               0,               0,      -34.745853,       49.848454 ],
  [               0,               0,               0,               0,       97.334579,      -142.13768 ],
  [               0,               0,               0,               0,      0.11585742,     -0.11945213 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.19184311,      0.29079115 ],
  [               0,               0,               0,               0,      -349.02158,       525.83954 ],
  [               0,               0,               0,               0,      -12.751662,       17.540455 ],
  [               0,               0,               0,               0,       55.038712,      -81.454773 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0019635535,    0.0010668876,   -0.0010165045,   -0.0040620551 ],
  [               0,               0,      -24.800365,       136.14082,      -248.27237,       151.59801 ],
  [               0,               0,      -9.8523426,       41.774521,      -56.533665,       22.909346 ],
  [               0,               0,      -6.2558303,       22.613464,      -22.795551,       3.2888582 ],
  [               0,               0,     -0.63305569,       3.1976223,      -5.5021653,       3.2427423 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0079228338,    -0.012426827 ],
  [               0,               0,               0,               0,      -4.4950895,       10.889928 ],
  [               0,               0,               0,               0,     0.096608423,      -1.3287916 ],
  [               0,               0,               0,               0,      0.40850946,      -1.9324583 ],
  [               0,               0,               0,               0,     -0.12946051,      0.27536878 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0099879662,    -0.025389779,    -0.048670921,      0.12697703 ],
  [               0,               0,       9.0844135,      -46.617294,       80.709564,      -46.045662 ],
  [               0,               0,       22.526958,      -138.89246,       283.76157,      -193.18381 ],
  [               0,               0,       -5.624002,       36.101158,      -77.592552,       54.723717 ],
  [               0,               0,       8.7439432,      -56.898987,       123.48751,      -89.339539 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0013414148,    0.0035875952 ],
  [               0,               0,               0,               0,      -101.74971,       235.41937 ],
  [               0,               0,               0,               0,       52.809364,      -122.56087 ],
  [               0,               0,               0,               0,      0.15285493,     -0.10005235 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00056125637,     0.002046945 ],
  [               0,               0,               0,               0,      -2.7649796,       9.1031027 ],
  [               0,               0,               0,               0,       2.6295409,      -7.3891587 ],
  [               0,               0,               0,               0,     -0.22444525,     -0.64620018 ],
  [               0,               0,               0,               0,     -0.11172296,      0.30994561 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,    0.0014958936 ],
  [               0,               0,               0,               0,               0,       2.3426714 ],
  [               0,               0,               0,               0,               0,     -0.88536024 ],
  [               0,               0,               0,               0,               0,      -2.1506803 ],
  [               0,               0,               0,               0,               0,     -0.87657189 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.027226685,   0.00087170111,      0.17049897 ],
  [               0,               0,               0,       21.670172,      -72.319672,       53.374771 ],
  [               0,               0,               0,       345.23013,      -1686.6538,       2057.8354 ],
  [               0,               0,               0,      -10.033906,       63.490627,       -94.51207 ],
  [               0,               0,               0,    0.0092231417,   -0.0063093975,    -0.034474459 ],
  [               0,               0,               0,       28.043791,      -151.33913,       202.99393 ],
  [               0,               0,               0,       18.282133,      -74.047935,       68.507507 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,        -5.32127,         13.6764 ],
  [               0,               0,               0,               0,       96.114502,      -237.54117 ],
  [               0,               0,               0,               0,       6621.0771,      -16478.039 ],
  [               0,               0,               0,               0,      -80.510223,       198.79817 ],
  [               0,               0,               0,               0,       12.770927,       -31.74176 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam100_layer1_etaoffv3_37_b = [
 [
  [               0,               0,       13817.017,       7.9554772,     -0.87469125,    0.0014814871 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00048714428,   0.00068174925 ],
  [               0,               0,               0,               0,       -2.100739,       4.6463795 ],
  [               0,               0,               0,               0,     0.040466525,      0.79100972 ],
  [               0,               0,               0,               0,     -0.83643657,      0.68965054 ],
  [               0,               0,               0,               0,       -6.117641,       -1.883827 ],
  [               0,               0,               0,               0,    -0.005927729,   -0.0041279825 ],
  [               0,               0,               0,               0,    -0.021220688,     0.093693502 ],
  [               0,               0,               0,               0,     -0.72176194,      -1.8851064 ],
 ],
 [
  [               0,               0,               0,   -0.0050566341,    0.0056208405,   -0.0026624529 ],
  [               0,               0,               0,       10.583702,      -11.289656,     -0.25329795 ],
  [               0,               0,               0,        1.564321,      -2.2530012,       1.7515661 ],
  [               0,               0,               0,      -1.7033325,       1.9334399,      0.72548288 ],
  [               0,               0,               0,     -0.27123848,      0.51975942,    -0.079691589 ],
  [               0,               0,               0,  -0.00047831476,   0.00046335487,  -9.3199007e-05 ],
  [               0,               0,               0,     -0.46867156,      0.83546513,       2.6629527 ],
  [               0,               0,               0,      -9.4598598,     -0.13058256,      -0.1690181 ],
 ],
 [
  [               0,               0,               0,               0,     0.021947265,    0.0088755293 ],
  [               0,               0,               0,               0,       13.535138,      -10.218669 ],
  [               0,               0,               0,               0,      -27.801102,       21.607914 ],
  [               0,               0,               0,               0,        15.31071,      -12.177553 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0012348488,   0.00084781513,   5.1990137e-05 ],
  [               0,               0,               0,       1.5989469,       -5.362617,       5.9195175 ],
  [               0,               0,               0,      0.14662838,     -0.16732842,     -0.95567787 ],
  [               0,               0,               0,      0.10516955,      0.22559461,      -1.6163572 ],
  [               0,               0,               0,     -0.14904617,      0.47533289,     -0.45160875 ],
  [               0,               0,               0,  -0.00017553281,   0.00040522104,  -0.00022829141 ],
  [               0,               0,               0,        5.499783,       1.6160018,      0.69208646 ],
  [               0,               0,               0,     -0.43312803,     -0.78249925,      -2.0561805 ],
 ],
 [
  [               0,               0,               0,               0,               0,    0.0066122334 ],
  [               0,               0,               0,               0,               0,       1.4875946 ],
  [               0,               0,               0,               0,               0,      -1.3185656 ],
  [               0,               0,               0,               0,               0,      -1.1264307 ],
  [               0,               0,               0,               0,               0,    -0.027325926 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.004936276,    0.0041579157,   -0.0019894394,    -0.018735712 ],
  [               0,               0,     -0.19884855,       4.7848787,      -2.8597593,      -6.8843689 ],
  [               0,               0,       8322.2061,       3791.7297,      -5604.1753,      -22997.098 ],
  [               0,               0,               0,               0,               0,              -1 ],
  [               0,               0,       45.470322,       29.781376,       76.587219,       270.78619 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.73840284,      -1.0272567,       -1.496218,       2.0880241 ],
  [               0,               0,       435.16312,      -650.69147,      -906.44183,       1360.7793 ],
  [               0,               0,       853.33948,      -1145.3751,        -2243.02,       3053.3745 ],
  [               0,               0,       289.75876,      -331.38815,      -1021.0661,       1286.0658 ],
  [               0,               0,       3286.0225,      -4668.0444,      -6694.5449,       9508.0049 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam100_layer1_etaoffv3_37_e = [
 [
  [               0,               0,               0,               0,               0,    -0.022821324 ],
  [               0,               0,               0,               0,               0,       1.1507297 ],
  [               0,               0,               0,               0,               0,      -0.7719987 ],
  [               0,               0,               0,               0,               0,    -0.081465401 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,       0.2158969,     -0.31230503 ],
  [               0,               0,               0,               0,      -19.546537,       34.625237 ],
  [               0,               0,               0,               0,      -88.969353,       129.62012 ],
  [               0,               0,               0,               0,       120.62475,      -174.39195 ],
  [               0,               0,               0,               0,      0.15773885,    -0.090661682 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.26246235,      0.39709061 ],
  [               0,               0,               0,               0,      -295.92587,       445.96265 ],
  [               0,               0,               0,               0,       7.0032854,      -11.800196 ],
  [               0,               0,               0,               0,       29.817257,      -43.987774 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,  -0.00045981759,  -0.00019027203,   0.00069345615,    0.0034462742 ],
  [               0,               0,      0.33487779,      0.45265993,     -0.88553411,      0.99092221 ],
  [               0,               0,       2.1753573,       -3.399303,      -6.8309736,       9.5676317 ],
  [               0,               0,      0.77598327,      -1.5442731,      -2.0345221,       2.9583588 ],
  [               0,               0,     -0.34500426,      0.57259232,      0.79404169,      -1.2499009 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0050079105,   -0.0073380405 ],
  [               0,               0,               0,               0,      -3.3951337,       9.0842848 ],
  [               0,               0,               0,               0,     -0.22786766,     -0.75793517 ],
  [               0,               0,               0,               0,     -0.42676893,     -0.29031965 ],
  [               0,               0,               0,               0,     -0.31654996,      0.66090888 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   0.00027879153,   -0.0013283887,   -0.0010249899,    0.0083415927 ],
  [               0,               0,      -2.1995621,       17.538357,      -43.742092,       37.870152 ],
  [               0,               0,        23.33523,      -148.98674,       317.07379,       -226.1813 ],
  [               0,               0,      -17.726542,       111.52159,      -232.97949,       160.46312 ],
  [               0,               0,       12.026373,      -78.838455,       172.13309,      -125.11136 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.09861552,      0.23101746 ],
  [               0,               0,               0,               0,      -181.11835,       418.36789 ],
  [               0,               0,               0,               0,       44.441032,      -103.57121 ],
  [               0,               0,               0,               0,       15.845361,      -36.047024 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00055176037,    0.0020386504 ],
  [               0,               0,               0,               0,     -0.35246369,       3.5785263 ],
  [               0,               0,               0,               0,     -0.52223837,    -0.038600594 ],
  [               0,               0,               0,               0,     -0.34865525,      -0.3297807 ],
  [               0,               0,               0,               0,      0.24357215,     -0.49918583 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,   0.00091088621 ],
  [               0,               0,               0,               0,               0,       3.2429302 ],
  [               0,               0,               0,               0,               0,      -1.0619931 ],
  [               0,               0,               0,               0,               0,      -2.2174644 ],
  [               0,               0,               0,               0,               0,      -1.2163603 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   0.00082529528,    0.0020517453,   -0.0018718051 ],
  [               0,               0,               0,      -5.2240949,       3.3018341,       30.591652 ],
  [               0,               0,               0,      0.52876675,      -16.345018,       34.887337 ],
  [               0,               0,               0,       1.3919296,       3.0668743,      -15.494097 ],
  [               0,               0,               0,     0.012540033,  -0.00023542541,    -0.072968692 ],
  [               0,               0,               0,       3.4195106,      -1.0314569,      -17.260677 ],
  [               0,               0,               0,      -8.8338966,     -0.34747234,       51.025993 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0014550096,     0.001588588,   -0.0011026151,    -0.014792951 ],
  [               0,               0,      -158.65752,      -130.37326,       328.26923,       2431.8369 ],
  [               0,               0,     -0.27299824,     -0.64166737,      -1.5675428,      -4.6117783 ],
  [               0,               0,      -6.9900351,     -0.69953448,       20.595135,        58.24778 ],
  [               0,               0,     0.010881772,     0.027162265,     0.060274206,     -0.17170633 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam200_layer1_etaoffv3_37_b = [
 [
  [               0,               0,       25894.432,      -155.82994,      -0.2263234,   0.00074602128 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,  -0.00023329027,    0.0010037679 ],
  [               0,               0,               0,               0,      0.10864315,       4.1743803 ],
  [               0,               0,               0,               0,      0.85284698,     -0.99403417 ],
  [               0,               0,               0,               0,     -0.64709127,      -0.9810589 ],
  [               0,               0,               0,               0,     -0.64701116,    -0.013802752 ],
  [               0,               0,               0,               0,   5.0788854e-06,  -0.00017012841 ],
  [               0,               0,               0,               0,     -0.64413404,       0.8647601 ],
  [               0,               0,               0,               0,       -1.829309,      -1.6201766 ],
 ],
 [
  [               0,               0,               0,   2.4027326e-05,   0.00065191399,   0.00063782907 ],
  [               0,               0,               0,     -0.63805425,      -2.2063789,       5.2747126 ],
  [               0,               0,               0,      -11.524716,       13.961176,      -4.6531134 ],
  [               0,               0,               0,      -2.9004467,       3.3149731,      -2.2737706 ],
  [               0,               0,               0,      -4.1214538,      -3.5979421,       1.7596029 ],
  [               0,               0,               0,   -0.0082340641,   1.4991422e-05,    0.0011344668 ],
  [               0,               0,               0,      0.60285693,      -1.3239689,      0.78795558 ],
  [               0,               0,               0,      -4.9080367,       5.2510071,      -4.2622504 ],
 ],
 [
  [               0,               0,               0,               0,     0.020438513,    0.0088081304 ],
  [               0,               0,               0,               0,       13.171142,      -9.9001265 ],
  [               0,               0,               0,               0,      -29.734283,       23.121359 ],
  [               0,               0,               0,               0,       17.079969,      -13.585447 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0012462281,   -0.0017472639,    0.0021363767 ],
  [               0,               0,               0,       0.9490518,      -3.9381988,       5.5126157 ],
  [               0,               0,               0,      -2.0712731,       4.8724599,      -3.7407391 ],
  [               0,               0,               0,      0.96972609,      -1.5359739,     -0.73375732 ],
  [               0,               0,               0,      0.45794159,     -0.91969949,      0.30386981 ],
  [               0,               0,               0,   0.00045993124,   -0.0009343003,   0.00050729164 ],
  [               0,               0,               0,      -5.2519341,       10.780732,      -3.2612317 ],
  [               0,               0,               0,      -8.4615088,       20.037663,      -7.9750657 ],
 ],
 [
  [               0,               0,               0,               0,               0,       2.2838681 ],
  [               0,               0,               0,               0,               0,      0.12967804 ],
  [               0,               0,               0,               0,               0,     -0.12840863 ],
  [               0,               0,               0,               0,               0,     -0.12919508 ],
  [               0,               0,               0,               0,               0,  -0.00015147519 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    -0.053644329,    -0.025018938,      0.03522649,      0.14827628 ],
  [               0,               0,       129.85231,       1006.9587,      -1669.3557,       15.898386 ],
  [               0,               0,       -4.770637,      -2.9326761,       1.0883611,       8.3274632 ],
  [               0,               0,   1.1753236e-05,  -0.00065379881,    -0.018025452,       -1.007636 ],
  [               0,               0,      0.52754128,       0.7554993,       1.0560673,       1.1428635 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.73997122,      -1.0346719,      -1.5063467,       2.1130154 ],
  [               0,               0,       724.77087,      -1054.2791,      -1479.1538,       2158.3674 ],
  [               0,               0,       908.35828,      -1215.4471,      -2405.3018,       3267.8904 ],
  [               0,               0,       223.53809,      -226.59207,      -940.32697,        1149.717 ],
  [               0,               0,       3613.7004,      -5132.4131,      -7369.3252,       10464.149 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam200_layer1_etaoffv3_37_e = [
 [
  [               0,               0,               0,               0,               0,    0.0081073074 ],
  [               0,               0,               0,               0,               0,      -1.9720365 ],
  [               0,               0,               0,               0,               0,      0.83835709 ],
  [               0,               0,               0,               0,               0,      0.34462401 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.056220356,     0.083466664 ],
  [               0,               0,               0,               0,      -973.77441,       1437.1495 ],
  [               0,               0,               0,               0,      -1271.5306,       1874.4008 ],
  [               0,               0,               0,               0,       1138.4277,      -1654.9062 ],
  [               0,               0,               0,               0,      0.37375066,    -0.073542818 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.37571448,      0.56727242 ],
  [               0,               0,               0,               0,       -300.4231,       452.59229 ],
  [               0,               0,               0,               0,       29.991451,      -45.964649 ],
  [               0,               0,               0,               0,       2.4580657,      -3.3438978 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,   0.00012978938 ],
  [               0,               0,               0,               0,               0,      -39.793232 ],
  [               0,               0,               0,               0,               0,       6.3080935 ],
  [               0,               0,               0,               0,               0,      -1.5497059 ],
  [               0,               0,               0,               0,               0,       10.338521 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,  -0.00033265987,   -0.0001731576,   0.00054207171,    0.0031456719 ],
  [               0,               0,       1.7904142,      -2.2055063,      -4.9921274,       8.4726429 ],
  [               0,               0,     0.022462765,      0.31086141,      -1.2989641,    0.0063633178 ],
  [               0,               0,     -0.14923404,     0.091458715,      0.49290901,      -1.4893014 ],
  [               0,               0,     -0.12426284,      0.23461926,      0.14512001,     -0.24104778 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0064596445,    -0.010137269 ],
  [               0,               0,               0,               0,      -6.3906374,       15.116877 ],
  [               0,               0,               0,               0,      0.27981332,      -1.7932245 ],
  [               0,               0,               0,               0,     -0.28024772,     -0.63071465 ],
  [               0,               0,               0,               0,      -0.3903299,      0.81144446 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0023373016,   -0.0055290665,    -0.009663838,     0.025957959 ],
  [               0,               0,      -3.2448809,       17.679199,      -31.032963,       20.389818 ],
  [               0,               0,       22.829145,       -146.4258,        312.8577,      -223.94632 ],
  [               0,               0,      -19.422037,       118.86092,      -241.42638,       161.51585 ],
  [               0,               0,       11.750873,      -77.468277,       170.01834,      -124.15486 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.19227098,      0.44530448 ],
  [               0,               0,               0,               0,      -163.40643,       378.17444 ],
  [               0,               0,               0,               0,       9.7243567,      -24.007456 ],
  [               0,               0,               0,               0,       45.610455,      -104.29651 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00042790256,    0.0019619423 ],
  [               0,               0,               0,               0,      -1.5624856,       6.8638344 ],
  [               0,               0,               0,               0,     -0.50536084,     -0.13833351 ],
  [               0,               0,               0,               0,     0.033793591,      -1.2386193 ],
  [               0,               0,               0,               0,     0.019638665,     0.038947087 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,    0.0011838173 ],
  [               0,               0,               0,               0,               0,       2.5097494 ],
  [               0,               0,               0,               0,               0,     -0.79709762 ],
  [               0,               0,               0,               0,               0,      -2.0953331 ],
  [               0,               0,               0,               0,               0,     -0.99946076 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.063956387,      0.21216194,     -0.12853269 ],
  [               0,               0,               0,       -101.7903,       549.76758,      -727.44574 ],
  [               0,               0,               0,        119.2757,      -595.45349,       740.62305 ],
  [               0,               0,               0,       -118.9855,       588.12268,       -726.1936 ],
  [               0,               0,               0,    0.0099279964,    -0.046506219,     0.056327272 ],
  [               0,               0,               0,       203.97859,      -994.35522,        1212.699 ],
  [               0,               0,               0,      -30.645039,       177.72632,      -254.33057 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,       1.5734752,      -3.8942926 ],
  [               0,               0,               0,               0,      -3165.4402,       7874.8384 ],
  [               0,               0,               0,               0,       7078.0879,      -17619.637 ],
  [               0,               0,               0,               0,      -198.02409,         490.082 ],
  [               0,               0,               0,               0,      0.10268239,      0.11761421 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam500_layer1_etaoffv3_37_b = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   3.8636113e-05,   0.00026682598 ],
  [               0,               0,               0,               0,       12.597256,       5.3188286 ],
  [               0,               0,               0,               0,     -0.83771658,      0.46027535 ],
  [               0,               0,               0,               0,      -10.671377,       2.5495193 ],
  [               0,               0,               0,               0,      -2.0351007,       0.2266252 ],
  [               0,               0,               0,               0,    0.0002208193,   0.00013683891 ],
  [               0,               0,               0,               0,      -2.7089801,       2.1751268 ],
  [               0,               0,               0,               0,       1.1335465,      -1.8917817 ],
 ],
 [
  [               0,               0,               0,    0.0024158603,   -0.0027670157,    0.0017208242 ],
  [               0,               0,               0,       -21.63122,       24.480267,      -2.3811257 ],
  [               0,               0,               0,       22.166348,       -17.79171,       6.2167044 ],
  [               0,               0,               0,       35.803394,      -34.917416,       10.456251 ],
  [               0,               0,               0,       62.025154,      -49.241371,      -1.2791649 ],
  [               0,               0,               0,      0.04810556,     -0.06476856,     0.035998583 ],
  [               0,               0,               0,     -0.42579985,      0.33442128,     -0.14867175 ],
  [               0,               0,               0,      -5.1527758,       4.6481881,      -1.9446197 ],
 ],
 [
  [               0,               0,               0,               0,     0.090691485,     -0.04400273 ],
  [               0,               0,               0,               0,       4.0272369,      -2.6091835 ],
  [               0,               0,               0,               0,      -17.710983,       13.547357 ],
  [               0,               0,               0,               0,       13.365988,      -10.631066 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00073644985,   0.00061531673 ],
  [               0,               0,               0,               0,      -1.3280489,       4.7948337 ],
  [               0,               0,               0,               0,      0.19035403,      -1.2597471 ],
  [               0,               0,               0,               0,     -0.35203496,      -1.1118306 ],
  [               0,               0,               0,               0,    -0.040069405,     -0.14742127 ],
  [               0,               0,               0,               0,   -1.286856e-05,   7.3005045e-05 ],
  [               0,               0,               0,               0,      0.99470824,       1.2080683 ],
  [               0,               0,               0,               0,      -3.0513389,       5.8041501 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.45684165,     -0.68231809,     -0.96275759,        1.444119 ],
  [               0,               0,       205.36401,      -281.19266,       -410.7038,       568.09613 ],
  [               0,               0,      -449.82175,        786.2901,       266.59274,      -674.16168 ],
  [               0,               0,       222.46835,      -235.03981,      -931.10883,       1156.8163 ],
  [               0,               0,       3659.9087,      -5202.2925,      -7469.8345,       10615.642 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam500_layer1_etaoffv3_37_e = [
 [
  [               0,               0,               0,               0,      0.12101049,     -0.17231604 ],
  [               0,               0,               0,               0,        270.6489,      -979.53558 ],
  [               0,               0,               0,               0,      -283.18912,       411.53662 ],
  [               0,               0,               0,               0,      -9.3859882,       14.097916 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.007038557,   -0.0038792633 ],
  [               0,               0,               0,               0,       486.18768,      -705.83307 ],
  [               0,               0,               0,               0,      -41.102207,       58.900597 ],
  [               0,               0,               0,               0,       57.543415,      -83.743355 ],
  [               0,               0,               0,               0,      0.16201118,    -0.052811727 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      -0.4023433,      0.60591084 ],
  [               0,               0,               0,               0,      -372.59567,       561.42444 ],
  [               0,               0,               0,               0,       8.2345514,      -13.652378 ],
  [               0,               0,               0,               0,       27.002945,      -39.795742 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   -0.0032426729,    0.0015493097,    0.0075185266,    0.0014843433 ],
  [               0,               0,       42.739246,      -196.29707,       302.83673,      -155.41103 ],
  [               0,               0,       14.295844,      -67.067429,       103.93402,       -54.29203 ],
  [               0,               0,       11.215853,      -49.004452,       69.722313,      -33.104473 ],
  [               0,               0,       1.0128381,      -4.0703487,       5.2820406,      -2.1336975 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0063769175,   -0.0098972889 ],
  [               0,               0,               0,               0,      -7.2675853,       16.937656 ],
  [               0,               0,               0,               0,      0.32537267,      -1.9388767 ],
  [               0,               0,               0,               0,       0.2492218,      -1.7187381 ],
  [               0,               0,               0,               0,     -0.40223908,      0.83579963 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,  -0.00035641322,  -0.00010236068,   0.00090194191,    0.0047745756 ],
  [               0,               0,       2.6327844,       -4.544487,      -13.592034,       27.567438 ],
  [               0,               0,       1.4934622,       -3.056076,       -7.477437,       13.991908 ],
  [               0,               0,     -0.54609364,       2.4546523,      -1.8450499,      -3.0926983 ],
  [               0,               0,     -0.41518489,      0.70912433,        2.907125,      -5.3069353 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.001859326,  -0.00052795542 ],
  [               0,               0,               0,               0,      -318.83585,       735.52545 ],
  [               0,               0,               0,               0,       72.793228,      -169.05414 ],
  [               0,               0,               0,               0,       11.683495,      -26.282862 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00075295055,    0.0015633241 ],
  [               0,               0,               0,               0,       3.2440214,      -4.4843993 ],
  [               0,               0,               0,               0,     -0.52808672,     -0.17286012 ],
  [               0,               0,               0,               0,       -1.367126,        2.054065 ],
  [               0,               0,               0,               0,     -0.32253584,      0.85188526 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,  -0.00013884839,   0.00033333746 ],
  [               0,               0,               0,               0,      -9.8771086,      -15.977899 ],
  [               0,               0,               0,               0,      -2080.1558,       6245.6133 ],
  [               0,               0,               0,               0,       5573.7588,      -15732.429 ],
  [               0,               0,               0,               0,      -528.36902,      -1266.9474 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.022731965,     0.065680318,    -0.017838838 ],
  [               0,               0,               0,      -434.51205,       2146.5991,      -2640.3916 ],
  [               0,               0,               0,       111.64486,      -548.18567,        671.1684 ],
  [               0,               0,               0,      -42.789822,       1.6289268,       221.95705 ],
  [               0,               0,               0,     -0.18190128,    -0.029825864,       1.3952038 ],
  [               0,               0,               0,       3.4243779,      -16.845161,       20.711231 ],
  [               0,               0,               0,       3.1394031,      -23.192057,       25.869276 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.022455318,    -0.038618553 ],
  [               0,               0,               0,               0,       -3943.334,       9805.9736 ],
  [               0,               0,               0,               0,      0.56182396,      0.94093198 ],
  [               0,               0,               0,               0,      -234.95398,       580.50641 ],
  [               0,               0,               0,               0,      0.18433344,     -0.46530917 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam1000_layer1_etaoffv3_37_b = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00012474235,   0.00088439789 ],
  [               0,               0,               0,               0,     -0.59530139,       4.7855902 ],
  [               0,               0,               0,               0,      0.47328517,      -1.2469947 ],
  [               0,               0,               0,               0,     -0.41999272,      -1.2098908 ],
  [               0,               0,               0,               0,     -0.34791502,   -0.0085502407 ],
  [               0,               0,               0,               0,  -1.0778083e-05,   2.6179461e-05 ],
  [               0,               0,               0,               0,      -3.0656762,       3.9582365 ],
  [               0,               0,               0,               0,     -0.39764264,      -2.3467042 ],
 ],
 [
  [               0,               0,               0,               0,   0.00015159837,   0.00010247697 ],
  [               0,               0,               0,               0,      -20.764126,       21.436214 ],
  [               0,               0,               0,               0,      -3.6135175,       4.4222288 ],
  [               0,               0,               0,               0,      0.14061087,       2.0245538 ],
  [               0,               0,               0,               0,     -0.17325173,     -0.52300876 ],
  [               0,               0,               0,               0,  -1.8813471e-05,   0.00022102593 ],
  [               0,               0,               0,               0,       0.1852493,       1.7238497 ],
  [               0,               0,               0,               0,      -1.1381149,      -1.2292475 ],
 ],
 [
  [               0,               0,               0,               0,      0.47959515,     -0.38001648 ],
  [               0,               0,               0,               0,      -579.65271,       464.35617 ],
  [               0,               0,               0,               0,      -30.014803,       22.325289 ],
  [               0,               0,               0,               0,       144.47778,      -114.90541 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0057125306,    -0.013173819,    0.0093099028 ],
  [               0,               0,               0,      -2.6528356,       5.7871428,     -0.54288763 ],
  [               0,               0,               0,      -2.3057683,       5.2000451,      -3.8590262 ],
  [               0,               0,               0,      0.76028389,      -1.3271664,     -0.84747744 ],
  [               0,               0,               0,      0.69847435,       -1.566926,      0.70371807 ],
  [               0,               0,               0,   0.00043232655,  -0.00090882526,   0.00052103796 ],
  [               0,               0,               0,      -1.0830269,       5.1292505,      -6.7928796 ],
  [               0,               0,               0,       6.4176593,      -13.256733,       3.1634297 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.52994943,     -0.84042108,      -1.1466206,       1.8193893 ],
  [               0,               0,        832.5083,      -1179.4055,       -1642.795,       2331.9595 ],
  [               0,               0,      -894.64868,       1452.8326,       1097.5201,      -1924.7231 ],
  [               0,               0,      -168.97382,       371.10916,      -327.32635,       198.28027 ],
  [               0,               0,        3959.468,       -5635.208,      -8070.2378,       11483.483 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam1000_layer1_etaoffv3_37_e = [
 [
  [               0,               0,               0,               0,               0,    0.0005492107 ],
  [               0,               0,               0,               0,               0,   -7.486983e+08 ],
  [               0,               0,               0,               0,               0,      -3.1552298 ],
  [               0,               0,               0,               0,               0,       5.4115071 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.088844597,      0.13278303 ],
  [               0,               0,               0,               0,      -652.32751,       965.21252 ],
  [               0,               0,               0,               0,      -471.14578,       692.96381 ],
  [               0,               0,               0,               0,       381.64868,      -555.06299 ],
  [               0,               0,               0,               0,      0.31300226,     0.030019281 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.42219126,      0.63610619 ],
  [               0,               0,               0,               0,      -425.09854,       639.50159 ],
  [               0,               0,               0,               0,       18.719688,      -29.228092 ],
  [               0,               0,               0,               0,       15.602714,       -22.86865 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,  -0.00069241849,  -0.00025732582,    0.0011488444,    0.0051526483 ],
  [               0,               0,       2.3522987,      -2.6930001,      -7.6121411,       11.248885 ],
  [               0,               0,     -0.28460681,      0.98759389,      -1.3938087,      -0.2950646 ],
  [               0,               0,      -1.0320756,       1.3347929,       3.0512371,      -5.1425972 ],
  [               0,               0,   -0.0026603681,     0.059300043,     -0.19668463,      0.22447857 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,  -0.00052378693,    0.0029866798 ],
  [               0,               0,               0,               0,     -0.47952029,       4.2150617 ],
  [               0,               0,               0,               0,       1.2035543,      -3.5925081 ],
  [               0,               0,               0,               0,     -0.17199987,     -0.87840271 ],
  [               0,               0,               0,               0,     -0.27651533,      0.60305882 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,  -0.00045273069,  -0.00037666655,   0.00097885297,    0.0075557339 ],
  [               0,               0,       2.5444319,      -4.7481046,      -12.241094,       25.997639 ],
  [               0,               0,       0.3993293,     -0.64707977,      -2.1336019,       2.2681053 ],
  [               0,               0,     -0.24109009,       1.0103289,    -0.060331926,      -3.3175015 ],
  [               0,               0,     -0.17841198,      0.21642011,       1.6724347,      -2.7466433 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.32615307,      0.75329977 ],
  [               0,               0,               0,               0,      -133.03558,       308.42581 ],
  [               0,               0,               0,               0,      -57.753231,       130.99969 ],
  [               0,               0,               0,               0,       92.181488,      -211.26282 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.014138082,    -0.030340821 ],
  [               0,               0,               0,               0,      -12.691314,       33.456963 ],
  [               0,               0,               0,               0,      -2.5424359,       4.5475869 ],
  [               0,               0,               0,               0,       2.2777681,       -6.543077 ],
  [               0,               0,               0,               0,       0.4202407,     -0.88309038 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    3.755976e-06,    0.0028615706 ],
  [               0,               0,               0,               0,      -359.29474,       863.14038 ],
  [               0,               0,               0,               0,       4.9768806,      -12.523632 ],
  [               0,               0,               0,               0,       17.419291,      -43.353619 ],
  [               0,               0,               0,               0,    -0.027893161,     -0.36191839 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,      0.35864091,      -1.6969523,       2.0154836 ],
  [               0,               0,               0,      -711.71423,       3403.3225,      -4058.9319 ],
  [               0,               0,               0,       -35.47773,       153.84367,      -166.39305 ],
  [               0,               0,               0,      -124.97235,       617.95367,      -763.24939 ],
  [               0,               0,               0,      0.56278169,      -2.6826117,       3.1982968 ],
  [               0,               0,               0,      -80.353577,       379.17737,       -447.6167 ],
  [               0,               0,               0,       151.12067,      -743.70563,       912.18219 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0096667521,     0.010222266 ],
  [               0,               0,               0,               0,      0.23397563,       10.576326 ],
  [               0,               0,               0,               0,               0,     -0.40000001 ],
  [               0,               0,               0,               0,       -53.35416,       131.42422 ],
  [               0,               0,               0,               0,      0.70125103,     -0.94335681 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]


gam_layer1_etaoffv3_37_b = [
    gam5_layer1_etaoffv3_37_b,
    gam10_layer1_etaoffv3_37_b,
    gam25_layer1_etaoffv3_37_b,
    gam50_layer1_etaoffv3_37_b,
    gam75_layer1_etaoffv3_37_b,
    gam100_layer1_etaoffv3_37_b,
    gam200_layer1_etaoffv3_37_b,
    gam500_layer1_etaoffv3_37_b,
    gam1000_layer1_etaoffv3_37_b,
    ]

gam_layer1_etaoffv3_37_e = [
    gam5_layer1_etaoffv3_37_e,
    gam10_layer1_etaoffv3_37_e,
    gam25_layer1_etaoffv3_37_e,
    gam50_layer1_etaoffv3_37_e,
    gam75_layer1_etaoffv3_37_e,
    gam100_layer1_etaoffv3_37_e,
    gam200_layer1_etaoffv3_37_e,
    gam500_layer1_etaoffv3_37_e,
    gam1000_layer1_etaoffv3_37_e,
    ]



###############################################################################
# Tables for v3 sshape correction, layer 2, 5x5 electron clusters
#

ele5_layer2_etaoffv3_55_b = [
 [
    [0, 0, 0, 0, 0, 0], # Removed by hand, see below
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele5_layer2_etaoffv3_55_e = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele10_layer2_etaoffv3_55_b = [
 [
  [               0,               0,               0,     -0.21592221,     0.072746456,    0.0063402136 ],
  [               0,               0,               0,       35.770397,      -12.719185,       3.1045809 ],
  [               0,               0,               0,       4.1983519,      -1.2788233,      -1.0126278 ],
  [               0,               0,               0,      -4.5248036,       1.5975167,     -0.13813946 ],
  [               0,               0,               0,     0.034511443,    -0.011306059,   0.00096232252 ],
  [               0,               0,               0,      -9.7182064,       4.5155182,      0.67069179 ],
  [               0,               0,               0,      -29.412901,       7.9977522,    -0.026894659 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.018182639,    -0.030707115,     0.019057918 ],
  [               0,               0,               0,      -2.1179562,        4.466888,      0.74464422 ],
  [               0,               0,               0,      0.74425834,      -1.1640966,     -0.88453072 ],
  [               0,               0,               0,       3.1749866,      -3.5126832,        1.069416 ],
  [               0,               0,               0,      0.02177004,    -0.026557717,    0.0082743019 ],
  [               0,               0,               0,       -54.04129,       60.016815,      -15.737385 ],
  [               0,               0,               0,      -6.9335728,       14.044318,      -7.8805871 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,      0.00746794 ],
  [               0,               0,               0,               0,               0,       1.7848724 ],
  [               0,               0,               0,               0,               0,     -0.87981361 ],
  [               0,               0,               0,               0,               0,    -0.086403862 ],
  [               0,               0,               0,               0,               0,   0.00089218654 ],
  [               0,               0,               0,               0,               0,        1.187429 ],
  [               0,               0,               0,               0,               0,     -0.78656119 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.021649413,     0.059684094,    -0.029911799 ],
  [               0,               0,               0,       7.6246409,      -19.833441,       14.454478 ],
  [               0,               0,               0,      0.52920491,     -0.67002428,      -1.1754981 ],
  [               0,               0,               0,       2.0322714,      -3.8088534,       2.0952578 ],
  [               0,               0,               0,    -0.022138599,     0.040773522,     -0.01985213 ],
  [               0,               0,               0,      -8.5432034,       19.487934,      -10.687587 ],
  [               0,               0,               0,       -1.139637,       3.3583753,      -2.5230265 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     -0.71267247,     -0.29973087,      0.47670022,       1.8102669 ],
  [               0,               0,      -508.56509,        803.4903,       646.90991,      -1085.7642 ],
  [               0,               0,       268.09448,      -358.57425,      -508.70053,       679.89105 ],
  [               0,               0,       104.74524,      -143.62013,      -198.78703,       272.56265 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.078257248,     -0.10907812 ],
  [               0,               0,               0,               0,       237.23174,         354.194 ],
  [               0,               0,               0,               0,      -673.01117,       934.60919 ],
  [               0,               0,               0,               0,      -2.1825886,        4.986783 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele10_layer2_etaoffv3_55_e = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0075912634,     0.015295829 ],
  [               0,               0,               0,               0,      -7.5756946,       16.063869 ],
  [               0,               0,               0,               0,       7.3288541,      -12.312897 ],
  [               0,               0,               0,               0,      -3.8369305,       5.9768329 ],
  [               0,               0,               0,               0,    0.0074901818,    -0.010776109 ],
  [               0,               0,               0,               0,     -0.70966327,       2.2692988 ],
  [               0,               0,               0,               0,      -1.0009131,      0.73172843 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele25_layer2_etaoffv3_55_b = [
 [
  [               0,               0,               0,     0.046941973,    -0.010519748,    0.0073225196 ],
  [               0,               0,               0,       -16.67001,       4.1667066,       3.7126751 ],
  [               0,               0,               0,       6.0353284,      -2.9400792,      -1.0573288 ],
  [               0,               0,               0,      -2.1569343,       1.4774761,     -0.14894113 ],
  [               0,               0,               0,      0.00766443,    -0.003027929,   0.00079872168 ],
  [               0,               0,               0,      -5.8242741,      0.90928054,       0.8083328 ],
  [               0,               0,               0,       10.865144,      -11.525428,     -0.33960989 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   0.00081210013,      0.01248185,  -0.00071760861 ],
  [               0,               0,               0,       2.2059803,      -6.7829742,       6.7541647 ],
  [               0,               0,               0,        4.276618,      -5.6225071,      0.40701947 ],
  [               0,               0,               0,      -1.3941894,       2.2747946,     -0.75187624 ],
  [               0,               0,               0,    0.0082397927,    -0.012859874,    0.0053589279 ],
  [               0,               0,               0,      -3.1839983,       2.2727394,      0.65701103 ],
  [               0,               0,               0,      -15.654344,       12.550714,      -3.7728381 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.11698078,    -0.089082956 ],
  [               0,               0,               0,               0,        49.20977,      -35.843216 ],
  [               0,               0,               0,               0,       9.5550346,      -9.3615294 ],
  [               0,               0,               0,               0,       9.6499081,      -7.5120096 ],
  [               0,               0,               0,               0,     0.024307653,     -0.01743314 ],
  [               0,               0,               0,               0,      -29.703125,       24.729244 ],
  [               0,               0,               0,               0,       -55.80447,        36.39492 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0059276619,    0.0082585467,    0.0055854991 ],
  [               0,               0,               0,       5.5677714,       -11.54591,       8.9340792 ],
  [               0,               0,               0,       2.0399399,      -5.2561216,       1.8225913 ],
  [               0,               0,               0,     -0.08413697,       1.2330142,     -0.99642211 ],
  [               0,               0,               0,    0.0090479879,    -0.017910223,    0.0092145037 ],
  [               0,               0,               0,      0.76030457,      -3.8985229,       4.0020084 ],
  [               0,               0,               0,       20.641687,      -49.132641,       26.565302 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     -0.17216735,     0.003640987,      0.32750317 ],
  [               0,               0,               0,       62.249691,      -204.26654,       165.60532 ],
  [               0,               0,               0,       109.19464,      -250.19563,       136.80124 ],
  [               0,               0,               0,      0.96086103,      -39.077106,       51.732105 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.22543272,     -0.30504701 ],
  [               0,               0,               0,               0,      -2522.9663,       3631.1333 ],
  [               0,               0,               0,               0,      -35.221775,       47.181892 ],
  [               0,               0,               0,               0,      0.17531389,     0.061218914 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele25_layer2_etaoffv3_55_e = [
 [
  [               0,               0,       1.3928456,      -1.9682544,       -3.050488,       4.3162279 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,   0.00073115522 ],
  [               0,               0,               0,               0,               0,           0.001 ],
  [               0,               0,               0,               0,               0,      -1.8612366 ],
  [               0,               0,               0,               0,               0,      0.24670029 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,       1.1062567,      -1.6598538 ],
  [               0,               0,               0,               0,       101.90692,      -151.49249 ],
  [               0,               0,               0,               0,       77.226135,      -118.07364 ],
  [               0,               0,               0,               0,      -78.450684,       118.64229 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     -0.10533445,      0.35616609,     -0.29130128 ],
  [               0,               0,               0,       36.940304,      -125.33754,       108.85417 ],
  [               0,               0,               0,      -7.1522923,        24.10828,      -21.466576 ],
  [               0,               0,               0,       3.0742879,      -10.170813,          8.4066 ],
  [               0,               0,               0,     0.012416489,    -0.040614557,     0.033272397 ],
  [               0,               0,               0,        5.715961,      -33.467983,       42.049576 ],
  [               0,               0,               0,       84.551231,      -326.14615,       253.87891 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0065628234,    -0.017903212,     0.022004977 ],
  [               0,               0,               0,       3.2947996,      -15.604857,       20.127483 ],
  [               0,               0,               0,     -0.69874799,       3.4368389,      -5.2193155 ],
  [               0,               0,               0,     -0.40810686,       1.5636501,      -1.4600756 ],
  [               0,               0,               0,   -0.0079491353,     0.031304147,    -0.030651644 ],
  [               0,               0,               0,      -30.173925,       123.82858,      -125.95235 ],
  [               0,               0,               0,      -48.832272,       203.95663,      -215.70676 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0037533436,   -0.0057978085,     0.045681767 ],
  [               0,               0,               0,        11.05311,       -51.66378,       62.423138 ],
  [               0,               0,               0,       6.0938869,      -28.971466,       33.293335 ],
  [               0,               0,               0,       1.5361321,       15.684652,      -39.122166 ],
  [               0,               0,               0,     0.020629769,    0.0083387652,    -0.063507766 ],
  [               0,               0,               0,      -11.851832,       56.700516,      -67.791862 ],
  [               0,               0,               0,       14.152196,      -64.055229,       69.221786 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,       0.1268651 ],
  [               0,               0,               0,               0,               0,      0.61260015 ],
  [               0,               0,               0,               0,               0,     -0.55984038 ],
  [               0,               0,               0,               0,               0,     0.016736798 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0049259751,    -0.012174342 ],
  [               0,               0,               0,               0,      -6.0455599,      -5.6773171 ],
  [               0,               0,               0,               0,               0,     -0.40000001 ],
  [               0,               0,               0,               0,       13.941402,       26.713413 ],
  [               0,               0,               0,               0,        15.66595,        38.39238 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele50_layer2_etaoffv3_55_b = [
 [
  [               0,               0,               0,   -0.0055836504,     0.001439097,    0.0084260479 ],
  [               0,               0,               0,       1.1200368,     -0.25197345,       3.7814579 ],
  [               0,               0,               0,      -0.4185212,     0.060750589,      -1.3127302 ],
  [               0,               0,               0,       5.3557377,      -2.2153704,      0.21017833 ],
  [               0,               0,               0,     0.014489706,   -0.0068179774,    0.0016447918 ],
  [               0,               0,               0,       2.6137433,      -1.0103003,      0.13744605 ],
  [               0,               0,               0,      -45.011822,       18.942493,      -3.3667836 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0085560232,    0.0026530134 ],
  [               0,               0,               0,               0,      -3.5213962,       5.9107046 ],
  [               0,               0,               0,               0,      0.19296138,       -1.588769 ],
  [               0,               0,               0,               0,       0.2060058,   -0.0035199674 ],
  [               0,               0,               0,               0,  -0.00095787342,     0.001148076 ],
  [               0,               0,               0,               0,       -1.216946,       1.5145432 ],
  [               0,               0,               0,               0,      -3.6143823,       -6.689146 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,       0.1090119,    -0.081288271 ],
  [               0,               0,               0,               0,     -0.24049713,       4.1916738 ],
  [               0,               0,               0,               0,      0.94107753,       -2.194165 ],
  [               0,               0,               0,               0,       3.0664876,      -2.4026902 ],
  [               0,               0,               0,               0,    0.0066875531,   -0.0042025442 ],
  [               0,               0,               0,               0,       -22.78414,       19.220016 ],
  [               0,               0,               0,               0,      -49.802082,       38.029121 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,  -0.00023147152,    0.0019212796,    0.0070626521 ],
  [               0,               0,               0,       1.5082481,      -4.6503053,        6.338964 ],
  [               0,               0,               0,      0.40703475,      -1.1545616,     -0.68921047 ],
  [               0,               0,               0,      0.61871499,       2.4158549,      0.31225958 ],
  [               0,               0,               0,     0.011801759,    0.0099113621,    0.0066281054 ],
  [               0,               0,               0,     -0.15971079,      0.38515607,     -0.15581079 ],
  [               0,               0,               0,      0.22501424,     -0.55654806,      -2.7854598 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   -0.0040680775,     -0.03669152,     -0.01998909,       0.1164346 ],
  [               0,               0,       553.07428,       -761.5614,      -1053.1566,        1452.627 ],
  [               0,               0,      -155.06541,       221.80888,       276.68073,      -397.94882 ],
  [               0,               0,      -42.854019,       55.688858,       75.815346,        -97.9972 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,       0.3524183,     -0.49086842 ],
  [               0,               0,               0,               0,      -13704.146,       19700.848 ],
  [               0,               0,               0,               0,      -99.736374,        136.6273 ],
  [               0,               0,               0,               0,     0.034391686,      0.54266459 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele50_layer2_etaoffv3_55_e = [
 [
  [               0,               0,      0.65125734,     -0.89320976,      -1.4878814,       2.0504551 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0017043849,    -0.003551696,   -0.0045810845,      0.01151061 ],
  [               0,               0,      -370.80356,       543.67902,       839.60889,      -1227.3976 ],
  [               0,               0,      -122.69881,       185.17397,       255.44853,      -386.80838 ],
  [               0,               0,      -38.211529,       65.794075,       87.591019,      -149.91856 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,       0.1167916,     -0.17323565 ],
  [               0,               0,               0,               0,      -191.25793,       293.29385 ],
  [               0,               0,               0,               0,      -72.188728,       108.18667 ],
  [               0,               0,               0,               0,       62.388622,      -94.215096 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.096058615,     -0.32197791,      0.27739054 ],
  [               0,               0,               0,      -21.947111,       73.145569,      -57.463779 ],
  [               0,               0,               0,       6.9909186,      -24.380035,        19.92808 ],
  [               0,               0,               0,    -0.078189284,       1.1121541,      -1.5999022 ],
  [               0,               0,               0,     0.035185363,      -0.1163229,     0.096446455 ],
  [               0,               0,               0,       -1.348105,        1.954808,       1.2064246 ],
  [               0,               0,               0,       18.361662,       -72.58223,       68.275635 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.080264769,     -0.32588503,      0.34007657 ],
  [               0,               0,               0,      -9.5198212,       37.428356,      -34.080894 ],
  [               0,               0,               0,       3.3448064,      -13.514761,       12.395406 ],
  [               0,               0,               0,      -1.5636593,       6.2656174,      -6.1773357 ],
  [               0,               0,               0,    -0.013013159,     0.051621895,    -0.050567102 ],
  [               0,               0,               0,      -13.031173,       54.362991,      -56.199429 ],
  [               0,               0,               0,      -21.745218,       86.767845,      -89.104164 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0072899419,    -0.075008661,      0.14802504 ],
  [               0,               0,               0,       54.988678,      -257.33356,       303.18054 ],
  [               0,               0,               0,      -1.2044148,       4.9628453,      -6.1418719 ],
  [               0,               0,               0,     0.058839448,       31.837265,      -78.402092 ],
  [               0,               0,               0,        0.118044,    -0.022656964,     -0.65550649 ],
  [               0,               0,               0,      0.56648153,      -2.5814223,       2.9387798 ],
  [               0,               0,               0,       2.1262543,       -13.51261,       16.069948 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.30231458,      0.76295662 ],
  [               0,               0,               0,               0,      -12.318874,       32.387791 ],
  [               0,               0,               0,               0,       22.017136,      -55.428944 ],
  [               0,               0,               0,               0,      -18.191549,       44.943996 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0055295248,    -0.013705546 ],
  [               0,               0,               0,               0,      -4.3818364,       1.9908174 ],
  [               0,               0,               0,               0,   3.9635415e-05,     -0.42718172 ],
  [               0,               0,               0,               0,       168.15211,       3.6498992 ],
  [               0,               0,               0,               0,       70.051537,       173.03514 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele75_layer2_etaoffv3_55_b = [
 [
  [               0,               0,               0,    -0.012419532,    0.0023951689,    0.0072094435 ],
  [               0,               0,               0,       7.6306062,      -2.2070365,       4.4578743 ],
  [               0,               0,               0,       1.3569862,      -1.0237063,      -1.2487199 ],
  [               0,               0,               0,      -1.4266803,       0.8800925,    -0.089433923 ],
  [               0,               0,               0,    0.0041425121,   -0.0012229909,   0.00053640507 ],
  [               0,               0,               0,        0.070868,      0.35750073,      0.73154801 ],
  [               0,               0,               0,       19.685446,      -10.844016,     -0.54696822 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0056153783,   -0.0053916713,    0.0096521582 ],
  [               0,               0,               0,      -7.2904191,       7.2775683,       2.0658171 ],
  [               0,               0,               0,      -1.4318513,       1.4347045,        -1.69528 ],
  [               0,               0,               0,      -1.6569742,       4.4092603,      -1.5196561 ],
  [               0,               0,               0,    0.0096125584,    0.0017093041,   2.3581741e-05 ],
  [               0,               0,               0,      0.78732473,     -0.66657102,      0.16800573 ],
  [               0,               0,               0,       13.488319,      -20.016027,      -8.2025423 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.085705973,     -0.06078472 ],
  [               0,               0,               0,               0,       5.2791448,     -0.35081998 ],
  [               0,               0,               0,               0,       14.916129,      -13.403839 ],
  [               0,               0,               0,               0,       -23.89513,       21.103743 ],
  [               0,               0,               0,               0,     0.023437098,   -0.0035483364 ],
  [               0,               0,               0,               0,      -4.9413867,       4.0579014 ],
  [               0,               0,               0,               0,      0.48494342,      -3.4581449 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   0.00034474133,   -0.0019363624,     0.010119608 ],
  [               0,               0,               0,       0.7646758,      -2.2056091,       4.8448648 ],
  [               0,               0,               0,       1.3501164,      -3.2032659,       0.3778041 ],
  [               0,               0,               0,      0.27451086,       1.1652257,     -0.85671198 ],
  [               0,               0,               0,    0.0049224785,   0.00060380984,   -0.0014253467 ],
  [               0,               0,               0,     -0.79709947,       1.5122013,     -0.49799138 ],
  [               0,               0,               0,        1.016995,      -2.2875361,      -1.7874674 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   -0.0066779349,    -0.033087954,    -0.015098611,      0.10949051 ],
  [               0,               0,       438.84705,      -603.46838,      -834.50232,       1150.3188 ],
  [               0,               0,      -98.772301,       143.74377,       172.72324,      -253.79861 ],
  [               0,               0,      -74.128433,       98.493279,       134.93839,      -178.92714 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.50740153,     -0.69942033 ],
  [               0,               0,               0,               0,       261.64307,      -346.10538 ],
  [               0,               0,               0,               0,      -36.738297,       49.059483 ],
  [               0,               0,               0,               0,      0.70795292,     -0.32919538 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele75_layer2_etaoffv3_55_e = [
 [
  [               0,               0,       1.0279359,      -1.4225214,      -2.2674639,       3.1456301 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0056892065,    0.0041914382,    -0.002405962,    -0.019131763 ],
  [               0,               0,       14.672039,      -14.061628,      -28.071724,       27.229313 ],
  [               0,               0,      -43.964142,        62.74147,       93.715424,      -134.43599 ],
  [               0,               0,      -7.0707703,       12.272361,       16.806433,      -28.789774 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,    0.0060043274 ],
  [               0,               0,               0,               0,               0,       3.4458547 ],
  [               0,               0,               0,               0,               0,      -1.2494185 ],
  [               0,               0,               0,               0,               0,      0.11498272 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.022614952,     0.059823424,    -0.029781796 ],
  [               0,               0,               0,       20.692734,      -64.362396,       53.532444 ],
  [               0,               0,               0,       1.5910388,      -5.4030323,       3.2887762 ],
  [               0,               0,               0,      0.62706745,      -2.2058134,       1.9786587 ],
  [               0,               0,               0,     0.021824727,    -0.070137575,     0.056820765 ],
  [               0,               0,               0,      -6.0362868,       22.634907,      -20.220739 ],
  [               0,               0,               0,      -12.176076,       44.074619,      -47.617054 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.031850532,     -0.11882292,      0.12006124 ],
  [               0,               0,               0,       1.3371327,      -8.8656082,       15.119208 ],
  [               0,               0,               0,      0.54883277,      -1.7141415,    -0.010930114 ],
  [               0,               0,               0,      -3.1062665,       12.443521,      -12.371446 ],
  [               0,               0,               0,     0.039212637,     -0.15742165,      0.15742639 ],
  [               0,               0,               0,      -3.2904613,       15.695569,      -18.591324 ],
  [               0,               0,               0,       26.616653,      -110.65194,       114.22417 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,      0.01802977,   -0.0021447029,    -0.080897301 ],
  [               0,               0,               0,       58.552563,      -287.30228,       353.91763 ],
  [               0,               0,               0,      -17.827591,       87.162834,      -107.52765 ],
  [               0,               0,               0,       4.2234402,      -20.423416,       24.714155 ],
  [               0,               0,               0,    0.0011754542,    -0.013002829,     0.024313938 ],
  [               0,               0,               0,       1.1444592,       8.9934597,      -26.345472 ],
  [               0,               0,               0,       4.9789886,      -32.741234,       45.741158 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.012687846,     0.043847702 ],
  [               0,               0,               0,               0,      -94.441666,       235.01775 ],
  [               0,               0,               0,               0,       34.361534,      -85.883499 ],
  [               0,               0,               0,               0,      -13.420498,       33.266857 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   0.00068592798,     0.012011858,    -0.034002405 ],
  [               0,               0,               0,      -2.0299294,       8.8010302,       4.1350121 ],
  [               0,               0,               0,       59998.742,        49773515,        18051391 ],
  [               0,               0,               0,        36.49791,       43.389404,      -137.74527 ],
  [               0,               0,               0,       17.889259,       25.608965,      -11.267609 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele100_layer2_etaoffv3_55_b = [
 [
  [               0,               0,               0,    -0.058440022,     0.026345961,    0.0052667907 ],
  [               0,               0,               0,       18.963688,      -8.6599598,       5.0838847 ],
  [               0,               0,               0,     -0.61312252,      0.27593309,      -1.3735085 ],
  [               0,               0,               0,     -0.18051142,      0.10874522,    -0.014710248 ],
  [               0,               0,               0,     0.019686999,    -0.009054211,    0.0011382353 ],
  [               0,               0,               0,       9.5272655,       -4.268033,       1.1772224 ],
  [               0,               0,               0,       2.2984724,     -0.74380821,      -1.5673399 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0049331216,   -0.0064959475,     0.010222296 ],
  [               0,               0,               0,       -6.551106,       6.8844438,       2.2781217 ],
  [               0,               0,               0,      -1.4185894,       1.1395965,      -1.5707819 ],
  [               0,               0,               0,      0.52434325,     -0.15660627,    -0.020220775 ],
  [               0,               0,               0,    0.0023430279,   -0.0039818445,    0.0011071912 ],
  [               0,               0,               0,       2.2618306,     -0.92238861,      0.20052792 ],
  [               0,               0,               0,       12.522064,      -18.462299,      -5.3897486 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.035268206,    -0.019863622 ],
  [               0,               0,               0,               0,       13.751793,      -7.2322125 ],
  [               0,               0,               0,               0,       6.3309002,      -6.5379124 ],
  [               0,               0,               0,               0,      -5.7187872,       4.6880312 ],
  [               0,               0,               0,               0,     0.014169994,    -0.012258376 ],
  [               0,               0,               0,               0,     -0.79860938,       1.3057426 ],
  [               0,               0,               0,               0,       -2.683208,       2.3123085 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   0.00027238941,  -0.00036624703,    0.0086412057 ],
  [               0,               0,               0,       1.3903824,      -3.9783053,         6.08143 ],
  [               0,               0,               0,      0.54465121,      -1.5336064,     -0.45077163 ],
  [               0,               0,               0,      0.57173944,          1.1899,      -1.2320256 ],
  [               0,               0,               0,    0.0079209739,    0.0010960588,   -0.0052056261 ],
  [               0,               0,               0,   -0.0012433321,     -0.14708498,      0.33801144 ],
  [               0,               0,               0,      0.67008281,      -1.7629473,      -1.9346271 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    -0.013497811,    -0.020615233,   0.00033127455,     0.084072441 ],
  [               0,               0,       294.49069,      -406.93372,      -561.94702,       778.97168 ],
  [               0,               0,      -93.941139,       135.22704,       169.06189,      -245.17245 ],
  [               0,               0,      -30.672129,       38.981541,       54.200153,      -68.402382 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.70790678,     -0.98595637 ],
  [               0,               0,               0,               0,      -210.99615,       347.48828 ],
  [               0,               0,               0,               0,      -42.697929,       56.958569 ],
  [               0,               0,               0,               0,      0.37140936,      0.32968521 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele100_layer2_etaoffv3_55_e = [
 [
  [               0,               0,      0.52062684,     -0.69054443,      -1.2033935,        1.610781 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.018672727,   -0.0089695444,    -0.026053898,    0.0027658625 ],
  [               0,               0,      -1064.6146,       1557.8757,       2361.4983,      -3453.0889 ],
  [               0,               0,       28.156021,      -45.610001,      -56.062286,       90.734955 ],
  [               0,               0,       85.371124,      -124.84866,      -183.24773,       267.94431 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0023168761,    0.0052016834 ],
  [               0,               0,               0,               0,       26.648993,      -37.609238 ],
  [               0,               0,               0,               0,      -5.8841805,       7.7338204 ],
  [               0,               0,               0,               0,      -1.7899024,       2.7810686 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.012603488,    -0.051288433,     0.058564164 ],
  [               0,               0,               0,      -3.5173876,        12.65919,      -7.6971035 ],
  [               0,               0,               0,       3.6305835,       -12.36293,       9.1665525 ],
  [               0,               0,               0,      -1.9317738,       6.5612144,       -5.486165 ],
  [               0,               0,               0,   -0.0031176505,     0.012809131,    -0.012300215 ],
  [               0,               0,               0,      -8.6288738,        31.68162,      -28.176071 ],
  [               0,               0,               0,      -4.5883312,       21.104698,      -31.016426 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0014939257,    0.0063065137 ],
  [               0,               0,               0,               0,      -2.0045056,       6.9256568 ],
  [               0,               0,               0,               0,      0.30447277,      -1.8645173 ],
  [               0,               0,               0,               0,      -9.3530693,       16.974062 ],
  [               0,               0,               0,               0,      0.02854987,   -0.0014821327 ],
  [               0,               0,               0,               0,    0.0024781043,   -0.0027908648 ],
  [               0,               0,               0,               0,       1.5280819,      -4.3250966 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0069443849,    0.0027004336,     0.042938724 ],
  [               0,               0,               0,       45.247181,      -211.30492,       249.06714 ],
  [               0,               0,               0,      -4.7098284,       21.433277,       -25.53042 ],
  [               0,               0,               0,      -17.671534,       69.560326,      -75.104797 ],
  [               0,               0,               0,    0.0043189703,     0.019001858,     0.045420844 ],
  [               0,               0,               0,      0.29731202,      -1.4328371,       1.7241791 ],
  [               0,               0,               0,      -1.9110467,       12.032457,      -17.248188 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.43461221,       1.0797718 ],
  [               0,               0,               0,               0,       21.706734,      -50.954662 ],
  [               0,               0,               0,               0,      -21.730099,       52.222855 ],
  [               0,               0,               0,               0,       28.421755,      -69.706047 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.001539881,   -0.0038143301 ],
  [               0,               0,               0,               0,      -11.335269,      -11.688085 ],
  [               0,               0,               0,               0,      -528.32153,       3356074.2 ],
  [               0,               0,               0,               0,       573.32434,        478.6962 ],
  [               0,               0,               0,               0,       180.29138,       667.37292 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele200_layer2_etaoffv3_55_b = [
 [
  [               0,               0,               0,    -0.035718646,     0.016448624,    0.0066010873 ],
  [               0,               0,               0,       15.456675,      -6.8659086,       4.9704809 ],
  [               0,               0,               0,     -0.46575329,      0.23766652,      -1.3657993 ],
  [               0,               0,               0,     -0.39892778,      0.13009834,    -0.017971005 ],
  [               0,               0,               0,  -0.00027487607,   -0.0017154058,   0.00045535099 ],
  [               0,               0,               0,       9.7238455,      -8.1535196,       1.2258413 ],
  [               0,               0,               0,      -10.549534,       2.9878027,       -1.763045 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.011331962,    -0.019081205,     0.014720155 ],
  [               0,               0,               0,      -6.3561678,       8.5238285,       1.6547617 ],
  [               0,               0,               0,      -2.2705469,       2.1270702,      -1.8259317 ],
  [               0,               0,               0,       1.8507429,      -1.8340541,      0.44423473 ],
  [               0,               0,               0,   -0.0029591427,    0.0056183538,   -0.0019059789 ],
  [               0,               0,               0,      -1.7566584,       2.7923868,     -0.19813271 ],
  [               0,               0,               0,       8.3037529,      -14.223573,       4.0075812 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.010366442,   -0.0026958061 ],
  [               0,               0,               0,               0,        20.45163,       -11.36322 ],
  [               0,               0,               0,               0,       4.0213461,      -4.6721306 ],
  [               0,               0,               0,               0,      -4.3591952,       3.4965198 ],
  [               0,               0,               0,               0,    0.0037620894,    -0.001931269 ],
  [               0,               0,               0,               0,       1.3201075,    -0.037126936 ],
  [               0,               0,               0,               0,      -3.0139349,      0.59240013 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0084982915,     0.018591428,   -0.0015634818 ],
  [               0,               0,               0,       3.6275244,       -8.542881,       8.5576363 ],
  [               0,               0,               0,      0.77867132,      -1.8706752,     -0.35125712 ],
  [               0,               0,               0,     0.046054412,      0.41801813,     -0.35298562 ],
  [               0,               0,               0,    0.0027919067,   -0.0039410037,    0.0016187752 ],
  [               0,               0,               0,     -0.97094887,      0.56753743,       1.1522233 ],
  [               0,               0,               0,       8.4874687,      -19.561913,        8.350399 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   0.00080457411,    -0.045566853,    -0.030667735,      0.13542666 ],
  [               0,               0,        660.4986,      -907.79553,      -1249.0421,       1719.5972 ],
  [               0,               0,      -69.807472,       106.74615,       109.91345,      -172.79095 ],
  [               0,               0,       -71.82196,       95.632507,       130.12315,      -172.93774 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.97904032,      -1.3646872 ],
  [               0,               0,               0,               0,       50.289627,       14.811093 ],
  [               0,               0,               0,               0,      -38.302914,       50.694866 ],
  [               0,               0,               0,               0,       0.4558523,       0.4345493 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele200_layer2_etaoffv3_55_e = [
 [
  [               0,               0,       0.5190438,     -0.68206906,      -1.1856292,        1.572698 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   -0.0047051776,   -0.0061129453,    0.0020788189,     0.034184471 ],
  [               0,               0,      -194.55865,       292.63702,       436.04474,      -653.29663 ],
  [               0,               0,       38.490833,      -60.009426,       -82.26181,       127.41269 ],
  [               0,               0,       8.1571941,      -11.730886,       -16.74069,       24.087475 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.012365122,     0.021677842 ],
  [               0,               0,               0,               0,       157.48096,      -233.51283 ],
  [               0,               0,               0,               0,      -197.08289,       296.74084 ],
  [               0,               0,               0,               0,       178.05891,      -269.04169 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0026956084,   -0.0086342432,     0.015317437 ],
  [               0,               0,               0,      -1.8676288,       4.9899526,      0.63434249 ],
  [               0,               0,               0,     -0.25714353,      0.98037821,      -2.2735751 ],
  [               0,               0,               0,      -1.4767165,      0.13674089,       7.0615439 ],
  [               0,               0,               0,    -0.012520821,     0.002606974,     0.056999303 ],
  [               0,               0,               0,       1.4327657,      -4.8803616,       4.1890588 ],
  [               0,               0,               0,      -2.7332439,       9.5269756,      -11.287821 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0032233184,    -0.012637985,     0.022203349 ],
  [               0,               0,               0,       2.3754859,      -11.882056,       17.181486 ],
  [               0,               0,               0,     0.075736307,    -0.019873712,      -1.5507351 ],
  [               0,               0,               0,     -0.66607976,       3.6314244,      -4.4339662 ],
  [               0,               0,               0,    -0.001705226,   -0.0039125695,     0.013537982 ],
  [               0,               0,               0,       1.6165646,      -7.8312106,        9.494607 ],
  [               0,               0,               0,      -6.7326465,       30.137655,      -39.425083 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.004083313,  -0.00037645968,     0.035077725 ],
  [               0,               0,               0,       9.8477373,      -44.520256,       52.619888 ],
  [               0,               0,               0,       12.183997,      -58.540997,       69.090118 ],
  [               0,               0,               0,      -7.9708843,       38.571129,      -46.569508 ],
  [               0,               0,               0,    0.0029182134,   -0.0052090921,   -0.0037785268 ],
  [               0,               0,               0,      -1.5687966,     -0.15115912,       9.7258186 ],
  [               0,               0,               0,       3.8764586,      0.50605041,      -26.106117 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0034195343,    0.0058783963 ],
  [               0,               0,               0,               0,      -49.222275,       123.33144 ],
  [               0,               0,               0,               0,       7.6755891,       -20.14559 ],
  [               0,               0,               0,               0,       1.1600654,      -2.6633804 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0025773265,   -0.0063834665 ],
  [               0,               0,               0,               0,      -10.735966,      -13.323017 ],
  [               0,               0,               0,               0,      -16.356407,      -411363.44 ],
  [               0,               0,               0,               0,       514.37036,       173.80659 ],
  [               0,               0,               0,               0,       160.93422,       402.54111 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele500_layer2_etaoffv3_55_b = [
 [
  [               0,               0,               0,    -0.045373429,     0.019728661,     0.006006489 ],
  [               0,               0,               0,       20.704252,       -9.228322,       5.6777191 ],
  [               0,               0,               0,     -0.67257684,      0.34400761,      -1.3908995 ],
  [               0,               0,               0,     0.060912743,    -0.051618323,   -0.0069892113 ],
  [               0,               0,               0,     0.013984593,   -0.0061705727,   0.00071623648 ],
  [               0,               0,               0,       31.035801,      -13.144604,       1.4294014 ],
  [               0,               0,               0,      -3.1385043,       1.4995855,      -1.7810237 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.019735448,     0.027534921,   -0.0022139559 ],
  [               0,               0,               0,       5.6321497,      -9.3360758,       8.3432188 ],
  [               0,               0,               0,      -2.4248042,       2.7708912,      -2.1419337 ],
  [               0,               0,               0,        1.424583,      -1.5084912,      0.38187584 ],
  [               0,               0,               0,    0.0077101584,    -0.010332892,    0.0038678271 ],
  [               0,               0,               0,      -2.7233083,       2.9090102,      0.24636874 ],
  [               0,               0,               0,      -9.1602592,       10.127395,      -4.3652015 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.011639907,     0.018170191 ],
  [               0,               0,               0,               0,       18.593155,      -10.822879 ],
  [               0,               0,               0,               0,       3.8236613,      -4.3316307 ],
  [               0,               0,               0,               0,      -3.6101899,        2.838181 ],
  [               0,               0,               0,               0,   -0.0026329581,    0.0021062922 ],
  [               0,               0,               0,               0,       64.875046,      -39.329418 ],
  [               0,               0,               0,               0,       47.776058,      -38.668224 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   0.00073156081,   -0.0015002217,    0.0092342738 ],
  [               0,               0,               0,      0.70287114,      -2.2882442,       5.5238595 ],
  [               0,               0,               0,       0.4651368,      -1.2947106,     -0.54962319 ],
  [               0,               0,               0,      0.32688001,      0.87069857,     -0.94229078 ],
  [               0,               0,               0,    0.0050139129,   0.00076023355,   -0.0038201481 ],
  [               0,               0,               0,     -0.47880051,       1.0078425,     -0.35394284 ],
  [               0,               0,               0,       1.7476059,      -4.3543029,      -6.6130581 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.010195198,     -0.04848228,    -0.041608702,      0.13179417 ],
  [               0,               0,       658.36438,      -908.20209,      -1247.7629,       1724.2671 ],
  [               0,               0,      -88.710854,       133.91965,        144.1124,      -222.02769 ],
  [               0,               0,      -43.153549,       56.203274,       75.706581,       -98.13311 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,       3.1928632,      -4.4540992 ],
  [               0,               0,               0,               0,       230.59003,        35.90979 ],
  [               0,               0,               0,               0,      -13.656695,       16.065247 ],
  [               0,               0,               0,               0,      0.59170127,       0.5730682 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele500_layer2_etaoffv3_55_e = [
 [
  [      0.10568984,   -0.0083606075,     -0.14998166,     -0.28475124,     -0.32864875,      0.89407086 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,       8.5804014,      -12.823335,      -18.768145,       28.055319 ],
  [               0,               0,       525.95282,       -743.7392,      -1118.8085,       1582.9368 ],
  [               0,               0,       126.35229,      -193.72542,       -280.6026,        428.8161 ],
  [               0,               0,      -31.147619,       47.453182,       69.943039,      -106.37272 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.036389183,    -0.050492257 ],
  [               0,               0,               0,               0,       144.35716,      -214.02245 ],
  [               0,               0,               0,               0,      -100.07172,        150.0287 ],
  [               0,               0,               0,               0,       86.711632,       -131.0284 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0073742792,    -0.023995973,     0.028110923 ],
  [               0,               0,               0,      -1.0251611,       1.6006099,       4.1269641 ],
  [               0,               0,               0,      -1.4091469,       4.7839646,      -5.4132872 ],
  [               0,               0,               0,      -1.4445766,     -0.20046121,       6.7816181 ],
  [               0,               0,               0,    -0.014033312,    0.0046794699,     0.051916607 ],
  [               0,               0,               0,       2.0115869,      -6.6330981,       5.4863448 ],
  [               0,               0,               0,      -4.2867308,       14.568152,      -15.335172 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0027907239,   -0.0094027808,     0.016773436 ],
  [               0,               0,               0,      0.89435929,       -5.950984,       11.547205 ],
  [               0,               0,               0,       1.7627989,      -7.0832391,       5.7571979 ],
  [               0,               0,               0,      -1.0453594,     -0.65107673,       7.2766294 ],
  [               0,               0,               0,    -0.011191873,    0.0032250094,     0.054120257 ],
  [               0,               0,               0,      -0.5178166,       2.3127806,      -2.4607866 ],
  [               0,               0,               0,       1.8936425,      -7.4452128,       4.2516031 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   0.00067265576,    -0.014941828,     0.042071048 ],
  [               0,               0,               0,       20.105804,      -94.653976,       113.96567 ],
  [               0,               0,               0,      0.42919129,       -2.442091,       2.1503482 ],
  [               0,               0,               0,      -1.0022324,       16.943062,      -32.125179 ],
  [               0,               0,               0,     0.011285599,   -0.0055635115,     -0.02285929 ],
  [               0,               0,               0,       1.1347177,      -5.2033839,       5.9691138 ],
  [               0,               0,               0,      -5.5269046,       21.485125,      -22.553234 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.016699819,    -0.023030341 ],
  [               0,               0,               0,               0,      -21.981005,       56.520908 ],
  [               0,               0,               0,               0,       38.000076,      -94.887268 ],
  [               0,               0,               0,               0,      -32.389156,       79.910332 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.082044639,     0.057736125 ],
  [               0,               0,               0,               0,      -20.560999,       52.464203 ],
  [               0,               0,               0,               0,               0,     -0.40000001 ],
  [               0,               0,               0,               0,      -36.598011,       89.585609 ],
  [               0,               0,               0,               0,       0.1020441,     -0.23859331 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele1000_layer2_etaoffv3_55_b = [
 [
  [               0,               0,               0,   -0.0029779426,    0.0022948568,    0.0076665343 ],
  [               0,               0,               0,       5.3812218,       -3.537508,       5.3355885 ],
  [               0,               0,               0,     -0.20254453,      0.15550686,      -1.3755631 ],
  [               0,               0,               0,   -0.0074340524,    -0.016799418,    -0.014898331 ],
  [               0,               0,               0,   -0.0047239689,    0.0019024046,   -0.0001669347 ],
  [               0,               0,               0,      -44.862316,       17.320555,       1.3014297 ],
  [               0,               0,               0,       15.461471,      -23.450006,     -0.14523634 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0032330453,   -0.0034470884,    0.0090614315 ],
  [               0,               0,               0,      -2.5092018,       2.0351126,       4.3333278 ],
  [               0,               0,               0,      0.32851717,     -0.61475146,      -1.0716397 ],
  [               0,               0,               0,       2.0565567,      -1.0548571,      -1.3909045 ],
  [               0,               0,               0,   -0.0081265178,   -4.003516e-05,     0.013243919 ],
  [               0,               0,               0,      0.53689027,     -0.88331199,      0.38523501 ],
  [               0,               0,               0,     -0.11320733,     -0.43546724,      0.23668686 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.023190755,     0.027204899 ],
  [               0,               0,               0,               0,       23.265959,      -14.244079 ],
  [               0,               0,               0,               0,       3.4212606,      -4.0000997 ],
  [               0,               0,               0,               0,       -3.747927,       2.9334676 ],
  [               0,               0,               0,               0,    0.0037770607,   -0.0029599119 ],
  [               0,               0,               0,               0,       1.8477958,      0.76586401 ],
  [               0,               0,               0,               0,      -3.9245379,     -0.76143497 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,    0.0075252224 ],
  [               0,               0,               0,               0,               0,       4.2364836 ],
  [               0,               0,               0,               0,               0,      -1.3885731 ],
  [               0,               0,               0,               0,               0,     0.036216903 ],
  [               0,               0,               0,               0,               0,   0.00044223043 ],
  [               0,               0,               0,               0,               0,      0.94140106 ],
  [               0,               0,               0,               0,               0,      -20.665421 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.017003665,     -0.05115759,    -0.049786896,       0.1304916 ],
  [               0,               0,       661.87201,      -915.16669,      -1253.3217,       1736.0553 ],
  [               0,               0,      -77.099068,       119.01849,       119.87452,      -190.71638 ],
  [               0,               0,      -43.800842,       57.259468,       76.801193,      -99.964142 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.64617616,     -0.90078872 ],
  [               0,               0,               0,               0,       370.15918,      -473.56384 ],
  [               0,               0,               0,               0,      -55.959248,       75.627922 ],
  [               0,               0,               0,               0,      0.31195176,      0.28569755 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele1000_layer2_etaoffv3_55_e = [
 [
  [               0,       320.32877,      -921.73962,       15.503384,       1863.4478,      -1340.5668 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,    -0.011772962,     0.002774199,     0.017670354,     0.021639878,    -0.011105894 ],
  [               0,      -98.597603,       75.487526,       222.08479,        163.0358,      -497.05142 ],
  [               0,       136.44327,      -103.30817,      -297.68332,      -214.19783,       648.16174 ],
  [               0,      -78.139076,       57.587269,       170.66298,       126.40734,       -372.5668 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.017626738,     -0.02255128 ],
  [               0,               0,               0,               0,       177.57076,      -263.95392 ],
  [               0,               0,               0,               0,      -148.85336,       223.76923 ],
  [               0,               0,               0,               0,        135.1161,      -204.21793 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0023674348,     0.010490474,   -0.0023583705 ],
  [               0,               0,               0,       3.9000187,      -16.099106,       20.051395 ],
  [               0,               0,               0,       2.1685829,      -7.2680111,       4.6623969 ],
  [               0,               0,               0,      -1.0185249,      -1.4515955,       6.8106179 ],
  [               0,               0,               0,    -0.012577269,    0.0033164618,     0.042173587 ],
  [               0,               0,               0,      -1.6349238,        5.762002,      -4.9627128 ],
  [               0,               0,               0,       5.5892215,      -18.813835,       12.786831 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0011014763,   -0.0059464145,     0.016999396 ],
  [               0,               0,               0,      0.14175218,      -1.9976318,       6.6556106 ],
  [               0,               0,               0,      0.14858156,     -0.34326133,      -1.2575895 ],
  [               0,               0,               0,      -10.800904,       21.813063,       9.2863722 ],
  [               0,               0,               0,     -0.05671265,    0.0091621112,      0.30025187 ],
  [               0,               0,               0,     0.079261564,     -0.34377375,      0.40292177 ],
  [               0,               0,               0,       1.1774035,      -4.4809852,       1.1367383 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.003569094,    -0.055234067,      0.11856435 ],
  [               0,               0,               0,       49.795193,      -232.33504,       273.83783 ],
  [               0,               0,               0,       26.672403,      -125.75481,       146.90228 ],
  [               0,               0,               0,       -16.61784,       78.527702,      -92.667282 ],
  [               0,               0,               0,    0.0085061621,    -0.031035341,     0.026631828 ],
  [               0,               0,               0,      -54.455498,       260.93933,      -311.59317 ],
  [               0,               0,               0,       24.322836,      -108.54614,       118.85003 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.27758798,      0.69008625 ],
  [               0,               0,               0,               0,      -277.98013,       687.76208 ],
  [               0,               0,               0,               0,       9.8685455,      -25.209335 ],
  [               0,               0,               0,               0,        60.88446,      -149.54062 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0034838682,   -0.0086280033 ],
  [               0,               0,               0,               0,      -11.267318,      -11.756555 ],
  [               0,               0,               0,               0,      -5433.8086,       3623993.2 ],
  [               0,               0,               0,               0,       381.35217,       99.005554 ],
  [               0,               0,               0,               0,       137.60776,       170.56635 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]


ele_layer2_etaoffv3_55_b = [
    ele5_layer2_etaoffv3_55_b,
    ele10_layer2_etaoffv3_55_b,
    ele25_layer2_etaoffv3_55_b,
    ele50_layer2_etaoffv3_55_b,
    ele75_layer2_etaoffv3_55_b,
    ele100_layer2_etaoffv3_55_b,
    ele200_layer2_etaoffv3_55_b,
    ele500_layer2_etaoffv3_55_b,
    ele1000_layer2_etaoffv3_55_b,
    ]

ele_layer2_etaoffv3_55_e = [
    ele5_layer2_etaoffv3_55_e,
    ele10_layer2_etaoffv3_55_e,
    ele25_layer2_etaoffv3_55_e,
    ele50_layer2_etaoffv3_55_e,
    ele75_layer2_etaoffv3_55_e,
    ele100_layer2_etaoffv3_55_e,
    ele200_layer2_etaoffv3_55_e,
    ele500_layer2_etaoffv3_55_e,
    ele1000_layer2_etaoffv3_55_e,
    ]


###############################################################################
# Tables for v3 sshape correction, layer 2, 5x5 photon clusters
#

gam5_layer2_etaoffv3_55_b = [
 [
  [               0,               0,               0,      0.12570305,     -0.10376265,     0.024374546 ],
  [               0,               0,               0,        19.47975,       2.0897017,      0.56254029 ],
  [               0,               0,               0,       7.8577147,      -5.3103576,     -0.40859526 ],
  [               0,               0,               0,       0.8084656,       0.3753868,    -0.011173123 ],
  [               0,               0,               0,    0.0029913024,   -0.0014235053,   0.00084042718 ],
  [               0,               0,               0,      -5.2924213,      -2.8037939,       1.5695468 ],
  [               0,               0,               0,       25.303343,      -16.446495,     -0.78460991 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,      0.10191956,      -0.1421953,     0.051715508 ],
  [               0,               0,               0,       53.175919,      -45.318283,       11.271863 ],
  [               0,               0,               0,      -1.3003792,     -0.30187505,     -0.63518959 ],
  [               0,               0,               0,       1.5205704,      -3.5313127,       2.1681066 ],
  [               0,               0,               0,      0.11914721,     -0.16461018,     0.057441752 ],
  [               0,               0,               0,      -18.262014,       17.588753,      -4.1676745 ],
  [               0,               0,               0,       28.742947,      -28.875368,       3.7476721 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0056826281,      0.12948552 ],
  [               0,               0,               0,               0,       16.413895,      -12.709696 ],
  [               0,               0,               0,               0,      -14.704931,       11.356423 ],
  [               0,               0,               0,               0,     -0.86838812,       2.0382764 ],
  [               0,               0,               0,               0,     0.093035273,     -0.24224699 ],
  [               0,               0,               0,               0,      -4.8089519,       3.8429298 ],
  [               0,               0,               0,               0,     -0.84271264,      0.72595394 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
    [0, 0, 0, 0, 0, 0], # Removed by hand, see below
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam5_layer2_etaoffv3_55_e = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam10_layer2_etaoffv3_55_b = [
 [
  [               0,               0,               0,   -0.0020664781,      0.01267682,    0.0079529099 ],
  [               0,               0,               0,      0.65749526,      -2.4713964,       3.0933645 ],
  [               0,               0,               0,        4.373239,      -1.3963215,      -1.1580642 ],
  [               0,               0,               0,      -1.8460844,      0.68997848,    -0.033143505 ],
  [               0,               0,               0,    0.0040128399,   -0.0046699471,   0.00056643644 ],
  [               0,               0,               0,      -9.6392307,       6.2400794,     0.034533441 ],
  [               0,               0,               0,      -75.633179,       36.537884,      -5.5604987 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.010601449,    -0.011612462,      0.01102163 ],
  [               0,               0,               0,      -7.6436634,        7.346355,       1.2613587 ],
  [               0,               0,               0,       2.5647466,      -3.0723653,     -0.39986116 ],
  [               0,               0,               0,      -2.0182018,       2.7744422,     -0.84302515 ],
  [               0,               0,               0,   -0.0074045914,    0.0065346053,   -0.0010631477 ],
  [               0,               0,               0,       55.177746,      -53.990475,       13.307181 ],
  [               0,               0,               0,       2.3954675,      -21.497551,       2.4656107 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.12709311,      0.10604672 ],
  [               0,               0,               0,               0,       133.69229,      -102.74131 ],
  [               0,               0,               0,               0,      -23.193953,       17.039833 ],
  [               0,               0,               0,               0,      -14.519671,       11.441028 ],
  [               0,               0,               0,               0,     0.098723747,    -0.077737913 ],
  [               0,               0,               0,               0,       29.699915,      -23.497486 ],
  [               0,               0,               0,               0,      -45.629223,       23.360247 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,      0.02089042,    -0.028942136,    0.0099089742 ],
  [               0,               0,               0,        3.164253,      -11.563024,       12.438294 ],
  [               0,               0,               0,       -9.903039,       23.482395,      -14.981807 ],
  [               0,               0,               0,       6.0047579,      -14.433335,       8.7791672 ],
  [               0,               0,               0,     0.008439078,    -0.023102839,     0.016362656 ],
  [               0,               0,               0,       1.4693826,      -2.9355547,       2.5188968 ],
  [               0,               0,               0,       11.503361,      -19.848635,       6.9930606 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0059437859,     -0.13560788,    -0.087076329,       0.3645843 ],
  [               0,               0,       3645.8386,       -5773.144,      -4355.8623,       7433.6084 ],
  [               0,               0,       99.359062,      -133.17773,      -177.56029,       236.43954 ],
  [               0,               0,      -44.628826,       55.252525,       78.690147,      -96.631119 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.29169637,      0.42778394 ],
  [               0,               0,               0,               0,      -8682.0391,       12465.298 ],
  [               0,               0,               0,               0,      -2.9647017,       2.7096274 ],
  [               0,               0,               0,               0,   -0.0028674048,     -0.28447145 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam10_layer2_etaoffv3_55_e = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.18761846,      0.28520575 ],
  [               0,               0,               0,               0,   7.4852157e+09,   1.1119532e+10 ],
  [               0,               0,               0,               0,      -459.27509,       690.37823 ],
  [               0,               0,               0,               0,        229.1738,      -345.04959 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0035746964,     0.012298349 ],
  [               0,               0,               0,               0,      -5.3014765,        12.26264 ],
  [               0,               0,               0,               0,       4.4197936,      -8.6744204 ],
  [               0,               0,               0,               0,      -1.6761082,       2.8486159 ],
  [               0,               0,               0,               0,     0.011471081,     -0.01928678 ],
  [               0,               0,               0,               0,     -0.27758086,       1.5612171 ],
  [               0,               0,               0,               0,     -0.97796303,       1.1397932 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam25_layer2_etaoffv3_55_b = [
 [
  [               0,               0,               0,    -0.015802432,     0.021016451,    0.0023526687 ],
  [               0,               0,               0,        18.74864,      -14.906534,       6.7562633 ],
  [               0,               0,               0,      -3.7637098,       2.2401173,      -1.6312916 ],
  [               0,               0,               0,       1.7722152,      -0.8773244,     0.084159181 ],
  [               0,               0,               0,    0.0083904713,   -0.0084631862,    0.0021938409 ],
  [               0,               0,               0,     -0.73064446,     -0.80892104,       1.1058131 ],
  [               0,               0,               0,     -0.84465092,       1.7319174,       -1.727403 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,      0.02025488,     -0.01972243,     0.013463335 ],
  [               0,               0,               0,      -9.1093779,       8.3853493,       1.6166091 ],
  [               0,               0,               0,       1.7925582,      -2.0808911,     -0.74305087 ],
  [               0,               0,               0,    -0.015997378,      0.21878417,    -0.076505601 ],
  [               0,               0,               0,     0.013699591,    -0.015922904,    0.0044708042 ],
  [               0,               0,               0,       5.8983741,       1.3596433,      -1.7803959 ],
  [               0,               0,               0,      -37.496243,       35.504936,      -13.028331 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.24893597,     -0.19240996 ],
  [               0,               0,               0,               0,      -87.873993,       74.682655 ],
  [               0,               0,               0,               0,       3.7990334,      -4.3838644 ],
  [               0,               0,               0,               0,       1.6329505,       -1.368978 ],
  [               0,               0,               0,               0,    -0.068584919,     0.055166017 ],
  [               0,               0,               0,               0,      -34.571194,        28.33761 ],
  [               0,               0,               0,               0,      -44.539673,      -41.299404 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0047170324,    0.0094088903,   -0.0010610893 ],
  [               0,               0,               0,       21.934486,      -47.456196,       29.956865 ],
  [               0,               0,               0,      -9.2188177,       19.165155,      -11.468602 ],
  [               0,               0,               0,       4.4436698,      -8.6650219,       4.4219189 ],
  [               0,               0,               0,  -0.00016213222,  -0.00047175289,     0.002238082 ],
  [               0,               0,               0,      -1.2474439,       2.6063571,     -0.31730732 ],
  [               0,               0,               0,       -6.423995,       13.766895,      -8.8732634 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0019081837,     -0.06510058,    -0.045387413,      0.18910757 ],
  [               0,               0,        970.7403,      -1329.9636,      -1849.3914,       2536.0747 ],
  [               0,               0,      -172.94585,       252.07709,       292.08182,      -429.79382 ],
  [               0,               0,      -119.66134,       161.51515,       218.38123,       -294.4577 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,       0.2889891,     -0.40023369 ],
  [               0,               0,               0,               0,       -5966.064,         8575.29 ],
  [               0,               0,               0,               0,      -77.248993,        105.7041 ],
  [               0,               0,               0,               0,      0.12532918,      0.10652179 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam25_layer2_etaoffv3_55_e = [
 [
  [               0,               0,       1.0443954,      -1.4387304,      -2.2957914,       3.1721261 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     -0.15384431,      0.22778118,      0.34297195,     -0.50668758 ],
  [               0,               0,        748.4942,      -1179.0204,      -1904.6868,       2981.9033 ],
  [               0,               0,       104.13048,      -112.95607,       -198.3196,       201.03819 ],
  [               0,               0,      -28.614908,       57.774906,       73.822296,       -142.6736 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.021633057,    0.0015817099 ],
  [               0,               0,               0,               0,       2.6914766,      -2.7642677 ],
  [               0,               0,               0,               0,       17.154537,      -26.871922 ],
  [               0,               0,               0,               0,      -19.372509,       29.324278 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.066264451,     -0.21692595,      0.18582749 ],
  [               0,               0,               0,       -8.442358,       25.837889,      -16.445225 ],
  [               0,               0,               0,       2.1955228,      -7.2591982,        4.724288 ],
  [               0,               0,               0,     -0.99737805,        3.446732,      -2.9384434 ],
  [               0,               0,               0,     0.018574003,    -0.064981073,     0.056470361 ],
  [               0,               0,               0,       -18.17573,       49.896053,      -32.834229 ],
  [               0,               0,               0,       112.72521,      -412.66983,       356.49588 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.001108376,    0.0047393693,   -0.0035915573 ],
  [               0,               0,               0,      0.95652479,      -6.7674737,       12.251308 ],
  [               0,               0,               0,     -0.12089611,      0.97973466,      -2.6729372 ],
  [               0,               0,               0,     -0.28764033,       1.1908277,      -1.2025847 ],
  [               0,               0,               0,     0.010025085,    -0.041345388,     0.042559188 ],
  [               0,               0,               0,       63.175339,      -266.82089,       279.57104 ],
  [               0,               0,               0,       44.182041,       -162.9062,       144.62991 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,      0.11234816,     -0.54418433,      0.65967619 ],
  [               0,               0,               0,        152.5417,      -667.03979,       731.56317 ],
  [               0,               0,               0,       654.34686,      -3085.9951,        3636.426 ],
  [               0,               0,               0,      -382.67148,       1809.7878,       -2139.158 ],
  [               0,               0,               0,     0.036907393,     -0.17014167,      0.19827229 ],
  [               0,               0,               0,      -5.3363757,       25.439468,      -29.319923 ],
  [               0,               0,               0,       86.690224,       -409.1492,       481.12979 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,     0.016148796 ],
  [               0,               0,               0,               0,               0,       1.7164907 ],
  [               0,               0,               0,               0,               0,      -1.1416347 ],
  [               0,               0,               0,               0,               0,      0.13393834 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0067900545,    -0.016815891 ],
  [               0,               0,               0,               0,      -9.8465118,       -14.31118 ],
  [               0,               0,               0,               0,  -0.00039998072,     -0.41368863 ],
  [               0,               0,               0,               0,       43.724751,       85.373436 ],
  [               0,               0,               0,               0,       33.113777,       81.507957 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam50_layer2_etaoffv3_55_b = [
 [
  [               0,               0,               0,    -0.067277871,     0.030970179,    0.0042457958 ],
  [               0,               0,               0,       22.938322,      -11.005666,       5.4991498 ],
  [               0,               0,               0,     -0.68833071,      0.33232114,      -1.3874599 ],
  [               0,               0,               0,     -0.21313125,       0.1141409,     -0.01374892 ],
  [               0,               0,               0,     0.023323791,    -0.010743386,    0.0014799562 ],
  [               0,               0,               0,       8.5656013,      -3.7905869,       1.1759777 ],
  [               0,               0,               0,      0.90102315,      -0.1622249,      -1.6178128 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0040251361,    0.0039434158,    0.0073314314 ],
  [               0,               0,               0,      -3.1975281,        3.121372,       3.2591314 ],
  [               0,               0,               0,     -0.61183274,      0.55333662,      -1.4690183 ],
  [               0,               0,               0,       -18.58502,       22.441284,        -5.58743 ],
  [               0,               0,               0,      0.10908201,     0.018724063,    -0.019688208 ],
  [               0,               0,               0,    -0.075868577,      0.10319759,      -0.0365717 ],
  [               0,               0,               0,       1.4987762,      -3.4479566,      -15.433163 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.059168849,    -0.042397399 ],
  [               0,               0,               0,               0,      -1.5656117,       6.3014293 ],
  [               0,               0,               0,               0,        6.985723,      -7.0395498 ],
  [               0,               0,               0,               0,      -5.7819223,       4.6329389 ],
  [               0,               0,               0,               0,   -0.0088439286,     0.008245918 ],
  [               0,               0,               0,               0,      -1.8313121,       2.4678929 ],
  [               0,               0,               0,               0,      0.97469467,       -2.442848 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0014119436,    -0.002102748,    0.0089796027 ],
  [               0,               0,               0,       1.5793045,      -4.7112942,       6.6873569 ],
  [               0,               0,               0,     -0.29784718,      0.27158579,      -1.3690913 ],
  [               0,               0,               0,     -0.60313368,        1.771102,      0.70241189 ],
  [               0,               0,               0,   -0.0006687956,    0.0041322969,     0.011439807 ],
  [               0,               0,               0,     0.028988287,       0.1039528,    -0.071170665 ],
  [               0,               0,               0,      0.30676624,     -0.90643454,      -2.4619882 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   -0.0020188508,    -0.038894121,    -0.023359159,      0.12025204 ],
  [               0,               0,       628.17584,      -865.66772,      -1192.6703,       1646.1267 ],
  [               0,               0,      -125.66596,       182.37914,       219.22501,      -320.81976 ],
  [               0,               0,      -58.373814,       77.020927,       104.77079,      -137.83067 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,       0.4055652,     -0.56479329 ],
  [               0,               0,               0,               0,      -10329.327,       14859.345 ],
  [               0,               0,               0,               0,      -79.469048,       108.39107 ],
  [               0,               0,               0,               0,      0.47114441,     -0.10138593 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam50_layer2_etaoffv3_55_e = [
 [
  [               0,               0,      0.51281947,     -0.68312669,      -1.1914679,       1.6015579 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,  -0.00085628883,   0.00032926237,    0.0020416803,    0.0047154329 ],
  [               0,               0,       36.142635,      -40.846794,      -71.321083,       80.039345 ],
  [               0,               0,      -48.882595,       69.559319,       103.45913,       -147.8638 ],
  [               0,               0,      -15.979487,       25.583603,       36.189915,      -57.781334 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.11340004,     -0.16793594 ],
  [               0,               0,               0,               0,      -86.949547,       134.97655 ],
  [               0,               0,               0,               0,      -122.14848,       183.65765 ],
  [               0,               0,               0,               0,       104.35247,      -157.70515 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.016464395,    -0.054083932,     0.052637991 ],
  [               0,               0,               0,      -1.4550459,       2.9163716,         2.73613 ],
  [               0,               0,               0,        4.811965,      -16.127069,       12.120039 ],
  [               0,               0,               0,      0.76665092,      -3.5038249,       3.8885064 ],
  [               0,               0,               0,     0.021275412,    -0.079229034,     0.074019022 ],
  [               0,               0,               0,      -3.8467224,       15.444443,      -14.583417 ],
  [               0,               0,               0,       37.035397,      -120.67775,       95.299408 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.089889526,     -0.36768794,      0.38434562 ],
  [               0,               0,               0,      -12.199315,       48.398212,      -44.978981 ],
  [               0,               0,               0,       3.1557322,       -12.70539,       11.474655 ],
  [               0,               0,               0,       -3.119956,       12.740525,      -12.855454 ],
  [               0,               0,               0,    -0.027022419,      0.11057333,     -0.11216035 ],
  [               0,               0,               0,      -4.6216574,       18.573612,      -18.259209 ],
  [               0,               0,               0,      -27.040104,       111.19973,      -116.81214 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.024391619,        0.132388,     -0.16532406 ],
  [               0,               0,               0,       6.2532082,      -32.999516,       45.383324 ],
  [               0,               0,               0,      0.90602791,      -3.9059911,       2.9994125 ],
  [               0,               0,               0,      -6.1733403,       29.640631,      -35.522198 ],
  [               0,               0,               0,    -0.022646379,      0.11500537,     -0.14521836 ],
  [               0,               0,               0,      -11.565279,       50.153572,      -53.507549 ],
  [               0,               0,               0,      -40.679028,       222.14229,      -301.33319 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0022237904,    0.0072622728 ],
  [               0,               0,               0,               0,      -82.761559,       205.95563 ],
  [               0,               0,               0,               0,       16.431078,      -41.659962 ],
  [               0,               0,               0,               0,       2.2567952,      -5.3476253 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.012681214,    -0.031439628 ],
  [               0,               0,               0,               0,       3.0759046,      -20.531124 ],
  [               0,               0,               0,               0,       14196.199,        -2483618 ],
  [               0,               0,               0,               0,       929.40906,      -2090.2502 ],
  [               0,               0,               0,               0,       263.93906,      -473.77875 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam75_layer2_etaoffv3_55_b = [
 [
  [               0,               0,               0,    -0.017205028,     0.010443126,    0.0068982346 ],
  [               0,               0,               0,       10.781531,      -5.3846183,       4.7825217 ],
  [               0,               0,               0,     -0.11598346,   -0.0088645369,      -1.3330218 ],
  [               0,               0,               0,      -0.2225284,      0.16574205,    -0.029768666 ],
  [               0,               0,               0,    0.0096343867,   -0.0054209637,   0.00076857168 ],
  [               0,               0,               0,      -46.351898,       13.588249,       0.0293904 ],
  [               0,               0,               0,       7.4489651,      -1.9040027,      -1.4475667 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.012827407,     0.017552175,    0.0027675598 ],
  [               0,               0,               0,       1.0722412,      -3.3263869,       5.5463943 ],
  [               0,               0,               0,     -0.43748233,      0.40740854,      -1.4294715 ],
  [               0,               0,               0,      0.53201795,     -0.44606432,     0.098107241 ],
  [               0,               0,               0,   -0.0017343604,   0.00086395291,   0.00021365048 ],
  [               0,               0,               0,      -11.793948,       9.9222984,      -1.7853723 ],
  [               0,               0,               0,       -74.12632,       70.166435,      -37.169685 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.03827709,    -0.024840133 ],
  [               0,               0,               0,               0,        23.74876,      -14.311745 ],
  [               0,               0,               0,               0,      -11.526801,       7.8833947 ],
  [               0,               0,               0,               0,       9.1787882,      -7.3492074 ],
  [               0,               0,               0,               0,     0.013426185,   -0.0095327105 ],
  [               0,               0,               0,               0,      -16.439911,        14.15866 ],
  [               0,               0,               0,               0,      -61.882889,       47.839401 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0034373139,     0.006846345,    0.0051820483 ],
  [               0,               0,               0,       3.3902645,      -7.8376856,        7.988791 ],
  [               0,               0,               0,     -0.12529264,     -0.12936296,      -1.1334753 ],
  [               0,               0,               0,      -1.7346107,      0.79176605,       2.9105046 ],
  [               0,               0,               0,    -0.015221926,    0.0041174111,     0.027485322 ],
  [               0,               0,               0,      0.15593541,    -0.022001794,    -0.079450466 ],
  [               0,               0,               0,       1.1769725,       -2.761709,       -1.480965 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0069793565,    -0.039689038,    -0.032455221,      0.11072315 ],
  [               0,               0,       640.19171,      -883.59509,      -1213.1398,       1677.1565 ],
  [               0,               0,      -76.662804,        115.1299,       127.04914,      -194.39075 ],
  [               0,               0,      -69.804245,       92.635162,       126.46049,      -167.46227 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.41349849,     -0.57619274 ],
  [               0,               0,               0,               0,      -23511.518,       33800.504 ],
  [               0,               0,               0,               0,      -89.522842,       122.43674 ],
  [               0,               0,               0,               0,      0.24830635,      0.27113613 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam75_layer2_etaoffv3_55_e = [
 [
  [               0,               0,      0.72851014,     -0.98748261,      -1.6357578,       2.2284606 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    -0.010835653,   -0.0048229238,    0.0098842904,     0.040906124 ],
  [               0,               0,      -11.076895,       26.045906,       31.737455,      -66.349991 ],
  [               0,               0,       12.567993,      -21.380142,      -29.176565,       48.325924 ],
  [               0,               0,       1.1139238,      -1.2347341,       -1.999746,       2.1004913 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.021039965,     0.032934394 ],
  [               0,               0,               0,               0,         484.867,      -727.76062 ],
  [               0,               0,               0,               0,      -865.86035,       1309.0861 ],
  [               0,               0,               0,               0,       770.22089,      -1164.3376 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0029859901,    -0.010847114,     0.018325213 ],
  [               0,               0,               0,       1.0490024,      -4.4870167,       8.1753569 ],
  [               0,               0,               0,       3.2585044,      -11.204064,       8.2341394 ],
  [               0,               0,               0,      -6.0355682,       11.873326,      0.65232521 ],
  [               0,               0,               0,     -0.02507641,    0.0084781647,     0.086453497 ],
  [               0,               0,               0,      -1.6374642,       5.7090149,      -4.8913102 ],
  [               0,               0,               0,       3.9824502,      -13.316751,       8.0172634 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0089544738,    -0.029478727,      0.03203246 ],
  [               0,               0,               0,     0.029225258,      -3.3385191,        9.623313 ],
  [               0,               0,               0,      0.72311831,      -2.6200321,       1.0706999 ],
  [               0,               0,               0,      -47.901833,       191.28748,      -187.92349 ],
  [               0,               0,               0,    -0.018201496,   -0.0074104741,   -0.0029852199 ],
  [               0,               0,               0,    0.0063569741,    -0.030202461,     0.036652222 ],
  [               0,               0,               0,       4.5615091,      -18.170809,       13.085874 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,  -0.00085696077,   6.1075989e-05,     0.014650389 ],
  [               0,               0,               0,       14.470684,      -69.702545,       86.519531 ],
  [               0,               0,               0,        1.126192,      -5.3136344,       5.0410166 ],
  [               0,               0,               0,      -9.5482759,       64.040497,       -96.50515 ],
  [               0,               0,               0,     0.039677724,  -0.00078199391,     -0.20509072 ],
  [               0,               0,               0,     -0.22239806,      0.57739729,    -0.073570669 ],
  [               0,               0,               0,      0.72711533,      -0.4412559,       -6.064703 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.72061908,       1.7902868 ],
  [               0,               0,               0,               0,       13.547202,      -30.859859 ],
  [               0,               0,               0,               0,       24.566643,      -61.781643 ],
  [               0,               0,               0,               0,      -20.842957,       51.500584 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.032942303,    -0.081588842 ],
  [               0,               0,               0,               0,      -6.6573744,      -15.837554 ],
  [               0,               0,               0,               0,               0,     -0.40000001 ],
  [               0,               0,               0,               0,       3813.1982,      -9356.1787 ],
  [               0,               0,               0,               0,       10.497108,       32.010742 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam100_layer2_etaoffv3_55_b = [
 [
  [               0,               0,               0,   -0.0073707234,    0.0032358225,    0.0077830954 ],
  [               0,               0,               0,       4.9033322,      -2.2203965,       4.6137276 ],
  [               0,               0,               0,      0.22623911,     -0.13403536,      -1.3234307 ],
  [               0,               0,               0,       100.34234,       -41.48193,      0.15947916 ],
  [               0,               0,               0,     -0.81652623,      0.33783093,   -0.0012804153 ],
  [               0,               0,               0,      0.34630445,     -0.17438138,     0.031262666 ],
  [               0,               0,               0,       -7.368093,       3.1538835,     -0.43477598 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0062981537,     0.014351493,   0.00059509563 ],
  [               0,               0,               0,       3.0361414,      -7.2266502,       7.8421426 ],
  [               0,               0,               0,      0.03317799,      -0.3371926,       -1.220149 ],
  [               0,               0,               0,      0.45733476,     -0.18584649,    -0.020791644 ],
  [               0,               0,               0,    0.0045651202,   -0.0074022608,    0.0033595194 ],
  [               0,               0,               0,      -3.1724708,       2.8874948,      0.35566947 ],
  [               0,               0,               0,      -9.8885937,       8.8230066,       -3.506382 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.031778302,    -0.016845183 ],
  [               0,               0,               0,               0,       5.6627893,     -0.63665968 ],
  [               0,               0,               0,               0,       5.5171089,      -5.7898188 ],
  [               0,               0,               0,               0,      -4.6442084,       3.7334187 ],
  [               0,               0,               0,               0,    0.0076599051,   -0.0064543611 ],
  [               0,               0,               0,               0,       5.9261861,      -3.8602884 ],
  [               0,               0,               0,               0,      -1.6801513,       1.5945605 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   0.00081953412,   -0.0022795715,    0.0098344386 ],
  [               0,               0,               0,      0.82380545,      -2.4603586,       5.3654065 ],
  [               0,               0,               0,       0.4595736,      -1.3279935,      -0.5254606 ],
  [               0,               0,               0,      0.94965625,      0.93258756,      -1.2984459 ],
  [               0,               0,               0,    0.0093197282,    0.0019609779,   -0.0068160426 ],
  [               0,               0,               0,     -0.49384382,       1.0943989,     -0.48678064 ],
  [               0,               0,               0,       1.6293365,      -4.0417366,     -0.56936097 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0046382598,    -0.039369877,    -0.029961621,      0.11302963 ],
  [               0,               0,       604.58783,      -834.29071,      -1147.4888,       1586.2484 ],
  [               0,               0,      -90.853851,       135.42274,       151.73822,      -229.78641 ],
  [               0,               0,      -50.522957,        66.22187,       89.963402,      -117.48455 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.67639273,     -0.94189221 ],
  [               0,               0,               0,               0,       154.38467,      -176.68394 ],
  [               0,               0,               0,               0,      -42.614384,       56.895462 ],
  [               0,               0,               0,               0,      0.34997863,      0.29023448 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam100_layer2_etaoffv3_55_e = [
 [
  [               0,               0,      0.58435494,     -0.77953261,      -1.3286393,       1.7855341 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.001377249,   -0.0026857196,   -0.0021227146,     0.012891855 ],
  [               0,               0,      -176.98901,       265.58694,       400.34546,      -598.34698 ],
  [               0,               0,       31.583504,      -49.926128,      -69.954445,       109.52364 ],
  [               0,               0,      -12.330008,       19.148214,       27.789961,      -43.037952 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.079417318,      0.12573293 ],
  [               0,               0,               0,               0,        111.6057,      -165.73976 ],
  [               0,               0,               0,               0,      -87.031052,        130.4818 ],
  [               0,               0,               0,               0,       67.920174,       -102.6259 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0035935601,     0.013429414,   -0.0040186457 ],
  [               0,               0,               0,       3.5491188,      -13.677119,       16.674994 ],
  [               0,               0,               0,       2.4345071,      -8.2178202,       5.5321722 ],
  [               0,               0,               0,      -1.3379749,       4.2444615,      -3.1872091 ],
  [               0,               0,               0,  -0.00075564865,  -7.0703964e-06,    0.0030595292 ],
  [               0,               0,               0,      -9.6273708,        33.27718,      -28.244919 ],
  [               0,               0,               0,       27.596897,      -92.011276,       73.806114 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.011099524,     -0.03884742,     0.041758675 ],
  [               0,               0,               0,    -0.076356992,      -2.6368241,       8.7773781 ],
  [               0,               0,               0,      0.46541536,      -1.3849953,     -0.42398086 ],
  [               0,               0,               0,      -1.8573868,        7.435184,      -7.3308029 ],
  [               0,               0,               0,   -0.0014650302,    0.0063361111,   -0.0062707672 ],
  [               0,               0,               0,       7.0039639,      -31.132715,       34.782646 ],
  [               0,               0,               0,       31.210188,      -126.11229,       125.06454 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0025102959,  -0.00075068645,     0.026607253 ],
  [               0,               0,               0,       20.455198,      -96.648933,       116.63627 ],
  [               0,               0,               0,       3.9859126,      -19.200487,       21.903641 ],
  [               0,               0,               0,      -6.9556613,       41.937378,      -59.719681 ],
  [               0,               0,               0,     0.020356121,   -0.0021917147,     -0.10318644 ],
  [               0,               0,               0,     -0.61055785,       2.0288084,      -1.3076117 ],
  [               0,               0,               0,       1.4205185,      -1.9643261,      -6.3098078 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0022913439,    0.0060742428 ],
  [               0,               0,               0,               0,      -56.408756,       141.29628 ],
  [               0,               0,               0,               0,       9.2259283,      -24.047943 ],
  [               0,               0,               0,               0,       1.4379839,      -3.3099563 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.068093114,      -0.1688939 ],
  [               0,               0,               0,               0,      -9.3767366,       6.2835374 ],
  [               0,               0,               0,               0,   0.00022627418,     -0.40000001 ],
  [               0,               0,               0,               0,         3328.46,      -8183.7954 ],
  [               0,               0,               0,               0,       7.8462582,       29.984718 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam200_layer2_etaoffv3_55_b = [
 [
  [               0,               0,               0,   -0.0089016138,    0.0049296883,    0.0073454482 ],
  [               0,               0,               0,       3.0806255,       -1.817699,       4.8846407 ],
  [               0,               0,               0,        1.475111,      -0.7376135,      -1.2500607 ],
  [               0,               0,               0,      0.21663903,    -0.072011411,    -0.043317433 ],
  [               0,               0,               0,   -0.0025603762,   0.00085475855,   0.00024376223 ],
  [               0,               0,               0,       18.523312,      -9.3494368,       1.4206172 ],
  [               0,               0,               0,      -10.602669,       5.8658504,      -1.2288485 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0032152312,   -0.0034434446,    0.0078414092 ],
  [               0,               0,               0,      -1.1359535,      0.47056815,       4.9316492 ],
  [               0,               0,               0,      -0.9415822,      0.79864943,      -1.5172219 ],
  [               0,               0,               0,      0.84759957,     -0.74204057,       0.1439524 ],
  [               0,               0,               0,   -0.0029323462,    0.0034209453,  -0.00044879579 ],
  [               0,               0,               0,      -1.3369285,       1.2232757,      0.72427583 ],
  [               0,               0,               0,      -2.0709794,       1.0381728,      -1.5394255 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.00949449,    0.0015039257 ],
  [               0,               0,               0,               0,       8.7026491,      -3.0775344 ],
  [               0,               0,               0,               0,       3.6127443,      -4.1628757 ],
  [               0,               0,               0,               0,      -3.2875426,       2.5881598 ],
  [               0,               0,               0,               0,   0.00010512336,  -0.00013696453 ],
  [               0,               0,               0,               0,      -11.890064,       14.157783 ],
  [               0,               0,               0,               0,      -63.268234,       52.200775 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0028341145,   -0.0058831354,     0.010902193 ],
  [               0,               0,               0,       1.0213075,      -3.0271814,       6.0163536 ],
  [               0,               0,               0,      0.82011753,       -2.121135,     -0.10448708 ],
  [               0,               0,               0,     -0.36494896,        1.153597,     -0.72940105 ],
  [               0,               0,               0,  -0.00082401413,    0.0023971288,   -0.0012190512 ],
  [               0,               0,               0,      -1.5801659,        2.833492,     -0.39211833 ],
  [               0,               0,               0,       3.6394851,      -10.085352,      -1.8982062 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0085952319,    -0.046271287,    -0.038504642,      0.12775476 ],
  [               0,               0,       650.55133,      -898.33337,      -1232.7231,       1705.1021 ],
  [               0,               0,       -94.88913,       142.33266,       155.97244,      -238.15833 ],
  [               0,               0,      -37.016964,       47.874939,       64.232674,      -82.561592 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.89158458,      -1.2425165 ],
  [               0,               0,               0,               0,       76.487251,      -30.599337 ],
  [               0,               0,               0,               0,      -38.919994,        51.61665 ],
  [               0,               0,               0,               0,      0.42214951,      0.39469975 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam200_layer2_etaoffv3_55_e = [
 [
  [               0,               0,      0.47077763,     -0.61416835,      -1.0888255,       1.4366126 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   -0.0052332394,     0.024628965,     0.025622357,    -0.064249605 ],
  [               0,               0,      -332.69519,       496.05322,       738.82391,      -1099.2747 ],
  [               0,               0,       259.28568,      -391.00204,       -553.6991,       834.29309 ],
  [               0,               0,      -146.91518,       217.11928,       321.44714,       -474.9509 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.035059564,     0.054675508 ],
  [               0,               0,               0,               0,       479.15201,      -718.39355 ],
  [               0,               0,               0,               0,      -570.46368,       862.02966 ],
  [               0,               0,               0,               0,       529.67236,      -800.75073 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0029449139,   -0.0086575849,     0.014679338 ],
  [               0,               0,               0,       2.1721239,      -9.5263786,       13.772657 ],
  [               0,               0,               0,      -1.3721813,        4.652369,      -5.2989707 ],
  [               0,               0,               0,      -1.4249032,      -7.6460552,       20.909782 ],
  [               0,               0,               0,    -0.032176718,    0.0075794682,      0.11207326 ],
  [               0,               0,               0,       1.9311692,      -6.3826218,       5.2913113 ],
  [               0,               0,               0,      -3.4457412,       11.832578,      -13.142787 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0023622119,     0.013500125,   -0.0083699701 ],
  [               0,               0,               0,       1.9742922,       -10.66786,        16.63723 ],
  [               0,               0,               0,      0.88286209,      -3.3820703,       1.9224796 ],
  [               0,               0,               0,       1.5123338,      -20.696568,       39.796139 ],
  [               0,               0,               0,      -0.0318298,    0.0091450233,      0.14944772 ],
  [               0,               0,               0,     -0.28808117,       1.2391152,      -1.2983575 ],
  [               0,               0,               0,       4.7276154,      -19.129616,       16.287189 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,  -0.00026410338,   -0.0077688112,     0.029797992 ],
  [               0,               0,               0,       31.139215,      -147.49127,       177.27605 ],
  [               0,               0,               0,       2.9154735,      -14.069036,       15.717748 ],
  [               0,               0,               0,      -24.199945,       123.64793,      -155.49719 ],
  [               0,               0,               0,     0.023713345,   -0.0066327932,     -0.10089664 ],
  [               0,               0,               0,     -0.84380203,       3.7802577,      -4.1928153 ],
  [               0,               0,               0,       3.1277013,      -11.015889,       5.7751603 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.041643932,      0.11154142 ],
  [               0,               0,               0,               0,      -87.259819,       217.65477 ],
  [               0,               0,               0,               0,      -1.1789443,       1.4708116 ],
  [               0,               0,               0,               0,       15.136479,      -36.966846 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0003177707,  -6.9432252e-05,   -0.0017771536 ],
  [               0,               0,               0,       0.9553048,        2.107506,       17.731333 ],
  [               0,               0,               0,       3.9877403,       9968.5547,       9839.5479 ],
  [               0,               0,               0,        1.889205,       240.00928,       353.63538 ],
  [               0,               0,               0,       9.1585178,       112.63925,       174.34013 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam500_layer2_etaoffv3_55_b = [
 [
  [               0,               0,               0,    0.0085101835,   -0.0023857136,    0.0078352736 ],
  [               0,               0,               0,      -4.7491364,      0.43220767,       5.1687403 ],
  [               0,               0,               0,     -0.83631349,      0.60867536,      -1.4286617 ],
  [               0,               0,               0,       25.358171,      -11.221725,      0.95893317 ],
  [               0,               0,               0,     -0.20088276,     0.088041641,   -0.0076088272 ],
  [               0,               0,               0,       7.8149152,       -3.628114,      0.26227587 ],
  [               0,               0,               0,       -9.420207,       3.4954271,    -0.098826669 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0050362647,    0.0072177616,    0.0040414738 ],
  [               0,               0,               0,     0.041032717,      -1.7389817,       6.2857709 ],
  [               0,               0,               0,      -2.6662595,       3.7762029,      -2.6375017 ],
  [               0,               0,               0,       1.3902336,      -1.8844419,      0.59386122 ],
  [               0,               0,               0,    0.0030779387,    -0.003822403,    0.0018467869 ],
  [               0,               0,               0,      -2.5055904,       2.7762985,      0.26365253 ],
  [               0,               0,               0,      -4.2442646,       7.5462155,      -4.5321207 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.013721225,     0.019585077 ],
  [               0,               0,               0,               0,       15.631916,      -8.1105576 ],
  [               0,               0,               0,               0,       3.2098079,      -3.8552792 ],
  [               0,               0,               0,               0,      -3.4363644,       2.6938915 ],
  [               0,               0,               0,               0,    0.0054847477,   -0.0042643156 ],
  [               0,               0,               0,               0,       4.6340022,      -2.0384254 ],
  [               0,               0,               0,               0,      -9.1375046,       4.1830096 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0092616063,    -0.017948475,     0.015636083 ],
  [               0,               0,               0,      -2.2444704,        3.402384,       3.4054289 ],
  [               0,               0,               0,       1.6002793,      -3.7819908,      0.79985225 ],
  [               0,               0,               0,      -0.8023144,        2.035465,      -1.2163923 ],
  [               0,               0,               0,   -0.0026068345,    0.0051753698,   -0.0019920338 ],
  [               0,               0,               0,       -1.255662,       2.3993587,        -0.17298 ],
  [               0,               0,               0,       4.6507459,      -11.911607,       5.5742626 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.016792495,    -0.055643667,    -0.052820615,      0.14319851 ],
  [               0,               0,       716.43964,      -986.96088,      -1357.5884,       1873.4847 ],
  [               0,               0,      -72.103127,       113.69291,       106.28547,      -174.94427 ],
  [               0,               0,      -63.081242,       83.650848,       112.73781,       -149.1561 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.67794645,     -0.94490707 ],
  [               0,               0,               0,               0,       142.22385,      -138.16095 ],
  [               0,               0,               0,               0,      -52.950207,       71.354012 ],
  [               0,               0,               0,               0,       0.3414031,      0.30996859 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam500_layer2_etaoffv3_55_e = [
 [
  [      0.13062891,    -0.015882324,     -0.19074981,     -0.34192017,      -0.3478443,       1.0401254 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   -0.0087538781,    -0.011447301,    0.0028646393,     0.059449527 ],
  [               0,               0,      -434.99734,       645.28723,       968.88812,      -1434.6139 ],
  [               0,               0,       99.800835,      -152.07758,      -214.24522,       325.57809 ],
  [               0,               0,       3.4009333,      -4.2177272,      -5.8790579,        6.968473 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.030910522,    -0.043127265 ],
  [               0,               0,               0,               0,       238.50275,      -355.04425 ],
  [               0,               0,               0,               0,      -160.04736,       240.61755 ],
  [               0,               0,               0,               0,       145.78198,      -220.32135 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.010098238,     0.042859849,    -0.035561003 ],
  [               0,               0,               0,     -0.63143718,      -2.5025744,       10.328293 ],
  [               0,               0,               0,      -4.3781719,       15.442056,      -14.996903 ],
  [               0,               0,               0,      0.29666948,      -1.0269663,       1.0606198 ],
  [               0,               0,               0,    0.0033659961,    -0.011580197,     0.010864649 ],
  [               0,               0,               0,       19.006067,      -66.120216,       57.754299 ],
  [               0,               0,               0,       9.5154505,       -34.85759,        29.08853 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0002700586,   -0.0021628931,     0.012488707 ],
  [               0,               0,               0,      -1.9853827,       6.9176459,      -2.6134226 ],
  [               0,               0,               0,     -0.96824902,       4.4285631,      -6.3305125 ],
  [               0,               0,               0,      -8.3823614,       18.356136,        4.486249 ],
  [               0,               0,               0,    -0.040356264,     0.015447223,      0.20085604 ],
  [               0,               0,               0,      0.60177207,      -2.6007562,       2.8263974 ],
  [               0,               0,               0,       2.4631717,      -9.3519945,       5.7618914 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0071804509,    0.0019915304,     0.047193155 ],
  [               0,               0,               0,         6.49718,      -26.472202,       28.755541 ],
  [               0,               0,               0,     0.036806319,      -1.3414258,       1.7343421 ],
  [               0,               0,               0,      -66.169571,       294.23544,      -323.17111 ],
  [               0,               0,               0,    -0.064023137,    0.0025760923,      0.38182253 ],
  [               0,               0,               0,       0.9630999,      -3.6689997,       3.3337443 ],
  [               0,               0,               0,       30.122389,      -142.47694,       165.36034 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.002193585,    0.0070494483 ],
  [               0,               0,               0,               0,      -84.509346,          210.55 ],
  [               0,               0,               0,               0,         12.1907,      -31.299314 ],
  [               0,               0,               0,               0,        1.409335,      -3.2570484 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.020661537,    -0.051169142 ],
  [               0,               0,               0,               0,      -9.1578531,      -16.776548 ],
  [               0,               0,               0,               0,               0,     -0.40000001 ],
  [               0,               0,               0,               0,       58.727657,       75.361572 ],
  [               0,               0,               0,               0,       22.337795,       54.830505 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam1000_layer2_etaoffv3_55_b = [
 [
  [               0,               0,               0,   -0.0004342405,   0.00026923916,    0.0075441129 ],
  [               0,               0,               0,        1.608795,      -1.5491163,       5.5735931 ],
  [               0,               0,               0,      -1.5945489,      0.62514758,      -1.3227735 ],
  [               0,               0,               0,        241.0143,      -97.682793,       2.1508749 ],
  [               0,               0,               0,      -1.8286318,       0.7409299,    -0.016621687 ],
  [               0,               0,               0,      0.76888084,      -0.4256753,       0.1081524 ],
  [               0,               0,               0,      -1.5229131,      0.50719422,    -0.041602634 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.017265083,    -0.011093803,     0.006661634 ],
  [               0,               0,               0,       0.2090456,      -6.6573291,       9.5414228 ],
  [               0,               0,               0,      -1.5298563,       2.0382061,       -1.980606 ],
  [               0,               0,               0,       1.1048893,      -1.2591649,      0.29601666 ],
  [               0,               0,               0,   -0.0064005372,    0.0043152645,   0.00051207416 ],
  [               0,               0,               0,      -4.7848992,       5.0651512,     -0.25078338 ],
  [               0,               0,               0,        3.503582,      -2.9689493,     -0.93949014 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.024339361,     0.025568243 ],
  [               0,               0,               0,               0,       16.027807,      -7.1394048 ],
  [               0,               0,               0,               0,       15.310232,      -13.291528 ],
  [               0,               0,               0,               0,       -9.634388,         7.49543 ],
  [               0,               0,               0,               0,     0.022119092,    -0.016648011 ],
  [               0,               0,               0,               0,       2.0517278,     -0.59865749 ],
  [               0,               0,               0,               0,       8.8263311,      -8.2054539 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.025880259,    -0.056167521,     0.037012476 ],
  [               0,               0,               0,      -9.0878773,       19.175215,      -5.3199315 ],
  [               0,               0,               0,      0.76563323,      -1.8773892,     -0.18664463 ],
  [               0,               0,               0,     -0.19354422,      0.59743911,     -0.44143239 ],
  [               0,               0,               0,    -0.007500662,     0.016400056,   -0.0082669808 ],
  [               0,               0,               0,      -1.9836447,       4.2134056,       -1.256307 ],
  [               0,               0,               0,       8.6902533,      -19.847654,       9.8027372 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      -0.0111421,   -0.0051927711,    0.0085243508,     0.036151309 ],
  [               0,               0,      -11.321499,       9.0458136,       16.449827,      -6.6040516 ],
  [               0,               0,      -62.142902,       88.794388,       119.93634,      -172.53786 ],
  [               0,               0,      -13.489533,       14.557299,       22.757839,      -23.748442 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.54595351,     -0.76113904 ],
  [               0,               0,               0,               0,       939.34705,       -1309.637 ],
  [               0,               0,               0,               0,      -56.732922,       76.878708 ],
  [               0,               0,               0,               0,      0.20489272,      0.14021152 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam1000_layer2_etaoffv3_55_e = [
 [
  [       152.57526,      -317.34119,      -221.80489,       378.64981,       799.58923,      -854.46686 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,     0.016815638,     0.033008896,     0.031633943,    -0.023443617,     -0.20655978 ],
  [               0,       291.10941,      -213.57446,      -635.90216,      -475.82925,       1394.7228 ],
  [               0,       194.62912,      -137.07788,      -433.89035,      -340.21786,        963.2403 ],
  [               0,      -166.42786,       123.31179,       361.77643,        265.2301,      -786.03448 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0083604334,   0.00095778314 ],
  [               0,               0,               0,               0,       28.438906,      -39.847107 ],
  [               0,               0,               0,               0,       33.552616,      -52.027428 ],
  [               0,               0,               0,               0,      -34.044632,       51.539536 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0016325681,   -0.0025201947,    0.0084390836 ],
  [               0,               0,               0,      -1.1072088,      0.43717822,       6.8078227 ],
  [               0,               0,               0,      0.39175656,      -1.1849781,     -0.48754629 ],
  [               0,               0,               0,     0.029139876,     -0.11466041,      0.17433392 ],
  [               0,               0,               0,    0.0030427726,    -0.010408017,     0.008641663 ],
  [               0,               0,               0,      -1.1862825,       2.2245972,       1.6268721 ],
  [               0,               0,               0,       1.8864685,      -7.1106582,      -1.5607544 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.016745992,    -0.065151744,     0.072300449 ],
  [               0,               0,               0,     -0.90398902,       1.0931032,       4.8681583 ],
  [               0,               0,               0,     -0.71399099,        3.398448,      -5.3476372 ],
  [               0,               0,               0,      -12.196774,       54.132233,      -53.080631 ],
  [               0,               0,               0,    0.0087757157,     0.025374008,     -0.02908108 ],
  [               0,               0,               0,       0.6661889,       -2.838872,       3.0649779 ],
  [               0,               0,               0,     -0.80985081,       3.3123264,      -6.4814091 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0012012159,   -0.0042836731,      0.01047023 ],
  [               0,               0,               0,       12.264187,      -59.988754,       76.465935 ],
  [               0,               0,               0,      0.48224923,      -2.6240907,       2.2294888 ],
  [               0,               0,               0,      -1.5118653,       7.4783058,      -9.1669865 ],
  [               0,               0,               0,   0.00061264617,   -0.0039895396,    0.0069613941 ],
  [               0,               0,               0,       2.0080686,      -10.588606,       14.753151 ],
  [               0,               0,               0,     -0.19568771,     -0.71232355,        1.356866 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.01595407,     0.048310563 ],
  [               0,               0,               0,               0,       -126.0871,       313.33746 ],
  [               0,               0,               0,               0,      -3.1624813,       6.4344726 ],
  [               0,               0,               0,               0,       21.865314,      -53.547112 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,       1.1451569,      -2.8366802 ],
  [               0,               0,               0,               0,       -9.686677,      -12.921847 ],
  [               0,               0,               0,               0,               0,     -0.40000001 ],
  [               0,               0,               0,               0,        537.0708,       -1326.644 ],
  [               0,               0,               0,               0,   -0.0039111986,       1.3270005 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]


gam_layer2_etaoffv3_55_b = [
    gam5_layer2_etaoffv3_55_b,
    gam10_layer2_etaoffv3_55_b,
    gam25_layer2_etaoffv3_55_b,
    gam50_layer2_etaoffv3_55_b,
    gam75_layer2_etaoffv3_55_b,
    gam100_layer2_etaoffv3_55_b,
    gam200_layer2_etaoffv3_55_b,
    gam500_layer2_etaoffv3_55_b,
    gam1000_layer2_etaoffv3_55_b,
    ]

gam_layer2_etaoffv3_55_e = [
    gam5_layer2_etaoffv3_55_e,
    gam10_layer2_etaoffv3_55_e,
    gam25_layer2_etaoffv3_55_e,
    gam50_layer2_etaoffv3_55_e,
    gam75_layer2_etaoffv3_55_e,
    gam100_layer2_etaoffv3_55_e,
    gam200_layer2_etaoffv3_55_e,
    gam500_layer2_etaoffv3_55_e,
    gam1000_layer2_etaoffv3_55_e,
    ]


###############################################################################
# Tables for v3 sshape correction, layer 2, 3x5 electron clusters
#

ele5_layer2_etaoffv3_35_b = [
 [
    [0, 0, 0, 0, 0, 0], # Removed by hand, see below
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele5_layer2_etaoffv3_35_e = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele10_layer2_etaoffv3_35_b = [
 [
  [               0,               0,               0,   -0.0012702509,   0.00057442486,    0.0091407737 ],
  [               0,               0,               0,       1.4594194,     -0.55764592,       3.2778711 ],
  [               0,               0,               0,      0.24583644,     -0.21291742,      -1.2404712 ],
  [               0,               0,               0,       901.15527,       -346.8735,       10.412189 ],
  [               0,               0,               0,      -8.2959518,       3.1946754,    -0.096129239 ],
  [               0,               0,               0,     -0.34007955,   -0.0085761715,     0.018302415 ],
  [               0,               0,               0,      -2.2522926,       1.3674352,     -0.16240941 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.035273775,    -0.020272842,    0.0037623651 ],
  [               0,               0,               0,       27.871023,      -49.177177,       24.187229 ],
  [               0,               0,               0,       22.531063,      -29.082083,       7.7150803 ],
  [               0,               0,               0,      -9.2063751,       12.330435,      -3.8946781 ],
  [               0,               0,               0,   -0.0026362718,   -0.0050321706,    0.0053380677 ],
  [               0,               0,               0,       3.3701675,      -4.1746526,       2.2511842 ],
  [               0,               0,               0,      -16.917343,       14.581879,      -4.4503541 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,     0.001675247 ],
  [               0,               0,               0,               0,               0,       7.1210146 ],
  [               0,               0,               0,               0,               0,      -1.3020595 ],
  [               0,               0,               0,               0,               0,    -0.087003283 ],
  [               0,               0,               0,               0,               0,     0.002224918 ],
  [               0,               0,               0,               0,               0,       1.0259796 ],
  [               0,               0,               0,               0,               0,      -1.4984072 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.014956992,     0.021635329,    0.0024703054 ],
  [               0,               0,               0,      -0.3859823,      -1.1052858,       4.3606448 ],
  [               0,               0,               0,    -0.062556602,      0.54630578,      -1.7111405 ],
  [               0,               0,               0,      0.82134527,      -1.4378649,      0.68049532 ],
  [               0,               0,               0,     0.010832563,     -0.02142979,     0.011014554 ],
  [               0,               0,               0,      0.41382357,      -1.4569631,       2.2462912 ],
  [               0,               0,               0,       14.568821,       -37.28334,       22.731569 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.13722029,     0.046067066,    -0.092410035,     -0.28487107 ],
  [               0,               0,      -669.23126,       935.91095,       1135.6414,      -1590.4825 ],
  [               0,               0,        383.4451,      -516.64636,       -711.0578,       956.96771 ],
  [               0,               0,       175.84521,      -240.36601,      -328.96051,       449.73502 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,       1490.0577,      -1714.4417 ],
  [               0,               0,               0,               0,      -2.6579025,       3.7695076 ],
  [               0,               0,               0,               0,       2.6538665,      -3.7755845 ],
  [               0,               0,               0,               0,     -0.31426531,      0.45172349 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele10_layer2_etaoffv3_35_e = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  # Copied from 55, see below
  [               0,               0,               0,               0,   -0.0075912634,     0.015295829 ],
  [               0,               0,               0,               0,      -7.5756946,       16.063869 ],
  [               0,               0,               0,               0,       7.3288541,      -12.312897 ],
  [               0,               0,               0,               0,      -3.8369305,       5.9768329 ],
  [               0,               0,               0,               0,    0.0074901818,    -0.010776109 ],
  [               0,               0,               0,               0,     -0.70966327,       2.2692988 ],
  [               0,               0,               0,               0,      -1.0009131,      0.73172843 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele25_layer2_etaoffv3_35_b = [
 [
  [               0,               0,               0,    -0.061558302,     0.030785386,    0.0039121206 ],
  [               0,               0,               0,       19.072161,      -11.096759,       5.9922643 ],
  [               0,               0,               0,      -2.6697352,      0.80207735,      -1.3892612 ],
  [               0,               0,               0,       1.2011367,     -0.17959744,    -0.033946782 ],
  [               0,               0,               0,     0.016267413,   -0.0086012166,    0.0016422346 ],
  [               0,               0,               0,       21.611568,       -8.918786,       1.2175205 ],
  [               0,               0,               0,      -14.773012,       4.8317642,      -1.6843382 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0061757015,    -0.005663936,    0.0069065019 ],
  [               0,               0,               0,      -12.070657,       13.265753,       1.4267465 ],
  [               0,               0,               0,      0.34529141,      -1.2639077,     -0.94245315 ],
  [               0,               0,               0,      0.46015427,       0.1793808,     -0.12999125 ],
  [               0,               0,               0,  -0.00018328523,  -0.00034451624,    0.0013175897 ],
  [               0,               0,               0,         1.07591,      -2.1373043,       1.7411077 ],
  [               0,               0,               0,        1.218556,      -3.8180387,     -0.13405995 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.033926532,    -0.018914681 ],
  [               0,               0,               0,               0,       36.667377,      -25.666222 ],
  [               0,               0,               0,               0,       20.992886,      -18.555777 ],
  [               0,               0,               0,               0,       -3.664351,       3.4318967 ],
  [               0,               0,               0,               0,    -0.032854918,     0.022547118 ],
  [               0,               0,               0,               0,      -22.398861,       18.378422 ],
  [               0,               0,               0,               0,      -19.611696,       9.4475374 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0046563027,   -0.0082750041,     0.010342505 ],
  [               0,               0,               0,       1.3229159,      -4.4692845,       7.1610079 ],
  [               0,               0,               0,       3.4728839,      -8.3745632,       3.4312363 ],
  [               0,               0,               0,     -0.72654295,       2.6967568,      -1.8326788 ],
  [               0,               0,               0,     0.011336284,    -0.022305962,     0.011678484 ],
  [               0,               0,               0,       1.6859221,      -5.0913615,       4.1349306 ],
  [               0,               0,               0,       7.6781516,      -21.121227,       11.630667 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,       6.2650585,      -16.486767,       10.839023 ],
  [               0,               0,               0,      -4695.5352,        12806.44,      -8727.4209 ],
  [               0,               0,               0,      -2114.7236,       5863.2402,      -4065.2607 ],
  [               0,               0,               0,       1616.8887,      -4437.4824,       3044.7727 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.23003963,       0.3345091 ],
  [               0,               0,               0,               0,      -6443.2505,       9244.5977 ],
  [               0,               0,               0,               0,       74.932968,      -102.91829 ],
  [               0,               0,               0,               0,      -1.1267139,      -2.1697879 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele25_layer2_etaoffv3_35_e = [
 [
  [               0,               0,      -2.3763123,       3.3871841,       4.7522449,      -6.7687397 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,   0.00099558663 ],
  [               0,               0,               0,               0,               0,           0.001 ],
  [               0,               0,               0,               0,               0,      0.58591759 ],
  [               0,               0,               0,               0,               0,       1.1852388 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      -2.4527848,       3.7444608 ],
  [               0,               0,               0,               0,         34.0159,      -49.719723 ],
  [               0,               0,               0,               0,       32.504955,      -50.131763 ],
  [               0,               0,               0,               0,      -34.705379,        52.50211 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     -0.11216329,      0.37489405,     -0.30299667 ],
  [               0,               0,               0,       40.817455,      -136.38931,       116.93659 ],
  [               0,               0,               0,      -7.5752859,       25.294388,      -22.325258 ],
  [               0,               0,               0,      0.52400923,     -0.98110068,      0.24101679 ],
  [               0,               0,               0,     0.003357708,   -0.0085698385,    0.0056946618 ],
  [               0,               0,               0,       23.919407,      -80.865578,       68.136276 ],
  [               0,               0,               0,       66.325493,      -233.89413,       203.03909 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.014919634,      0.06137656,    -0.053379141 ],
  [               0,               0,               0,       4.5093293,      -20.285576,       25.493689 ],
  [               0,               0,               0,     0.070736356,    -0.063828565,      -1.4020326 ],
  [               0,               0,               0,       -473.7998,       2025.1147,      -2085.0361 ],
  [               0,               0,               0,     -0.33781663,      -0.4781273,       3.4788356 ],
  [               0,               0,               0,     0.011261183,    -0.044293731,     0.043636389 ],
  [               0,               0,               0,     -0.85375011,      0.86363423,     -0.50064528 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.023680605,    0.0027828414,      0.14390504 ],
  [               0,               0,               0,       60.720688,        -289.173,        346.4075 ],
  [               0,               0,               0,       45.912777,      -217.09796,        255.4659 ],
  [               0,               0,               0,      -33.156891,       157.09537,      -186.03574 ],
  [               0,               0,               0,    -0.006846698,   -0.0047871349,     0.050861061 ],
  [               0,               0,               0,      -30.159626,       139.89896,      -161.09827 ],
  [               0,               0,               0,       53.780109,      -242.84218,       269.01422 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,     0.021022756 ],
  [               0,               0,               0,               0,               0,       1.5655369 ],
  [               0,               0,               0,               0,               0,      -1.0077024 ],
  [               0,               0,               0,               0,               0,     0.026744185 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele50_layer2_etaoffv3_35_b = [
 [
  [               0,               0,               0,    -0.033813395,      0.01600036,    0.0059360694 ],
  [               0,               0,               0,       13.223563,      -6.3823605,       5.4480939 ],
  [               0,               0,               0,      -1.1839981,      0.40531644,      -1.3783612 ],
  [               0,               0,               0,       1.2143677,     -0.42398784,   -0.0035632404 ],
  [               0,               0,               0,    0.0023664837,   -0.0016121909,   0.00079510815 ],
  [               0,               0,               0,        15.46614,      -7.0938187,       1.1012505 ],
  [               0,               0,               0,      -17.052414,       7.1154938,      -1.8187577 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0014205822,    0.0069963131 ],
  [               0,               0,               0,               0,      -1.8104659,       5.5724669 ],
  [               0,               0,               0,               0,      -0.2784687,      -1.2631785 ],
  [               0,               0,               0,               0,    -0.066430666,      0.25234038 ],
  [               0,               0,               0,               0,   -0.0037577252,    0.0038667831 ],
  [               0,               0,               0,               0,      0.77122062,     -0.23964921 ],
  [               0,               0,               0,               0,      -1.4629945,      -20.591629 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.022033848,   -0.0094135478 ],
  [               0,               0,               0,               0,       18.207735,      -10.448276 ],
  [               0,               0,               0,               0,       6.5849662,      -6.7781005 ],
  [               0,               0,               0,               0,      -5.8019762,       4.7760959 ],
  [               0,               0,               0,               0,    -0.010148245,    0.0090977531 ],
  [               0,               0,               0,               0,      -1.6049153,        1.885065 ],
  [               0,               0,               0,               0,       6.5982819,      -8.1015148 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0051554106,     0.011308856,    0.0018200997 ],
  [               0,               0,               0,       3.1964369,      -8.1817951,       9.0255442 ],
  [               0,               0,               0,      -0.3105447,      0.20705603,      -1.3480186 ],
  [               0,               0,               0,     -0.48853064,       0.4286285,      0.90441781 ],
  [               0,               0,               0,   -0.0042138989,    0.0012250508,    0.0092384452 ],
  [               0,               0,               0,      0.47064272,     -0.55366832,      0.20097563 ],
  [               0,               0,               0,       1.9677852,      -4.5778699,     -0.32168961 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.24830651,     -0.17958446,     -0.35888851,      0.20047112 ],
  [               0,               0,      -4697.3623,       6596.4829,       8443.5381,      -11863.853 ],
  [               0,               0,      -1417.3074,       1970.3962,       2656.5745,      -3695.2424 ],
  [               0,               0,       700.15851,       -957.8606,      -1315.3882,       1799.7227 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0036022053,    0.0061313962 ],
  [               0,               0,               0,               0,      -2240.7002,       3223.7563 ],
  [               0,               0,               0,               0,      -19.206692,       26.156256 ],
  [               0,               0,               0,               0,     -0.25949875,     -0.61108416 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele50_layer2_etaoffv3_35_e = [
 [
  [               0,               0,     -0.50247002,      0.75138468,      0.89021307,      -1.3388066 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    -0.016888903,     0.010038682,     0.026286446,   -0.0019122158 ],
  [               0,               0,       93.474312,      -111.57193,      -186.39809,       221.34955 ],
  [               0,               0,      -126.62128,       189.50945,       272.85181,      -409.29736 ],
  [               0,               0,      -39.419262,       59.327885,       85.881912,      -129.08499 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0033859573,    0.0077482392 ],
  [               0,               0,               0,               0,       56.619732,      -80.466843 ],
  [               0,               0,               0,               0,      -226.61635,       341.75037 ],
  [               0,               0,               0,               0,       192.84113,      -291.40985 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.035930444,     -0.11250413,     0.093723595 ],
  [               0,               0,               0,      -11.887013,       35.918468,      -22.100824 ],
  [               0,               0,               0,       5.5997195,       -19.73204,        16.01086 ],
  [               0,               0,               0,      -1.9335907,       7.0440135,      -6.3388815 ],
  [               0,               0,               0,    0.0011633653,   -0.0062692664,    0.0081376629 ],
  [               0,               0,               0,      -6.7104654,       21.230474,      -15.865605 ],
  [               0,               0,               0,      -1.0553232,      0.26613533,       1.0058161 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   3.7596568e-05,    0.0013938521,    0.0056036869 ],
  [               0,               0,               0,      -1.1538041,       2.6477921,       2.7760518 ],
  [               0,               0,               0,      0.99415636,      -3.9143624,       2.5538006 ],
  [               0,               0,               0,       -29.79734,       123.93982,      -128.42561 ],
  [               0,               0,               0,    0.0019888685,   -0.0066523701,      0.01581057 ],
  [               0,               0,               0,      0.15535185,      -0.6344012,      0.66597635 ],
  [               0,               0,               0,       24.703785,      -102.76179,       104.95409 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0050929408,   -0.0077613597,     0.056695241 ],
  [               0,               0,               0,       79.891243,      -374.18759,       440.88797 ],
  [               0,               0,               0,       5.2886209,      -25.648869,       29.845234 ],
  [               0,               0,               0,      -4.5339952,       28.937841,      -44.084766 ],
  [               0,               0,               0,    -0.014592549,   -0.0019694138,     0.098956957 ],
  [               0,               0,               0,       3.9489918,      -19.093714,       23.039867 ],
  [               0,               0,               0,      -19.255625,       97.932564,      -123.29497 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.043429434,      0.10943855 ],
  [               0,               0,               0,               0,      -40.874214,       105.24885 ],
  [               0,               0,               0,               0,       -345.0144,       848.35394 ],
  [               0,               0,               0,               0,       337.29764,      -830.31879 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0093163373,     0.023069229 ],
  [               0,               0,               0,               0,        6.422967,       23.501253 ],
  [               0,               0,               0,               0,               0,     -0.40000001 ],
  [               0,               0,               0,               0,      -15.821627,      -38.658875 ],
  [               0,               0,               0,               0,      -7.8269143,      -20.296989 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele75_layer2_etaoffv3_35_b = [
 [
  [               0,               0,               0,   -0.0021843782,    0.0012806999,    0.0072052367 ],
  [               0,               0,               0,       3.7938662,      -1.9295355,       5.2892394 ],
  [               0,               0,               0,      0.10651699,     -0.14254233,      -1.3480403 ],
  [               0,               0,               0,       205.21324,       -105.2821,      -1.4543481 ],
  [               0,               0,               0,      -1.4852057,      0.77600741,     0.010473983 ],
  [               0,               0,               0,      0.24558675,     -0.18704137,     0.042824905 ],
  [               0,               0,               0,      -3.4468596,       1.1185262,     -0.21114643 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.032964364,     0.033658177,  -0.00089975965 ],
  [               0,               0,               0,        9.484951,      -10.297766,       7.5903983 ],
  [               0,               0,               0,      -2.8104975,       2.8252499,      -2.1178043 ],
  [               0,               0,               0,       2.1823761,      -2.3877478,      0.70428067 ],
  [               0,               0,               0,     0.015335636,    -0.017295217,    0.0053934208 ],
  [               0,               0,               0,     -0.62683827,       2.4334478,     -0.56831145 ],
  [               0,               0,               0,       2.1894169,      -1.0215433,      -2.2667203 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.061251041,     -0.04183837 ],
  [               0,               0,               0,               0,       6.3189893,     -0.25193831 ],
  [               0,               0,               0,               0,       22.218018,      -19.400166 ],
  [               0,               0,               0,               0,      -14.476538,       11.741024 ],
  [               0,               0,               0,               0,    -0.051462166,     0.042218465 ],
  [               0,               0,               0,               0,       2.7644067,      -1.4425848 ],
  [               0,               0,               0,               0,        18.99984,      -17.731253 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0036214902,     0.010013152,   0.00072531682 ],
  [               0,               0,               0,       2.7605522,      -7.7992144,       9.5365725 ],
  [               0,               0,               0,       1.1546901,      -2.5481608,     -0.16224338 ],
  [               0,               0,               0,      -0.1832457,      0.83503419,      -0.4932906 ],
  [               0,               0,               0,     0.004439069,   -0.0086867502,    0.0050201616 ],
  [               0,               0,               0,      -1.4923091,        2.024199,      0.21303143 ],
  [               0,               0,               0,       3.6293607,       -9.134551,       3.1363852 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.18687908,    -0.010264596,     -0.18087737,      -0.1965141 ],
  [               0,               0,       333.46091,      -444.91183,      -619.00018,       829.25549 ],
  [               0,               0,      -205.45833,       301.05194,       377.08334,      -554.98651 ],
  [               0,               0,       588.10315,      -803.85675,      -1106.9708,       1513.1621 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.037864745,    -0.041778218 ],
  [               0,               0,               0,               0,      -2224.3472,       3202.8069 ],
  [               0,               0,               0,               0,      -20.641668,       27.928972 ],
  [               0,               0,               0,               0,      -0.1819845,     -0.50996226 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele75_layer2_etaoffv3_35_e = [
 [
  [               0,               0,      -1.0310912,       1.4980905,       1.9666122,      -2.8585083 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.023915013,    -0.035634805,    -0.051046237,     0.082317829 ],
  [               0,               0,      -1796.1115,       2624.2473,       3966.8438,      -5792.6084 ],
  [               0,               0,      -154.17809,       228.68294,         349.047,      -518.45544 ],
  [               0,               0,       282.30847,      -417.19464,      -613.17859,        906.1507 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,    0.0074097817 ],
  [               0,               0,               0,               0,               0,       3.4892135 ],
  [               0,               0,               0,               0,               0,      -1.2234724 ],
  [               0,               0,               0,               0,               0,      0.10872961 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.021209255,      0.05903231,    -0.032916632 ],
  [               0,               0,               0,       27.917276,      -88.207733,       74.134567 ],
  [               0,               0,               0,       1.6360999,      -5.3153653,       2.9696472 ],
  [               0,               0,               0,      0.50752997,      -1.9286834,       1.8553886 ],
  [               0,               0,               0,     0.019569915,     -0.06278836,     0.051109511 ],
  [               0,               0,               0,      -5.1214266,       19.343599,      -17.262186 ],
  [               0,               0,               0,      -9.4622107,       34.192135,      -32.321621 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.026928313,      0.12785849,     -0.13964204 ],
  [               0,               0,               0,       12.839385,      -58.984028,       70.219032 ],
  [               0,               0,               0,     -0.35446236,       2.1958449,      -4.3146014 ],
  [               0,               0,               0,      -2.7238605,       10.924037,       -10.83597 ],
  [               0,               0,               0,    0.0061112568,    -0.025065305,     0.026478693 ],
  [               0,               0,               0,       11.994094,      -51.952354,       56.364075 ],
  [               0,               0,               0,       37.646362,      -152.89737,        152.8829 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0029337716,   -0.0060959714,     0.040660068 ],
  [               0,               0,               0,       26.038521,      -120.92324,       143.31012 ],
  [               0,               0,               0,      -9.7618484,        46.89093,      -57.574638 ],
  [               0,               0,               0,      -7.3178668,       42.766991,      -56.111423 ],
  [               0,               0,               0,   0.00021794646,    0.0042137364,     0.028719077 ],
  [               0,               0,               0,      0.98081654,      -5.2036815,       6.8745031 ],
  [               0,               0,               0,       18.577883,      -86.041092,       96.499901 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.059978314,      0.15068217 ],
  [               0,               0,               0,               0,      -211.10576,       525.34741 ],
  [               0,               0,               0,               0,      -196.63419,       482.91888 ],
  [               0,               0,               0,               0,       223.58192,      -550.32318 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.021762567,    -0.053899903 ],
  [               0,               0,               0,               0,      -9.5578499,      -15.524657 ],
  [               0,               0,               0,               0,   0.00087349798,       6976.3789 ],
  [               0,               0,               0,               0,       23.017895,       41.638466 ],
  [               0,               0,               0,               0,       10.098588,       24.477526 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele100_layer2_etaoffv3_35_b = [
 [
  [               0,               0,               0,    -0.035921067,     0.016189076,    0.0059560332 ],
  [               0,               0,               0,       16.466816,       -7.568141,       5.8474102 ],
  [               0,               0,               0,      0.67324036,     -0.31084013,      -1.3291807 ],
  [               0,               0,               0,      0.21348147,    -0.057914406,     -0.03946026 ],
  [               0,               0,               0,    0.0059656207,   -0.0029183319,    0.0008393529 ],
  [               0,               0,               0,       17.381098,      -7.7704282,       1.1714898 ],
  [               0,               0,               0,      -12.959927,       5.7359471,       -1.590654 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0088969516,    -0.011557684,     0.010762932 ],
  [               0,               0,               0,      -10.486644,       11.514216,       2.0655055 ],
  [               0,               0,               0,     -0.87664312,      0.46644503,      -1.4112308 ],
  [               0,               0,               0,      0.91122442,     -0.62244308,     0.092523046 ],
  [               0,               0,               0,   -0.0013318224,    0.0026338794,  -0.00051337568 ],
  [               0,               0,               0,      0.33830291,   -0.0097702574,      0.61946398 ],
  [               0,               0,               0,       2.3638856,      -5.5072894,      0.52175295 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.054558478,      -0.0361274 ],
  [               0,               0,               0,               0,         4.23525,       1.3734722 ],
  [               0,               0,               0,               0,       11.166495,      -10.499044 ],
  [               0,               0,               0,               0,      -8.8187046,       7.2135658 ],
  [               0,               0,               0,               0,     -0.02778537,     0.023361962 ],
  [               0,               0,               0,               0,      -2.4465461,       2.5966079 ],
  [               0,               0,               0,               0,      -5.3372316,       1.5134972 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0085837571,     0.017735856,   -0.0011920426 ],
  [               0,               0,               0,       5.2566662,      -12.212519,       11.258621 ],
  [               0,               0,               0,     -0.16290209,       -0.111555,      -1.1836497 ],
  [               0,               0,               0,     0.065150686,     0.012223965,      0.12245794 ],
  [               0,               0,               0,    0.0017222462,   -0.0050064297,    0.0045317062 ],
  [               0,               0,               0,      0.23264752,      0.39455158,     -0.29761323 ],
  [               0,               0,               0,        3.289526,      -7.0783792,       1.0901384 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.19148716,    -0.036494501,     -0.20388901,     -0.12923655 ],
  [               0,               0,       591.85168,      -797.07166,      -1084.6912,       1464.1678 ],
  [               0,               0,      -482.97247,       681.69293,       897.21332,       -1268.475 ],
  [               0,               0,         624.948,      -853.06024,      -1176.1534,       1605.5833 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.27815199,     -0.38814774 ],
  [               0,               0,               0,               0,      -3035.5806,       4367.6709 ],
  [               0,               0,               0,               0,      -39.163605,       51.755127 ],
  [               0,               0,               0,               0,     -0.19784781,     -0.48885834 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele100_layer2_etaoffv3_35_e = [
 [
  [               0,               0,      -1.4654979,       2.1298251,       2.8588016,      -4.1558676 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   -0.0088876616,     0.093629397,     0.080514006,     -0.28614792 ],
  [               0,               0,       343.15701,      -523.59082,      -747.89557,       1144.2153 ],
  [               0,               0,       360.70557,      -541.27856,      -756.67749,       1134.9553 ],
  [               0,               0,      -366.71735,       538.59924,        800.8681,      -1175.9691 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0099478187,     0.026890995 ],
  [               0,               0,               0,               0,      -13.066182,       22.600121 ],
  [               0,               0,               0,               0,       18.316025,      -28.885197 ],
  [               0,               0,               0,               0,      -20.041512,       30.386171 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.007379571,    -0.023326755,     0.026141981 ],
  [               0,               0,               0,       -2.867666,       7.6257668,     -0.27447614 ],
  [               0,               0,               0,  -0.00052418414,    0.0056038485,      -1.3788483 ],
  [               0,               0,               0,     -0.23393786,      -6.1338558,       13.129611 ],
  [               0,               0,               0,    -0.016139278,    0.0020018667,     0.059404574 ],
  [               0,               0,               0,       1.6667542,      -5.4666624,       4.5145903 ],
  [               0,               0,               0,      -3.8777471,       13.281098,      -14.202544 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0036889568,   0.00099011522 ],
  [               0,               0,               0,               0,      -3.0903594,       9.8877106 ],
  [               0,               0,               0,               0,      0.32836097,      -1.9815505 ],
  [               0,               0,               0,               0,       182.88031,      -703.46991 ],
  [               0,               0,               0,               0,       1.4034406,       1.3189304 ],
  [               0,               0,               0,               0,  -5.0335417e-05,   0.00017339269 ],
  [               0,               0,               0,               0,     -0.41451141,     0.012897416 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0025536483,   0.00088536448,     0.021762818 ],
  [               0,               0,               0,       31.196678,      -147.31012,       176.92723 ],
  [               0,               0,               0,        4.576828,      -22.077885,       25.356157 ],
  [               0,               0,               0,      -9.3291636,       21.577087,      -15.197237 ],
  [               0,               0,               0,   -0.0026928617,     0.011766999,      0.14609651 ],
  [               0,               0,               0,      0.31468394,      -1.3895646,       1.5329326 ],
  [               0,               0,               0,      -6.6011415,        32.60825,      -40.350899 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.17204967,      0.42981821 ],
  [               0,               0,               0,               0,       61.471149,      -148.13533 ],
  [               0,               0,               0,               0,      -81.519905,          199.52 ],
  [               0,               0,               0,               0,       79.337418,      -195.24869 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.038248286,     -0.15412299 ],
  [               0,               0,               0,               0,       107.52182,      -268.69446 ],
  [               0,               0,               0,               0,               0,     -0.40000001 ],
  [               0,               0,               0,               0,        52.02137,      -127.52151 ],
  [               0,               0,               0,               0,     0.018447889,    -0.026247885 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele200_layer2_etaoffv3_35_b = [
 [
  [               0,               0,               0,    -0.010329748,    0.0048260316,    0.0071590198 ],
  [               0,               0,               0,       9.0107794,      -4.0290074,       5.6852827 ],
  [               0,               0,               0,    -0.022027543,   -0.0058453865,      -1.3668706 ],
  [               0,               0,               0,      -38.478619,       19.037693,       -4.089705 ],
  [               0,               0,               0,      0.29152876,     -0.14492139,     0.031500325 ],
  [               0,               0,               0,     -0.83158767,      0.37157559,    0.0055104299 ],
  [               0,               0,               0,       -4.105854,       1.9969897,     -0.40798554 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,      0.02266678,    -0.029699625,     0.016442342 ],
  [               0,               0,               0,      -14.838837,       17.761047,      0.24608058 ],
  [               0,               0,               0,      -1.7206054,       1.5993586,      -1.7122304 ],
  [               0,               0,               0,       1.5464208,      -1.5027266,      0.32810012 ],
  [               0,               0,               0,   -0.0066935718,    0.0091220764,   -0.0024453695 ],
  [               0,               0,               0,       -5.359355,       7.1281009,      -1.5290542 ],
  [               0,               0,               0,      -1.9091274,     -0.76081425,     -0.25048184 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.037207436,     -0.02401052 ],
  [               0,               0,               0,               0,      0.68871051,       5.3830061 ],
  [               0,               0,               0,               0,       14.604513,      -13.238323 ],
  [               0,               0,               0,               0,      -9.0712852,       7.3166046 ],
  [               0,               0,               0,               0,    -0.015513477,     0.013557299 ],
  [               0,               0,               0,               0,       1.9056079,     -0.53174162 ],
  [               0,               0,               0,               0,         10.6999,      -10.441708 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.015668888,      0.03427276,    -0.011156346 ],
  [               0,               0,               0,       8.1888046,      -18.708038,       15.178732 ],
  [               0,               0,               0,     -0.44827226,       0.7578702,      -1.7834553 ],
  [               0,               0,               0,     -0.10994733,      0.49895027,     -0.30071184 ],
  [               0,               0,               0,    0.0026117337,   -0.0056342399,    0.0036193293 ],
  [               0,               0,               0,       2.4628475,       -5.772881,       4.0909219 ],
  [               0,               0,               0,       6.7254558,      -14.928622,       6.0047469 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.20135012,    -0.043430142,     -0.21809851,     -0.12192363 ],
  [               0,               0,       442.50766,      -575.58124,      -807.03711,       1052.6436 ],
  [               0,               0,      -285.85468,       412.87878,       525.48828,      -761.62769 ],
  [               0,               0,       580.40302,      -790.82367,      -1092.2094,       1488.3098 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.55261666,      0.80381727 ],
  [               0,               0,               0,               0,      -8676.5576,       12472.166 ],
  [               0,               0,               0,               0,       51.349869,      -72.594658 ],
  [               0,               0,               0,               0,     -0.42410612,     -0.42022675 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele200_layer2_etaoffv3_35_e = [
 [
  [               0,               0,      -2.7986677,       4.0521774,       5.5532746,      -8.0404444 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     -0.59866744,      0.88710284,       1.3199615,      -1.9478602 ],
  [               0,               0,      -27.118032,       38.589912,       60.563507,      -83.014427 ],
  [               0,               0,       29.462675,      -42.967667,      -62.775803,         90.3545 ],
  [               0,               0,       2.2305162,       -4.292119,      -5.4598994,       10.389797 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.026695073,    -0.036386814 ],
  [               0,               0,               0,               0,       97.606361,      -142.45609 ],
  [               0,               0,               0,               0,      -115.63422,       173.38042 ],
  [               0,               0,               0,               0,        108.6203,      -164.04761 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   0.00026300299,   0.00026473726,    0.0067149326 ],
  [               0,               0,               0,      -0.8592301,      0.95870304,       5.3951173 ],
  [               0,               0,               0,     -0.38922226,       1.5493264,      -2.9203992 ],
  [               0,               0,               0,      0.52907479,      -3.4580839,       4.9991622 ],
  [               0,               0,               0,   -0.0035791814,   0.00015257201,     0.015094591 ],
  [               0,               0,               0,       3.0783176,      -10.512434,        9.077467 ],
  [               0,               0,               0,      -4.4721837,       16.100628,      -17.177485 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0077472166,    -0.025934881,     0.028961867 ],
  [               0,               0,               0,       1.4835733,      -9.9612942,       17.858189 ],
  [               0,               0,               0,     -0.22787282,       1.6285523,      -3.7656605 ],
  [               0,               0,               0,       -2.175338,       8.6959229,      -8.5440874 ],
  [               0,               0,               0,   -0.0046533076,     0.019817539,    -0.020199448 ],
  [               0,               0,               0,       3.0345011,      -14.278526,       17.020447 ],
  [               0,               0,               0,       21.313534,      -85.523209,       83.335228 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.002676866,    -0.015060536,     0.061127234 ],
  [               0,               0,               0,        29.09326,      -133.20668,       155.25211 ],
  [               0,               0,               0,       25.924709,      -124.10889,       147.21321 ],
  [               0,               0,               0,      -12.598253,        60.84684,      -73.352715 ],
  [               0,               0,               0,    0.0054147597,    -0.013450829,    0.0018985342 ],
  [               0,               0,               0,       11.432074,      -62.253849,        83.96743 ],
  [               0,               0,               0,      -14.768464,       84.145195,      -119.33137 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.032670379,    -0.075513549 ],
  [               0,               0,               0,               0,      -204.70131,       507.76443 ],
  [               0,               0,               0,               0,      -99.373871,       243.47539 ],
  [               0,               0,               0,               0,       119.49025,      -294.08414 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.021291822,    -0.052726358 ],
  [               0,               0,               0,               0,      -13.549132,      -6.3208418 ],
  [               0,               0,               0,               0,               0,     -0.40000001 ],
  [               0,               0,               0,               0,        70.52655,      -9.7024803 ],
  [               0,               0,               0,               0,        15.67291,       38.521301 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele500_layer2_etaoffv3_35_b = [
 [
  [               0,               0,               0,     -0.03862289,     0.016622959,    0.0059008398 ],
  [               0,               0,               0,        23.30526,      -10.305702,       6.7484865 ],
  [               0,               0,               0,        0.673926,     -0.21133678,      -1.3662454 ],
  [               0,               0,               0,     0.062632866,    -0.060710602,    -0.025342213 ],
  [               0,               0,               0,     0.010652519,     -0.00466089,   0.00076504709 ],
  [               0,               0,               0,       24.759472,      -10.423994,       1.3054096 ],
  [               0,               0,               0,      -17.377256,       7.6523519,       -1.821947 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     -0.01382475,      0.01928148,   0.00040019129 ],
  [               0,               0,               0,       4.5738335,      -8.1141996,       8.9408531 ],
  [               0,               0,               0,      -2.0278749,       2.3333218,      -2.0242176 ],
  [               0,               0,               0,       1.2317362,      -1.3355789,      0.32957345 ],
  [               0,               0,               0,    0.0058435607,    -0.007774658,    0.0030646634 ],
  [               0,               0,               0,      -2.6722615,       2.7135868,      0.26940635 ],
  [               0,               0,               0,      -8.7352839,       10.120061,      -4.3651409 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.028766975,    -0.016359035 ],
  [               0,               0,               0,               0,       5.1868172,       1.4755408 ],
  [               0,               0,               0,               0,       10.586811,      -9.9342804 ],
  [               0,               0,               0,               0,      -6.8460164,       5.4981575 ],
  [               0,               0,               0,               0,    -0.017340668,     0.014689313 ],
  [               0,               0,               0,               0,       1.1016645,      0.12365881 ],
  [               0,               0,               0,               0,       4.8367133,      -5.8185019 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0041501373,    0.0095297135,    0.0031748048 ],
  [               0,               0,               0,       3.0981398,      -7.5254397,       9.1528206 ],
  [               0,               0,               0,    -0.095738113,      0.12480906,      -1.3941867 ],
  [               0,               0,               0,    -0.014504253,      0.15820414,     -0.13538072 ],
  [               0,               0,               0,  -0.00021935444,   0.00067155983,  -0.00027576694 ],
  [               0,               0,               0,       2.8548155,      -6.2237525,       5.1892018 ],
  [               0,               0,               0,        1.481747,      -4.0467186,      -8.5897722 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.23604487,     -0.11503442,     -0.30152428,     0.035458766 ],
  [               0,               0,       598.40778,       -746.4837,       -1071.469,       1336.7284 ],
  [               0,               0,      -889.88086,       1243.4344,       1658.6783,       -2319.925 ],
  [               0,               0,       765.78851,      -1041.9851,      -1441.2572,       1961.2267 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.55791825,     -0.78087461 ],
  [               0,               0,               0,               0,      -12.998324,       38.495209 ],
  [               0,               0,               0,               0,       3.6964722,      -8.5226316 ],
  [               0,               0,               0,               0,     0.084394433,      -0.1901336 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele500_layer2_etaoffv3_35_e = [
 [
  [       102.81869,       -207.0269,       -165.3568,       242.48276,       592.48718,      -608.68811 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0086897667,    0.0047507747,   -0.0046467814,    -0.023193985 ],
  [               0,               0,     0.029346738,      -3.2289202,      -2.5047312,       13.968841 ],
  [               0,               0,       4.5112262,      -6.8465128,      -10.184351,       14.203538 ],
  [               0,               0,      0.43455791,      -1.5167305,      -1.5289299,       4.3208618 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.033916969,     0.055515755 ],
  [               0,               0,               0,               0,       230.94371,      -344.02054 ],
  [               0,               0,               0,               0,      -146.45502,       220.23302 ],
  [               0,               0,               0,               0,       131.45114,      -198.66676 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.020918768,    -0.070472419,     0.067260802 ],
  [               0,               0,               0,      -9.3751526,       29.793968,       -18.71373 ],
  [               0,               0,               0,     -0.81934291,        2.731461,      -3.6982288 ],
  [               0,               0,               0,      0.11004893,     -0.98454797,       1.7585087 ],
  [               0,               0,               0,  -0.00025671575,   -0.0043321815,     0.011032863 ],
  [               0,               0,               0,       3.1400876,      -10.154177,       8.3291283 ],
  [               0,               0,               0,      -8.2939053,       28.322481,      -26.878071 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0016887134,   -0.0050013023,     0.011805518 ],
  [               0,               0,               0,       1.2599853,      -7.9290671,       14.762443 ],
  [               0,               0,               0,       2.0172079,      -8.2081203,       6.9376316 ],
  [               0,               0,               0,      -1.0871216,        1.323191,       2.9207425 ],
  [               0,               0,               0,   -0.0068729282,    0.0030095016,     0.031461239 ],
  [               0,               0,               0,     -0.92122716,         4.05827,      -4.3120327 ],
  [               0,               0,               0,       3.6819315,       -14.67889,       11.638134 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0047051422,    -0.033494048,     0.058704074 ],
  [               0,               0,               0,       39.269249,      -182.65263,         216.351 ],
  [               0,               0,               0,       1.2854109,      -7.3972149,       8.9490995 ],
  [               0,               0,               0,      -5.9400625,       28.913395,      -35.070999 ],
  [               0,               0,               0,    0.0045196675,    -0.019377491,     0.021922551 ],
  [               0,               0,               0,       11.076485,      -53.030903,       64.369781 ],
  [               0,               0,               0,      -3.5003116,       15.344618,      -18.274517 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,       0.1158307,     -0.27863705 ],
  [               0,               0,               0,               0,      -85.262383,       213.66071 ],
  [               0,               0,               0,               0,       -35.26012,       85.366692 ],
  [               0,               0,               0,               0,       46.773502,      -115.05012 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.031719439,     0.071480922 ],
  [               0,               0,               0,               0,      -2.7333939,       8.3342915 ],
  [               0,               0,               0,               0,               0,     -0.40000001 ],
  [               0,               0,               0,               0,      -36.394592,       89.141769 ],
  [               0,               0,               0,               0,      0.10979512,     -0.20038515 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele1000_layer2_etaoffv3_35_b = [
 [
  [               0,               0,               0,   -0.0058295145,    0.0031808529,    0.0072298213 ],
  [               0,               0,               0,       6.3727379,      -3.9031966,       6.2991562 ],
  [               0,               0,               0,      0.33434007,    -0.063055061,      -1.3603889 ],
  [               0,               0,               0,      -10.958637,       4.7522078,     -0.64052856 ],
  [               0,               0,               0,     0.084603757,    -0.036868043,    0.0048361472 ],
  [               0,               0,               0,      -6.7216835,       2.8264701,    -0.030502917 ],
  [               0,               0,               0,      -7.0066309,       3.1391828,     -0.49366972 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0063043232,   -0.0096332049,     0.010739708 ],
  [               0,               0,               0,      -5.9197378,       7.1216669,       3.6998467 ],
  [               0,               0,               0,      -1.3130603,        1.094918,      -1.5196158 ],
  [               0,               0,               0,      0.91885722,     -0.76939648,     0.083799005 ],
  [               0,               0,               0,    0.0024799739,   -0.0024741818,   0.00097026594 ],
  [               0,               0,               0,      0.38821232,     0.029675828,      0.67788798 ],
  [               0,               0,               0,       6.5502815,      -13.431037,       4.4673691 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.054227479,     0.051162146 ],
  [               0,               0,               0,               0,       29.639948,      -18.329464 ],
  [               0,               0,               0,               0,       5.3315897,      -5.5966768 ],
  [               0,               0,               0,               0,      -4.4002261,       3.4820943 ],
  [               0,               0,               0,               0,     0.018430525,    -0.014438647 ],
  [               0,               0,               0,               0,       12.984304,       -9.515461 ],
  [               0,               0,               0,               0,       8.8976727,      -9.0144196 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,    0.0070061944 ],
  [               0,               0,               0,               0,               0,       5.2324781 ],
  [               0,               0,               0,               0,               0,      -1.4184836 ],
  [               0,               0,               0,               0,               0,     0.026785752 ],
  [               0,               0,               0,               0,               0,   0.00059906882 ],
  [               0,               0,               0,               0,               0,      0.90968657 ],
  [               0,               0,               0,               0,               0,      -1270.9379 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.24757791,     -0.12952952,      -0.3223221,     0.061597198 ],
  [               0,               0,        571.7652,      -695.87109,      -1011.6593,       1228.8069 ],
  [               0,               0,      -960.36646,       1342.9514,       1784.3578,      -2497.6499 ],
  [               0,               0,       790.66669,      -1075.8403,      -1487.2319,       2023.7672 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,       0.5680325,      -0.7948314 ],
  [               0,               0,               0,               0,      -13.596085,       44.713165 ],
  [               0,               0,               0,               0,      -4.5985603,       3.1722536 ],
  [               0,               0,               0,               0,      0.11395386,     -0.13316518 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele1000_layer2_etaoffv3_35_e = [
 [
  [               0,       308.74326,      -885.10638,       10.360812,       1788.6254,       -1281.729 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,     -0.54027122,      0.41586074,       1.2086304,      0.87887615,      -2.6926284 ],
  [               0,       329.61893,      -252.29184,      -695.96759,      -547.04608,       1573.3239 ],
  [               0,       153.47318,      -103.05467,      -344.66733,      -283.47305,       770.84613 ],
  [               0,       -131.1911,       97.652641,       283.45135,        205.5892,      -612.03601 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.039231468,     0.064204834 ],
  [               0,               0,               0,               0,       242.10539,      -360.92267 ],
  [               0,               0,               0,               0,      -112.96713,       169.52058 ],
  [               0,               0,               0,               0,       104.33784,      -157.67691 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0017550346,     0.010297268,   -0.0049947277 ],
  [               0,               0,               0,       7.6151657,      -30.455862,       34.811165 ],
  [               0,               0,               0,       4.1755977,      -14.306171,       10.745706 ],
  [               0,               0,               0,      -1.6764812,       5.9676738,      -5.1664839 ],
  [               0,               0,               0,    0.0038129557,    -0.012754745,     0.011374789 ],
  [               0,               0,               0,      -6.5941811,       21.041449,      -15.934689 ],
  [               0,               0,               0,       13.116314,      -47.512169,        40.73328 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0027273835,    -0.011636226,     0.021103164 ],
  [               0,               0,               0,     -0.28969643,     -0.71905178,       6.5863633 ],
  [               0,               0,               0,       0.1539984,     -0.36180654,      -1.2939584 ],
  [               0,               0,               0,      -3.8234615,       9.3389482,    -0.097280785 ],
  [               0,               0,               0,    -0.014385551,    0.0046114507,     0.076583542 ],
  [               0,               0,               0,    -0.047033932,      0.14234094,    -0.038337257 ],
  [               0,               0,               0,       2.5429459,       -9.651638,       6.0796189 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,  -0.00028676901,   -0.0020635473,     0.016423127 ],
  [               0,               0,               0,       8.0434809,      -39.109852,       50.723427 ],
  [               0,               0,               0,       5.2961302,      -24.779018,       27.654669 ],
  [               0,               0,               0,      -9.9597301,       52.144943,      -59.796127 ],
  [               0,               0,               0,    0.0036534497,    0.0093736025,     0.034646995 ],
  [               0,               0,               0,      -2.2158823,       10.312507,       -11.95834 ],
  [               0,               0,               0,       10.085189,      -47.018715,        51.69392 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.050195772,      0.13013545 ],
  [               0,               0,               0,               0,       22.344393,      -51.331383 ],
  [               0,               0,               0,               0,       -56.11834,       136.81641 ],
  [               0,               0,               0,               0,       57.072353,      -140.42323 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0017299433,   -0.0042838221 ],
  [               0,               0,               0,               0,      -8.4866905,      -18.542097 ],
  [               0,               0,               0,               0,   2.2627417e-05,     -0.71678382 ],
  [               0,               0,               0,               0,       318030.97,      -785996.12 ],
  [               0,               0,               0,               0,        199.3864,       884.74451 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]


ele_layer2_etaoffv3_35_b = [
    ele5_layer2_etaoffv3_35_b,
    ele10_layer2_etaoffv3_35_b,
    ele25_layer2_etaoffv3_35_b,
    ele50_layer2_etaoffv3_35_b,
    ele75_layer2_etaoffv3_35_b,
    ele100_layer2_etaoffv3_35_b,
    ele200_layer2_etaoffv3_35_b,
    ele500_layer2_etaoffv3_35_b,
    ele1000_layer2_etaoffv3_35_b,
    ]

ele_layer2_etaoffv3_35_e = [
    ele5_layer2_etaoffv3_35_e,
    ele10_layer2_etaoffv3_35_e,
    ele25_layer2_etaoffv3_35_e,
    ele50_layer2_etaoffv3_35_e,
    ele75_layer2_etaoffv3_35_e,
    ele100_layer2_etaoffv3_35_e,
    ele200_layer2_etaoffv3_35_e,
    ele500_layer2_etaoffv3_35_e,
    ele1000_layer2_etaoffv3_35_e,
    ]


###############################################################################
# Tables for v3 sshape correction, layer 2, 3x5 photon clusters
#

gam5_layer2_etaoffv3_35_b = [
 [
  [               0,               0,               0,    -0.052829798,      0.03602777,    0.0055409758 ],
  [               0,               0,               0,         7.74929,      -6.2215319,       3.5738437 ],
  [               0,               0,               0,     -0.83786374,      0.67053419,      -1.3111513 ],
  [               0,               0,               0,     -0.76740479,      0.40597901,    -0.034745481 ],
  [               0,               0,               0,     0.023463564,   -0.0094258571,   0.00070873776 ],
  [               0,               0,               0,      -33.002289,      -2.3834174,    -0.023222834 ],
  [               0,               0,               0,      -120.70652,      -0.5963788,      -5.0727372 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.029967694,     0.024215113,    0.0035041037 ],
  [               0,               0,               0,      -15.344456,       17.141649,       -1.601635 ],
  [               0,               0,               0,      -8.2923985,       7.9919505,      -3.1519599 ],
  [               0,               0,               0,       6.1935248,      -6.0314994,       1.4797004 ],
  [               0,               0,               0,     0.028424948,    -0.028911553,    0.0075009246 ],
  [               0,               0,               0,       7.4541278,      -7.7089586,       2.5870373 ],
  [               0,               0,               0,       19.231724,      -25.170155,     -0.50412273 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.74275088,      0.62240875 ],
  [               0,               0,               0,               0,       62.435459,      -48.261173 ],
  [               0,               0,               0,               0,      -22.552927,       17.281286 ],
  [               0,               0,               0,               0,      -7.1164465,       5.5078368 ],
  [               0,               0,               0,               0,      0.10594329,    -0.079823688 ],
  [               0,               0,               0,               0,      -38.651691,       31.762604 ],
  [               0,               0,               0,               0,       35.117325,      -27.778919 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
    [0, 0, 0, 0, 0, 0], # Removed by hand, see below
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam5_layer2_etaoffv3_35_e = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam10_layer2_etaoffv3_35_b = [
 [
  [               0,               0,               0,    -0.039177407,     0.021976279,    0.0061745378 ],
  [               0,               0,               0,       3.2930567,      -2.7595029,       3.9513404 ],
  [               0,               0,               0,      0.26545221,       1.0287536,      -1.5554808 ],
  [               0,               0,               0,     -0.27958238,     -0.37419224,       0.1091206 ],
  [               0,               0,               0,   0.00092449115,   -0.0052614426,    0.0011882132 ],
  [               0,               0,               0,       9.0212221,      -4.5276904,       1.3311539 ],
  [               0,               0,               0,      -12.977424,       2.4568257,      -2.4784293 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0084621655,     0.019237099,    0.0015874498 ],
  [               0,               0,               0,       8.8642292,      -14.727587,       8.6297188 ],
  [               0,               0,               0,      0.13987902,   -0.0096395863,       -1.344697 ],
  [               0,               0,               0,    -0.030656246,      0.21401936,    -0.077452756 ],
  [               0,               0,               0,   -0.0075225467,      0.00966084,   -0.0028818096 ],
  [               0,               0,               0,      -11.773398,       15.801623,      -2.3440001 ],
  [               0,               0,               0,      -3.7320485,     -0.51269788,      -6.1801324 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.10624393,     0.091963857 ],
  [               0,               0,               0,               0,        65.22541,       -48.54361 ],
  [               0,               0,               0,               0,      -2.8303399,       1.0666173 ],
  [               0,               0,               0,               0,      0.72304815,      -0.6806131 ],
  [               0,               0,               0,               0,   -0.0029466399,    0.0035278797 ],
  [               0,               0,               0,               0,       18.024179,       -13.49675 ],
  [               0,               0,               0,               0,      -67.119598,        53.07486 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.024042848,    -0.046353038,      0.02930822 ],
  [               0,               0,               0,      -5.9188523,       11.240377,      -1.8904378 ],
  [               0,               0,               0,       1.1253968,      -2.3794217,     -0.15646634 ],
  [               0,               0,               0,     -0.16617703,      0.58601564,     -0.32227844 ],
  [               0,               0,               0,   -0.0014313078,   0.00083328754,    0.0010990055 ],
  [               0,               0,               0,      -6.0543475,       11.511209,       -4.407712 ],
  [               0,               0,               0,       14.739678,      -25.865105,       9.5602245 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.076512411,     -0.31405613,      0.42970407,     -0.19598296 ],
  [               0,               0,      -144825.45,      -219880.56,      -325475.28,      -473856.19 ],
  [               0,               0,       150545.95,        61580.84,      -161036.31,      -327226.28 ],
  [               0,               0,      -465845.75,       386603.59,       360583.16,       11183.828 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.87882566,       1.2717035 ],
  [               0,               0,               0,               0,       -294.1012,       425.78387 ],
  [               0,               0,               0,               0,       107.60248,      -151.12569 ],
  [               0,               0,               0,               0,      -0.2909838,      -1.1309777 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam10_layer2_etaoffv3_35_e = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.030142806,     0.046234105 ],
  [               0,               0,               0,               0,        115.0718,       178.28043 ],
  [               0,               0,               0,               0,       183.52014,      -281.59772 ],
  [               0,               0,               0,               0,        169.0939,       -251.4128 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
    # Copied from 55; see below.
  [               0,               0,               0,               0,   -0.0035746964,     0.012298349 ],
  [               0,               0,               0,               0,      -5.3014765,        12.26264 ],
  [               0,               0,               0,               0,       4.4197936,      -8.6744204 ],
  [               0,               0,               0,               0,      -1.6761082,       2.8486159 ],
  [               0,               0,               0,               0,     0.011471081,     -0.01928678 ],
  [               0,               0,               0,               0,     -0.27758086,       1.5612171 ],
  [               0,               0,               0,               0,     -0.97796303,       1.1397932 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam25_layer2_etaoffv3_35_b = [
 [
  [               0,               0,               0,    -0.025978673,     0.015570193,    0.0045048543 ],
  [               0,               0,               0,       18.546488,      -10.175748,       6.4238219 ],
  [               0,               0,               0,      -4.8207302,       1.8800999,      -1.5168492 ],
  [               0,               0,               0,       3.4868884,      -1.2527955,     0.062689371 ],
  [               0,               0,               0,    0.0072510773,    -0.004672565,    0.0013868689 ],
  [               0,               0,               0,       3.4233925,      -2.2690129,        1.053211 ],
  [               0,               0,               0,      -25.194954,       9.1078377,      -2.0545795 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0014881088,   -0.0010780434,    0.0088664824 ],
  [               0,               0,               0,      -7.8621445,       7.9620013,       2.1949649 ],
  [               0,               0,               0,     0.028105821,    -0.094659336,      -1.3229392 ],
  [               0,               0,               0,       1.5985992,      -1.8390014,      0.55117685 ],
  [               0,               0,               0,     0.011742373,    -0.014893349,     0.004688039 ],
  [               0,               0,               0,       26.550339,      -32.847328,       9.3550434 ],
  [               0,               0,               0,      -127.77859,       109.90502,      -57.663414 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.060221471,    -0.041506197 ],
  [               0,               0,               0,               0,       25.816162,      -15.960548 ],
  [               0,               0,               0,               0,       4.2080684,       -4.738553 ],
  [               0,               0,               0,               0,      -3.3413286,       2.6433911 ],
  [               0,               0,               0,               0,   -0.0013472654,    0.0018638904 ],
  [               0,               0,               0,               0,      -30.139111,       24.921818 ],
  [               0,               0,               0,               0,      -8.3658857,       5.4931936 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.021947686,     0.042763114,    -0.012813473 ],
  [               0,               0,               0,       12.784545,      -26.976467,         18.0991 ],
  [               0,               0,               0,      -3.5053937,       6.7344565,       -4.639339 ],
  [               0,               0,               0,       2.0090098,      -3.8960271,       2.0308747 ],
  [               0,               0,               0,    0.0099492827,    -0.021955529,     0.012813664 ],
  [               0,               0,               0,       1.4776113,      -1.7375571,      0.57439995 ],
  [               0,               0,               0,       2.7650163,      -5.8583221,      0.61844695 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,       0.1165752,    -0.060385447,     -0.15099856,     0.026820801 ],
  [               0,               0,      -10117.458,        14015.72,       18568.732,      -25720.506 ],
  [               0,               0,      -2568.9802,       3542.3103,       4836.9609,      -6671.5986 ],
  [               0,               0,       294.61792,      -401.16058,      -544.41901,       741.66431 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.027433479,    -0.019654591 ],
  [               0,               0,               0,               0,      -1.9462692,       10.231026 ],
  [               0,               0,               0,               0,       6.3162789,      -9.9286118 ],
  [               0,               0,               0,               0,    -0.067039564,     -0.41939828 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam25_layer2_etaoffv3_35_e = [
 [
  [               0,               0,      0.33117577,     -0.46383741,     -0.84630966,       1.1932093 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     -0.72663832,       1.1150388,         1.62271,      -2.4847629 ],
  [               0,               0,       2443.5208,      -3812.5706,      -5542.1875,       8642.6992 ],
  [               0,               0,       2369.3398,      -3458.8347,      -5200.1211,       7589.5093 ],
  [               0,               0,      -1444.5447,       2131.4927,        3137.783,       -4628.959 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      -3.1757126,       4.8415036 ],
  [               0,               0,               0,               0,      -1.4122514,       3.8372972 ],
  [               0,               0,               0,               0,       47.396389,      -72.575874 ],
  [               0,               0,               0,               0,      -34.849327,       52.718075 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,      -0.0879746,      0.28807786,     -0.22793835 ],
  [               0,               0,               0,       56.025616,      -185.70525,       157.83138 ],
  [               0,               0,               0,       -10.03246,       32.925648,      -28.338787 ],
  [               0,               0,               0,       6.0953183,      -20.323021,       16.982492 ],
  [               0,               0,               0,     0.043387383,     -0.14492902,      0.12139315 ],
  [               0,               0,               0,      -8.2564583,       30.936434,      -27.994049 ],
  [               0,               0,               0,      0.63646668,      -1.0880553,      -2.0085256 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.003168945,   -0.0015904504,  -0.00097831187 ],
  [               0,               0,               0,       6.6789494,      -32.415859,       41.533882 ],
  [               0,               0,               0,      0.73075747,      -2.6119516,       1.0062817 ],
  [               0,               0,               0,      -5.3265862,       22.115309,      -22.847635 ],
  [               0,               0,               0,     0.001297237,   -0.0044224323,    0.0045839343 ],
  [               0,               0,               0,      0.37012804,      -1.9471161,       2.6215718 ],
  [               0,               0,               0,       48.803249,      -202.63948,       208.09151 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0086946329,   -0.0041504162,     0.071764305 ],
  [               0,               0,               0,       53.870251,      -249.43854,       290.90829 ],
  [               0,               0,               0,       14.210186,      -68.531822,       81.405022 ],
  [               0,               0,               0,      -12.406686,       59.230618,      -70.650963 ],
  [               0,               0,               0,    0.0019037516,   -0.0069915392,    0.0056994949 ],
  [               0,               0,               0,       2.2174847,       1.4264809,      -11.536813 ],
  [               0,               0,               0,       5.1800346,       3.8365664,      -47.997726 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,     0.011328281 ],
  [               0,               0,               0,               0,               0,        2.555552 ],
  [               0,               0,               0,               0,               0,      -1.2030926 ],
  [               0,               0,               0,               0,               0,     0.043970432 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam50_layer2_etaoffv3_35_b = [
 [
  [               0,               0,               0,    -0.038533878,     0.017738814,    0.0054720226 ],
  [               0,               0,               0,       15.570335,      -7.6602879,       6.0271039 ],
  [               0,               0,               0,       0.3226867,     -0.11255523,      -1.3574494 ],
  [               0,               0,               0,      0.24122033,    -0.091479257,    -0.022827266 ],
  [               0,               0,               0,    0.0074698227,   -0.0036447721,   0.00098325766 ],
  [               0,               0,               0,       18.584984,      -7.9418797,        1.216182 ],
  [               0,               0,               0,      -9.2225933,       4.2076578,      -1.6959033 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.021596592,     0.026296502,   -0.0010546098 ],
  [               0,               0,               0,       5.5096731,      -8.1400099,       8.0999622 ],
  [               0,               0,               0,      -2.8133612,       3.1641192,      -2.2944295 ],
  [               0,               0,               0,        1.785834,       -1.901374,      0.51675671 ],
  [               0,               0,               0,    0.0088680424,    -0.010660272,    0.0037428038 ],
  [               0,               0,               0,       2.5943983,      -3.4183023,       1.9097357 ],
  [               0,               0,               0,      -6.0437469,       6.8982992,      -3.5923667 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.047707513,    -0.031950228 ],
  [               0,               0,               0,               0,      0.09042985,       5.2797618 ],
  [               0,               0,               0,               0,       11.512329,      -10.827723 ],
  [               0,               0,               0,               0,      -8.3139381,       6.7630458 ],
  [               0,               0,               0,               0,      0.01560673,    -0.013496607 ],
  [               0,               0,               0,               0,     -0.66931033,     -0.35818177 ],
  [               0,               0,               0,               0,       -8.064024,       5.4629602 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.001232723,    0.0039384677,     0.004413614 ],
  [               0,               0,               0,       3.0520625,      -8.2970648,       9.8654423 ],
  [               0,               0,               0,     -0.37257031,      0.34553868,      -1.4327461 ],
  [               0,               0,               0,      0.29375303,    -0.077171683,     -0.12288316 ],
  [               0,               0,               0,    0.0038840121,   -0.0073173395,    0.0039946679 ],
  [               0,               0,               0,       1.1023808,      -2.9243112,       2.5023205 ],
  [               0,               0,               0,       0.5687654,      -3.6514912,       1.0543051 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.17832571,    -0.076056957,     -0.21996965,    -0.003508802 ],
  [               0,               0,      -2903.1448,       4024.7581,       5358.4512,      -7424.5854 ],
  [               0,               0,      -1255.0585,       1744.8365,       2352.4424,      -3272.3879 ],
  [               0,               0,        572.0788,      -781.12885,       -1075.743,       1469.0265 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.40093264,      0.58517033 ],
  [               0,               0,               0,               0,      -3626.4626,       5215.5732 ],
  [               0,               0,               0,               0,       49.722435,      -69.982811 ],
  [               0,               0,               0,               0,     -0.32013687,     -0.89905286 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam50_layer2_etaoffv3_35_e = [
 [
  [               0,               0,      -1.1045623,        1.618802,       2.1335049,       -3.129508 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.015082964,    0.0086116185,   -0.0092301778,    -0.048005398 ],
  [               0,               0,       -46.17075,       58.723698,       95.173645,      -116.43411 ],
  [               0,               0,      -152.29582,       227.94405,       330.88123,      -496.27686 ],
  [               0,               0,      -34.924004,       50.219967,       75.000771,      -107.63922 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.01843885,     0.029550891 ],
  [               0,               0,               0,               0,       461.50647,       -691.8302 ],
  [               0,               0,               0,               0,      -551.36212,       833.25092 ],
  [               0,               0,               0,               0,       484.44528,      -732.38525 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.064924374,     -0.22148876,      0.19479778 ],
  [               0,               0,               0,       -26.84004,       89.483673,      -69.464844 ],
  [               0,               0,               0,       2.9199572,      -9.8758135,       6.9024868 ],
  [               0,               0,               0,     -0.97487694,       3.3212698,      -2.7304454 ],
  [               0,               0,               0,     -0.01842159,     0.063506491,    -0.053715225 ],
  [               0,               0,               0,      -2.0278845,       7.4170408,      -5.9182892 ],
  [               0,               0,               0,      -6.9554114,       24.420107,      -23.149374 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,  -0.00053013401,    0.0023541406,    0.0057906145 ],
  [               0,               0,               0,       1.0520854,      -6.4529791,       12.382227 ],
  [               0,               0,               0,      0.53303742,      -1.9065509,      0.32015014 ],
  [               0,               0,               0,       -561.8045,        2387.145,      -2503.4966 ],
  [               0,               0,               0,      -4.6776967,         19.8885,      -20.868975 ],
  [               0,               0,               0,      0.17189638,     -0.79279453,      0.92167461 ],
  [               0,               0,               0,       2.3678997,      -9.5812979,       6.5919437 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0031483418,    -0.012906452,     0.058556955 ],
  [               0,               0,               0,       31.444462,       -146.0452,       172.37108 ],
  [               0,               0,               0,      0.19938914,      -1.5410844,       1.2864343 ],
  [               0,               0,               0,      -5.1347723,        31.23436,      -42.503551 ],
  [               0,               0,               0,   0.00028577808,    0.0026171878,     0.019421827 ],
  [               0,               0,               0,      -9.4820004,       45.301876,      -54.063671 ],
  [               0,               0,               0,       26.290331,      -122.33543,       139.21147 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.085672587,      0.21814457 ],
  [               0,               0,               0,               0,      -29.488449,       75.639931 ],
  [               0,               0,               0,               0,       -49.72559,       121.21271 ],
  [               0,               0,               0,               0,       53.854504,      -132.51486 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0032350861,    -0.010528187 ],
  [               0,               0,               0,               0,       821.80139,      -2040.5872 ],
  [               0,               0,               0,               0,               0,     -0.40000001 ],
  [               0,               0,               0,               0,      -202.44133,       502.25723 ],
  [               0,               0,               0,               0,      0.12627542,     -0.88972926 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam75_layer2_etaoffv3_35_b = [
 [
  [               0,               0,               0,    -0.023240898,     0.010894905,    0.0065268138 ],
  [               0,               0,               0,        16.13409,      -7.2937055,       5.8691096 ],
  [               0,               0,               0,     -0.51859748,     0.056527335,      -1.3385068 ],
  [               0,               0,               0,     -0.81656229,      0.51335543,     -0.10123174 ],
  [               0,               0,               0,     0.018664859,   -0.0092923995,    0.0013697967 ],
  [               0,               0,               0,      -12.239709,       5.2356133,      0.14585488 ],
  [               0,               0,               0,      -8.3101492,       3.2487454,       -1.287443 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.013458975,    -0.015805645,     0.011667332 ],
  [               0,               0,               0,      -10.253846,       10.577398,       2.6114895 ],
  [               0,               0,               0,       1.2757289,      -1.9509414,     -0.70031279 ],
  [               0,               0,               0,     -0.38608429,      0.84507352,       -0.352052 ],
  [               0,               0,               0,   -0.0018612441,    0.0020048914,  -1.7408507e-05 ],
  [               0,               0,               0,      -5.0106173,       6.4544902,      -1.1864839 ],
  [               0,               0,               0,       14.492624,      -20.093691,        5.089551 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0043045008,    0.0046001598 ],
  [               0,               0,               0,               0,        17.23613,      -8.7384825 ],
  [               0,               0,               0,               0,       5.3637924,      -5.6289039 ],
  [               0,               0,               0,               0,      -3.8772261,       3.0788853 ],
  [               0,               0,               0,               0,   -0.0038667053,    0.0033514807 ],
  [               0,               0,               0,               0,      -18.579807,       16.864569 ],
  [               0,               0,               0,               0,      -22.362192,        -5.98634 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.003948323,   -0.0083868494,     0.011868198 ],
  [               0,               0,               0,       1.0813491,      -3.2437625,       6.7348499 ],
  [               0,               0,               0,      0.31534308,      -1.2056134,     -0.54744929 ],
  [               0,               0,               0,     -0.11546019,      0.80681968,     -0.62333298 ],
  [               0,               0,               0,   -0.0023750172,    0.0040512369,   -0.0021750231 ],
  [               0,               0,               0,      -1.5301853,       3.3023019,       -1.119153 ],
  [               0,               0,               0,       5.3288455,      -14.962933,        4.611093 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.19784127,     0.003652347,     -0.18135539,      -0.2485873 ],
  [               0,               0,       147.59871,      -188.95558,      -276.20764,       357.09888 ],
  [               0,               0,       -98.89579,       156.23064,       176.12112,      -281.86209 ],
  [               0,               0,       527.61646,      -720.81665,      -994.53577,       1358.7775 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.51368767,      -0.7187888 ],
  [               0,               0,               0,               0,      -33.556767,       59.702629 ],
  [               0,               0,               0,               0,        19.43347,      -30.484749 ],
  [               0,               0,               0,               0,   -0.0030739235,     -0.34804922 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam75_layer2_etaoffv3_35_e = [
 [
  [               0,               0,      -1.9577112,       2.8341324,       3.8708119,      -5.6035175 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.017093953,     0.073718064,      0.03698672,     -0.26045004 ],
  [               0,               0,      -204.85081,       289.37585,       455.22824,      -640.73425 ],
  [               0,               0,       254.29855,      -383.93228,      -511.68549,       772.63916 ],
  [               0,               0,      -357.59674,       524.42108,       780.29047,      -1144.0183 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.085264988,      0.13191617 ],
  [               0,               0,               0,               0,       256.56448,      -383.44101 ],
  [               0,               0,               0,               0,      -270.90347,       408.73056 ],
  [               0,               0,               0,               0,       249.59509,      -377.23187 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.044255391,     -0.14979185,      0.13406511 ],
  [               0,               0,               0,      -17.769115,       58.657593,      -43.667633 ],
  [               0,               0,               0,       5.9009395,      -20.228327,       15.830148 ],
  [               0,               0,               0,      -2.4234459,       8.2953348,      -6.8629012 ],
  [               0,               0,               0,   0.00018940668,  -0.00077794562,    0.0020885877 ],
  [               0,               0,               0,      -11.067889,       38.154171,      -32.389317 ],
  [               0,               0,               0,       8.2491989,      -28.287901,       21.546516 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0027920401,      0.01609363,    -0.012753431 ],
  [               0,               0,               0,       1.8685344,      -11.251642,       18.847467 ],
  [               0,               0,               0,      0.92723268,       -3.567199,        2.082006 ],
  [               0,               0,               0,      -6.3138046,       25.572916,      -25.692835 ],
  [               0,               0,               0,     0.003708543,    -0.014759349,     0.017005591 ],
  [               0,               0,               0,       1.1379151,      -4.7970762,        5.145926 ],
  [               0,               0,               0,       24.190266,      -98.285904,       97.727707 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.002600004,  -0.00085890398,   -0.0028628472 ],
  [               0,               0,               0,       14.959888,      -75.391876,        97.78125 ],
  [               0,               0,               0,      -0.7881822,       3.7733765,      -5.7841277 ],
  [               0,               0,               0,      -9.7032652,       52.751518,      -69.807106 ],
  [               0,               0,               0,     0.013104517,  -0.00020528569,    -0.065917052 ],
  [               0,               0,               0,       9.6352234,      -45.235989,       53.124119 ],
  [               0,               0,               0,      -3.9681401,       13.029164,      -11.259424 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.25455695,      0.63758594 ],
  [               0,               0,               0,               0,       35.692554,      -84.864189 ],
  [               0,               0,               0,               0,      -2.9880846,       6.0741615 ],
  [               0,               0,               0,               0,      0.34639624,     -0.78087813 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,       1.4350919,      -3.5536964 ],
  [               0,               0,               0,               0,       1294.3553,      -3229.6909 ],
  [               0,               0,               0,               0,               0,     -0.40000001 ],
  [               0,               0,               0,               0,       142.31461,       -350.5871 ],
  [               0,               0,               0,               0,       1.5195934,      -4.0126295 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam100_layer2_etaoffv3_35_b = [
 [
  [               0,               0,               0,    -0.032686181,      0.01499988,    0.0059491997 ],
  [               0,               0,               0,       18.470282,      -8.6157484,        6.295877 ],
  [               0,               0,               0,      0.36067536,     -0.11108346,      -1.3599626 ],
  [               0,               0,               0,       0.5092001,     -0.21906444,    -0.022941707 ],
  [               0,               0,               0,    0.0040270388,   -0.0023284187,   0.00076795183 ],
  [               0,               0,               0,       18.249683,      -7.7850776,       1.1270971 ],
  [               0,               0,               0,      -16.350126,       7.4836788,      -1.8014815 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0059963246,   -0.0039653638,    0.0068391687 ],
  [               0,               0,               0,      -4.3311167,       2.3282237,        5.778935 ],
  [               0,               0,               0,      0.23163147,      -0.5897786,      -1.1515276 ],
  [               0,               0,               0,      0.30261782,    -0.054658752,    -0.060833715 ],
  [               0,               0,               0,   -0.0024212692,    0.0019433906,    0.0004010979 ],
  [               0,               0,               0,      -2.7679696,        2.647332,      0.29271153 ],
  [               0,               0,               0,      -3.0050488,       1.8533992,       -1.711706 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,       0.0410023,    -0.025087975 ],
  [               0,               0,               0,               0,      -1.2647907,       5.9126515 ],
  [               0,               0,               0,               0,       9.0043678,      -8.6949463 ],
  [               0,               0,               0,               0,      -6.4114146,       5.1941495 ],
  [               0,               0,               0,               0,    -0.015218581,     0.012770037 ],
  [               0,               0,               0,               0,        1.812664,     -0.60637856 ],
  [               0,               0,               0,               0,      -2.8551326,     -0.27983961 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   0.00016469297,   0.00027586959,    0.0065415879 ],
  [               0,               0,               0,        1.292209,      -4.0186467,       7.6209965 ],
  [               0,               0,               0,      0.70560795,      -1.9892923,       -0.161864 ],
  [               0,               0,               0,     -0.28057078,       1.0500065,     -0.70868641 ],
  [               0,               0,               0,    0.0010162368,    -0.001885883,    0.0014599516 ],
  [               0,               0,               0,     -0.28264862,      0.14984623,      0.94568932 ],
  [               0,               0,               0,       2.2228501,      -6.9135818,       2.8745091 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.17763767,    -0.020842271,     -0.18002082,     -0.15616786 ],
  [               0,               0,       64.090973,      -76.976608,      -121.11992,       149.84505 ],
  [               0,               0,      -464.46268,       658.39496,       862.80579,      -1225.1943 ],
  [               0,               0,       563.11646,      -768.88422,      -1060.6348,       1448.2971 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,        0.227745,     -0.31767204 ],
  [               0,               0,               0,               0,      -5074.1084,       7294.0508 ],
  [               0,               0,               0,               0,      -63.482079,       85.904968 ],
  [               0,               0,               0,               0,     -0.34629548,     -0.62958735 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam100_layer2_etaoffv3_35_e = [
 [
  [               0,               0,      -2.0293019,       2.9382956,        4.002542,      -5.7953415 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0014993302,     0.072409116,     0.053543314,     -0.23260476 ],
  [               0,               0,      -113.99232,       158.49385,       246.28371,      -339.26431 ],
  [               0,               0,        257.4006,      -384.21848,      -542.61676,       808.99683 ],
  [               0,               0,      -265.71973,       390.26508,       578.67523,      -849.64832 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.09627565,      0.14853151 ],
  [               0,               0,               0,               0,       324.43448,      -486.22086 ],
  [               0,               0,               0,               0,      -337.70938,       509.92703 ],
  [               0,               0,               0,               0,       296.70673,      -448.53613 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.012308521,    -0.039399333,      0.03889231 ],
  [               0,               0,               0,      -2.8891029,        7.252377,      0.71957028 ],
  [               0,               0,               0,       3.4052882,      -11.528255,        8.292038 ],
  [               0,               0,               0,      -3.1771739,       10.825152,      -9.0747013 ],
  [               0,               0,               0,   -0.0048237424,     0.016440835,    -0.013242376 ],
  [               0,               0,               0,      -8.9187422,       29.885025,      -24.407623 ],
  [               0,               0,               0,       32.713985,      -111.33578,       92.272713 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.015868515,    -0.065174341,     0.075332992 ],
  [               0,               0,               0,      -3.5125186,       12.500486,      -7.1377673 ],
  [               0,               0,               0,      0.86947006,      -3.4378593,       2.0584314 ],
  [               0,               0,               0,      -23.809052,        98.23365,      -100.75799 ],
  [               0,               0,               0,     0.013845362,    -0.056985144,     0.069707766 ],
  [               0,               0,               0,     -0.17767341,      0.75305134,      -0.7770052 ],
  [               0,               0,               0,       18.423832,      -76.032967,       76.477066 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0003339047,    0.0045170276,   0.00079415337 ],
  [               0,               0,               0,       16.981853,       -83.09111,       104.67464 ],
  [               0,               0,               0,       9.1416264,      -43.379539,       50.185036 ],
  [               0,               0,               0,     -0.70285952,       29.580067,      -68.186905 ],
  [               0,               0,               0,    -0.041735634,    -0.088737808,      0.46769178 ],
  [               0,               0,               0,       4.4409819,      -20.720039,       24.174139 ],
  [               0,               0,               0,      -7.2864728,        34.82275,      -42.110149 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.029299423,     0.077467456 ],
  [               0,               0,               0,               0,      -37.148376,       95.054115 ],
  [               0,               0,               0,               0,      -97.035355,       237.60796 ],
  [               0,               0,               0,               0,       99.628311,      -245.19145 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0093183191,    -0.023076141 ],
  [               0,               0,               0,               0,      -8.6427307,      -17.366016 ],
  [               0,               0,               0,               0,   0.00022627418,       514.37372 ],
  [               0,               0,               0,               0,       252.91606,       -302.2764 ],
  [               0,               0,               0,               0,       30.897924,       76.578285 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam200_layer2_etaoffv3_35_b = [
 [
  [               0,               0,               0,    -0.013293795,    0.0084467381,    0.0062784492 ],
  [               0,               0,               0,       7.3707042,      -4.8651609,       6.3400602 ],
  [               0,               0,               0,      0.69380814,      -0.4432897,      -1.3079834 ],
  [               0,               0,               0,      0.81222093,     -0.21022172,    -0.037161253 ],
  [               0,               0,               0,    0.0019337572,   -0.0014291551,   0.00063238619 ],
  [               0,               0,               0,       8.2692919,      -5.4318924,       1.1645509 ],
  [               0,               0,               0,      -23.169811,       8.2952271,      -1.7131435 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.012576091,    -0.016056238,     0.011860414 ],
  [               0,               0,               0,      -6.2612128,       7.0858407,       3.7718036 ],
  [               0,               0,               0,     -0.68384826,      0.62676638,      -1.5103451 ],
  [               0,               0,               0,      0.63715076,     -0.59245628,      0.11206983 ],
  [               0,               0,               0,    -0.006598006,    0.0084112957,   -0.0020590664 ],
  [               0,               0,               0,       -5.624722,       6.6699767,      -1.0537122 ],
  [               0,               0,               0,       2.2172556,      -3.5576897,       -0.219788 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.016283318,     0.021550721 ],
  [               0,               0,               0,               0,       18.193546,      -9.5904484 ],
  [               0,               0,               0,               0,       6.0124297,      -6.1557202 ],
  [               0,               0,               0,               0,      -4.1350746,       3.2843182 ],
  [               0,               0,               0,               0,    0.0031562196,   -0.0026765848 ],
  [               0,               0,               0,               0,       40.822475,      -30.712364 ],
  [               0,               0,               0,               0,       1.2714421,      -2.4254687 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0035742803,    0.0091797244,    0.0013196138 ],
  [               0,               0,               0,       4.0075164,      -10.173725,       11.270399 ],
  [               0,               0,               0,      0.16284615,     -0.65873712,     -0.93923646 ],
  [               0,               0,               0,    -0.033740107,       0.3805638,     -0.30527258 ],
  [               0,               0,               0,    0.0015738005,    -0.003540742,    0.0025892546 ],
  [               0,               0,               0,     -0.29716095,      0.11103256,       1.0687543 ],
  [               0,               0,               0,    -0.091329344,      -1.1286093,      -6.8633809 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.18134911,     -0.03362903,     -0.19261406,     -0.12499351 ],
  [               0,               0,       219.36604,      -280.00055,      -400.86914,       515.39429 ],
  [               0,               0,      -504.77316,       713.04614,       938.58258,      -1327.9994 ],
  [               0,               0,       544.44751,      -741.64954,      -1025.1617,       1396.6019 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.38985083,       0.5734129 ],
  [               0,               0,               0,               0,      -4989.5522,       7182.7441 ],
  [               0,               0,               0,               0,       24.025063,      -34.648434 ],
  [               0,               0,               0,               0,    -0.083934441,     -0.47935116 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam200_layer2_etaoffv3_35_e = [
 [
  [               0,               0,      -3.4722402,       5.0204554,       6.9138899,      -9.9958372 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.026212264,     0.014243205,    -0.016607083,    -0.081847817 ],
  [               0,               0,      -4.9341245,      -5.0681291,        1.603599,       27.686192 ],
  [               0,               0,       24.202305,      -35.521317,      -52.269806,       75.538826 ],
  [               0,               0,       4.5622611,      -7.8606381,      -10.666637,       18.336079 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.050107773,     0.078698084 ],
  [               0,               0,               0,               0,       317.40201,      -473.82343 ],
  [               0,               0,               0,               0,      -273.74142,       412.87527 ],
  [               0,               0,               0,               0,       257.05746,      -388.55182 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0088623343,    -0.028955424,     0.031512573 ],
  [               0,               0,               0,      -1.2589436,       1.7684054,       5.3771167 ],
  [               0,               0,               0,     -0.95928562,       3.2123923,      -4.1033807 ],
  [               0,               0,               0,    -0.097359665,      -2.9610362,       6.4872866 ],
  [               0,               0,               0,   -0.0083654476,    0.0028889263,      0.02844459 ],
  [               0,               0,               0,       2.7818329,      -9.1370449,       7.5609617 ],
  [               0,               0,               0,      -5.7002215,       19.605225,      -19.679014 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,  -0.00022812792,    0.0032166655,     0.003102032 ],
  [               0,               0,               0,         1.19595,      -7.5759158,        14.26864 ],
  [               0,               0,               0,       1.0251638,      -4.0388861,        2.612963 ],
  [               0,               0,               0,      0.99060005,      -220.83905,         511.577 ],
  [               0,               0,               0,     -0.47552675,      0.20002139,       2.1387098 ],
  [               0,               0,               0,    -0.063322105,      0.27803293,     -0.29487914 ],
  [               0,               0,               0,       1.2404422,       -5.008132,       1.9430054 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0091436626,    -0.078072645,      0.13975596 ],
  [               0,               0,               0,       113.74158,      -529.12396,       619.05133 ],
  [               0,               0,               0,       43.156616,      -204.33228,       240.41066 ],
  [               0,               0,               0,      -26.022121,       123.28789,       -145.8952 ],
  [               0,               0,               0,     0.014370304,    -0.057954218,     0.057898574 ],
  [               0,               0,               0,      -26.957985,       129.92673,      -155.60101 ],
  [               0,               0,               0,       26.935987,      -124.32694,       141.61372 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.058065388,      0.14742725 ],
  [               0,               0,               0,               0,      0.31621256,       3.5970333 ],
  [               0,               0,               0,               0,      -130.81734,       320.72922 ],
  [               0,               0,               0,               0,       129.23615,      -318.06357 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0012949164,   0.00029086461,   -0.0060388301 ],
  [               0,               0,               0,      -276.99341,      -1.8835512,       1713.4868 ],
  [               0,               0,               0,               0,               0,     -0.40000001 ],
  [               0,               0,               0,       41.538101,       3.0302005,      -262.61539 ],
  [               0,               0,               0,     0.052231357,      0.12755223,      0.42965817 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam500_layer2_etaoffv3_35_b = [
 [
  [               0,               0,               0,  -1.7205919e-05,    1.822352e-05,    0.0073924814 ],
  [               0,               0,               0,     -0.64240146,     -0.79441965,       6.2049108 ],
  [               0,               0,               0,       2.1783576,     -0.69148046,      -1.3132052 ],
  [               0,               0,               0,       2840.2776,      -1294.9445,       30.223663 ],
  [               0,               0,               0,      -21.008636,       9.5783939,     -0.22381355 ],
  [               0,               0,               0,      0.60150051,     -0.26305956,      0.03520786 ],
  [               0,               0,               0,    -0.019867308,    0.0058903513,    -0.012707893 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     -0.01430033,     0.016622752,     0.001993109 ],
  [               0,               0,               0,        4.796145,      -6.6988778,       8.3610582 ],
  [               0,               0,               0,      -2.7221224,       3.5723822,      -2.4744141 ],
  [               0,               0,               0,       1.3229989,        -1.72018,      0.49645606 ],
  [               0,               0,               0,    0.0061422354,   -0.0065688831,    0.0022825175 ],
  [               0,               0,               0,      -1.1065903,       1.3127856,      0.53904086 ],
  [               0,               0,               0,      -10.942826,       14.631967,      -6.1093459 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.020339929,     0.024542199 ],
  [               0,               0,               0,               0,         13.8993,      -5.8550792 ],
  [               0,               0,               0,               0,        11.13453,      -10.170345 ],
  [               0,               0,               0,               0,      -30.578457,        24.01071 ],
  [               0,               0,               0,               0,     -0.20267162,      0.15887152 ],
  [               0,               0,               0,               0,       1.9961962,      -1.4376037 ],
  [               0,               0,               0,               0,       6.2268963,      -8.2335358 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0070135463,     0.015070612,   -0.0010448169 ],
  [               0,               0,               0,       4.0140204,      -9.5578079,       10.925795 ],
  [               0,               0,               0,    -0.015793707,     -0.32982171,      -1.0368705 ],
  [               0,               0,               0,    -0.026703902,      0.34071401,     -0.31592965 ],
  [               0,               0,               0,    0.0028393148,   -0.0058693616,    0.0035752673 ],
  [               0,               0,               0,      0.70097893,      -1.5889628,       1.7775929 ],
  [               0,               0,               0,       1.8426802,      -5.1953917,       1.7475405 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.18356252,    -0.061485704,     -0.21446346,    -0.050491884 ],
  [               0,               0,       166.90768,      -186.76662,      -285.94028,       319.20306 ],
  [               0,               0,      -926.26648,       1291.3231,       1729.7222,      -2413.5625 ],
  [               0,               0,        493.5452,      -669.63593,      -926.77631,       1257.5988 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.02697196,    -0.017901652 ],
  [               0,               0,               0,               0,       4.9260883,       14.240418 ],
  [               0,               0,               0,               0,      -7.2648745,       8.7286797 ],
  [               0,               0,               0,               0,     0.055348042,     -0.20963977 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam500_layer2_etaoffv3_35_e = [
 [
  [       112.78837,      -226.85432,      -181.49127,       265.36215,       649.47412,       -666.4903 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.020964889,     0.011404873,    -0.012948323,    -0.063987799 ],
  [               0,               0,       -4.918951,      -4.7074013,       1.8505378,       26.768549 ],
  [               0,               0,       1.2674907,     -0.91183925,      -2.2310283,    -0.023687478 ],
  [               0,               0,     -0.91038978,      0.10770782,       1.1412139,       1.1521249 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.018760387,     0.032342605 ],
  [               0,               0,               0,               0,       329.68842,      -492.29333 ],
  [               0,               0,               0,               0,       -163.4202,       245.77904 ],
  [               0,               0,               0,               0,       148.81572,      -224.91199 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.012666527,     0.049111094,    -0.039162621 ],
  [               0,               0,               0,     -0.87978041,      -1.9004493,       10.913184 ],
  [               0,               0,               0,      -4.7370429,       16.522732,      -15.829348 ],
  [               0,               0,               0,      0.58300561,      -1.9717836,       1.8277467 ],
  [               0,               0,               0,    0.0042235255,    -0.014658168,      0.01362828 ],
  [               0,               0,               0,       18.780745,      -65.151611,       56.781017 ],
  [               0,               0,               0,       5.4865813,      -20.497395,       16.539082 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,  -0.00026601963,    0.0014385364,    0.0066453656 ],
  [               0,               0,               0,      -2.5303521,       8.5584574,      -2.9200664 ],
  [               0,               0,               0,      -1.4023079,        6.294796,      -8.3884382 ],
  [               0,               0,               0,      -18.981026,      0.82896006,       111.39495 ],
  [               0,               0,               0,     -0.17608061,     0.087162755,      0.84369689 ],
  [               0,               0,               0,      0.29558918,      -1.2791579,       1.3931621 ],
  [               0,               0,               0,       1.0558192,      -3.9334512,      0.54052752 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0011296762,   -0.0077892509,     0.022124838 ],
  [               0,               0,               0,       6.1076999,      -28.733143,       36.999428 ],
  [               0,               0,               0,       3.3183603,      -15.897397,       17.741816 ],
  [               0,               0,               0,     -0.30623981,       10.772055,      -13.243718 ],
  [               0,               0,               0,     0.006500632,     0.015824378,     0.032697819 ],
  [               0,               0,               0,      0.77056253,      -3.5467825,       4.0865941 ],
  [               0,               0,               0,      -7.5629845,       34.467026,      -42.181637 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.18485723,     -0.44443852 ],
  [               0,               0,               0,               0,      -101.29396,       252.46857 ],
  [               0,               0,               0,               0,       12.172992,      -31.228142 ],
  [               0,               0,               0,               0,       1.2122629,      -2.9119916 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.021405246,    -0.052999958 ],
  [               0,               0,               0,               0,       40.733406,      -140.74243 ],
  [               0,               0,               0,               0,               0,     -0.40000001 ],
  [               0,               0,               0,               0,       29.493858,       16.497355 ],
  [               0,               0,               0,               0,       7.1945777,       17.466059 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam1000_layer2_etaoffv3_35_b = [
 [
  [               0,               0,               0,  -0.00012940065,    5.881532e-05,    0.0072907126 ],
  [               0,               0,               0,       2.7757471,      -2.0351946,       6.5879908 ],
  [               0,               0,               0,      0.25769484,     -0.20961802,      -1.2929279 ],
  [               0,               0,               0,       2799.3398,      -1140.1014,       19.845242 ],
  [               0,               0,               0,      -20.432196,       8.3216372,     -0.14524697 ],
  [               0,               0,               0,      0.35603666,     -0.18941918,     0.036588859 ],
  [               0,               0,               0,     -0.60009247,       0.1848003,    -0.018952163 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,      0.01101262,     -0.00681603,    0.0065988703 ],
  [               0,               0,               0,      -4.2765107,     -0.31463426,        8.224184 ],
  [               0,               0,               0,      -2.4405155,       3.3554528,      -2.4439676 ],
  [               0,               0,               0,       1.2947361,      -1.6928585,      0.47885206 ],
  [               0,               0,               0,   -0.0015318234,  -6.4970656e-05,    0.0012702497 ],
  [               0,               0,               0,      -6.0143666,       6.1387005,     -0.50907183 ],
  [               0,               0,               0,        5.647902,      -3.7509599,      -1.0975134 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.02408834,     0.025175696 ],
  [               0,               0,               0,               0,       17.945787,      -7.4568448 ],
  [               0,               0,               0,               0,       20.575871,      -17.621317 ],
  [               0,               0,               0,               0,      -12.073364,        9.508173 ],
  [               0,               0,               0,               0,     0.021592554,    -0.016108794 ],
  [               0,               0,               0,               0,      0.10434268,      0.96635652 ],
  [               0,               0,               0,               0,       20.938318,      -18.065466 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0052941656,    -0.011205335,     0.012364343 ],
  [               0,               0,               0,      -3.6554048,       7.2865567,       2.1546273 ],
  [               0,               0,               0,     -0.28521594,      0.29715395,      -1.3237441 ],
  [               0,               0,               0,      0.33726403,     -0.51755303,      0.13107374 ],
  [               0,               0,               0,   -0.0015357503,    0.0033442993,   -0.0010531204 ],
  [               0,               0,               0,      0.53209162,      -1.3871439,       1.8099679 ],
  [               0,               0,               0,      -1.2981389,       1.9144861,      -1.9653174 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.22977313,     -0.11498305,     -0.29551554,     0.042968091 ],
  [               0,               0,      -10.979516,       75.018379,       61.259762,      -190.00256 ],
  [               0,               0,      -613.47021,       859.89905,       1146.8248,      -1609.5392 ],
  [               0,               0,       579.61566,      -786.98352,      -1090.0889,       1480.2159 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.19065781,     -0.25333807 ],
  [               0,               0,               0,               0,      0.78231186,       23.177319 ],
  [               0,               0,               0,               0,       -20.82477,       27.591036 ],
  [               0,               0,               0,               0,     0.058938794,     -0.18427226 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam1000_layer2_etaoffv3_35_e = [
 [
  [       134.62169,      -270.81088,      -215.94885,       316.52728,       772.14026,      -792.51666 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,    -0.001745426,     0.028082592,     0.045034666,     0.006649218,     -0.17930739 ],
  [               0,       559.33923,      -416.60959,      -1224.9218,      -899.84418,       2684.3901 ],
  [               0,       114.59543,      -76.467651,      -258.19324,        -213.659,        578.4787 ],
  [               0,      -109.98797,         81.5867,       237.45578,       172.73994,      -512.34155 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   4.6826361e-05,     0.010102753 ],
  [               0,               0,               0,               0,       21.366488,       -28.24272 ],
  [               0,               0,               0,               0,       22.434996,      -35.268131 ],
  [               0,               0,               0,               0,      -20.032005,       30.365896 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0041297469,   -0.0091198487,    0.0099501424 ],
  [               0,               0,               0,       2.9652078,       -16.07966,        24.61713 ],
  [               0,               0,               0,       1.7897979,      -5.5700803,       2.7550738 ],
  [               0,               0,               0,     -0.89301634,       2.7816322,      -2.0097935 ],
  [               0,               0,               0,   -0.0018328369,    0.0041721822,  -0.00076451123 ],
  [               0,               0,               0,      0.04529823,     -0.78414291,       2.1049356 ],
  [               0,               0,               0,       7.3894849,      -24.307608,       18.106005 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   0.00077292934,  -0.00047511535,     0.006387121 ],
  [               0,               0,               0,       1.6225253,      -9.6100149,       16.891195 ],
  [               0,               0,               0,     -0.66822863,       3.2220304,      -5.2349725 ],
  [               0,               0,               0,     -0.12249692,      0.34944841,       0.8781727 ],
  [               0,               0,               0,   0.00011583099,    0.0014191006,    0.0054219388 ],
  [               0,               0,               0,      0.97859216,      -4.2678986,       4.7715464 ],
  [               0,               0,               0,     -0.90045321,       3.7618918,      -6.9170251 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0014593065,    0.0013660558,     0.014673753 ],
  [               0,               0,               0,       10.077457,      -48.556217,       61.825054 ],
  [               0,               0,               0,       8.9642153,      -42.136448,       48.147552 ],
  [               0,               0,               0,      -4.7033081,       29.454929,      -35.818424 ],
  [               0,               0,               0,    0.0049011903,     0.008871519,     0.024489021 ],
  [               0,               0,               0,      -3.0588467,       14.339792,      -16.763056 ],
  [               0,               0,               0,       7.8732729,      -36.526592,       39.249374 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.026973002,    -0.061727639 ],
  [               0,               0,               0,               0,      -214.47571,       532.63446 ],
  [               0,               0,               0,               0,      -105.24407,       257.82245 ],
  [               0,               0,               0,               0,       120.00353,      -295.33658 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.008620711,    -0.021351239 ],
  [               0,               0,               0,               0,      -9.1559744,      -16.595743 ],
  [               0,               0,               0,               0,       0.2522178,      -1633.0286 ],
  [               0,               0,               0,               0,       103.13293,       196.58005 ],
  [               0,               0,               0,               0,       37.373318,       92.255913 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]


gam_layer2_etaoffv3_35_b = [
    gam5_layer2_etaoffv3_35_b,
    gam10_layer2_etaoffv3_35_b,
    gam25_layer2_etaoffv3_35_b,
    gam50_layer2_etaoffv3_35_b,
    gam75_layer2_etaoffv3_35_b,
    gam100_layer2_etaoffv3_35_b,
    gam200_layer2_etaoffv3_35_b,
    gam500_layer2_etaoffv3_35_b,
    gam1000_layer2_etaoffv3_35_b,
    ]

gam_layer2_etaoffv3_35_e = [
    gam5_layer2_etaoffv3_35_e,
    gam10_layer2_etaoffv3_35_e,
    gam25_layer2_etaoffv3_35_e,
    gam50_layer2_etaoffv3_35_e,
    gam75_layer2_etaoffv3_35_e,
    gam100_layer2_etaoffv3_35_e,
    gam200_layer2_etaoffv3_35_e,
    gam500_layer2_etaoffv3_35_e,
    gam1000_layer2_etaoffv3_35_e,
    ]


###############################################################################
# Tables for v3 sshape correction, layer 2, 3x7 electron clusters
#

ele5_layer2_etaoffv3_37_b = [
 [
    [0, 0, 0, 0, 0, 0], # Removed by hand, see below
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele5_layer2_etaoffv3_37_e = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele10_layer2_etaoffv3_37_b = [
 [
  [               0,               0,               0,    -0.039629567,     0.002417333,    0.0079941647 ],
  [               0,               0,               0,       27.902538,      -5.3516641,       3.5514698 ],
  [               0,               0,               0,       1.5910122,      -2.5809231,     -0.87565607 ],
  [               0,               0,               0,     -0.55783284,       1.1380693,      -0.1976575 ],
  [               0,               0,               0,     0.023488445,   -0.0075390306,     0.001280928 ],
  [               0,               0,               0,       4.8455734,      -1.8146991,       1.0560098 ],
  [               0,               0,               0,       18.371094,       -12.07008,      0.15372968 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.059645753,     0.076913647,     -0.01577512 ],
  [               0,               0,               0,       3.7307315,      -6.6664429,       5.7811813 ],
  [               0,               0,               0,       1.9058845,      -2.6586118,     -0.39890748 ],
  [               0,               0,               0,       0.5062663,    -0.030925516,     -0.12587722 ],
  [               0,               0,               0,    0.0067141829,    -0.011693566,    0.0047829309 ],
  [               0,               0,               0,        30.69302,      -34.303677,       10.051933 ],
  [               0,               0,               0,      -48.733135,        49.72224,       -13.41208 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,     0.010305527 ],
  [               0,               0,               0,               0,               0,       2.4951453 ],
  [               0,               0,               0,               0,               0,       -1.055777 ],
  [               0,               0,               0,               0,               0,    -0.061326772 ],
  [               0,               0,               0,               0,               0,   0.00058030593 ],
  [               0,               0,               0,               0,               0,       1.1295832 ],
  [               0,               0,               0,               0,               0,     -0.12593579 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.010949301,    -0.018057862,     0.016603054 ],
  [               0,               0,               0,     -0.25117889,      -1.7489433,        4.680418 ],
  [               0,               0,               0,       2.3183384,      -4.3823466,      0.76901495 ],
  [               0,               0,               0,      -1.0682375,       2.3462479,      -1.1723946 ],
  [               0,               0,               0,    0.0077074575,    -0.014229057,    0.0063671353 ],
  [               0,               0,               0,       6.6101761,      -15.841561,       10.616986 ],
  [               0,               0,               0,       12.069868,      -34.944359,       25.085155 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.62757015,      0.31314448,     -0.36730871,      -1.6595755 ],
  [               0,               0,       70.041153,      -44.582512,      -263.41693,       265.09656 ],
  [               0,               0,       -101.0907,       137.00311,        192.9213,      -262.24203 ],
  [               0,               0,        3.845521,      -6.8003879,      -7.9183269,       13.902239 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,        83.23172,      -116.58435 ],
  [               0,               0,               0,               0,       5.4162245,      -6.3673563 ],
  [               0,               0,               0,               0,       12.799503,      -119.18933 ],
  [               0,               0,               0,               0,      -2675.8672,       3846.5522 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele10_layer2_etaoffv3_37_e = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
    # Copied from 55
  [               0,               0,               0,               0,   -0.0075912634,     0.015295829 ],
  [               0,               0,               0,               0,      -7.5756946,       16.063869 ],
  [               0,               0,               0,               0,       7.3288541,      -12.312897 ],
  [               0,               0,               0,               0,      -3.8369305,       5.9768329 ],
  [               0,               0,               0,               0,    0.0074901818,    -0.010776109 ],
  [               0,               0,               0,               0,     -0.70966327,       2.2692988 ],
  [               0,               0,               0,               0,      -1.0009131,      0.73172843 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele25_layer2_etaoffv3_37_b = [
 [
  [               0,               0,               0,    -0.059519559,     0.031667013,    0.0035070397 ],
  [               0,               0,               0,       19.464464,      -12.518871,       6.3310266 ],
  [               0,               0,               0,      -2.7716622,      0.87005842,      -1.4029831 ],
  [               0,               0,               0,        1.144944,     -0.14290915,    -0.035594322 ],
  [               0,               0,               0,       0.0181327,    -0.009803853,     0.001831107 ],
  [               0,               0,               0,       20.623638,      -8.5136557,       1.2128699 ],
  [               0,               0,               0,      -14.351976,       4.4659944,      -1.6782701 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0070583122,   -0.0068714856,    0.0076361713 ],
  [               0,               0,               0,      -11.331571,       12.389533,       1.4714108 ],
  [               0,               0,               0,      0.17880604,     -0.99749172,      -1.0049603 ],
  [               0,               0,               0,      0.47683829,      0.12598126,     -0.11861662 ],
  [               0,               0,               0,  -0.00040843559,   9.9293822e-05,    0.0010453945 ],
  [               0,               0,               0,       1.2373102,      -2.4157481,       1.8173809 ],
  [               0,               0,               0,       1.9007838,      -4.6678424,      0.12757409 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,       0.1001395,    -0.075989269 ],
  [               0,               0,               0,               0,       46.051003,      -31.991072 ],
  [               0,               0,               0,               0,       24.270847,      -21.393864 ],
  [               0,               0,               0,               0,       16.711184,      -13.041541 ],
  [               0,               0,               0,               0,     0.060386274,    -0.045608465 ],
  [               0,               0,               0,               0,      -34.287365,        28.40234 ],
  [               0,               0,               0,               0,       -53.55439,       34.581074 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0012398966,   -0.0039213444,     0.010513989 ],
  [               0,               0,               0,       3.3075659,      -8.1240206,       8.5060301 ],
  [               0,               0,               0,     -0.54449338,      0.91425586,       -1.814533 ],
  [               0,               0,               0,       5.2688451,      -4.7883391,      0.53189957 ],
  [               0,               0,               0,     0.030184928,    -0.018513829,   -0.0042541074 ],
  [               0,               0,               0,        1.750967,      -4.1397305,       2.5014434 ],
  [               0,               0,               0,      -2.4335158,       5.0532947,      -5.4940004 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,       3.0611231,      -7.9041023,       5.0920482 ],
  [               0,               0,               0,      -1480.7426,       3817.8113,      -2448.4431 ],
  [               0,               0,               0,      -243.73672,       714.49304,      -523.11841 ],
  [               0,               0,               0,       45.987286,      -113.16079,       69.160019 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.10907422,      0.15789561 ],
  [               0,               0,               0,               0,      -94972.047,       136140.02 ],
  [               0,               0,               0,               0,        242.1264,      -327.88501 ],
  [               0,               0,               0,               0,      -3.8051653,      -7.7768774 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele25_layer2_etaoffv3_37_e = [
 [
  [               0,               0,      -2.9336343,       4.2060995,        5.915719,      -8.4777842 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,   0.00096285134 ],
  [               0,               0,               0,               0,               0,           0.001 ],
  [               0,               0,               0,               0,               0,      0.91628003 ],
  [               0,               0,               0,               0,               0,      0.93324161 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.094165273,     -0.13763034 ],
  [               0,               0,               0,               0,       123.90232,      -184.56273 ],
  [               0,               0,               0,               0,      -71.473503,       106.88964 ],
  [               0,               0,               0,               0,       47.433403,      -71.626762 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,      -0.0893448,      0.29143259,     -0.22801012 ],
  [               0,               0,               0,       39.890076,      -131.09032,       110.81627 ],
  [               0,               0,               0,      -12.202379,       40.272503,      -34.430836 ],
  [               0,               0,               0,        6.855185,      -22.662174,       18.726748 ],
  [               0,               0,               0,     0.037785064,     -0.12494016,      0.10326006 ],
  [               0,               0,               0,      0.74748945,      -0.2876505,      -1.2728822 ],
  [               0,               0,               0,       86.397942,      -311.73346,       278.57724 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.007256032,    -0.021586804,     0.022763517 ],
  [               0,               0,               0,     -0.50477517,     -0.52886486,       6.1486568 ],
  [               0,               0,               0,      0.85822326,      -2.8697195,      0.97764945 ],
  [               0,               0,               0,      -3.6306207,       14.753362,      -14.877485 ],
  [               0,               0,               0,   0.00056261208,   -0.0014542546,    0.0014124505 ],
  [               0,               0,               0,       9.4949646,      -42.135082,       46.943584 ],
  [               0,               0,               0,       39.563343,      -161.02328,       161.39268 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0065404917,   -0.0054609841,     0.066334173 ],
  [               0,               0,               0,       84.760323,      -404.96945,       485.68106 ],
  [               0,               0,               0,      -45.388268,       215.55884,      -256.94199 ],
  [               0,               0,               0,       9.8871593,      -46.553566,       54.777348 ],
  [               0,               0,               0,   -0.0020181646,   -0.0075517963,     0.029956911 ],
  [               0,               0,               0,      -3.4922717,       19.604294,      -25.556461 ],
  [               0,               0,               0,        1.300525,      -30.555435,       59.245731 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,     0.021732476 ],
  [               0,               0,               0,               0,               0,       1.5376394 ],
  [               0,               0,               0,               0,               0,     -0.99606758 ],
  [               0,               0,               0,               0,               0,     0.025948245 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele50_layer2_etaoffv3_37_b = [
 [
  [               0,               0,               0,    -0.034914311,     0.016380591,    0.0059083374 ],
  [               0,               0,               0,       13.482039,      -6.4196925,       5.4362078 ],
  [               0,               0,               0,      -1.2647718,      0.45016864,      -1.3853991 ],
  [               0,               0,               0,       1.1819952,     -0.42147651,   1.6589258e-05 ],
  [               0,               0,               0,    0.0025127465,     -0.00165082,   0.00082176272 ],
  [               0,               0,               0,       15.624002,      -7.1381865,       1.0959517 ],
  [               0,               0,               0,      -15.351125,       6.4946713,      -1.8208793 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0074917497,    0.0023055065 ],
  [               0,               0,               0,               0,      -4.7476497,       7.7435956 ],
  [               0,               0,               0,               0,      0.10152411,      -1.5749391 ],
  [               0,               0,               0,               0,      0.20042436,    -0.018339157 ],
  [               0,               0,               0,               0,   -0.0019707584,    0.0019622245 ],
  [               0,               0,               0,               0,      -1.0063353,       1.4085526 ],
  [               0,               0,               0,               0,      -2.0800982,      -13.435364 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.021351181,   -0.0085985614 ],
  [               0,               0,               0,               0,        18.51642,      -10.790451 ],
  [               0,               0,               0,               0,       5.8850961,       -6.171052 ],
  [               0,               0,               0,               0,      -7.5318394,       8.1155777 ],
  [               0,               0,               0,               0,     0.015423011,    0.0052861683 ],
  [               0,               0,               0,               0,      -1.1564401,       1.0336657 ],
  [               0,               0,               0,               0,       1.3534755,      -4.1623597 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0022883709,    0.0053132367,    0.0049068397 ],
  [               0,               0,               0,       1.7850643,      -5.2336731,       7.4975634 ],
  [               0,               0,               0,    -0.076057605,     -0.28851435,       -1.091814 ],
  [               0,               0,               0,      -1.8553716,      0.56487668,       3.9972296 ],
  [               0,               0,               0,    -0.016954647,    0.0069670165,     0.030984528 ],
  [               0,               0,               0,      0.22246164,     -0.21724862,     0.058463495 ],
  [               0,               0,               0,       1.1861309,      -2.7665603,      -1.4357259 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.14007398,    -0.065193333,     -0.17676987,     0.012461085 ],
  [               0,               0,      -5737.2065,        7963.396,       10519.438,      -14598.751 ],
  [               0,               0,      -2418.9133,       3340.0703,       4545.3994,      -6278.2847 ],
  [               0,               0,       676.98151,      -925.48859,      -1268.6517,       1734.6633 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.12272504,     -0.16626084 ],
  [               0,               0,               0,               0,      -2190.1101,       3149.3928 ],
  [               0,               0,               0,               0,      -63.828503,       88.414467 ],
  [               0,               0,               0,               0,     -0.34504449,     -0.60712814 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele50_layer2_etaoffv3_37_e = [
 [
  [               0,               0,     -0.36268052,       0.5566045,      0.60733581,     -0.94513291 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    -0.023177244,     0.037634749,     0.053057637,    -0.082179852 ],
  [               0,               0,       13.625786,       -6.815567,      -18.303478,       2.6016698 ],
  [               0,               0,      -321.12674,       479.99268,        695.4079,      -1040.3053 ],
  [               0,               0,      -57.202324,       84.280083,       123.01023,      -181.03615 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0021960919,  -0.00062439247 ],
  [               0,               0,               0,               0,       21.346859,      -27.184044 ],
  [               0,               0,               0,               0,      -216.25351,       326.04446 ],
  [               0,               0,               0,               0,       185.20737,      -279.86169 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.035710882,     -0.11186987,      0.09332221 ],
  [               0,               0,               0,      -11.857295,       35.953075,      -22.284899 ],
  [               0,               0,               0,        5.050642,      -17.731035,       14.187762 ],
  [               0,               0,               0,      -1.6772385,       6.1114545,      -5.4903998 ],
  [               0,               0,               0,   0.00018309274,   -0.0026957756,    0.0049049859 ],
  [               0,               0,               0,      -5.8129854,       18.311037,      -13.512968 ],
  [               0,               0,               0,      -1.5768845,       2.1104531,     -0.66574508 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,      0.02077326,    -0.083797053,     0.092875399 ],
  [               0,               0,               0,      -6.7006426,       25.582384,      -20.868467 ],
  [               0,               0,               0,      0.94359952,      -3.5071018,       1.9131992 ],
  [               0,               0,               0,      -5.0181036,         19.5355,      -18.700867 ],
  [               0,               0,               0,    0.0025989241,   -0.0091749458,     0.010481995 ],
  [               0,               0,               0,       2.5413606,      -11.036286,       12.061861 ],
  [               0,               0,               0,       11.107567,      -40.954224,       34.933792 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0047899126,    -0.018942663,     0.082076743 ],
  [               0,               0,               0,       71.618599,      -333.43283,       390.68591 ],
  [               0,               0,               0,       5.6639709,      -27.619034,        32.41647 ],
  [               0,               0,               0,      -9.2100143,       33.555164,      -33.933617 ],
  [               0,               0,               0,    -0.049093127,     0.022516742,      0.29057851 ],
  [               0,               0,               0,      0.24707609,      -1.0137401,       1.0221266 ],
  [               0,               0,               0,      -8.6436281,       42.628952,      -52.851788 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.049118303,      0.12366898 ],
  [               0,               0,               0,               0,     -0.30235478,       5.1536322 ],
  [               0,               0,               0,               0,      -308.41144,       758.22125 ],
  [               0,               0,               0,               0,       302.05554,      -743.56354 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.75998026,       -1.882044 ],
  [               0,               0,               0,               0,      -5.5074382,      -6.4711256 ],
  [               0,               0,               0,               0,    -0.018572832,       7.4605265 ],
  [               0,               0,               0,               0,      -85.512901,       212.92497 ],
  [               0,               0,               0,               0,    -0.081641465,     -0.79815114 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele75_layer2_etaoffv3_37_b = [
 [
  [               0,               0,               0,    -0.032422766,     0.017319666,    0.0052146618 ],
  [               0,               0,               0,       21.795929,      -11.275863,       6.4056468 ],
  [               0,               0,               0,     -0.29429406,    -0.061510213,      -1.3400916 ],
  [               0,               0,               0,      0.93133652,     -0.11537865,    -0.056765839 ],
  [               0,               0,               0,    0.0030630499,   -0.0023010238,     0.001089938 ],
  [               0,               0,               0,       10.341709,      -5.6396074,       1.0409296 ],
  [               0,               0,               0,      -15.616896,        4.883348,      -1.5326411 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.031367052,     0.032236304,  -0.00057040725 ],
  [               0,               0,               0,       8.6598387,      -9.5502071,       7.4156761 ],
  [               0,               0,               0,       -2.865365,       2.8564088,      -2.1212778 ],
  [               0,               0,               0,       2.1820481,       -2.386256,       0.7119363 ],
  [               0,               0,               0,     0.014303228,     -0.01641109,    0.0052718273 ],
  [               0,               0,               0,     0.057254169,       1.6260763,     -0.36725533 ],
  [               0,               0,               0,       3.2480364,      -2.3937991,      -1.8929414 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.083841912,     -0.06023638 ],
  [               0,               0,               0,               0,       1.6659402,       3.5693719 ],
  [               0,               0,               0,               0,       16.145954,      -14.541373 ],
  [               0,               0,               0,               0,      -9.9259739,       8.0937862 ],
  [               0,               0,               0,               0,    -0.032156754,     0.026810938 ],
  [               0,               0,               0,               0,      -8.5109863,        7.548461 ],
  [               0,               0,               0,               0,      -14.693822,       9.1915798 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0013156804,    0.0048943092,    0.0035797588 ],
  [               0,               0,               0,       1.2298521,      -4.4493976,       7.7028565 ],
  [               0,               0,               0,        1.323128,      -2.9824407,      0.11036333 ],
  [               0,               0,               0,     -0.15319753,      0.85349208,     -0.54200584 ],
  [               0,               0,               0,    0.0048860102,   -0.0091387387,    0.0050294246 ],
  [               0,               0,               0,      -1.3244801,       1.6848652,      0.35788539 ],
  [               0,               0,               0,       3.8849339,      -9.8656311,        3.617451 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.19257811,    0.0075861565,     -0.17344749,     -0.25301206 ],
  [               0,               0,       300.01813,      -401.39536,       -559.1203,       751.22522 ],
  [               0,               0,      -136.50546,       206.10413,       247.96962,      -377.14413 ],
  [               0,               0,       569.75574,      -779.29303,      -1072.5435,       1467.0565 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.26073226,     -0.36439103 ],
  [               0,               0,               0,               0,      -2899.7544,       4169.3164 ],
  [               0,               0,               0,               0,      -19.888027,       24.046329 ],
  [               0,               0,               0,               0,      -0.2812162,     -0.61587483 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele75_layer2_etaoffv3_37_e = [
 [
  [               0,               0,     -0.90480036,       1.3200723,       1.7057935,      -2.4907944 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   -0.0019656795,   0.00080101134,    0.0040561827,    0.0068969796 ],
  [               0,               0,      -21.821453,       37.336868,       51.425484,      -84.498055 ],
  [               0,               0,      -88.343437,       130.72716,       191.79112,      -284.75525 ],
  [               0,               0,      -19.858583,       29.058273,       43.131996,      -63.011894 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,    0.0077412282 ],
  [               0,               0,               0,               0,               0,       3.3465798 ],
  [               0,               0,               0,               0,               0,      -1.2192165 ],
  [               0,               0,               0,               0,               0,       0.1047849 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.021165706,     0.058678605,    -0.032408316 ],
  [               0,               0,               0,       27.550434,      -86.791992,       72.748573 ],
  [               0,               0,               0,        1.813109,      -5.8667603,       3.3996317 ],
  [               0,               0,               0,       0.4481923,      -1.7660837,       1.7466655 ],
  [               0,               0,               0,     0.020133954,    -0.064730212,     0.052787159 ],
  [               0,               0,               0,      -4.8770547,       18.481447,      -16.507265 ],
  [               0,               0,               0,      -9.5600128,       34.532948,      -32.609558 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.034732599,       0.1493572,     -0.15052392 ],
  [               0,               0,               0,       10.430398,      -46.494217,       54.642403 ],
  [               0,               0,               0,      0.12073065,      -0.2986913,      -1.1649768 ],
  [               0,               0,               0,      -20.981573,       87.142204,       -90.30011 ],
  [               0,               0,               0,   0.00069004286,     0.001264052,    0.0015308833 ],
  [               0,               0,               0,      0.68181026,      -2.8415508,       2.9775538 ],
  [               0,               0,               0,       27.550713,      -114.39654,       116.99305 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0014429841,   -0.0035394232,       0.0270171 ],
  [               0,               0,               0,       14.250721,      -67.343201,       82.390594 ],
  [               0,               0,               0,      -19.480705,       93.570229,      -113.58534 ],
  [               0,               0,               0,      -6.4464526,       44.675098,      -59.246887 ],
  [               0,               0,               0,    0.0076324944,     0.013561239,     0.034293976 ],
  [               0,               0,               0,       3.1085091,      -15.218177,       18.633711 ],
  [               0,               0,               0,       11.567733,      -53.637634,       59.046391 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.16870375,      0.41999784 ],
  [               0,               0,               0,               0,      -363.21927,       899.14722 ],
  [               0,               0,               0,               0,      -15.950859,       38.441593 ],
  [               0,               0,               0,               0,        86.75811,       -213.4834 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0014271861,   -0.0035348232 ],
  [               0,               0,               0,               0,      -9.7415514,      -14.152486 ],
  [               0,               0,               0,               0,   1.6151776e+12,   3.3136673e+11 ],
  [               0,               0,               0,               0,        545.1795,       249.16081 ],
  [               0,               0,               0,               0,       312.32602,       102.02951 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele100_layer2_etaoffv3_37_b = [
 [
  [               0,               0,               0,    -0.035684194,     0.016122308,    0.0059566824 ],
  [               0,               0,               0,       16.350527,      -7.5450501,        5.845685 ],
  [               0,               0,               0,      0.68729746,     -0.31588721,      -1.3287066 ],
  [               0,               0,               0,      0.23588882,    -0.070635557,    -0.039036144 ],
  [               0,               0,               0,    0.0054420359,    -0.002679059,   0.00084090664 ],
  [               0,               0,               0,        17.25478,      -7.7323694,        1.164378 ],
  [               0,               0,               0,      -12.309258,        5.489646,       -1.581338 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0083424505,    -0.010863549,     0.010658515 ],
  [               0,               0,               0,      -10.257724,       11.209408,        2.111347 ],
  [               0,               0,               0,     -0.98086375,      0.60054469,      -1.4500763 ],
  [               0,               0,               0,      0.91784316,     -0.62441045,     0.092609979 ],
  [               0,               0,               0,   -0.0011077926,    0.0023944739,  -0.00045965079 ],
  [               0,               0,               0,      0.89218861,     -0.55745518,      0.69920802 ],
  [               0,               0,               0,       2.7987158,      -6.1532054,      0.72612637 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.044708457,    -0.028210957 ],
  [               0,               0,               0,               0,       8.2887926,      -1.9043406 ],
  [               0,               0,               0,               0,       10.355363,      -9.8431377 ],
  [               0,               0,               0,               0,      -8.4703569,       6.9251986 ],
  [               0,               0,               0,               0,    -0.026101332,     0.021974761 ],
  [               0,               0,               0,               0,     -0.68592685,        1.207195 ],
  [               0,               0,               0,               0,      -1.2798135,      -1.7020631 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0083913263,     0.017409163,   -0.0010438453 ],
  [               0,               0,               0,       5.0705724,      -11.870066,       11.091817 ],
  [               0,               0,               0,     -0.13110296,     -0.17675172,       -1.154828 ],
  [               0,               0,               0,     0.035769343,     0.085139684,     0.086003967 ],
  [               0,               0,               0,    0.0013074483,   -0.0040626405,    0.0040522376 ],
  [               0,               0,               0,      0.15377207,      0.50173163,     -0.32287508 ],
  [               0,               0,               0,        3.171387,      -6.8174696,      0.94495291 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.19299416,    -0.028966613,     -0.19996344,     -0.15199076 ],
  [               0,               0,       445.37567,      -600.19794,      -812.70123,       1098.5092 ],
  [               0,               0,      -501.23264,       706.99103,       931.95447,      -1316.5662 ],
  [               0,               0,       640.77869,      -875.38409,      -1205.8644,       1647.4677 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.31114075,     -0.43450928 ],
  [               0,               0,               0,               0,      -2326.9939,       3349.5586 ],
  [               0,               0,               0,               0,      -20.320961,       25.183357 ],
  [               0,               0,               0,               0,     -0.16815712,     -0.44113058 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele100_layer2_etaoffv3_37_e = [
 [
  [               0,               0,      -1.7488184,       2.5351996,        3.437041,      -4.9830942 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     -0.00625988,     0.099948451,     0.082701221,     -0.31071931 ],
  [               0,               0,       328.84427,       -500.5668,      -715.33917,       1091.8317 ],
  [               0,               0,       372.05426,      -557.42572,      -782.47644,       1171.7136 ],
  [               0,               0,       -402.4985,        591.8075,       878.67548,      -1291.6791 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      -0.2765874,      0.42950961 ],
  [               0,               0,               0,               0,       35.255642,      -50.509884 ],
  [               0,               0,               0,               0,        4.839602,      -8.4706507 ],
  [               0,               0,               0,               0,      -9.1336451,       13.889434 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.013982037,    -0.045667961,     0.044970777 ],
  [               0,               0,               0,      -6.0786886,       18.591143,      -9.6101685 ],
  [               0,               0,               0,      0.26891655,     -0.90991479,     -0.60475963 ],
  [               0,               0,               0,     -0.15336706,      -3.2427945,       7.1237612 ],
  [               0,               0,               0,   -0.0082606124,   -0.0013353918,     0.035524968 ],
  [               0,               0,               0,       1.9500819,      -6.3566575,       5.2265005 ],
  [               0,               0,               0,      -5.1210375,       17.443399,      -17.588606 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0020024297,    0.0045065978 ],
  [               0,               0,               0,               0,      -2.5846903,       8.8261566 ],
  [               0,               0,               0,               0,      0.27639672,      -1.8728839 ],
  [               0,               0,               0,               0,      -1.6010433,       2.3811092 ],
  [               0,               0,               0,               0,     0.061418723,    -0.094905794 ],
  [               0,               0,               0,               0,   -0.0093050739,     0.026643865 ],
  [               0,               0,               0,               0,    -0.027687926,      -1.2451237 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0025833058,   5.3120824e-05,     0.023859899 ],
  [               0,               0,               0,       32.303986,      -152.33205,       182.62057 ],
  [               0,               0,               0,       11.454291,      -54.593033,       63.780033 ],
  [               0,               0,               0,      -11.612725,       19.098227,       6.5846219 ],
  [               0,               0,               0,     0.025612419,   -0.0085369302,    0.0063187322 ],
  [               0,               0,               0,       1.5210794,      -7.0485744,       8.1684151 ],
  [               0,               0,               0,      -6.0820069,       31.321625,      -40.273647 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.22375032,      0.55825371 ],
  [               0,               0,               0,               0,       56.459171,      -135.98566 ],
  [               0,               0,               0,               0,      -57.139999,       139.50397 ],
  [               0,               0,               0,               0,       55.716774,      -137.09956 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0071329256,    0.0093248375 ],
  [               0,               0,               0,               0,        6.507946,       23.820698 ],
  [               0,               0,               0,               0,               0,     -0.40000001 ],
  [               0,               0,               0,               0,       37.275978,      -92.142311 ],
  [               0,               0,               0,               0,      0.36995152,      0.78015119 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele200_layer2_etaoffv3_37_b = [
 [
  [               0,               0,               0,    -0.010812913,    0.0050374516,    0.0071449545 ],
  [               0,               0,               0,       9.4903202,      -4.2299085,       5.6963534 ],
  [               0,               0,               0,    -0.066645168,     0.013229694,      -1.3670332 ],
  [               0,               0,               0,      -31.228029,       15.508408,      -3.3281064 ],
  [               0,               0,               0,      0.23800381,     -0.11890941,     0.025788916 ],
  [               0,               0,               0,     -0.93118006,      0.41907781,    0.0074310992 ],
  [               0,               0,               0,      -4.0797572,       2.0047014,     -0.43090662 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.021965986,    -0.028927604,     0.016261294 ],
  [               0,               0,               0,      -14.405105,       17.258747,      0.37207007 ],
  [               0,               0,               0,        -1.66471,       1.5260887,       -1.691222 ],
  [               0,               0,               0,         1.49536,      -1.4357201,      0.30820805 ],
  [               0,               0,               0,   -0.0062553715,     0.008618189,   -0.0023009733 ],
  [               0,               0,               0,      -5.1654143,        6.937078,      -1.4949328 ],
  [               0,               0,               0,       -1.032732,      -1.8158484,     0.052535418 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.011231342,  -0.00068744749 ],
  [               0,               0,               0,               0,       10.506271,      -3.4166007 ],
  [               0,               0,               0,               0,       4.1726356,      -4.7016754 ],
  [               0,               0,               0,               0,      -3.5529404,       2.8236003 ],
  [               0,               0,               0,               0,    0.0015875144,   -0.0014606934 ],
  [               0,               0,               0,               0,      -5.1344357,       6.1362581 ],
  [               0,               0,               0,               0,       11.488153,      -10.770435 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.017400319,     0.037657589,    -0.012678454 ],
  [               0,               0,               0,       8.5151606,      -19.283833,       15.379314 ],
  [               0,               0,               0,     -0.79952908,       1.4928914,      -2.1622269 ],
  [               0,               0,               0,     0.027761484,      0.21548736,     -0.15342931 ],
  [               0,               0,               0,    0.0029536132,   -0.0062253149,     0.003860451 ],
  [               0,               0,               0,       3.1854329,      -7.2901764,       4.8463707 ],
  [               0,               0,               0,       6.6749463,      -14.726322,       5.8079419 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.20016748,    -0.045442138,     -0.21843381,      -0.1151446 ],
  [               0,               0,       450.02554,      -592.65631,      -820.64429,       1083.9767 ],
  [               0,               0,      -537.55273,       758.20874,       999.16876,      -1411.5089 ],
  [               0,               0,       598.66144,       -815.9848,      -1126.2668,       1535.2551 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.11329667,     -0.14059612 ],
  [               0,               0,               0,               0,       -7.783772,       26.784828 ],
  [               0,               0,               0,               0,      -6.2224603,       7.1517391 ],
  [               0,               0,               0,               0,     0.045346584,     -0.15573856 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele200_layer2_etaoffv3_37_e = [
 [
  [               0,               0,      -2.8809376,       4.1703234,        5.724226,      -8.2860107 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0063779815,    0.0037081472,   -0.0029718101,    -0.016387425 ],
  [               0,               0,     -0.33056015,      -3.4627919,      -2.3235404,       15.298978 ],
  [               0,               0,       8.3620548,      -11.679837,       -17.77593,       23.632708 ],
  [               0,               0,       1.0193143,      -2.5463498,      -2.9155333,       6.7207031 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.05147269,     0.082552992 ],
  [               0,               0,               0,               0,        152.0358,      -225.15256 ],
  [               0,               0,               0,               0,      -78.651657,       117.58639 ],
  [               0,               0,               0,               0,       75.985641,      -114.73061 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   0.00050198665,  -0.00048817298,     0.007323049 ],
  [               0,               0,               0,      -0.8355788,      0.91124415,       5.3926239 ],
  [               0,               0,               0,      -0.1644607,      0.77500021,      -2.2534659 ],
  [               0,               0,               0,      0.59146303,      -3.1422169,       4.2537804 ],
  [               0,               0,               0,   -0.0023720721,   0.00030468797,     0.011164528 ],
  [               0,               0,               0,       2.4320245,      -8.3809843,       7.3260403 ],
  [               0,               0,               0,       -4.991847,       17.757376,      -18.471582 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0078624086,    -0.026030101,     0.028693598 ],
  [               0,               0,               0,       1.5258487,       -10.19174,       18.150759 ],
  [               0,               0,               0,     -0.22485214,       1.6289076,      -3.7803376 ],
  [               0,               0,               0,      -2.2321694,       8.9266853,       -8.776825 ],
  [               0,               0,               0,   -0.0045900703,     0.019560408,    -0.019913673 ],
  [               0,               0,               0,       3.1366456,      -14.720022,       17.495426 ],
  [               0,               0,               0,       21.525284,       -86.46386,       84.377525 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0027292073,    -0.016164554,     0.064102352 ],
  [               0,               0,               0,       29.905581,      -136.79515,       159.19182 ],
  [               0,               0,               0,       25.802891,      -123.57459,       146.63155 ],
  [               0,               0,               0,      -12.961827,        62.55835,      -75.367249 ],
  [               0,               0,               0,    0.0057086851,    -0.014743795,    0.0033166979 ],
  [               0,               0,               0,       11.930021,      -64.498352,       86.490463 ],
  [               0,               0,               0,      -14.950722,        85.23365,      -120.87122 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.031960953,    -0.073698051 ],
  [               0,               0,               0,               0,      -204.39796,       506.98679 ],
  [               0,               0,               0,               0,      -98.330505,       240.90826 ],
  [               0,               0,               0,               0,       118.81335,      -292.41486 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0025385632,   -0.0062988307 ],
  [               0,               0,               0,               0,      -9.9141407,      -14.692234 ],
  [               0,               0,               0,               0,         9787260,        -5165073 ],
  [               0,               0,               0,               0,       336.67026,        575.7132 ],
  [               0,               0,               0,               0,       308.78836,       493.70312 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele500_layer2_etaoffv3_37_b = [
 [
  [               0,               0,               0,    -0.038665745,     0.016615035,    0.0059112599 ],
  [               0,               0,               0,       23.247166,       -10.25654,       6.7349916 ],
  [               0,               0,               0,      0.64426482,     -0.19778441,      -1.3673534 ],
  [               0,               0,               0,     0.089243591,    -0.073715962,    -0.024073536 ],
  [               0,               0,               0,     0.010400258,    -0.004542639,   0.00076350244 ],
  [               0,               0,               0,       24.638494,      -10.377732,       1.3011957 ],
  [               0,               0,               0,      -16.809437,       7.4125886,      -1.8172776 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.011753778,     0.016888784,    0.0011185966 ],
  [               0,               0,               0,       3.5211327,      -6.8384953,       8.5357199 ],
  [               0,               0,               0,      -1.9930772,       2.2971604,      -2.0150316 ],
  [               0,               0,               0,       1.2484807,       -1.357554,      0.33696923 ],
  [               0,               0,               0,    0.0050431811,    -0.006825957,    0.0027821511 ],
  [               0,               0,               0,      -3.3705969,       3.4636171,      0.06532792 ],
  [               0,               0,               0,      -9.0606813,       10.522647,      -4.4866133 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.028086392,    -0.015767626 ],
  [               0,               0,               0,               0,       6.0094919,      0.78092599 ],
  [               0,               0,               0,               0,       10.404808,      -9.7791805 ],
  [               0,               0,               0,               0,       -6.776618,       5.4378581 ],
  [               0,               0,               0,               0,     -0.01689776,     0.014325499 ],
  [               0,               0,               0,               0,       1.1201541,      0.10456853 ],
  [               0,               0,               0,               0,       5.1479001,      -6.0557919 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0028328064,    0.0067019584,    0.0032779505 ],
  [               0,               0,               0,       2.6144085,      -6.8029284,       9.2794733 ],
  [               0,               0,               0,      0.26757923,     -0.97756463,     -0.71523649 ],
  [               0,               0,               0,     -0.14636455,      0.65099561,     -0.46958676 ],
  [               0,               0,               0,    0.0011225272,   -0.0021367134,    0.0015716857 ],
  [               0,               0,               0,      0.35020491,      -1.0811028,       1.5886347 ],
  [               0,               0,               0,        2.159404,      -6.2113619,      -4.0430012 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,       0.2197421,    -0.086900569,     -0.26636019,     -0.02321187 ],
  [               0,               0,       579.01648,      -727.53308,      -1039.9664,       1307.5659 ],
  [               0,               0,      -872.97644,       1220.9554,       1625.3767,      -2275.5608 ],
  [               0,               0,       769.84131,      -1047.9406,      -1448.8955,       1972.4434 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.31301945,     -0.43615654 ],
  [               0,               0,               0,               0,       -5185.334,       7465.4058 ],
  [               0,               0,               0,               0,      -64.726082,       89.011131 ],
  [               0,               0,               0,               0,       25.420338,      -36.895638 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele500_layer2_etaoffv3_37_e = [
 [
  [       96.148186,      -193.62019,      -154.77086,       226.89055,       554.81018,      -570.05505 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0082566952,    0.0044435086,   -0.0043880702,    -0.021428691 ],
  [               0,               0,      0.69194186,      -4.2022557,      -3.9589784,       16.095985 ],
  [               0,               0,       8.1788435,      -12.406739,       -18.28709,       26.481497 ],
  [               0,               0,       1.2760484,      -2.6448166,      -3.2820473,       6.6563144 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.033512313,     0.054693602 ],
  [               0,               0,               0,               0,       235.45389,      -350.73547 ],
  [               0,               0,               0,               0,      -164.21179,       247.09103 ],
  [               0,               0,               0,               0,       148.31149,      -224.15591 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.014913209,    -0.049730334,      0.04944174 ],
  [               0,               0,               0,      -6.1030154,        18.55607,      -9.1107779 ],
  [               0,               0,               0,      -1.1271558,       3.7894578,      -4.5985866 ],
  [               0,               0,               0,     -0.93328744,       1.9049262,       0.6787411 ],
  [               0,               0,               0,   -0.0017277497,   -0.0040535876,      0.02145665 ],
  [               0,               0,               0,       1.7763327,      -5.8107634,       4.8113203 ],
  [               0,               0,               0,      -4.8822188,       16.507948,      -16.824139 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.001722089,   -0.0050672553,     0.011816294 ],
  [               0,               0,               0,        1.305666,      -8.1273823,       14.978758 ],
  [               0,               0,               0,       2.0031633,      -8.1473522,       6.8722944 ],
  [               0,               0,               0,      -1.0973876,        1.427843,       2.7351408 ],
  [               0,               0,               0,    -0.006753209,     0.003068476,     0.030739823 ],
  [               0,               0,               0,     -0.94458002,       4.1497149,      -4.4009414 ],
  [               0,               0,               0,       3.8533669,      -15.363798,       12.328948 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.010132803,    -0.058380172,     0.090991385 ],
  [               0,               0,               0,       18.923372,      -88.797417,       107.36829 ],
  [               0,               0,               0,       3.5588143,      -17.186008,        19.44495 ],
  [               0,               0,               0,      -77.876144,       361.83884,      -419.35547 ],
  [               0,               0,               0,    0.0095514273,    -0.037040483,     0.044391509 ],
  [               0,               0,               0,       1.4159311,      -6.8697057,       8.3560791 ],
  [               0,               0,               0,       25.655815,      -110.93308,       116.49244 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.11738803,      -0.2824415 ],
  [               0,               0,               0,               0,      -86.933662,       217.77736 ],
  [               0,               0,               0,               0,      -35.099163,       84.964722 ],
  [               0,               0,               0,               0,       46.991165,      -115.58245 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.055052362,      0.12532263 ],
  [               0,               0,               0,               0,      -3.2161233,       9.3406448 ],
  [               0,               0,               0,               0,               0,     -0.40000001 ],
  [               0,               0,               0,               0,      -33.454994,       81.888107 ],
  [               0,               0,               0,               0,      0.10019938,     -0.21529946 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele1000_layer2_etaoffv3_37_b = [
 [
  [               0,               0,               0,   -0.0055297264,    0.0030330606,    0.0072475886 ],
  [               0,               0,               0,       6.1807737,      -3.7947683,       6.2828736 ],
  [               0,               0,               0,       0.2978566,    -0.045545772,      -1.3609002 ],
  [               0,               0,               0,      -10.808155,       4.6784563,     -0.63879186 ],
  [               0,               0,               0,     0.083512828,    -0.036332589,      0.00482724 ],
  [               0,               0,               0,      -6.6540093,       2.7998364,    -0.028861774 ],
  [               0,               0,               0,      -6.5672069,       2.9427323,     -0.48365042 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0064639277,   -0.0096610738,     0.010731502 ],
  [               0,               0,               0,      -5.9885287,       7.1218395,        3.704026 ],
  [               0,               0,               0,      -1.2483901,       1.0238166,      -1.5012447 ],
  [               0,               0,               0,      0.89430732,     -0.73862118,     0.074849136 ],
  [               0,               0,               0,    0.0024127052,   -0.0024494021,   0.00097535376 ],
  [               0,               0,               0,      0.55304307,     -0.18264604,      0.73307425 ],
  [               0,               0,               0,       6.2290592,      -13.047758,       4.3494101 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.054908168,     0.051683184 ],
  [               0,               0,               0,               0,        29.96069,      -18.585667 ],
  [               0,               0,               0,               0,       5.5673699,      -5.7796307 ],
  [               0,               0,               0,               0,      -4.5384197,       3.5885775 ],
  [               0,               0,               0,               0,     0.019115508,    -0.014967266 ],
  [               0,               0,               0,               0,       11.798368,      -8.5461378 ],
  [               0,               0,               0,               0,       7.4968677,      -7.8377628 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,    0.0070667989 ],
  [               0,               0,               0,               0,               0,       5.2017183 ],
  [               0,               0,               0,               0,               0,      -1.4205582 ],
  [               0,               0,               0,               0,               0,     0.028833566 ],
  [               0,               0,               0,               0,               0,   0.00059103256 ],
  [               0,               0,               0,               0,               0,      0.90041441 ],
  [               0,               0,               0,               0,               0,      -209.09825 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,       0.2525351,     -0.12821478,     -0.32599235,     0.052031375 ],
  [               0,               0,       687.40369,      -864.80347,      -1234.5048,       1553.3903 ],
  [               0,               0,      -836.21417,       1172.4565,       1551.3636,      -2177.6543 ],
  [               0,               0,       806.20142,      -1097.5626,      -1517.0651,        2065.449 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.59592903,     -0.83418244 ],
  [               0,               0,               0,               0,       -19.87221,       52.026047 ],
  [               0,               0,               0,               0,       1.7722521,       -5.829174 ],
  [               0,               0,               0,               0,      0.12160878,     -0.13280392 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele1000_layer2_etaoffv3_37_e = [
 [
  [               0,       282.62204,      -810.44489,       9.5325909,       1638.5383,      -1174.5079 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,     -0.57144821,      0.43664953,       1.2798367,      0.93866366,      -2.8547587 ],
  [               0,       395.08044,      -305.99124,      -821.73383,      -668.34589,       1888.2688 ],
  [               0,       153.06674,      -102.57184,      -344.12057,      -283.69778,       770.36523 ],
  [               0,       -132.7243,       98.870399,        286.7338,       207.78587,      -619.07007 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.039159492,     0.064088598 ],
  [               0,               0,               0,               0,       241.15118,      -359.48456 ],
  [               0,               0,               0,               0,      -114.02113,       171.11185 ],
  [               0,               0,               0,               0,       105.55665,      -159.51833 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0017448314,      0.01025028,   -0.0049348501 ],
  [               0,               0,               0,       7.6051283,      -30.388823,       34.720356 ],
  [               0,               0,               0,       4.1268559,      -14.134322,       10.593818 ],
  [               0,               0,               0,       -1.706934,        6.060113,      -5.2352309 ],
  [               0,               0,               0,    0.0034075435,    -0.011418269,     0.010283252 ],
  [               0,               0,               0,      -6.5575666,        20.96965,      -15.917474 ],
  [               0,               0,               0,       13.092109,      -47.328026,       40.494972 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0024001554,    -0.010170908,     0.019501481 ],
  [               0,               0,               0,     -0.14442843,      -1.3376139,       7.2415657 ],
  [               0,               0,               0,      0.08380904,    -0.062492169,      -1.6120112 ],
  [               0,               0,               0,      -4.2828903,       10.037147,      0.99408787 ],
  [               0,               0,               0,    -0.017017659,      0.00534607,      0.09140972 ],
  [               0,               0,               0,   -0.0067522945,    -0.030093821,      0.13988882 ],
  [               0,               0,               0,       2.4604075,      -9.3314314,       5.7650433 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0017492216,    0.0022212544,     0.014532462 ],
  [               0,               0,               0,       7.7525549,      -37.049339,       47.472626 ],
  [               0,               0,               0,     0.086255193,    0.0033198167,      -1.8095765 ],
  [               0,               0,               0,      -5.6396942,       35.542145,      -42.722546 ],
  [               0,               0,               0,    0.0048914221,     0.011497788,      0.04293244 ],
  [               0,               0,               0,     -0.15004998,      0.46810102,     -0.23738644 ],
  [               0,               0,               0,       7.9370542,      -36.713379,       39.338135 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00038157406,     0.006312923 ],
  [               0,               0,               0,               0,      -4.7017608,       15.174049 ],
  [               0,               0,               0,               0,      -34.193302,        82.80426 ],
  [               0,               0,               0,               0,       38.286945,      -94.169411 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.011670681,    -0.028904542 ],
  [               0,               0,               0,               0,     -0.40350592,      -37.961349 ],
  [               0,               0,               0,               0,   6.3356769e-05,     -0.60817224 ],
  [               0,               0,               0,               0,       182.22412,      -121.65421 ],
  [               0,               0,               0,               0,       29.055719,        71.80072 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]


ele_layer2_etaoffv3_37_b = [
    ele5_layer2_etaoffv3_37_b,
    ele10_layer2_etaoffv3_37_b,
    ele25_layer2_etaoffv3_37_b,
    ele50_layer2_etaoffv3_37_b,
    ele75_layer2_etaoffv3_37_b,
    ele100_layer2_etaoffv3_37_b,
    ele200_layer2_etaoffv3_37_b,
    ele500_layer2_etaoffv3_37_b,
    ele1000_layer2_etaoffv3_37_b,
    ]

ele_layer2_etaoffv3_37_e = [
    ele5_layer2_etaoffv3_37_e,
    ele10_layer2_etaoffv3_37_e,
    ele25_layer2_etaoffv3_37_e,
    ele50_layer2_etaoffv3_37_e,
    ele75_layer2_etaoffv3_37_e,
    ele100_layer2_etaoffv3_37_e,
    ele200_layer2_etaoffv3_37_e,
    ele500_layer2_etaoffv3_37_e,
    ele1000_layer2_etaoffv3_37_e,
    ]


###############################################################################
# Tables for v3 sshape correction, layer 2, 3x7 photon clusters
#

gam5_layer2_etaoffv3_37_b = [
 [
  [               0,               0,               0,     0.037538581,     0.024682095,    0.0010150467 ],
  [               0,               0,               0,       10.024294,      -12.386288,       5.1411605 ],
  [               0,               0,               0,      -5.0802455,        4.076911,      -1.9220359 ],
  [               0,               0,               0,     -0.66293979,     -0.46799627,      0.17658673 ],
  [               0,               0,               0,    -0.017339151,   -0.0032645438,    0.0019595153 ],
  [               0,               0,               0,      -3.8555171,      -1.4260368,       1.1866612 ],
  [               0,               0,               0,      -10.150219,      0.67572248,      -1.7589022 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0026661495,    0.0024889377,    0.0078136157 ],
  [               0,               0,               0,       2.3701074,      -5.2359557,       4.5303235 ],
  [               0,               0,               0,      -1.6083856,       2.4404345,       -2.004379 ],
  [               0,               0,               0,       1.7805887,      -2.1155851,      0.62834841 ],
  [               0,               0,               0,   -0.0095685758,     0.012988789,    -0.003984449 ],
  [               0,               0,               0,        112.5171,       -130.1423,       36.603832 ],
  [               0,               0,               0,       69.615211,      -104.13269,       31.675772 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.28899485,       0.2480794 ],
  [               0,               0,               0,               0,       53.103165,      -40.561378 ],
  [               0,               0,               0,               0,      -17.096333,       12.808679 ],
  [               0,               0,               0,               0,      -7.6104622,       5.8551803 ],
  [               0,               0,               0,               0,      0.05458634,    -0.040055621 ],
  [               0,               0,               0,               0,      -28.908585,       23.752657 ],
  [               0,               0,               0,               0,       22.665726,      -17.920906 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
    [0, 0, 0, 0, 0, 0], # Removed by hand, see below
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
    [0, 0, 0, 0, 0, 0],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam5_layer2_etaoffv3_37_e = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam10_layer2_etaoffv3_37_b = [
 [
  [               0,               0,               0,      -0.0387998,      0.02178639,    0.0065168217 ],
  [               0,               0,               0,       2.6148129,      -2.4622843,       3.7988465 ],
  [               0,               0,               0,      0.35433555,       0.9508673,      -1.5252717 ],
  [               0,               0,               0,     -0.54571629,     -0.27700377,     0.094136909 ],
  [               0,               0,               0,  -1.6117037e-05,   -0.0050944518,    0.0011210121 ],
  [               0,               0,               0,       7.8396373,      -4.4624877,       1.3535246 ],
  [               0,               0,               0,      -11.565409,       2.2013266,       -2.497849 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.035628904,    -0.034968495,     0.016839772 ],
  [               0,               0,               0,       -2.666539,      0.19336402,       4.4090586 ],
  [               0,               0,               0,       1.0004638,      -1.3592958,     -0.90026152 ],
  [               0,               0,               0,    -0.077173866,      0.33607337,     -0.12819149 ],
  [               0,               0,               0,   -0.0017892149,  -0.00076081249,   0.00094564515 ],
  [               0,               0,               0,        1.389145,      -3.3427696,       3.0156684 ],
  [               0,               0,               0,       45.598778,      -63.123211,       14.023105 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.10298485,    -0.076529115 ],
  [               0,               0,               0,               0,       9.6048117,      -4.1733823 ],
  [               0,               0,               0,               0,      -13.228107,       9.2216587 ],
  [               0,               0,               0,               0,       3.5860491,      -2.6619027 ],
  [               0,               0,               0,               0,      0.18935272,     -0.14982435 ],
  [               0,               0,               0,               0,       13.166796,      -8.5075436 ],
  [               0,               0,               0,               0,      -629.32648,      -140.03827 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.022723895,     0.054858159,    -0.026852246 ],
  [               0,               0,               0,       8.3530436,      -19.824171,       15.069177 ],
  [               0,               0,               0,       2.1691871,      -4.2510562,      0.59215248 ],
  [               0,               0,               0,      -1.1296419,       2.5275269,      -1.2510771 ],
  [               0,               0,               0,      0.01452117,    -0.034107316,     0.020689793 ],
  [               0,               0,               0,     -0.89418781,       1.6062104,      0.35118461 ],
  [               0,               0,               0,       5.1532736,      -9.5808468,       2.8279099 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    -0.002310378,    -0.013348596,   -0.0055390834,     0.048116866 ],
  [               0,               0,      -2469.1533,       3484.3555,         4237.48,      -5994.3979 ],
  [               0,               0,       358.91928,      -492.92416,      -669.18341,       917.84509 ],
  [               0,               0,       211.08505,       -280.6333,      -389.15887,        517.4032 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.21564652,     -0.29389587 ],
  [               0,               0,               0,               0,        2927.189,      -4104.3745 ],
  [               0,               0,               0,               0,      -25.176695,       32.022816 ],
  [               0,               0,               0,               0,      0.24555553,      0.13032442 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam10_layer2_etaoffv3_37_e = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.20225661,      0.30950287 ],
  [               0,               0,               0,               0,       4.9294014,       10.242701 ],
  [               0,               0,               0,               0,       204.18646,       -310.2869 ],
  [               0,               0,               0,               0,       -143.0771,       216.85683 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
    # Copied from 55.
  [               0,               0,               0,               0,   -0.0035746964,     0.012298349 ],
  [               0,               0,               0,               0,      -5.3014765,        12.26264 ],
  [               0,               0,               0,               0,       4.4197936,      -8.6744204 ],
  [               0,               0,               0,               0,      -1.6761082,       2.8486159 ],
  [               0,               0,               0,               0,     0.011471081,     -0.01928678 ],
  [               0,               0,               0,               0,     -0.27758086,       1.5612171 ],
  [               0,               0,               0,               0,     -0.97796303,       1.1397932 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam25_layer2_etaoffv3_37_b = [
 [
  [               0,               0,               0,     -0.03053713,     0.018190037,    0.0042767441 ],
  [               0,               0,               0,       19.740183,      -11.154501,       6.5104189 ],
  [               0,               0,               0,      -3.6647594,       1.3043265,      -1.4457579 ],
  [               0,               0,               0,       2.8267477,     -0.90366846,      0.01730028 ],
  [               0,               0,               0,    0.0076033752,   -0.0050074807,     0.001445647 ],
  [               0,               0,               0,       5.8103466,      -3.4436967,       1.1296664 ],
  [               0,               0,               0,      -22.707567,       7.8080859,      -1.8780638 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,  -5.3677606e-05,    0.0010505369,    0.0082336525 ],
  [               0,               0,               0,      -3.5007453,       2.1126406,       4.0752645 ],
  [               0,               0,               0,     -0.58225501,      0.62528133,      -1.5223082 ],
  [               0,               0,               0,      0.88371038,      -0.8636263,      0.22091812 ],
  [               0,               0,               0,    0.0036504732,   -0.0045483662,    0.0014634427 ],
  [               0,               0,               0,       4.7645535,      -8.3740015,       4.5334377 ],
  [               0,               0,               0,       -81.68251,       64.452377,      -29.622414 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.27441603,       -0.209408 ],
  [               0,               0,               0,               0,      -22.355978,       21.841986 ],
  [               0,               0,               0,               0,       5.0473566,      -5.4577212 ],
  [               0,               0,               0,               0,     -0.80601168,      0.66596282 ],
  [               0,               0,               0,               0,    -0.083102547,     0.065855592 ],
  [               0,               0,               0,               0,      -10.338518,       9.5868492 ],
  [               0,               0,               0,               0,      -17.323723,       12.759047 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.018891867,     0.036927234,    -0.010217745 ],
  [               0,               0,               0,       11.723901,      -25.003834,       17.210552 ],
  [               0,               0,               0,      -3.0084701,       5.6496434,      -4.0652637 ],
  [               0,               0,               0,       1.1033996,      -1.6548594,       0.6550504 ],
  [               0,               0,               0,    0.0028447881,   -0.0045460267,    0.0022054871 ],
  [               0,               0,               0,       2.5054038,       -4.968998,       3.0051017 ],
  [               0,               0,               0,       6.1483626,      -14.994691,       6.7095995 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.11446809,    -0.023610452,     -0.12283068,    -0.073145323 ],
  [               0,               0,      -7700.3301,       10324.371,       15075.493,      -20223.902 ],
  [               0,               0,      -1606.4761,       2228.3435,       3026.3015,      -4199.5181 ],
  [               0,               0,       575.33997,      -788.75073,      -1078.5964,        1478.885 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.032727923,    -0.025675325 ],
  [               0,               0,               0,               0,      -2.9920454,       10.664191 ],
  [               0,               0,               0,               0,       8.1662273,      -12.499125 ],
  [               0,               0,               0,               0,    -0.057374507,     -0.41325122 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam25_layer2_etaoffv3_37_e = [
 [
  [               0,               0,      -0.2599512,      0.39247105,      0.38501447,      -0.5902828 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.095875651,      0.52085227,      0.28589326,      -1.8270943 ],
  [               0,               0,       -908.5177,       1217.7329,       2427.6541,      -3313.8835 ],
  [               0,               0,       808.74011,       -1229.618,      -1689.6848,       2571.3245 ],
  [               0,               0,      -340.54495,       494.78925,       757.54175,      -1100.7765 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.19939584,       0.3041198 ],
  [               0,               0,               0,               0,       135.68829,      -203.12074 ],
  [               0,               0,               0,               0,      -268.58929,       406.83447 ],
  [               0,               0,               0,               0,       340.75366,      -515.20074 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.085200071,      0.27829471,     -0.21927167 ],
  [               0,               0,               0,       52.905716,      -174.97533,       148.53244 ],
  [               0,               0,               0,      -8.0593786,       26.641769,      -23.327007 ],
  [               0,               0,               0,       4.9293671,      -16.559574,       13.936051 ],
  [               0,               0,               0,     0.041568793,     -0.13896799,      0.11651722 ],
  [               0,               0,               0,      -8.1626329,       30.641844,      -27.774399 ],
  [               0,               0,               0,      -5.5580673,       20.095404,      -19.869104 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0044202222,  -0.00072518497,   -0.0069961254 ],
  [               0,               0,               0,       5.6982079,      -28.511568,       37.459412 ],
  [               0,               0,               0,      0.72275591,      -2.5055401,       0.8546738 ],
  [               0,               0,               0,      -5.0569515,       21.362154,      -22.529222 ],
  [               0,               0,               0,    0.0029833296,    -0.002976384,   -0.0073108957 ],
  [               0,               0,               0,       2.1372981,      -8.1478205,       7.6370254 ],
  [               0,               0,               0,       10.742925,      -38.553329,       32.099678 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,      0.33945537,      -1.5598723,       1.7935218 ],
  [               0,               0,               0,       284.85709,       -1394.903,       1710.9706 ],
  [               0,               0,               0,       123.43675,      -594.78589,       714.96423 ],
  [               0,               0,               0,      -63.267464,       305.86627,      -369.48993 ],
  [               0,               0,               0,     0.045185499,      -0.2310981,      0.29583561 ],
  [               0,               0,               0,      -21.151621,       98.321114,       -113.2796 ],
  [               0,               0,               0,      -180.04279,       844.17499,      -997.12067 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,     0.011899819 ],
  [               0,               0,               0,               0,               0,       2.4751382 ],
  [               0,               0,               0,               0,               0,      -1.1940567 ],
  [               0,               0,               0,               0,               0,     0.043975461 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam50_layer2_etaoffv3_37_b = [
 [
  [               0,               0,               0,    -0.036948688,     0.017045557,    0.0055430117 ],
  [               0,               0,               0,        14.58832,      -7.1879363,       5.9650645 ],
  [               0,               0,               0,      0.25023925,    -0.087051101,      -1.3601508 ],
  [               0,               0,               0,      0.30907014,     -0.11930052,    -0.019850664 ],
  [               0,               0,               0,    0.0062179426,   -0.0031161883,   0.00095940934 ],
  [               0,               0,               0,       18.516235,      -7.9230604,       1.2050503 ],
  [               0,               0,               0,      -8.9533482,       4.1014457,      -1.7018489 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.021690451,     0.026503664,   -0.0011057105 ],
  [               0,               0,               0,       5.6389809,      -8.3151245,       8.1242275 ],
  [               0,               0,               0,      -2.8694766,       3.2338719,      -2.3118491 ],
  [               0,               0,               0,       1.8312529,      -1.9559224,      0.53140044 ],
  [               0,               0,               0,    0.0086528184,    -0.010430326,    0.0036857447 ],
  [               0,               0,               0,       2.6378391,       -3.486197,       1.9239488 ],
  [               0,               0,               0,      -6.5951819,       7.5783305,      -3.7760556 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.14232898,     -0.10689671 ],
  [               0,               0,               0,               0,      -40.724934,       37.685547 ],
  [               0,               0,               0,               0,       17.012941,       -15.19467 ],
  [               0,               0,               0,               0,      -14.914042,       12.066582 ],
  [               0,               0,               0,               0,     -0.07447461,     0.060443185 ],
  [               0,               0,               0,               0,        -12.3703,       10.525709 ],
  [               0,               0,               0,               0,      -31.941936,       22.843136 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0065941075,    -0.014117456,     0.015101447 ],
  [               0,               0,               0,     -0.80764258,      0.51987463,       4.7493076 ],
  [               0,               0,               0,     -0.53360295,      0.68495631,      -1.5780997 ],
  [               0,               0,               0,       2.8973033,      -5.7688279,       3.3283091 ],
  [               0,               0,               0,     0.029146483,    -0.062801741,     0.037047226 ],
  [               0,               0,               0,     -0.24537186,      0.93470138,     -0.56285977 ],
  [               0,               0,               0,     -0.46360371,      0.01077685,       -2.234633 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.81938785,     -0.98042327,      -1.4176781,       1.6844188 ],
  [               0,               0,      -6291.6138,       8748.4492,       11494.772,      -15982.618 ],
  [               0,               0,      -1761.3481,       2440.7759,       3311.1995,      -4590.4023 ],
  [               0,               0,       482.85791,        -658.823,      -906.94836,        1237.731 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.16820231,     -0.23383674 ],
  [               0,               0,               0,               0,      -3213.5442,       4619.2896 ],
  [               0,               0,               0,               0,      -119.95941,       165.88385 ],
  [               0,               0,               0,               0,     -0.51611704,     -0.73897862 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam50_layer2_etaoffv3_37_e = [
 [
  [               0,               0,      -1.4789094,       2.1529012,       2.9001317,      -4.2231221 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    -0.084119253,      0.20951834,      0.24897449,     -0.54600042 ],
  [               0,               0,       1519.5966,      -2238.7686,       -3359.427,       4952.1929 ],
  [               0,               0,       581.57666,      -857.06305,      -1268.2695,       1867.6589 ],
  [               0,               0,      -648.72998,       956.84888,       1406.4949,      -2074.1069 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.031084044,     0.049221478 ],
  [               0,               0,               0,               0,       356.73825,      -534.40063 ],
  [               0,               0,               0,               0,      -379.00595,       572.40662 ],
  [               0,               0,               0,               0,       333.55948,       -504.2518 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.037769392,       -0.125817,       0.1121536 ],
  [               0,               0,               0,      -11.001996,       34.911816,       -23.08762 ],
  [               0,               0,               0,       3.1059408,       -10.36714,       7.2343764 ],
  [               0,               0,               0,      -1.0617301,       2.6579826,      -1.2659729 ],
  [               0,               0,               0,     0.018596776,    -0.068235993,     0.063479237 ],
  [               0,               0,               0,      -8.4100676,       28.784664,      -24.277874 ],
  [               0,               0,               0,       22.589781,       -72.60994,       55.695377 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0026503957,    -0.011211302,     0.020200005 ],
  [               0,               0,               0,     0.014968608,      -1.9729576,       7.5624137 ],
  [               0,               0,               0,      0.58632571,      -2.2710431,       0.8774271 ],
  [               0,               0,               0,      -76.210739,       317.04993,      -328.38251 ],
  [               0,               0,               0,  -0.00069189357,    0.0023231178,     0.039059225 ],
  [               0,               0,               0,    -0.024106167,      0.10915055,     -0.11664814 ],
  [               0,               0,               0,       15.712919,      -65.373802,       66.155373 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0012653983,   -0.0075662988,     0.034784559 ],
  [               0,               0,               0,       35.963264,      -169.91849,       203.66237 ],
  [               0,               0,               0,       5.2747855,      -24.631094,       27.446863 ],
  [               0,               0,               0,      -6.5235519,       43.381317,      -61.303871 ],
  [               0,               0,               0,     0.016395854,    -0.014212274,    -0.011354421 ],
  [               0,               0,               0,      -2.1151917,       9.5693626,      -10.746077 ],
  [               0,               0,               0,       15.719325,      -72.807175,       81.189758 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.091001503,      0.23249143 ],
  [               0,               0,               0,               0,      -28.611166,       73.354118 ],
  [               0,               0,               0,               0,      -28.278337,       68.404099 ],
  [               0,               0,               0,               0,       33.169384,      -81.589439 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0056263097,    -0.036504965 ],
  [               0,               0,               0,               0,         91.6175,      -228.94397 ],
  [               0,               0,               0,               0,               0,     -0.40000001 ],
  [               0,               0,               0,               0,       35.379204,      -86.379974 ],
  [               0,               0,               0,               0,     0.032650147,    -0.059897423 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam75_layer2_etaoffv3_37_b = [
 [
  [               0,               0,               0,    -0.023927161,     0.010896873,    0.0065571428 ],
  [               0,               0,               0,       16.567299,      -7.2096653,       5.8275661 ],
  [               0,               0,               0,     -0.53621441,     0.063628905,      -1.3407359 ],
  [               0,               0,               0,     -0.78189421,      0.49162313,    -0.097040921 ],
  [               0,               0,               0,     0.018820377,   -0.0093628261,    0.0013870832 ],
  [               0,               0,               0,      -11.803206,       5.1304417,      0.14412715 ],
  [               0,               0,               0,      -7.3173995,       2.8400428,      -1.2955078 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.012042805,    -0.014007242,     0.011107426 ],
  [               0,               0,               0,      -8.6341724,       8.5999279,       3.1952515 ],
  [               0,               0,               0,        1.122541,      -1.7893432,     -0.74414307 ],
  [               0,               0,               0,     -0.29512733,      0.74886531,     -0.32529098 ],
  [               0,               0,               0,   -0.0018371294,    0.0019998774,  -8.5716656e-06 ],
  [               0,               0,               0,      -5.2117715,       6.6305399,      -1.2170085 ],
  [               0,               0,               0,       12.663574,      -17.947565,       4.4560537 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.04854304,    -0.032999832 ],
  [               0,               0,               0,               0,       13.405977,      -4.9755335 ],
  [               0,               0,               0,               0,      -2.7032077,      0.76568341 ],
  [               0,               0,               0,               0,       4.6449733,      -3.7080286 ],
  [               0,               0,               0,               0,    0.0018463139,  -0.00032312379 ],
  [               0,               0,               0,               0,      -19.304546,       16.368492 ],
  [               0,               0,               0,               0,       -48.22353,       5.4030247 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0058443323,    -0.012338235,     0.014023716 ],
  [               0,               0,               0,     -0.07665249,     -0.74164671,       5.3435225 ],
  [               0,               0,               0,      0.56766707,       -1.741328,     -0.26122934 ],
  [               0,               0,               0,    0.0021276216,      0.65679526,     -0.58947051 ],
  [               0,               0,               0,   -0.0045758146,    0.0081164809,   -0.0040507484 ],
  [               0,               0,               0,      -1.2049731,       2.3126652,     -0.50414491 ],
  [               0,               0,               0,       6.9523168,      -19.018742,       7.0323186 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.20184509,      0.01418147,     -0.17757195,     -0.28272903 ],
  [               0,               0,       61.544296,      -72.728867,       -120.5555,       146.82838 ],
  [               0,               0,      -103.85941,       163.50397,       184.84085,      -294.65286 ],
  [               0,               0,       547.83539,      -749.05096,      -1032.8269,       1412.2212 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,       0.3824392,     -0.53417134 ],
  [               0,               0,               0,               0,      -566.23535,       826.69489 ],
  [               0,               0,               0,               0,      -16.383875,       19.863895 ],
  [               0,               0,               0,               0,    -0.047646251,     -0.33797494 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam75_layer2_etaoffv3_37_e = [
 [
  [               0,               0,      -1.9600003,        2.835211,       3.8779929,      -5.6094003 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    -0.019698573,      0.14294568,      0.12885126,     -0.42746145 ],
  [               0,               0,      -885.49005,       1295.4825,       1937.6722,      -2831.8892 ],
  [               0,               0,       390.48352,      -583.46289,      -801.17926,       1196.6395 ],
  [               0,               0,       -439.7518,       646.22485,       957.28204,      -1406.4163 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.08546868,      0.13220647 ],
  [               0,               0,               0,               0,       265.21896,      -396.66202 ],
  [               0,               0,               0,               0,       -280.5943,         423.405 ],
  [               0,               0,               0,               0,       257.43204,       -389.0845 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,      0.04536603,     -0.15328661,      0.13679668 ],
  [               0,               0,               0,       -17.96093,       59.224514,      -44.087608 ],
  [               0,               0,               0,       5.9734955,      -20.476086,       16.037928 ],
  [               0,               0,               0,      -2.4109986,       8.2926655,      -6.8933039 ],
  [               0,               0,               0,    -0.000279812,    0.0011678769,   0.00015566759 ],
  [               0,               0,               0,      -10.684133,       36.821541,      -31.229288 ],
  [               0,               0,               0,       6.8361387,      -23.632397,       17.730059 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0024084654,     0.015037089,    -0.012150267 ],
  [               0,               0,               0,       2.1622374,      -12.567692,       20.295103 ],
  [               0,               0,               0,      0.79672039,      -3.0015705,       1.4712176 ],
  [               0,               0,               0,      -5.8564129,       23.478102,      -23.284443 ],
  [               0,               0,               0,    0.0031236361,    -0.012954347,     0.015967824 ],
  [               0,               0,               0,      0.92486775,      -3.9327924,         4.27245 ],
  [               0,               0,               0,       22.664129,      -91.471069,       90.065552 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0032862336,    0.0013973544,    -0.011108675 ],
  [               0,               0,               0,       1.5001144,      -11.970584,       22.910789 ],
  [               0,               0,               0,       6.6655164,      -31.084375,       34.977795 ],
  [               0,               0,               0,       -11.68369,       55.397808,      -65.615631 ],
  [               0,               0,               0,   0.00087705092,   0.00089112972,   -0.0069771092 ],
  [               0,               0,               0,      -1.5889601,       5.8241138,      -4.4133582 ],
  [               0,               0,               0,       1.9807628,       27.726484,      -79.970444 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,  -0.00013372808,     0.010792043 ],
  [               0,               0,               0,               0,       -30.53882,       78.253799 ],
  [               0,               0,               0,               0,       3.3666439,      -9.6041098 ],
  [               0,               0,               0,               0,     0.073579043,     -0.10228628 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.024665205,    -0.061076399 ],
  [               0,               0,               0,               0,       3.8454952,       17.534506 ],
  [               0,               0,               0,               0,               0,     -0.40000001 ],
  [               0,               0,               0,               0,       16.156622,       36.129807 ],
  [               0,               0,               0,               0,       8.1835051,       19.899998 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam100_layer2_etaoffv3_37_b = [
 [
  [               0,               0,               0,    -0.031879026,     0.014603871,    0.0059974869 ],
  [               0,               0,               0,       17.989738,      -8.3343649,       6.2537308 ],
  [               0,               0,               0,      0.34402767,     -0.10412974,      -1.3609118 ],
  [               0,               0,               0,      0.56323999,     -0.24544683,    -0.020538563 ],
  [               0,               0,               0,    0.0032163083,   -0.0019758963,   0.00075345783 ],
  [               0,               0,               0,       17.965353,      -7.6983347,       1.1175117 ],
  [               0,               0,               0,      -16.115503,       7.4005327,      -1.8078904 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0069815628,   -0.0049788295,      0.00705584 ],
  [               0,               0,               0,      -4.2629795,       2.1698384,       5.8574986 ],
  [               0,               0,               0,      0.32439873,     -0.70432156,      -1.1198225 ],
  [               0,               0,               0,      0.29001972,    -0.032498367,    -0.067881443 ],
  [               0,               0,               0,   -0.0027398611,    0.0022750353,   0.00034328151 ],
  [               0,               0,               0,      -3.4329455,       3.3874514,     0.097420342 ],
  [               0,               0,               0,      -3.0935471,       1.9606987,      -1.7479839 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.043139953,     -0.02680216 ],
  [               0,               0,               0,               0,      -1.5135678,       6.0995717 ],
  [               0,               0,               0,               0,       9.0646973,      -8.7366743 ],
  [               0,               0,               0,               0,      -6.3803983,        5.164371 ],
  [               0,               0,               0,               0,     -0.01562333,     0.013085232 ],
  [               0,               0,               0,               0,       1.8621612,     -0.63580704 ],
  [               0,               0,               0,               0,        -2.56528,     -0.46214306 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   0.00039485379,  -0.00026853461,    0.0068779262 ],
  [               0,               0,               0,       1.0784924,      -3.5468979,       7.3501544 ],
  [               0,               0,               0,      0.70966876,      -2.0158308,     -0.13987942 ],
  [               0,               0,               0,      -0.2615203,       1.0281931,      -0.7049197 ],
  [               0,               0,               0,    0.0010943293,   -0.0019782754,    0.0014830991 ],
  [               0,               0,               0,     -0.31928307,      0.22020647,      0.90210342 ],
  [               0,               0,               0,       2.3597744,      -7.2567835,       3.0801539 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.20988192,     -0.11792141,     -0.27916417,      0.07498806 ],
  [               0,               0,      -2507.6965,       3472.8013,       4647.8867,      -6431.6548 ],
  [               0,               0,      -1360.0665,       1886.3005,       2553.3015,      -3543.0913 ],
  [               0,               0,       533.68268,      -727.22601,      -1003.5089,       1367.6335 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.28248662,     -0.39470837 ],
  [               0,               0,               0,               0,      -3423.9353,       4924.9546 ],
  [               0,               0,               0,               0,      -22.764318,       28.340633 ],
  [               0,               0,               0,               0,     -0.22520193,     -0.57095283 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam100_layer2_etaoffv3_37_e = [
 [
  [               0,               0,      -2.3434258,       3.3918035,       4.6435413,      -6.7208233 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0055312673,     0.077506855,     0.053051207,     -0.25506687 ],
  [               0,               0,      -160.96448,       229.27203,       353.26355,      -500.47452 ],
  [               0,               0,       323.46964,      -483.83548,      -680.63483,       1017.2665 ],
  [               0,               0,      -328.47141,       482.79053,       716.03967,      -1052.1772 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.099733345,      0.15355377 ],
  [               0,               0,               0,               0,       337.85449,      -506.49619 ],
  [               0,               0,               0,               0,      -377.81479,        570.6983 ],
  [               0,               0,               0,               0,       337.21582,      -509.78677 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.017712338,    -0.057729058,      0.05435114 ],
  [               0,               0,               0,      -4.6621699,       13.243306,      -4.3179073 ],
  [               0,               0,               0,       2.7833824,      -9.3801613,       6.4353166 ],
  [               0,               0,               0,     -0.18065131,      0.56978375,     -0.31475744 ],
  [               0,               0,               0,    0.0097131981,    -0.033496257,     0.029577175 ],
  [               0,               0,               0,      -14.802764,       50.304733,      -42.057877 ],
  [               0,               0,               0,       6.1851592,      -21.078007,       15.656253 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.017073821,    -0.070174471,     0.080515683 ],
  [               0,               0,               0,      -3.8853552,       14.067587,      -8.7762604 ],
  [               0,               0,               0,      0.91447413,       -3.625236,       2.2523105 ],
  [               0,               0,               0,      -25.966257,       107.02377,      -109.64547 ],
  [               0,               0,               0,     0.018043661,    -0.074320942,     0.089317694 ],
  [               0,               0,               0,     -0.16693059,      0.70723796,     -0.73014843 ],
  [               0,               0,               0,       17.380653,      -71.652161,       71.888657 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0010672318,    -0.010808421,     0.029393593 ],
  [               0,               0,               0,       19.031055,      -90.785477,       111.36832 ],
  [               0,               0,               0,       8.9252949,      -42.492207,        49.30051 ],
  [               0,               0,               0,      -16.186289,       101.69772,      -152.11609 ],
  [               0,               0,               0,    -0.054153871,     0.010592552,      0.30070162 ],
  [               0,               0,               0,      -6.1688552,       28.943016,      -33.961594 ],
  [               0,               0,               0,      -18.706493,       92.214752,      -113.10171 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.032924585,     0.086427964 ],
  [               0,               0,               0,               0,      -38.430031,       98.187614 ],
  [               0,               0,               0,               0,      -97.749336,       239.36798 ],
  [               0,               0,               0,               0,       100.59855,      -247.57674 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0021138801,  -7.4171985e-05,    -0.012780723 ],
  [               0,               0,               0,      -3.4544339,      -4.5568347,       -7.324122 ],
  [               0,               0,               0,               0,               0,     -0.40000001 ],
  [               0,               0,               0,       21.879156,       39.860474,       62.084213 ],
  [               0,               0,               0,       7.9627433,       19.669161,       48.328693 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam200_layer2_etaoffv3_37_b = [
 [
  [               0,               0,               0,    0.0058972295,   -0.0022791086,    0.0076790564 ],
  [               0,               0,               0,       -2.603725,      0.75460631,       5.6010094 ],
  [               0,               0,               0,       1.4789902,     -0.71309459,      -1.2915986 ],
  [               0,               0,               0,       7.1344242,      -3.1602039,    -0.017713021 ],
  [               0,               0,               0,    -0.062624395,     0.027959812,  -0.00017553987 ],
  [               0,               0,               0,       4.3320351,      -2.1536818,      0.35858148 ],
  [               0,               0,               0,       3.2731302,      -1.5534604,     -0.27709368 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.012886878,     -0.01669554,      0.01220075 ],
  [               0,               0,               0,      -6.5010819,        7.494916,       3.5752015 ],
  [               0,               0,               0,     -0.86737657,       0.8319087,       -1.563458 ],
  [               0,               0,               0,      0.72869205,     -0.69198114,      0.13736306 ],
  [               0,               0,               0,   -0.0062847706,    0.0081434883,   -0.0020319496 ],
  [               0,               0,               0,      -6.5363455,       7.8825283,      -1.4650218 ],
  [               0,               0,               0,       2.2298374,      -3.7753782,     -0.07462763 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.014826652,     0.020424305 ],
  [               0,               0,               0,               0,       17.099035,      -8.7381039 ],
  [               0,               0,               0,               0,       5.9341917,      -6.0900574 ],
  [               0,               0,               0,               0,      -4.1039486,       3.2577772 ],
  [               0,               0,               0,               0,    0.0032064496,   -0.0027162228 ],
  [               0,               0,               0,               0,       38.540653,      -28.899233 ],
  [               0,               0,               0,               0,       4.1815968,      -4.7429314 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.003485474,    0.0089115649,    0.0015191815 ],
  [               0,               0,               0,       3.8863966,      -9.8940907,       11.101535 ],
  [               0,               0,               0,      0.13635905,     -0.60687166,     -0.96666068 ],
  [               0,               0,               0,      -0.0183035,       0.3525117,     -0.29093653 ],
  [               0,               0,               0,    0.0014989553,   -0.0033300384,    0.0024611501 ],
  [               0,               0,               0,     -0.26064616,     0.043820065,       1.0938047 ],
  [               0,               0,               0,     0.022356613,      -1.3388307,      -6.7758079 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,       0.1860522,    -0.024074836,     -0.19016367,     -0.15721563 ],
  [               0,               0,       189.70927,       -243.4072,      -348.99823,       451.57809 ],
  [               0,               0,      -403.29285,       575.02148,        745.6897,      -1065.5685 ],
  [               0,               0,       566.11115,      -772.19342,      -1066.1426,       1454.3424 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.63983792,     -0.89684415 ],
  [               0,               0,               0,               0,        -21.4713,       44.130943 ],
  [               0,               0,               0,               0,       41.331245,      -61.592243 ],
  [               0,               0,               0,               0,     0.059869763,     -0.29536843 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam200_layer2_etaoffv3_37_e = [
 [
  [               0,               0,      -3.4299791,       4.9601846,       6.8307481,      -9.8774099 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.025721405,     0.014199533,    -0.016090931,    -0.080857582 ],
  [               0,               0,      -5.0772099,      -5.2234125,       1.6565332,       28.402046 ],
  [               0,               0,       32.241253,       -47.38726,      -69.579582,       101.08764 ],
  [               0,               0,       5.9878907,      -9.8653841,      -13.689773,       22.576702 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.046018921,     0.072409354 ],
  [               0,               0,               0,               0,       309.23792,      -461.39182 ],
  [               0,               0,               0,               0,      -290.69479,       438.50745 ],
  [               0,               0,               0,               0,         273.737,      -413.76376 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.009199352,    -0.030110098,     0.032506917 ],
  [               0,               0,               0,      -1.5105538,        2.661824,       4.5829473 ],
  [               0,               0,               0,     -0.88817382,       2.9707065,      -3.8996022 ],
  [               0,               0,               0,    -0.050486691,      -2.9022679,       6.1812558 ],
  [               0,               0,               0,   -0.0078296009,    0.0027124195,     0.026647579 ],
  [               0,               0,               0,       2.8050299,      -9.2081757,       7.6189642 ],
  [               0,               0,               0,      -5.7723708,       19.867971,      -19.905045 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   0.00051559176,    -0.000115745,    0.0068264185 ],
  [               0,               0,               0,      0.33986184,      -3.8793948,       10.292362 ],
  [               0,               0,               0,      0.69620275,      -2.6823926,       1.2304682 ],
  [               0,               0,               0,      -10.235041,       33.513626,      -21.957527 ],
  [               0,               0,               0,    -0.015009098,    0.0052176886,      0.08671122 ],
  [               0,               0,               0,    -0.055855237,      0.22635753,     -0.21541198 ],
  [               0,               0,               0,        7.101202,      -28.001596,       24.777306 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.009799704,    -0.081115708,      0.14325511 ],
  [               0,               0,               0,        113.1842,      -526.42822,       615.80676 ],
  [               0,               0,               0,       47.381908,      -224.23979,       263.84543 ],
  [               0,               0,               0,      -27.956829,       132.38963,      -156.59261 ],
  [               0,               0,               0,     0.014864369,    -0.060303729,     0.060728978 ],
  [               0,               0,               0,      -25.831461,       124.49827,       -149.0619 ],
  [               0,               0,               0,       27.556717,      -126.98653,       144.41576 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.051224999,      0.13221501 ],
  [               0,               0,               0,               0,        -13.1571,       36.285164 ],
  [               0,               0,               0,               0,       -58.20388,       141.93262 ],
  [               0,               0,               0,               0,       61.013073,      -150.10753 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0017179871,    -0.003976794,    -0.008656959 ],
  [               0,               0,               0,     -0.75365418,     -0.37557247,       3.6675758 ],
  [               0,               0,               0,               0,               0,     -0.40000001 ],
  [               0,               0,               0,   -0.0061762608,      0.23293738,      0.19458325 ],
  [               0,               0,               0,   -0.0040021208,    -0.010282572,     -0.32739896 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam500_layer2_etaoffv3_37_b = [
 [
  [               0,               0,               0,    -0.024846174,      0.01313216,    0.0057044849 ],
  [               0,               0,               0,       9.6012049,      -6.9947639,       7.1275716 ],
  [               0,               0,               0,       3.5397966,      -1.2132825,      -1.2721274 ],
  [               0,               0,               0,     0.074032746,     -0.15455186,     -0.05572978 ],
  [               0,               0,               0,    0.0052405549,   -0.0026051609,   0.00082361972 ],
  [               0,               0,               0,       23.085066,      -9.9511538,       1.3741522 ],
  [               0,               0,               0,      -17.628712,       9.1916714,      -1.6552374 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0084073953,     0.010780517,    0.0042708679 ],
  [               0,               0,               0,      0.78529394,      -2.3677235,        6.746058 ],
  [               0,               0,               0,      -1.2088982,       1.5086468,      -1.7899767 ],
  [               0,               0,               0,      0.52764261,     -0.79665786,    -0.040139489 ],
  [               0,               0,               0,    0.0022682699,   -0.0009782277,    0.0021256155 ],
  [               0,               0,               0,     -0.99937671,       1.3625108,     -0.27478319 ],
  [               0,               0,               0,      -6.5847015,       8.1186552,      -2.7278683 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.015940722,     0.021032317 ],
  [               0,               0,               0,               0,       12.288321,      -4.5746369 ],
  [               0,               0,               0,               0,       11.284949,      -10.288197 ],
  [               0,               0,               0,               0,      -24.765465,       19.456102 ],
  [               0,               0,               0,               0,      0.15807886,     -0.12397023 ],
  [               0,               0,               0,               0,       1.6451185,      -1.1229559 ],
  [               0,               0,               0,               0,       7.2366524,      -5.9194722 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0067152889,     0.014485692,  -0.00076033681 ],
  [               0,               0,               0,        3.799684,      -9.1480904,       10.734654 ],
  [               0,               0,               0,     0.091754287,     -0.56800735,     -0.90888983 ],
  [               0,               0,               0,    -0.087956548,      0.47963151,      -0.3920716 ],
  [               0,               0,               0,    0.0027274296,    -0.005640917,     0.003471188 ],
  [               0,               0,               0,      0.70195526,      -1.5902288,       1.7717605 ],
  [               0,               0,               0,       2.3826442,      -6.3828173,       2.3828814 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.19823465,    -0.070251964,     -0.23438732,    -0.043590188 ],
  [               0,               0,       121.22278,      -131.08386,      -205.68428,       221.80426 ],
  [               0,               0,      -731.03424,       1024.5627,       1361.5721,      -1910.4673 ],
  [               0,               0,       545.56665,      -741.77557,      -1025.8042,        1394.861 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.018027956,   -0.0045211972 ],
  [               0,               0,               0,               0,        4.694499,       12.954444 ],
  [               0,               0,               0,               0,      -4.0213141,       4.1801262 ],
  [               0,               0,               0,               0,     0.057428144,      -0.2123089 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam500_layer2_etaoffv3_37_e = [
 [
  [       104.78266,      -210.77917,      -168.75369,       246.67567,       604.15387,      -620.06586 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.020753803,       0.0113044,     -0.01278244,    -0.063276142 ],
  [               0,               0,      -4.8027396,      -4.8943448,       1.5843939,       27.192598 ],
  [               0,               0,       3.3956463,      -4.0849652,      -6.8640318,       6.8851333 ],
  [               0,               0,     -0.56829441,     -0.33703142,      0.43984723,       2.0525692 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.02462887,     0.041201364 ],
  [               0,               0,               0,               0,       340.87518,       -509.2713 ],
  [               0,               0,               0,               0,      -168.95187,       254.15533 ],
  [               0,               0,               0,               0,       153.72493,      -232.33463 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.013391485,     0.051625591,    -0.041322738 ],
  [               0,               0,               0,     -0.59215575,      -2.8666146,       11.712949 ],
  [               0,               0,               0,      -4.6568274,       16.242222,      -15.585173 ],
  [               0,               0,               0,      0.61442804,      -2.0785427,       1.9167087 ],
  [               0,               0,               0,    0.0047625331,    -0.016529294,     0.015242504 ],
  [               0,               0,               0,       18.525702,      -64.272209,       56.030949 ],
  [               0,               0,               0,       5.5156283,      -20.595364,        16.63368 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0003505862,    0.0017930792,    0.0062636756 ],
  [               0,               0,               0,      -2.5002279,       8.4452343,      -2.8037448 ],
  [               0,               0,               0,      -1.4181967,       6.3621292,      -8.4619465 ],
  [               0,               0,               0,      -13.437221,       2.8086872,       73.485786 ],
  [               0,               0,               0,     -0.11882562,     0.059157573,      0.57082933 ],
  [               0,               0,               0,      0.35602075,      -1.5415055,       1.6800548 ],
  [               0,               0,               0,       1.2991333,      -4.8392687,       1.3885248 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,  -0.00057096285,   -0.0034913323,     0.021574236 ],
  [               0,               0,               0,       2.8874099,      -12.593776,       16.834955 ],
  [               0,               0,               0,       3.3207419,      -15.928383,       17.804302 ],
  [               0,               0,               0,       1.5568779,       9.2071886,      -20.762054 ],
  [               0,               0,               0,    0.0090267733,     0.013596018,     0.018988978 ],
  [               0,               0,               0,      0.30498001,      -1.3761687,       1.5544335 ],
  [               0,               0,               0,      -5.4014626,       23.511377,      -28.261782 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0005221577,     0.012123076 ],
  [               0,               0,               0,               0,       -46.05286,       116.48267 ],
  [               0,               0,               0,               0,       6.1765962,      -16.475056 ],
  [               0,               0,               0,               0,     -0.39592126,       1.0481147 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.026934424,     0.068188816 ],
  [               0,               0,               0,               0,      -5268.6021,       13079.862 ],
  [               0,               0,               0,               0,      0.43538645,       2365.2466 ],
  [               0,               0,               0,               0,       307.46164,      -761.65997 ],
  [               0,               0,               0,               0,     0.002505285,       1.0601959 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam1000_layer2_etaoffv3_37_b = [
 [
  [               0,               0,               0,   -1.508562e-05,   6.8963459e-06,    0.0073381467 ],
  [               0,               0,               0,       2.1172643,      -1.7448035,        6.527524 ],
  [               0,               0,               0,      0.33111042,     -0.24840578,      -1.2867957 ],
  [               0,               0,               0,       24498.627,      -9962.0518,       170.15492 ],
  [               0,               0,               0,        -179.798,       73.112656,      -1.2491883 ],
  [               0,               0,               0,      0.12924662,     -0.06795492,     0.012706466 ],
  [               0,               0,               0,     -0.20133676,      0.06260056,   -0.0063399863 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0016023808,    0.0048839399,    0.0032026784 ],
  [               0,               0,               0,       2.3425014,      -8.4809008,       10.591855 ],
  [               0,               0,               0,      -1.2773577,       1.9444395,      -2.0274887 ],
  [               0,               0,               0,      0.47416213,     -0.72055751,      0.19841366 ],
  [               0,               0,               0,    0.0049513141,   -0.0077252169,    0.0034264254 ],
  [               0,               0,               0,      -5.9702501,       5.8847256,     -0.40042523 ],
  [               0,               0,               0,       11.794764,      -10.519927,      0.74472666 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.023232648,     0.024567652 ],
  [               0,               0,               0,               0,       19.189642,      -8.5107136 ],
  [               0,               0,               0,               0,        21.31517,      -18.193979 ],
  [               0,               0,               0,               0,      -12.382884,       9.7459927 ],
  [               0,               0,               0,               0,     0.022027111,     -0.01647065 ],
  [               0,               0,               0,               0,     -0.78001642,       1.6685078 ],
  [               0,               0,               0,               0,       22.484432,      -19.264086 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0059551713,    -0.012740002,     0.013276991 ],
  [               0,               0,               0,      -4.0191422,       8.1074982,       1.6787304 ],
  [               0,               0,               0,     -0.29931861,      0.32184276,      -1.3353056 ],
  [               0,               0,               0,      0.35681266,     -0.55601764,      0.15059854 ],
  [               0,               0,               0,   -0.0017777956,    0.0039353203,   -0.0014113139 ],
  [               0,               0,               0,      0.43347374,      -1.1648313,       1.6807357 ],
  [               0,               0,               0,      -1.2439721,       1.7471067,      -1.8524178 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.22679976,      -0.1047275,     -0.28548333,      0.01809644 ],
  [               0,               0,      -31.515209,       96.176613,       94.546684,      -222.82941 ],
  [               0,               0,      -545.58093,       767.28027,       1018.5395,      -1434.4929 ],
  [               0,               0,         592.151,        -804.617,      -1114.1263,        1513.998 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.023690641,    -0.012450825 ],
  [               0,               0,               0,               0,       5.5519652,       14.498221 ],
  [               0,               0,               0,               0,      -4.1540036,       4.3247685 ],
  [               0,               0,               0,               0,     0.068250254,      -0.1972225 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam1000_layer2_etaoffv3_37_e = [
 [
  [       123.89949,      -249.29216,      -198.80324,       291.48605,       711.09515,      -730.02014 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,     -0.57578886,      0.44905534,       1.2813636,      0.91415858,      -2.8405137 ],
  [               0,       520.49982,      -404.05417,      -1083.9097,      -873.41193,        2481.957 ],
  [               0,       111.64963,      -74.177025,      -252.19057,      -209.74525,       566.27173 ],
  [               0,      -106.87447,        79.46093,       230.85683,       167.63661,       -498.3945 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0013893521,     0.012170936 ],
  [               0,               0,               0,               0,       21.962606,      -29.137197 ],
  [               0,               0,               0,               0,       20.849901,      -32.871304 ],
  [               0,               0,               0,               0,      -18.440771,       27.960331 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0041464474,   -0.0092144972,     0.010102753 ],
  [               0,               0,               0,       3.0005956,      -16.084229,        24.49575 ],
  [               0,               0,               0,       1.8744082,      -5.8739271,       3.0246522 ],
  [               0,               0,               0,     -0.94503665,       2.9662938,       -2.171721 ],
  [               0,               0,               0,   -0.0020157413,    0.0048343115,   -0.0013633177 ],
  [               0,               0,               0,     -0.13364005,     -0.18957882,        1.608183 ],
  [               0,               0,               0,       7.4941235,      -24.704645,       18.467878 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.011610986,    -0.045125209,     0.052267939 ],
  [               0,               0,               0,      -1.0333999,       1.3512466,       5.6175804 ],
  [               0,               0,               0,     -0.83968508,       3.8880804,      -5.8729811 ],
  [               0,               0,               0,      -16.995886,       75.636391,       -70.40905 ],
  [               0,               0,               0,     0.015462701,     0.021064317,     0.004492979 ],
  [               0,               0,               0,      0.46091127,      -1.9686629,       2.1328237 ],
  [               0,               0,               0,     -0.74812663,        3.093452,      -6.2876792 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,  -0.00011913459,   -0.0024455686,     0.016181869 ],
  [               0,               0,               0,       12.113913,      -58.884678,       74.871666 ],
  [               0,               0,               0,       10.738715,      -50.535732,       58.083965 ],
  [               0,               0,               0,      -7.8702302,        43.71196,      -52.353348 ],
  [               0,               0,               0,    0.0079226308,    0.0077903043,    0.0049065971 ],
  [               0,               0,               0,      -4.2275968,       19.920996,      -23.423592 ],
  [               0,               0,               0,        8.853467,      -41.263458,       44.974258 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.028241614,    -0.064822972 ],
  [               0,               0,               0,               0,      -222.53168,       552.48706 ],
  [               0,               0,               0,               0,      -103.02482,       252.35466 ],
  [               0,               0,               0,               0,       119.09398,      -293.09167 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.016822979,    -0.041665178 ],
  [               0,               0,               0,               0,      -18.623928,       6.3073759 ],
  [               0,               0,               0,               0,      -5.5606685,        1.964081 ],
  [               0,               0,               0,               0,       119.82827,      -63.650463 ],
  [               0,               0,               0,               0,       20.169424,       49.077042 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]


gam_layer2_etaoffv3_37_b = [
    gam5_layer2_etaoffv3_37_b,
    gam10_layer2_etaoffv3_37_b,
    gam25_layer2_etaoffv3_37_b,
    gam50_layer2_etaoffv3_37_b,
    gam75_layer2_etaoffv3_37_b,
    gam100_layer2_etaoffv3_37_b,
    gam200_layer2_etaoffv3_37_b,
    gam500_layer2_etaoffv3_37_b,
    gam1000_layer2_etaoffv3_37_b,
    ]

gam_layer2_etaoffv3_37_e = [
    gam5_layer2_etaoffv3_37_e,
    gam10_layer2_etaoffv3_37_e,
    gam25_layer2_etaoffv3_37_e,
    gam50_layer2_etaoffv3_37_e,
    gam75_layer2_etaoffv3_37_e,
    gam100_layer2_etaoffv3_37_e,
    gam200_layer2_etaoffv3_37_e,
    gam500_layer2_etaoffv3_37_e,
    gam1000_layer2_etaoffv3_37_e,
    ]


###############################################################################
# Region tables.
#

layer1_etaoffv3_regions_b = [
  [ 0.0, 0.00625, 5, 0.003125, 0 ],
  [ 0.00625, 0.4, 11, 0.003125, 0 ],
  [ 0.4, 0.79375, 11, 0.003125, 0 ],
  [ 0.79365, 0.8, 3, 0.003125, 0 ],
  [ 0.8, 1.39375, 11, 0.003125, 0 ],
  [ 1.39375, 1.3955, 0, 0.003125, 0 ],
  [ 1.3955, 1.4, 0, 0.025, 0 ],
  [ 1.4, 1.45, 0, 0.025, 0 ],
]
layer1_etaoffv3_regions_e = [
  [ 1.425, 1.454, 3, 0.025, 0 ],
  [ 1.454, 1.475, 0, 0.025, 0 ],
  [ 1.475, 1.5, 3, 0.025, 0 ],
  [ 1.5, 1.503125, 0, 0.003125, 0 ],
  [ 1.503125, 1.8, 0, 0.003125, 0 ],
  [ 1.8, 2.0, 0, 0.00416666666667, 0 ],
  [ 2.0, 2.29375, 0, 0.00625, 0 ],
  [ 2.29375, 2.30625, 3, 0.00625, 0 ],
  [ 2.30625, 2.39375, 0, 0.00625, 0 ],
  [ 2.39375, 2.4, 0, 0.00625, 0 ],
  [ 2.4, 2.475, 10, 0.025, 0 ],
  [ 2.475, 2.5, 0, 0.025, 0 ],
]

layer2_etaoffv3_regions_b = [
  [ 0.0, 0.4, 10, 0.025, 0 ],
  [ 0.4, 0.775, 10, 0.025, 0 ],
  [ 0.775, 0.825, 10, 0.025, 0 ],
  [ 0.825, 1.35, 10, 0.025, 0 ],
  [ 1.35, 1.4, 3, 0.025, 0 ],
  [ 1.4, 1.437, 3, 0.075, -0.025 ],
]
layer2_etaoffv3_regions_e = [
  [ 1.4, 1.45, 5, 0.025, 0 ],
  [ 1.45, 1.5, 3, 0.025, 0 ],
  [ 1.5, 1.525, 3, 0.025, 0 ],
  [ 1.525, 1.9, 10, 0.025, 0 ],
  [ 1.9, 2.3, 10, 0.025, 0 ],
  [ 2.3, 2.45, 10, 0.025, 0 ],
  [ 2.45, 2.475, 3, 0.025, 0 ],
  [ 2.475, 2.5, 0, 0.025, 0 ],
]


ele_etaoffv3_energies = [
    5*GeV, 10*GeV, 25*GeV, 50*GeV, 75*GeV, 100*GeV, 200*GeV, 500*GeV, 1000*GeV
    ]
gam_etaoffv3_energies = [
    5*GeV, 10*GeV, 25*GeV, 50*GeV, 75*GeV, 100*GeV, 200*GeV, 500*GeV, 1000*GeV
    ]

etaoffv3_energies = {'ele55' : ele_etaoffv3_energies,
                     'ele35' : ele_etaoffv3_energies,
                     'ele37' : ele_etaoffv3_energies,
                     'gam55' : gam_etaoffv3_energies,
                     'gam35' : gam_etaoffv3_energies,
                     'gam37' : gam_etaoffv3_energies,
                     
                     # Use 5x5 for other cluster sizes.
                     'ele33' : ele_etaoffv3_energies,
                     'ele57' : ele_etaoffv3_energies,
                     'ele77' : ele_etaoffv3_energies,
                     'gam33' : gam_etaoffv3_energies,
                     'gam57' : gam_etaoffv3_energies,
                     'gam77' : gam_etaoffv3_energies,
                     }

class CaloSwEtaoff_v3_b1_parms:
    region = CALOCORR_EMB1
    regions = layer1_etaoffv3_regions_b
    energy_degree = 3
    forms = [[]]

    correction = {'ele55' : ele_layer1_etaoffv3_55_b,
                  'ele35' : ele_layer1_etaoffv3_35_b,
                  'ele37' : ele_layer1_etaoffv3_37_b,
                  'gam55' : gam_layer1_etaoffv3_55_b,
                  'gam35' : gam_layer1_etaoffv3_35_b,
                  'gam37' : gam_layer1_etaoffv3_37_b,

                  # Use 5x5 for other cluster sizes.
                  'ele33' : ele_layer1_etaoffv3_55_b,
                  'ele57' : ele_layer1_etaoffv3_55_b,
                  'ele77' : ele_layer1_etaoffv3_55_b,
                  'gam33' : gam_layer1_etaoffv3_55_b,
                  'gam57' : gam_layer1_etaoffv3_55_b,
                  'gam77' : gam_layer1_etaoffv3_55_b,
                  }

    energies = etaoffv3_energies


class CaloSwEtaoff_v3_b2_parms:
    region = CALOCORR_EMB2
    regions = layer2_etaoffv3_regions_b
    energy_degree = 3
    forms = [[]]

    correction = {'ele55' : ele_layer2_etaoffv3_55_b,
                  'ele35' : ele_layer2_etaoffv3_35_b,
                  'ele37' : ele_layer2_etaoffv3_37_b,
                  'gam55' : gam_layer2_etaoffv3_55_b,
                  'gam35' : gam_layer2_etaoffv3_35_b,
                  'gam37' : gam_layer2_etaoffv3_37_b,

                  # Use 5x5 for other cluster sizes.
                  'ele33' : ele_layer2_etaoffv3_55_b,
                  'ele57' : ele_layer2_etaoffv3_55_b,
                  'ele77' : ele_layer2_etaoffv3_55_b,
                  'gam33' : gam_layer2_etaoffv3_55_b,
                  'gam57' : gam_layer2_etaoffv3_55_b,
                  'gam77' : gam_layer2_etaoffv3_55_b,
                  }

    energies = etaoffv3_energies


class CaloSwEtaoff_v3_e1_parms:
    region = CALOCORR_EME1
    regions = layer1_etaoffv3_regions_e
    energy_degree = 3
    forms = [[]]

    correction = {'ele55' : ele_layer1_etaoffv3_55_e,
                  'ele35' : ele_layer1_etaoffv3_35_e,
                  'ele37' : ele_layer1_etaoffv3_37_e,
                  'gam55' : gam_layer1_etaoffv3_55_e,
                  'gam35' : gam_layer1_etaoffv3_35_e,
                  'gam37' : gam_layer1_etaoffv3_37_e,

                  # Use 5x5 for other cluster sizes.
                  'ele33' : ele_layer1_etaoffv3_55_e,
                  'ele57' : ele_layer1_etaoffv3_55_e,
                  'ele77' : ele_layer1_etaoffv3_55_e,
                  'gam33' : gam_layer1_etaoffv3_55_e,
                  'gam57' : gam_layer1_etaoffv3_55_e,
                  'gam77' : gam_layer1_etaoffv3_55_e,
                  }

    energies = etaoffv3_energies


class CaloSwEtaoff_v3_e2_parms:
    region = CALOCORR_EME2
    regions = layer2_etaoffv3_regions_e
    energy_degree = 3
    forms = [[]]

    correction = {'ele55' : ele_layer2_etaoffv3_55_e,
                  'ele35' : ele_layer2_etaoffv3_35_e,
                  'ele37' : ele_layer2_etaoffv3_37_e,
                  'gam55' : gam_layer2_etaoffv3_55_e,
                  'gam35' : gam_layer2_etaoffv3_35_e,
                  'gam37' : gam_layer2_etaoffv3_37_e,

                  # Use 5x5 for other cluster sizes.
                  'ele33' : ele_layer2_etaoffv3_55_e,
                  'ele57' : ele_layer2_etaoffv3_55_e,
                  'ele77' : ele_layer2_etaoffv3_55_e,
                  'gam33' : gam_layer2_etaoffv3_55_e,
                  'gam57' : gam_layer2_etaoffv3_55_e,
                  'gam77' : gam_layer2_etaoffv3_55_e,
                  }

    energies = etaoffv3_energies




CaloSwEtaoff_v3_parms = { EMB1 : CaloSwEtaoff_v3_b1_parms,
                          EMB2 : CaloSwEtaoff_v3_b2_parms,
                          EME1 : CaloSwEtaoff_v3_e1_parms,
                          EME2 : CaloSwEtaoff_v3_e2_parms,
                          }



#
# Notes on the derivation of the above tables.
# The following are the descriptions of the fits used.
# The D constructor has the signature:
#
#     def __init__ (self, sampling, cellsize, lo, hi, degree, form,
#                   ysize = 0.001, prange = (None, None),
#                   psize = None, cond = None,
#                   outlier_sigma = 5,
#                   phase = 0,
#                   initvals = {},
#                   cmds = []):
#
# Several fixups were then made by hand:
#  - The following regions/layers were removed because they were not completely
#    populated in eta (due to et falling below threshold):
#
#      ele5: 1/1, 0/2
#      gam5: 4/1, 3/2
#      ele10: 9/2
#      gam10: 9/2
#
#  - For region 13/2 in ele25 and gam25, the fits for the 5x5 cluster size
#    are copied to the 3x5 and 3x7 cluster sizes
#
#
#
# layer1_regions = [
#     D(EMB1, 0.025/8, 0.0,     0.00625, 3, 5, 0.004), # 0
#     D(EMB1, 0.025/8, 0.00625, 0.4,     1, 11, prange=(0.1, 0.15), psize=0.0001),#1
#     D(EMB1, 0.025/8, 0.4,     0.79375, {'ele1000':1,
#                                         'gam1000':1,
#                                         None: 2},
#                                        11, prange=(0.6, 0.65), psize=0.0001),#2
#     D(EMB1, 0.025/8, 0.79365, 0.8,     {'ele10':0,
#                                         'gam10':0,
#                                         None:   1}, 3), # 3
#     D(EMB1, 0.025/8, 0.8,     1.39375, {'ele500':1,
#                                         'gam500':1,
#                                         None: 2}, 11,
#       prange=(1.0, 1.05), psize=0.0001), # 4
#     D(EMB1, 0.025/8, 1.39375, 1.3955,  0, 0, 0.002, psize=0.0005), # 5
#     D(EMB1, 0.025,   1.3955,  1.4,     3, 0, 0.01), # 6
#     D(EMB1, 0.025,   1.4,     1.45,    {'ele10': 2,
#                                         'gam10': 2,
#                                         'gam50-55' : 2,
#                                         None: 3}, 0, 0.01,
#       cond = lambda c: not 1.4125<=abs(c.etasamp[1])<=1.414), # 7
#     D(EME1, 0.025,   1.425,   1.454,   {'55' : 1,
#                                         'ele1000' : 1,
#                                         'gam1000-35' : 1,
#                                         'ele500' : 1,
#                                         'gam500' : 1,
#                                         'ele75' : 1,
#                                         'gam75' : 1,
#                                         None: 0},
#                                        3, 0.015), #8
#     D(EME1, 0.025,   1.454,   1.475,   {'ele50' : 0,
#                                         'ele25' : 2,
#                                         None :    1}, 0, 0.01),  #9
#     D(EME1, 0.025,   1.475,   1.5,     {'ele25': 0,
#                                         None   : 1}, 3, 0.01),  #10
#     D(EME1, 0.025/8, 1.5,     1.503125,0, 0, 0.005), #11
#     D(EME1, 0.025/8, 1.503125,1.8,     {'ele10':1,None:3},0,prange=(1.6, 1.65),#12
#       cond = lambda c: aeta_adjust(abs(c.etasamp[EME1]),c.etamax[EME1],EME1)>1.504),
#     D(EME1, 0.025/6, 1.8,     2.0,     1, 0, prange=(1.85, 1.9),  #13
#       cond = lambda c: not 1.8006<=aeta_adjust(abs(c.etasamp[EME1]),c.etamax[EME1],EME1)<=1.8016),
#     D(EME1, 0.025/4, 2.0,     2.29375, 3, 0, prange=(2.1, 2.15)), #14
#     D(EME1, 0.025/4, 2.29375, 2.30625, 1, 3, 0.003),              #15
#     D(EME1, 0.025/4, 2.30625, 2.39375, 1, 0),                     #16
#     D(EME1, 0.025/4, 2.39375, 2.4,     {'ele200' : 0,
#                                         'gam200' : 0,
#                                         'ele75'  : 0,
#                                         'gam75'  : 0,
#                                         'ele50'  : 0,
#                                         'gam50'  : 0,
#                                         'gam100' : 0,
#                                         None: 1},
#                                        0, 0.004),              #17
#     D(EME1, 0.025,   2.4,     2.475,   {'ele500-55' : 1,
#                                         'gam500-35' : 1,
#                                         'ele200-55' : 1,
#                                         'ele200-35' : 1,
#                                         'ele200-37' : 0,
#                                         None:2},
#                                        10, 0.008),              #18
#     D(EME1, 0.025,   2.475,   2.5,     {'gam50-37':2,
#                                         'gam100':3,
#                                         None:1},
#       0, 0.008, outlier_sigma=3),#19
#     ]


# layer2_regions = [
#     D(EMB2, 0.025,   0.0,   0.4,       2, 10, psize=0.0003), # 0
#     D(EMB2, 0.025,   0.4,   0.775,     {'ele50' : 1,
#                                         None:2}, 10, psize=0.0003), # 1
#     D(EMB2, 0.025,   0.775, 0.825,     {'ele10': 0,
#                                         None: 1}, 10, 0.005, psize=0.001), # 2
#     D(EMB2, 0.025,   0.825, 1.35,      {'ele1000' : 0,
#                                         None:2}, 10, 0.002, psize=0.0003), # 3
#     D(EMB2, 0.025,   1.35,  1.4,       {'ele25':2,
#                                         None: 3}, 3, 0.005, psize=0.0004), # 4
#     D(EMB2, 0.075,   1.4,   1.437,     1, 3, 0.008, psize=0.005, phase=-0.025),

#     D(EME2, 0.025,   1.4,  1.45,      {'ele500':5,
#                                        'gam500':5,
#                                        'ele1000':4,
#                                        'gam1000':5,
#                                        None:3},
#       5, 0.02, psize=0.02, outlier_sigma=2),
#     D(EME2, 0.025,   1.45, 1.5,       {'ele25':0,
#                                        'ele1000': 4,
#                                        'gam1000': 4,
#                                        None: 3},
#       3, 0.01, psize=0.005, cmds={'ele25' : ['set par 2 1e-3',
#                                              'set par 2 1e-3',
#                                              'fix 2'],
#                                   None: []}), # 7
#     D(EME2, 0.025,   1.5,    1.525,   {'ele75' : 0,
#                                        None:1}, 3, 0.005, psize=0.003),  # 8
#     D(EME2, 0.025,   1.525,  1.9,     {'ele10': 1,
#                                        'ele10-35': 0,
#                                        'ele10-37': 0,
#                                        'gam10' : 1,
#                                        'gam10-35' : 0,
#                                        'gam10-37' : 0,
#                                        None: 2}, 10, 0.005, psize=0.001), # 9
#     D(EME2, 0.025,   1.9,  2.3,       {'ele100':1,
#                                        None:2}, 10, psize=0.0003), # 10
#     D(EME2, 0.025,   2.3,  2.45,      2, 10, psize=0.0003), # 11
#     D(EME2, 0.025,   2.45, 2.475,     {'ele25': 0,
#                                        'gam25': 0,
#                                        None:1},  3, 0.005, psize=0.001,
#       outlier_sigma = {'ele25': 2,
#                        'gam25': 2,
#                        'ele50': 3,
#                        'gam50': 3,
#                        'ele75': 3,
#                        'gam75': 3,
#                        None: 5}), # 12
#     D(EME2, 0.025,   2.475,2.5,       {'gam200': 2,
#                                        'gam100-37':2,
#                                        'ele75-55' : 2,
#                                        None:1}, 0, 0.005,
#       outlier_sigma={'gam200': 2.5,
#                      'ele100': 2.5,
#                      'ele50' : 2.5,
#                      None:3}), # 13
#     ]
